/* Log configuration directives.
 *
 * PegSoft log subsystem library (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

/** 
 * \file log/conf.h
 * \brief Log configuration directives header file.
 */

#ifndef LOG_CONF_H
#define LOG_CONF_H

/*************************************************************************/

# include <conf/directives.h>

/*************************************************************************/

/// Namespace for all log declarations.

namespace xlog
{

/*************************************************************************/

class LogLevel;

/*************************************************************************/

/// Class implementing log rules configuration directive.

/** This class implements the log rules configuration directive, that may
  * be used by log modules as a way to determine whether to
  * log a message based on its log level. */

class LogRules : public conf::ConfRules
{
	public:
		/// Type of rules.
		enum
		{
			/// Accept the message
			lrAccept = 1,
			/// Reject the message
			lrReject = 0
		};
		
		/// Constructor setting directive name and reloadability.
		LogRules(std::string const &name, bool reloadable) :
				conf::ConfRules(name, reloadable),
				m_accept("accept", reloadable, m_rules, lrAccept),
				m_reject("reject", reloadable, m_rules, lrReject) { }
		/// Copy constructor.
		LogRules(LogRules const &lr) : conf::ConfRules(lr),
				m_accept(lr.m_accept), m_reject(lr.m_reject) { }
		/// Assignment operator.
		virtual LogRules &operator=(LogRules const &right);
		
		/// Returns the first rule that matches the given log level.
		/** Returns the first rule that matches a given log level.
		  *
		  * \param[in] level The log level.
		  * \return A pointer to the first ConfRules::Rule object
		  * that matches the object, or 0 if none matched. */
		ConfRules::Rule const *match(LogLevel const &level) const;
	protected:
		virtual bool add(conf::Conf &conf, conf::Conf &new_conf);
	private:
		ConfRule m_accept;
		ConfRule m_reject;
};

/*************************************************************************/

} /* namespace xlog */

/*************************************************************************/

#endif /* LOG_CONF_H */
