/* Expressing durations.
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "misc/misc.h"

using namespace std;

/*************************************************************************/

namespace misc
{

/*************************************************************************/

char *duration_expr(char *buf, size_t len, time_t secs)
{
	if (!secs)
	{
		snprintf(buf, sizeof(buf), "0s");
		return buf;
	}

	char *end = buf;

	time_t res = secs / 604800;
	if (res) {
		end += snprintf(end, len-(end-buf), "%ldw", (long) res);
		secs -= (res * 604800);
	}
	res = secs / 86400;
	if (res) {
		end += snprintf(end, len-(end-buf), "%ldd", (long) res);
		secs -= (res * 86400);
	}
	res = secs / 3600;
	if (res) {
		end += snprintf(end, len-(end-buf), "%ldh", (long) res);
		secs -= (res * 3600);
	}
	res = secs / 60;
	if (res) {
		end += snprintf(end, len-(end-buf), "%ldm", (long) res);
		secs -= (res * 60);
	}
	if (secs)
		end += snprintf(end, len-(end-buf), "%lds", (long) secs);
	return buf;
} 

/*************************************************************************/

bool duration_scan(char const *expr, time_t *secs)
{	
	if (!expr || !*expr)
		return false;
	
	/* Be safe */
	if (secs)
		*secs = 0;

	long lv;
	char const *end = expr;

	do {
		if (*end != '-' && *end != '+' && !isdigit(*end))
			return false;
		
		lv = strtol(end, const_cast<char**>(&end), 10);
		if (errno == ERANGE)
			return false;
		
		switch (*end) 
		{
			case 's':
			case 'S':
				if (secs)
					*secs += lv;
				break;
			case 'm':
			case 'M':
				if (secs)
					*secs += lv * 60;
				break;
			case 'h':
			case 'H':
				if (secs)
					*secs += lv * 3600; 
				break;
			case 'd':
			case 'D':
				if (secs)
					*secs += lv * 86400;
				break;
			case 'w':
			case 'W':
				if (secs)
					*secs += lv * 604800;
				break;
			case '\0':
				if (secs) {
					*secs += lv;
					if (secs < 0)
						*secs = 0;
				}
				return true;
			default:
				return false;
		}
	} while (*++end);
	
	if (secs && *secs < 0)
		*secs = 0;
		
	return true;
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
