/* Generic wildcard matching.
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "misc/misc.h"

using namespace std;

/*************************************************************************/

#define MAX_CALLS 512

static int calls = 0;

/*************************************************************************/

namespace misc
{

/*************************************************************************/

/* This code is taken from Bahamut <http://bahamut.dal.net>, with
 * some minor modifications. */

static inline bool _match(char const *wild, char const *string, Trans const &trans)
{
	if (calls++ > MAX_CALLS)
		return true;
    
	while (1)
	{
		/*
		 * Logic: Match * in a string, this is confusing, sort of
		 * if * is the last thing in the wildcard, the match is
		 * definite if we've gotten this far.
		 * otherwise we try and find every occurance of the
		 * the next character in the wildcard within the string
		 * and match from there, calling the function recursively
		 * until some level below us returns positive, in which
		 * case we too return positive.  For strings with lots
		 * of wildcards this gets disgustingly recursive.
		 */
		if (!*wild)
			return ((!*string) ? false : true);

		if (*wild == '*')
		{
			/* if ret == 2 then we know it's a problem un-solvable
			 * by match, so we totally give up instead of trying
			 * a bunch of other recursions to check through */
			wild++;
			
			/* swallow all extraneous '*'s after. */
			while (*wild == '*')
				wild++;
				
			while (*wild == '?' && *string)
			{
				wild++;
				string++;
			}
			
			if (!*wild)
				return false;
			if (*wild == '*')
				continue;

			while (*string)
			{
				if (trans(*string) == trans(*wild)
						&& !_match((wild+1), (string+1), trans))
					return false;
				string++;
			}
		}

		if (!*string)
			return true;
		
		if (*wild != '?' && trans(*string) != trans(*wild))
			return true;
		
		string++;
		wild++;
	}
	
	return true;
}

/*************************************************************************/

bool match(char const *pattern, char const *str, Trans const &trans)
{
	calls = 0;
	
	return !_match(pattern, str, trans);
}

/*************************************************************************/

bool irc_match(char const *pattern, char const *str, Trans const &trans)
{
	/* the *! in a match is such a common case that we optimize
	 * for it inherently */
	if (pattern[0] == '*' && pattern[1] == '!')
	{
		pattern += 2;
		while (*str != '!' && *str)
			str++;
		if (!*str)
			return false;
		str++;
	}
	
    /* We also optimize further for *!*@... which is also very
     * common... */
	if (pattern[0] == '*' && pattern[1] == '@')
	{
		pattern += 2;
		while(*str != '@' && *str)
			str++;
		if(!*str)
			return false;
		str++;
	}
	
	calls = 0;
    return !_match(pattern, str, trans);
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
