/* Buffers base class.
 *
 * PegSoft sockets library (c) 2007 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include "sockets/buffer.h"

using namespace std;

/*************************************************************************/

namespace sockets
{

/*************************************************************************/

Buffer::Iterator::Iterator(Buffer const *buffer, int state) : m_buffer(buffer),
		m_idx(0), m_ch(0)
{
	switch (state)
	{
		case isBegin:
			char ch;
			
			if (buffer->read_byte(m_idx, &ch))
			{
				m_ch = ch;
				break;
			}
			
			/* Fall through */
		case isEnd:
			m_idx = buffer->size();
			break;
		default:
			break;	
	}
}

/*************************************************************************/

Buffer::Iterator::Iterator(Buffer const *buffer, Buffer::size_type idx) :
		m_buffer(buffer), m_idx(idx), m_ch(0)
{
	char ch;
	
	if (buffer->read_byte(m_idx, &ch))
		m_ch = ch;
}

/*************************************************************************/

Buffer::Iterator &Buffer::Iterator::operator=(Iterator const &right)
{
	if (this != &right)
	{
		m_buffer = right.m_buffer;
		m_idx = right.m_idx;
		m_ch = right.m_ch; 
	}
	
	return *this;
}

/*************************************************************************/

bool Buffer::Iterator::operator==(Iterator const &right) const
{
	if (this == &right)
		return true;
	
	return (m_buffer == right.m_buffer) && (m_idx == right.m_idx); 
}

/*************************************************************************/

Buffer::Iterator &Buffer::Iterator::operator++()
{
	Buffer::size_type sz = m_buffer->size();
	
	if (m_idx == sz)
		return *this;
	
	++m_idx;
	
	if (m_idx != sz)
	{
		char ch;
		
		if (m_buffer->read_byte(m_idx, &ch))
			m_ch = ch;
	}
	
	return *this;
}

/*************************************************************************/

Buffer::Iterator Buffer::Iterator::operator++(int)
{
	Iterator before(*this);
	operator++();
	return before;
}

/*************************************************************************/

Buffer::Iterator &Buffer::Iterator::operator--()
{
	if (m_idx == 0)
		return *this;
	
	--m_idx;
	
	char ch;
		
	if (m_buffer->read_byte(m_idx, &ch))
		m_ch = ch;
	
	return *this;
}

/*************************************************************************/

Buffer::Iterator Buffer::Iterator::operator--(int)
{
	Iterator before(*this);
	operator--();
	return before;
}

/*************************************************************************/

} /* namespace sockets */

/*************************************************************************/
