/* TCP socket class.
 *
 * PegSoft sockets library (c) 2007 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include "sockets/tcpsocket.h"

using namespace std;

/*************************************************************************/

namespace sockets
{

/*************************************************************************/

TCPSocket &TCPSocket::operator=(TCPSocket const &right)
{
	if (this != &right)
	{
		TCPClient::operator=(right);
		
		m_next = 0;
	}
	
	return *this;
}

/*************************************************************************/

bool TCPSocket::add_remote(char const *address, char const *port,
				char const **error)
{
	if (status() & (ssConnecting | ssConnected | ssListening))
		return true;
	
	if (!(status() & ssRemoteInfo))
		return set_remote(address, port, error);
	
	/* Resolve. The new peers get added to the list, we have nothing to do. */
	if (!resolve(address, port, error, &m_remote))
		return false;
	
	return true;
}

/*************************************************************************/

bool TCPSocket::connect()
{
	if (status() & (ssConnecting | ssConnected))
		return true;
	
	if (!(status() & ssRemoteInfo))
		return false;
	
	return connect(remote());
}

/*************************************************************************/

bool TCPSocket::connect(IPInfo const *ipi)
{
	if (!(status() & ssCreated) && !create())
		return false;

	while (::connect(fd(), ipi->addr(), ipi->addrlen()) == -1)
	{
		if (errno == EINTR)
			continue;
		
		if (errno == EINPROGRESS)
		{
			errno = 0;
			status_add(ssConnecting);
			
			if (!watch())
				return false;
			
			return true;
		}
		
		return false;
	}
	
	status_add(ssConnected | ssWritable);
	on_connect(0);
	
	if (status() & ssWritable)
		on_write();
	
	if (!watch())
		return false;
	
	return true;
}

/*************************************************************************/

bool TCPSocket::connect_next()
{
	if (status() & (ssConnecting | ssConnected))
		return true;
	
	if (!(status() & ssRemoteInfo))
		return false;
	
	m_next = ((m_next && m_next->next()) ? m_next->next() : remote());
	return connect(m_next);
}

/*************************************************************************/

char const *TCPSocket::raddress() const
{
	if (m_next)
		return m_next->cname();
	else
		return TCPClient::raddress();
}

/*************************************************************************/

unsigned short TCPSocket::rport() const
{
	if (m_next)
		return port(m_next);
	else
		return TCPClient::rport();
}

/*************************************************************************/

bool TCPSocket::set_remote(char const *address, char const *port,
		char const **error)
{
	if (TCPClient::set_remote(address, port, error))
	{
		m_next = 0;
		return true;
	}
	
	return false;
}

/*************************************************************************/

} /* namespace sockets */

/*************************************************************************/
