/* Channel handling.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * Based on the original code of Services by Andy Church.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef CHANNELS_H
#define CHANNELS_H

/*************************************************************************/

#include "chanserv.h"

/*************************************************************************/

typedef struct cbmode_ CBMode;
typedef struct cbmodeinfo_ CBModeInfo;
typedef struct cmmode_ CMMode;
typedef struct csmode_ CSMode;
typedef struct cumode_ CUMode;
typedef struct cumodeinfo_ CUModeInfo;

struct cbmode_ {
	uint32_t flag;			/* Long value that represents the mode */
	uint16_t flags;			/* Flags applying to this mode (CBM_* below) */
	
	/* Function to associate a value with the mode */
	void (*setvalue)	(Channel *chan, char const *value);
	void (*cssetvalue)	(ChannelInfo::MLockData *mld, char *value);
};

#define CBM_MINUS_NO_ARG 	0x0001		/* No argument for unset */
#define CBM_NO_MLOCK		0x0002		/* Can't be MLOCKed */
#define CBM_NO_USER_MLOCK   0x0004      /* Can't be MLOCKed by non-opers */

struct cbmodeinfo_ {
	char mode;				/* The mode */
	uint32_t flag;			/* Long value that represents the mode */
	uint16_t flags;			/* CBM_* above */
	
	/* Function to retrieve the value associated to the mode (optional) */
	char * (*getvalue) 		(Channel *chan);
	char const * (*csgetvalue) 	(ChannelInfo::MLockData const *mld);
};

struct cmmode_ {
	void (*addmask) (char const *source, Channel *chan, char const *mask);
	void (*delmask) (char const *source, Channel *chan, char const *mask);
};

struct cumode_ {
	int16_t status;			/* CUS_* below */
	int16_t flags;			/* CUF_* below */
	
	int (*is_valid) (User *user, Channel *chan, int servermode);
};

struct cumodeinfo_ {
	char mode;				/* The mode */
	uint16_t status;			/* CUS_* below */
};

/* User flags on channel */
#define CUS_OP			0x0001
#define CUS_VOICE		0x0002

#ifdef HAS_HALFOP
#define CUS_HALFOP		0x0004		/* Halfop (+h) */
#endif

/* Used by Unreal */
#ifdef IRC_UNREAL
#define CUS_OWNER		0x0008		/* Owner/Founder (+q) */
#define CUS_PROTECT		0x0010		/* Protected users (+a) */
#endif

/* Channel user mode flags */

#define CUF_PROTECT_BOTSERV 	0x0001

/* Useful value */

#if defined(IRC_ULTIMATE)
# define CHAN_MAX_SYMBOLS   3
#elif defined(IRC_UNREAL)
# define CHAN_MAX_SYMBOLS	5
#else
# define CHAN_MAX_SYMBOLS   2
#endif 

/* Binary modes that need to be cleared */

#if defined(IRC_BAHAMUT)
#define MODESTOREMOVE "-ciklmnpstOR"
#elif defined(IRC_ULTIMATE)
#define MODESTOREMOVE "-fiklmnpstxAIKLORS"
#elif defined(IRC_UNREAL)
#define MODESTOREMOVE "-cfiklmnpstuzACGHKLMNOQRSTV"
#else
#define MODESTOREMOVE "-iklmnpstR"
#endif

/*************************************************************************/

extern CBMode cbmodes[128];
extern CBModeInfo cbmodeinfos[];
extern CUModeInfo cumodeinfos[];

/*************************************************************************/

#endif /* CHANNELS_H */
