/* ChanServ functions.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */


#include <system/headers.h>

#include <core/directives.h>
#include <core/epona.h>
#include <databases/database.h>
#include <log/levels.h>
#include <mail/mail.h>
#include <modules/manager.h>
#include <modules/module.h>

#include "botserv.h"
#include "channels.h"
#include "chanserv.h"
#include "epona.h"
#include "nickserv.h"
#include "operserv.h"

#include "services.h"
#include "pseudo.h"

using namespace std;
using namespace core;

/*************************************************************************/

LOGLEVELH(DebugChanServ, xlog::LogDebug, "Debug::ChanServ", "debug: chanserv: ");

LOGLEVELH(InfoChanServSettings, InfoChanServ, "Info::ChanServ::Settings", "chanserv settings: ");

LOGLEVELH(NoticeChanServ, xlog::LogNotice, "Notice::ChanServ", "chanserv: ");
LOGLEVELH(NoticeChanServPrivs, NoticeChanServ, "Notice::ChanServ::Privileges", "chanserv privileged: ");

LOGLEVELH(WarningChanServ, xlog::LogWarning, "Warning::ChanServ", "warning: chanserv: ");

/*************************************************************************/

typedef struct 
{
    int what;
    char const *name;
    int desc;
} LevelInfo;

static LevelInfo levelinfo[] = {
    { CA_AUTODEOP,      "AUTODEOP",   	CHAN_LEVEL_AUTODEOP },
#ifdef HAS_HALFOP
	{ CA_AUTOHALFOP,    "AUTOHALFOP",   CHAN_LEVEL_AUTOHALFOP },
#endif
    { CA_AUTOOP,        "AUTOOP",     	CHAN_LEVEL_AUTOOP },
#ifdef IRC_UNREAL
	{ CA_AUTOPROTECT,   "AUTOPROTECT",  CHAN_LEVEL_AUTOPROTECT },
#endif
    { CA_AUTOVOICE,     "AUTOVOICE",  	CHAN_LEVEL_AUTOVOICE },
    { CA_NOJOIN,        "NOJOIN",     	CHAN_LEVEL_NOJOIN },
    { CA_SIGNKICK,      "SIGNKICK",     CHAN_LEVEL_SIGNKICK },
    
    { CA_ACCESS_LIST,   "ACC-LIST",   	CHAN_LEVEL_ACCESS_LIST },
    { CA_ACCESS_CHANGE, "ACC-CHANGE", 	CHAN_LEVEL_ACCESS_CHANGE },
    { CA_AKICK,         "AKICK",      	CHAN_LEVEL_AKICK },
    { CA_SET,           "SET",        	CHAN_LEVEL_SET },
    
    { CA_BAN,           "BAN",          CHAN_LEVEL_BAN },
	{ CA_BANME,         "BANME",        CHAN_LEVEL_BANME },
    { CA_CLEAR,         "CLEAR",      	CHAN_LEVEL_CLEAR },
    { CA_GETKEY,        "GETKEY",     	CHAN_LEVEL_GETKEY },
#ifdef HAS_HALFOP
	{ CA_HALFOP,        "HALFOP",       CHAN_LEVEL_HALFOP },
    { CA_HALFOPME,      "HALFOPME",     CHAN_LEVEL_HALFOPME },
#endif
    { CA_INFO,          "INFO",         CHAN_LEVEL_INFO },
	{ CA_KICK,          "KICK",         CHAN_LEVEL_KICK },
	{ CA_KICKME,        "KICKME",       CHAN_LEVEL_KICKME },
    { CA_INVITE,        "INVITE",     	CHAN_LEVEL_INVITE },
    { CA_OPDEOP,        "OPDEOP",     	CHAN_LEVEL_OPDEOP },
    { CA_OPDEOPME,      "OPDEOPME",     CHAN_LEVEL_OPDEOPME },
#ifdef IRC_UNREAL
	{ CA_PROTECT,       "PROTECT",      CHAN_LEVEL_PROTECT },
    { CA_PROTECTME,     "PROTECTME",    CHAN_LEVEL_PROTECTME },
#endif
    { CA_TOPIC,         "TOPIC",        CHAN_LEVEL_TOPIC },
    { CA_UNBAN,         "UNBAN",      	CHAN_LEVEL_UNBAN },
    { CA_VOICE,         "VOICE",      	CHAN_LEVEL_VOICE },
    { CA_VOICEME,       "VOICEME",      CHAN_LEVEL_VOICEME },
    
    { CA_MEMO,          "MEMO",       	CHAN_LEVEL_MEMO },
    
    { CA_ASSIGN,        "ASSIGN",     	CHAN_LEVEL_ASSIGN },
    { CA_BADWORDS,      "BADWORDS",   	CHAN_LEVEL_BADWORDS },
    { CA_FANTASIA,      "FANTASIA",   	CHAN_LEVEL_FANTASIA },
    { CA_GREET,			"GREET",	  	CHAN_LEVEL_GREET },
    { CA_NOKICK,        "NOKICK",     	CHAN_LEVEL_NOKICK },
    { CA_SAY,			"SAY",		  	CHAN_LEVEL_SAY },
    
    { -1 }
};
static int levelinfo_maxwidth = 0;

CSModeUtil csmodeutils[] = {
	{ "DEOP",   	"!deop",		"-o",   ChannelInfo::cfOpNotice,	CA_OPDEOP, CA_OPDEOPME },
	{ "OP",			"!op",			"+o",	ChannelInfo::cfOpNotice,	CA_OPDEOP, CA_OPDEOPME },
	{ "DEVOICE",	"!devoice", 	"-v",   0          ,	CA_VOICE,  CA_VOICEME  },
	{ "VOICE",		"!voice",   	"+v",   0          ,	CA_VOICE,  CA_VOICEME  },
#ifdef HAS_HALFOP
    { "DEHALFOP",	"!dehalfop",	"-h",	0          ,	CA_HALFOP, CA_HALFOPME },
	{ "HALFOP",		"!halfop",		"+h",	0          ,	CA_HALFOP, CA_HALFOPME },
#endif
#ifdef IRC_UNREAL
	{ "DEPROTECT",	"!deprotect",	"-a",	0          ,	CA_PROTECT, CA_PROTECTME },
	{ "PROTECT",	"!protect",		"+a",	0          ,	CA_PROTECT, CA_PROTECTME },
#endif
	{ NULL }
};

/*************************************************************************/

/*static void alpha_insert_chan(ChannelInfo *ci);
static ChannelInfo *makechan(const char *chan);
static int delchan(ChannelInfo *ci);*/
static int is_real_owner(User *user, ChannelInfo *ci);
static int is_identified(User *user, ChannelInfo *ci);
static void make_unidentified(User *u, ChannelInfo *ci);

static void do_help(User *u);
static void do_register(User *u);
static void do_identify(User *u);
static void do_logout(User *u);
static void do_drop(User *u);
static void do_set(User *u);
static void do_set_owner(User *u, ChannelInfo *ci, char *param);
static void do_set_successor(User *u, ChannelInfo *ci, char *param);
static void do_set_password(User *u, ChannelInfo *ci, char *param);
static void do_set_desc(User *u, ChannelInfo *ci, char *param);
static void do_set_url(User *u, ChannelInfo *ci, char *param);
static void do_set_email(User *u, ChannelInfo *ci, char *param);
static void do_set_entrymsg(User *u, ChannelInfo *ci, char *param);
static void do_set_bantype(User *u, ChannelInfo *ci, char *param);
static void do_set_mlock(User *u, ChannelInfo *ci, char *param);
static void do_set_keeptopic(User *u, ChannelInfo *ci, char *param);
static void do_set_topiclock(User *u, ChannelInfo *ci, char *param);
static void do_set_private(User *u, ChannelInfo *ci, char *param);
static void do_set_secureops(User *u, ChannelInfo *ci, char *param);
static void do_set_securefounder(User *u, ChannelInfo *ci, char *param);
static void do_set_restricted(User *u, ChannelInfo *ci, char *param);
static void do_set_secure(User *u, ChannelInfo *ci, char *param);
static void do_set_signkick(User *u, ChannelInfo *ci, char *param);
static void do_set_opnotice(User *u, ChannelInfo *ci, char *param);
static void do_set_peace(User *u, ChannelInfo *ci, char *param);
static void do_set_noexpire(User *u, ChannelInfo *ci, char *param);
static void do_access(User *u);
static void do_akick(User *u);
static void do_info(User *u);
static void do_list(User *u);
static void do_clist(User *u);
static void do_invite(User *u);
static void do_levels(User *u);
static void do_util(User *u, CSModeUtil *util);
static void do_op(User *u);
static void do_deop(User *u);
static void do_voice(User *u);
static void do_devoice(User *u);
#ifdef HAS_HALFOP
static void do_halfop(User *u);
static void do_dehalfop(User *u);
#endif
#ifdef IRC_UNREAL
static void do_protect(User *u);
static void do_deprotect(User *u);
static void do_owner(User *u);
static void do_deowner(User *u);
#endif
static void do_cs_kick(User *u);
static void do_ban(User *u);
static void do_cs_topic(User *u);
static void do_unban(User *u);
static void do_clear(User *u);
static void do_getkey(User *u);
static void do_getpass(User *u);
static void do_sendpass(User *u);
static void do_forbid(User *u);
static void do_status(User *u);

/*************************************************************************/

static int access_msgs[ListCmd::cmListCmdTotal];
static int akick_msgs[ListCmd::cmListCmdTotal];

/*************************************************************************/

namespace
{
	
/*************************************************************************/

class AccessCmd : public ListCmd
{
	public:
		/// Constructor.
		AccessCmd();
	protected:
		/// Overrides ListCmd::context().
		virtual char const *context(User *u) const { return m_context; }
		
		/// Overrides ListCmd::container().
		virtual container_type *container(User *u);
		
		/// Overrides ListCmd::allow_dispatch().
		virtual bool allow_dispatch(User *u, container_type &ctn,
				char const *sub);
		
		/// Overrides ListCmd::changeable().
		virtual bool changeable(User *u, container_type &ctn,
				char const *sub) const;
		
		/// Overrides ListCmd::add().
		virtual int add(User *u, container_type &ctn, char const *sub);
		
		/// Adds a record to the access list.
		int add_record(User *u, int16_t ulevel, ChannelInfo *owner,
				NickGroup *ng, int16_t level);
		
		/// Overrides ListCmd::allow_del().
		virtual bool allow_del(User *u, container_type &ctn,
				char const *sub, databases::Record const *rec);
		
		/// Overrides ListCmd::del_record().
		virtual bool del_record(User *u, container_type &ctn,
				char const *sub, databases::Record *rec);
		
		/// Overrides ListCmd::list_record().
		virtual bool list_record(User *u, container_type &ctn,
				char const *sub, uint16_t num, databases::Record const *rec,
				uint16_t count);
	private:
		char *m_context;
		ChannelInfo *ci;
		
		AccessCmd(AccessCmd const &);
		AccessCmd &operator=(AccessCmd const &);
};

/*************************************************************************/

class AkickCmd : public ListCmd
{
	public:
		/// Constructor.
		AkickCmd();
	protected:
		/// Overrides ListCmd::context().
		virtual char const *context(User *u) const { return m_context; }
		
		/// Overrides ListCmd::container().
		virtual container_type *container(User *u);
		
		/// Overrides ListCmd::allow_dispatch().
		virtual bool allow_dispatch(User *u, container_type &ctn,
				char const *sub);
		
		/// Overrides ListCmd::dispatch().
		virtual int dispatch(User *u, container_type &ctn, char const *sub);
		
		/// Overrides ListCmd::changeable().
		virtual bool changeable(User *u, container_type &ctn,
				char const *sub) const;
		
		/// Overrides ListCmd::add().
		virtual int add(User *u, container_type &ctn, char const *sub);
		
		/// Adds a record to the access list.
		int add_record(User *u, ChannelInfo *owner, NickGroup *ng,
				char const *mask, char const *creator, time_t since,
				time_t expires, char const *reason);
		
		/// Implements the STICK subcommand
		int stick(User *u, container_type &ctn, char const *sub);
		
		/// Implements the UNSTICK subcommand
		int unstick(User *u, container_type &ctn, char const *sub);
		
		/// Overrides ListCmd::del_record().
		virtual bool del_record(User *u, container_type &ctn,
				char const *sub, databases::Record *rec);
		
		/// Overrides ListCmd::list_record().
		virtual bool list_record(User *u, container_type &ctn,
				char const *sub, uint16_t num, databases::Record const *rec,
				uint16_t count);
		
		/// Implements the ENFORCE subcommand
		int enforce(User *u, container_type &ctn, char const *sub);
	private:
		char *m_context;
		ChannelInfo *ci;
		
		AkickCmd(AkickCmd const &);
		AkickCmd &operator=(AkickCmd const &);
};

/*************************************************************************/

}

/*************************************************************************/

static Command cmds[] = {
    { "HELP",     do_help,     NULL,  -1,                       -1,-1,-1,-1 },
    { "REGISTER", do_register, NULL,  CHAN_HELP_REGISTER,       -1,-1,-1,-1 },
    { "IDENTIFY", do_identify, NULL,  CHAN_HELP_IDENTIFY,       -1,-1,-1,-1 },
    { "LOGOUT",   do_logout,   NULL,  -1,
		CHAN_HELP_LOGOUT, CHAN_SERVADMIN_HELP_LOGOUT,
		CHAN_SERVADMIN_HELP_LOGOUT, CHAN_SERVADMIN_HELP_LOGOUT },
    { "DROP",     do_drop,     NULL,  -1,
		CHAN_HELP_DROP, CHAN_SERVADMIN_HELP_DROP,
		CHAN_SERVADMIN_HELP_DROP, CHAN_SERVADMIN_HELP_DROP },
    { "SET",      do_set,      NULL,  CHAN_HELP_SET,
		-1, CHAN_SERVADMIN_HELP_SET,
		CHAN_SERVADMIN_HELP_SET, CHAN_SERVADMIN_HELP_SET },
    { "SET FOUNDER",    NULL,  NULL,  CHAN_HELP_SET_FOUNDER,    -1,-1,-1,-1 },
    { "SET SUCCESSOR",  NULL,  NULL,  CHAN_HELP_SET_SUCCESSOR,  -1,-1,-1,-1 },
    { "SET PASSWORD",   NULL,  NULL,  CHAN_HELP_SET_PASSWORD,   -1,-1,-1,-1 },
    { "SET DESC",       NULL,  NULL,  CHAN_HELP_SET_DESC,       -1,-1,-1,-1 },
    { "SET URL",        NULL,  NULL,  CHAN_HELP_SET_URL,        -1,-1,-1,-1 },
    { "SET EMAIL",      NULL,  NULL,  CHAN_HELP_SET_EMAIL,      -1,-1,-1,-1 },
    { "SET ENTRYMSG",   NULL,  NULL,  CHAN_HELP_SET_ENTRYMSG,   -1,-1,-1,-1 },
    { "SET BANTYPE",    NULL,  NULL,  CHAN_HELP_SET_BANTYPE,    -1,-1,-1,-1 },
    { "SET PRIVATE",    NULL,  NULL,  CHAN_HELP_SET_PRIVATE,    -1,-1,-1,-1 },
    { "SET KEEPTOPIC",  NULL,  NULL,  CHAN_HELP_SET_KEEPTOPIC,  -1,-1,-1,-1 },
    { "SET TOPICLOCK",  NULL,  NULL,  CHAN_HELP_SET_TOPICLOCK,  -1,-1,-1,-1 },
    { "SET MLOCK",      NULL,  NULL,  CHAN_HELP_SET_MLOCK,      -1,-1,-1,-1 },
    { "SET RESTRICTED", NULL,  NULL,  CHAN_HELP_SET_RESTRICTED, -1,-1,-1,-1 },
    { "SET SECURE",     NULL,  NULL,  CHAN_HELP_SET_SECURE,     -1,-1,-1,-1 },
    { "SET SECUREOPS",  NULL,  NULL,  CHAN_HELP_SET_SECUREOPS,  -1,-1,-1,-1 },
    { "SET SECUREFOUNDER",	NULL,  NULL,  CHAN_HELP_SET_SECUREFOUNDER,  -1,-1,-1,-1 },
    { "SET SIGNKICK",   NULL,  NULL,  CHAN_HELP_SET_SIGNKICK,   -1,-1,-1,-1 },
    { "SET OPNOTICE",   NULL,  NULL,  CHAN_HELP_SET_OPNOTICE,   -1,-1,-1,-1 },
    { "SET PEACE",      NULL,  NULL,  CHAN_HELP_SET_PEACE,      -1,-1,-1,-1 },
    { "SET NOEXPIRE",   NULL,  NULL,  -1, -1,
		CHAN_SERVADMIN_HELP_SET_NOEXPIRE,
		CHAN_SERVADMIN_HELP_SET_NOEXPIRE,
		CHAN_SERVADMIN_HELP_SET_NOEXPIRE },
    { "ACCESS",   do_access,   NULL,  CHAN_HELP_ACCESS,         -1,-1,-1,-1 },
    { "ACCESS LEVELS",  NULL,  NULL,  CHAN_HELP_ACCESS_LEVELS,  -1,-1,-1,-1 },
    { "AKICK",    do_akick,    NULL,  CHAN_HELP_AKICK,          -1,-1,-1,-1 },
    { "LEVELS",   do_levels,   NULL,  CHAN_HELP_LEVELS,         -1,-1,-1,-1 },
    { "INFO",     do_info,     NULL,  CHAN_HELP_INFO,           
		-1, CHAN_SERVADMIN_HELP_INFO, CHAN_SERVADMIN_HELP_INFO, 
		CHAN_SERVADMIN_HELP_INFO },
    { "LIST",     do_list,     NULL,  -1,
		CHAN_HELP_LIST, CHAN_SERVADMIN_HELP_LIST,
		CHAN_SERVADMIN_HELP_LIST, CHAN_SERVADMIN_HELP_LIST },
	{ "CLIST",     do_clist,   NULL,  CHAN_HELP_CLIST,
		-1, CHAN_SERVADMIN_HELP_CLIST,
		CHAN_SERVADMIN_HELP_CLIST, CHAN_SERVADMIN_HELP_CLIST },
    { "OP",       do_op,       NULL,  CHAN_HELP_OP,             -1,-1,-1,-1 },
    { "DEOP",     do_deop,     NULL,  CHAN_HELP_DEOP,           -1,-1,-1,-1 },
    { "VOICE",    do_voice,    NULL,  CHAN_HELP_VOICE,          -1,-1,-1,-1 },
    { "DEVOICE",  do_devoice,  NULL,  CHAN_HELP_DEVOICE,        -1,-1,-1,-1 },
#ifdef HAS_HALFOP
	{ "HALFOP",   do_halfop,   NULL,  CHAN_HELP_HALFOP,         -1,-1,-1,-1 },
    { "DEHALFOP", do_dehalfop, NULL,  CHAN_HELP_DEHALFOP,       -1,-1,-1,-1 },
#endif
#ifdef IRC_UNREAL
    { "PROTECT",  do_protect,  NULL,  CHAN_HELP_PROTECT,        -1,-1,-1,-1 },
    { "DEPROTECT",do_deprotect,NULL,  CHAN_HELP_DEPROTECT,      -1,-1,-1,-1 },
    { "OWNER",    do_owner,    NULL,  CHAN_HELP_OWNER,          -1,-1,-1,-1 },
    { "DEOWNER",  do_deowner,  NULL,  CHAN_HELP_DEOWNER,        -1,-1,-1,-1 },
#endif
    { "KICK",     do_cs_kick,  NULL,  CHAN_HELP_KICK,           -1,-1,-1,-1 },
    { "BAN",      do_ban,      NULL,  CHAN_HELP_BAN,            -1,-1,-1,-1 },
    { "TOPIC",    do_cs_topic, NULL,  CHAN_HELP_TOPIC,          -1,-1,-1,-1 },
    { "INVITE",   do_invite,   NULL,  CHAN_HELP_INVITE,         -1,-1,-1,-1 },
    { "UNBAN",    do_unban,    NULL,  CHAN_HELP_UNBAN,          -1,-1,-1,-1 },
    { "CLEAR",    do_clear,    NULL,  CHAN_HELP_CLEAR,          -1,-1,-1,-1 },
    { "GETKEY",   do_getkey,   NULL,  CHAN_HELP_GETKEY,         -1,-1,-1,-1 },
    { "SENDPASS", do_sendpass, NULL,  CHAN_HELP_SENDPASS,       -1,-1,-1,-1 },
    { "GETPASS",  do_getpass,  is_services_admin,  -1,
		-1, CHAN_SERVADMIN_HELP_GETPASS,
		CHAN_SERVADMIN_HELP_GETPASS, CHAN_SERVADMIN_HELP_GETPASS },
    { "FORBID",   do_forbid,   is_services_admin,  -1,          -1, -1, -1, -1 },
    { "STATUS",   do_status,   is_services_admin,  -1,
		-1, CHAN_SERVADMIN_HELP_STATUS,
		CHAN_SERVADMIN_HELP_STATUS, CHAN_SERVADMIN_HELP_STATUS },
    { NULL }
};

/*************************************************************************/
/*************************************************************************/

/* Display total number of registered channels and info about each; or, if
 * a specific channel is given, display information about that channel
 * (like /msg ChanServ INFO <channel>).  If count_only != 0, then only
 * display the number of registered channels (the channel parameter is
 * ignored).
 */

void listchans(int count_only, const char *chan)
{
    int count = 0;
    ChannelInfo *ci;

    if (count_only) {
		printf("%d channels registered.\n", channels.size());
    } else if (chan) {

	time_t t;
	struct tm *tm;
	char buf[BUFSIZE];

	if (!(ci = channels.find(chan))) {
	    printf("Channel %s not registered.\n", chan);
	    return;
	}
	
    printf("Information about channel %s:\n", ci->name());
    printf("        Founder: %s\n", ci->owner()->name());
    printf("    Description: %s\n", ci->desc());
    t = ci->created();
    tm = localtime(&t);
    strftime(buf, sizeof(buf), getstring3(STRFTIME_DATE_TIME_FORMAT), tm);
    printf("     Registered: %s\n", buf);
    t = ci->last_used();
    tm = localtime(&t);
    strftime(buf, sizeof(buf), getstring3(STRFTIME_DATE_TIME_FORMAT), tm);
    printf("      Last used: %s\n", buf);
    if (ci->topic()) {
	printf("     Last topic: %s\n", ci->topic());
	printf("   Topic set by: %s\n", ci->topic_setter());
    }
    if (ci->url())
	printf("            URL: %s\n", ci->url());
    if (ci->email())
	printf(" E-mail address: %s\n", ci->email());
    printf("        Options: ");
    if (!ci->flags()) {
	printf("None\n");
    } else {
	int need_comma = 0;
	static const char commastr[] = ", ";
	if (ci->flags() & ChannelInfo::cfPrivate) {
	    printf("Private");
	    need_comma = 1;
	}
	if (ci->flags() & ChannelInfo::cfKeepTopic) {
	    printf("%sTopic Retention", need_comma ? commastr : "");
	    need_comma = 1;
	}
	if (ci->flags() & ChannelInfo::cfTopicLock) {
	    printf("%sTopic Lock", need_comma ? commastr : "");
	    need_comma = 1;
	}
	if (ci->flags() & ChannelInfo::cfSecureOps) {
	    printf("%sSecure Ops", need_comma ? commastr : "");
	    need_comma = 1;
	}
	if (ci->flags() & ChannelInfo::cfRestricted) {
	    printf("%sRestricted Access", need_comma ? commastr : "");
	    need_comma = 1;
	}
	if (ci->flags() & ChannelInfo::cfSecure) {
	    printf("%sSecure", need_comma ? commastr : "");
	    need_comma = 1;
	}
	if (ci->flags() & ChannelInfo::cfNoExpire) {
	    printf("%sNo Expire", need_comma ? commastr : "");
	    need_comma = 1;
	}
	printf("\n");
	}
	if (ci->mlock_on() || ci->mlock_off())
		printf("      Mode lock: %s\n", ci->mlock_modes());

    } else {

	Channels::const_niterator it, end;
	
	for (it = channels.nbegin(), end = channels.nend(); it != end; ++it)
	{
		ci = (ChannelInfo *)(*it);
		printf("  %s %-20s  %s\n", ci->flags() & ChannelInfo::cfNoExpire ? "!" : " ",
			    ci->name(),
			    ci->desc());
		count++;
	}
	printf("%d channels registered.\n", count);

    }
}

/*************************************************************************/
/*************************************************************************/

/* ChanServ initialization. */

void cs_init(void)
{
    Command *cmd;

    cmd = lookup_cmd(cmds, "REGISTER");
    if (cmd)
	cmd->help_param1 = s_NickServ;
    cmd = lookup_cmd(cmds, "SET SECURE");
    if (cmd)
	cmd->help_param1 = s_NickServ;
    cmd = lookup_cmd(cmds, "SET SUCCESSOR");
    if (cmd)
	cmd->help_param1 = (char *)(long)epona->cd().chanserv.max_reg;
}

/*************************************************************************/

bool chanserv_load()
{
	/* Load password module */
	password::Encrypter *enc = 0;
		
	if (!epona->cd().chanserv.password_module.value().empty())
	{
		enc = channels.load_encrypter(epona->cd().chanserv.password_module.value().c_str());
		if (!enc)
			return false;
	}
	
	channels.encrypter(enc);
	
	/* Set up channels database */
	
	if (!epona->modules().load(epona->cd().chanserv.database_channels))
	{
		epona->log(ErrorChanServ(), "Can't load database module %s: %s",
				epona->cd().chanserv.database_channels.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	/* Set up levels database */
	
	if (!epona->modules().load(epona->cd().chanserv.database_levels))
	{
		epona->log(ErrorChanServ(), "Can't load database module %s: %s",
				epona->cd().chanserv.database_levels.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	/* Set up access database */
	
	if (!epona->modules().load(epona->cd().chanserv.database_access))
	{
		epona->log(ErrorChanServ(), "Can't load database module %s: %s",
				epona->cd().chanserv.database_access.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	/* Set up akicks database */
	
	if (!epona->modules().load(epona->cd().chanserv.database_akicks))
	{
		epona->log(ErrorChanServ(), "Can't load database module %s: %s",
				epona->cd().chanserv.database_akicks.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	/* Set up kbb database */
	
	if (!epona->modules().load(epona->cd().botserv.database_kbb))
	{
		epona->log(ErrorChanServ(), "Can't load database module %s: %s",
				epona->cd().botserv.database_kbb.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	/* Set up badwords database */
	
	if (!epona->modules().load(epona->cd().botserv.database_badwords))
	{
		epona->log(ErrorChanServ(), "Can't load database module %s: %s",
				epona->cd().botserv.database_badwords.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	/* Configure ACCESS command */
	
	access_msgs[ListCmd::cmSyntax] = CHAN_ACCESS_SYNTAX;
	access_msgs[ListCmd::cmDisabled] = CHAN_ACCESS_DISABLED;
	access_msgs[ListCmd::cmEmpty] = CHAN_ACCESS_EMPTY;
	access_msgs[ListCmd::cmSyntaxDel] = CHAN_ACCESS_DEL_SYNTAX;
	access_msgs[ListCmd::cmNoSuchEntry] = CHAN_ACCESS_NO_SUCH_ENTRY;
	access_msgs[ListCmd::cmNoMatch] = CHAN_ACCESS_NO_MATCH;
	access_msgs[ListCmd::cmDeletedOne] = CHAN_ACCESS_DELETED_ONE;
	access_msgs[ListCmd::cmDeletedSeveral] = CHAN_ACCESS_DELETED_SEVERAL;
	access_msgs[ListCmd::cmNotFound] = CHAN_ACCESS_NOT_FOUND;
	access_msgs[ListCmd::cmDeleted] = CHAN_ACCESS_DELETED;
	access_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for ACCESS */
	access_msgs[ListCmd::cmListHeader] = CHAN_ACCESS_LIST_HEADER;
	access_msgs[ListCmd::cmListedOne] = CHAN_ACCESS_LISTED_ONE;
	access_msgs[ListCmd::cmListedSeveral] = CHAN_ACCESS_LISTED_SEVERAL;
	access_msgs[ListCmd::cmClearPartially] = CHAN_ACCESS_CLEAR_PARTIALLY;
	access_msgs[ListCmd::cmClear] = CHAN_ACCESS_CLEAR;
	
	/* Configure AKICK command */
	
	akick_msgs[ListCmd::cmSyntax] = CHAN_AKICK_SYNTAX;
	akick_msgs[ListCmd::cmDisabled] = CHAN_AKICK_DISABLED;
	akick_msgs[ListCmd::cmEmpty] = CHAN_AKICK_EMPTY;
	akick_msgs[ListCmd::cmSyntaxDel] = CHAN_AKICK_DEL_SYNTAX;
	akick_msgs[ListCmd::cmNoSuchEntry] = CHAN_AKICK_NO_SUCH_ENTRY;
	akick_msgs[ListCmd::cmNoMatch] = CHAN_AKICK_NO_MATCH;
	akick_msgs[ListCmd::cmDeletedOne] = CHAN_AKICK_DELETED_ONE;
	akick_msgs[ListCmd::cmDeletedSeveral] = CHAN_AKICK_DELETED_SEVERAL;
	akick_msgs[ListCmd::cmNotFound] = CHAN_AKICK_NOT_FOUND;
	akick_msgs[ListCmd::cmDeleted] = CHAN_AKICK_DELETED;
	akick_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for AKICK */
	akick_msgs[ListCmd::cmListHeader] = CHAN_AKICK_LIST_HEADER;
	akick_msgs[ListCmd::cmListedOne] = CHAN_AKICK_LISTED_ONE;
	akick_msgs[ListCmd::cmListedSeveral] = CHAN_AKICK_LISTED_SEVERAL;
	akick_msgs[ListCmd::cmClearPartially] = CHAN_AKICK_CLEAR_PARTIALLY;
	akick_msgs[ListCmd::cmClear] = CHAN_AKICK_CLEAR;
	
	return true;
}

/*************************************************************************/

void chanserv_notify(modules::Module &sender, int msg, void *arg)
{
	switch (msg)
	{
		case modules::Module::mnLoaded:
			if (sender.name() == epona->cd().chanserv.database_channels.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database chanserv_channels: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				channels.setup_database(db);
			}
			
			if (sender.name() == epona->cd().chanserv.database_levels.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database chanserv_levels: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				levels.setup_database(db);
			}
			
			if (sender.name() == epona->cd().chanserv.database_access.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database chanserv_access: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				accesses.setup_database(db);
			}
			
			if (sender.name() == epona->cd().chanserv.database_akicks.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database chanserv_akicks: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				akicks.setup_database(db);
			}
			
			if (sender.name() == epona->cd().botserv.database_kbb.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database botserv_kbb: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				kbbs.setup_database(db);
			}
			
			if (sender.name() == epona->cd().botserv.database_badwords.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database botserv_badwords: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				badwords.setup_database(db);
			}
		default:
			break;
	}
}

/*************************************************************************/

void chanserv_start()
{
	channels.setup_owner();
	
	if (!channels.db() || !channels.db()->start())
		epona->fatal("Can't start database chanserv_channels");
	
	levels.setup_map();
	levels.setup_owner();
	
	if (!levels.db() || !levels.db()->start())
		epona->fatal("Can't start database chanserv_levels");
	
	accesses.setup_owner();
	
	if (!accesses.db() || !accesses.db()->start())
		epona->fatal("Can't start database chanserv_access");
	
	akicks.setup_owner();
	
	if (!akicks.db() || !akicks.db()->start())
		epona->fatal("Can't start database chanserv_akicks");
	
	kbbs.setup_map();
	kbbs.setup_owner();
	
	if (!kbbs.db() || !kbbs.db()->start())
		epona->fatal("Can't start database botserv_kbb");
	
	badwords.setup_owner();
	
	if (!badwords.db() || !badwords.db()->start())
		epona->fatal("Can't start database botserv_badwords");
}

/*************************************************************************/

/* Main ChanServ routine. */

void chanserv(User *u, char *buf)
{
    char *cmd, *s;

    cmd = strtok(buf, " ");

    if (!cmd) {
		return;
    } else if (stricmp(cmd, "\1PING") == 0) {
		s = strtok(NULL, "\1");
	    
		notice(s_ChanServ, u->nick, "\1PING%s%s\1", (s ? " " : ""),
				(s ? s : ""));
    } else {
		run_cmd(s_ChanServ, u, cmds, cmd);
    }
}

/*************************************************************************/

/* Check the current modes on a channel; if they conflict with a mode lock,
 * fix them. */

void check_modes(Channel *c)
{
	char modebuf[64], argbuf[BUFSIZE], *end = modebuf, *end2 = argbuf;
	uint32_t modes;
	ChannelInfo *ci;
	CBModeInfo *cbmi;
	CBMode *cbm;
	
	if (c->bouncy_modes)
		return;

    /* Check for mode bouncing */
    if (c->server_modecount >= 3 && c->chanserv_modecount >= 3) {
		wallops(NULL, 	"Warning: unable to set modes on channel %s.  "
						"Are your servers' U:lines configured correctly?", c->name);
		epona->log(WarningChanServ(),
				"Bouncy modes on channel %s", c->name);
		c->bouncy_modes = 1;
		return;
    }

    if (c->chanserv_modetime != time(NULL)) {
		c->chanserv_modecount = 0;
		c->chanserv_modetime = time(NULL);
    }
    c->chanserv_modecount++;

    if (!(ci = c->ci)) 
    {
		if (c->mode & CMODE_r) 
		{
	    	c->mode &= ~CMODE_r;
	    	s_mode(whosends(ci), c->name, "-r", NULL);
		}
		return;
    }
	
	modes = ~c->mode & ci->mlock_data().on();
	
	*end++ = '+';
	cbmi = cbmodeinfos;
		
	do {
		if (modes & cbmi->flag) 
		{
			*end++ = cbmi->mode;
			c->mode |= cbmi->flag;
			
			/* Add the eventual parameter and modify the Channel structure */
			if (cbmi->getvalue && cbmi->csgetvalue) {
				char const *value = cbmi->csgetvalue(&ci->mlock_data());
				
				cbm = &cbmodes[(int)cbmi->mode];
				cbm->setvalue(c, value);
				
				if (value) {
					*end2++ = ' ';
					while (*value)
						*end2++ = *value++;
				}
			}
		} else if (cbmi->getvalue && cbmi->csgetvalue &&
				(ci->mlock_data().on() & cbmi->flag) && (c->mode & cbmi->flag)) {
			char *value = cbmi->getvalue(c);
			char const *csvalue = cbmi->csgetvalue(&ci->mlock_data());
			
			/* Lock and actual values don't match, so fix the mode */
			if (value && csvalue && strcmp(value, csvalue)) {
				*end++ = cbmi->mode;
				
				cbm = &cbmodes[(int)cbmi->mode];
				cbm->setvalue(c, csvalue);
				
				*end2++ = ' ';
				while (*csvalue)
					*end2++ = *csvalue++;
			}
		}
	} while ((++cbmi)->flag != 0);
	
	if (*(end-1) == '+')
		end--; 
	
	modes = c->mode & ci->mlock_data().off();
	
	if (modes) {
		*end++ = '-';
		cbmi = cbmodeinfos;
		
		do {
			if (modes & cbmi->flag) {
				*end++ = cbmi->mode;
				c->mode &= ~cbmi->flag;
				
				/* Add the eventual parameter and clean up the Channel structure */
				if (cbmi->getvalue) {
					cbm = &cbmodes[(int)cbmi->mode];
					
					if (!(cbm->flags & CBM_MINUS_NO_ARG)) {					
						char *value = cbmi->getvalue(c);
					
						if (value) {
							*end2++ = ' ';
							while (*value)
								*end2++ = *value++;
						}
					}
					
					cbm->setvalue(c, NULL);
				}
			}
		} while ((++cbmi)->flag != 0);
	}
	
	if (end == modebuf)
		return;
	
	*end = 0; *end2 = 0;
	
#ifdef IRC_BAHAMUT
	send_cmd(whosends(ci), "MODE %s 0 %s%s", c->name, modebuf, (end2 == argbuf ? "" : argbuf));
#else
	send_cmd(whosends(ci), "MODE %s %s%s", c->name, modebuf, (end2 == argbuf ? "" : argbuf));
#endif
}

/*************************************************************************/

#ifdef HAS_HALFOP

/* Check whether a user is allowed to be halfopped on a channel; if they
 * aren't, dehalfop them.  If serverop is 1, the +h was done by a server.
 * Return 1 if the user is allowed to be opped, 0 otherwise. */

int check_valid_halfop(User *user, Channel *chan, int servermode)
{
	if (!chan->ci)
		return 1;

    if (servermode && !check_access(user, chan->ci, CA_AUTOHALFOP)) {
		notice_lang(s_ChanServ, user, CHAN_IS_REGISTERED, s_ChanServ);
		
		s_mode(whosends(chan->ci), chan->name, "-h", user->nick);
		return 0;
    }

    if (check_access(user, chan->ci, CA_AUTODEOP)) {
		s_mode(whosends(chan->ci), chan->name, "-h", user->nick);
		return 0;
    }
    
    //chan->ci->last_used(time(NULL));
    return 1;
}

#endif

/*************************************************************************/

/* Check whether a user is allowed to be opped on a channel; if they
 * aren't, deop them.  If serverop is 1, the +o was done by a server.
 * Return 1 if the user is allowed to be opped, 0 otherwise. */

int check_valid_op(User *user, Channel *chan, int servermode)
{
	if (!chan->ci)
		return 1;

    if (servermode && !check_access(user, chan->ci, CA_AUTOOP)) {
		notice_lang(s_ChanServ, user, CHAN_IS_REGISTERED, s_ChanServ);
#ifdef IRC_UNREAL
		send_cmd(whosends(chan->ci), "MODE %s -aoq %s %s %s %s", chan->name, user->nick, user->nick, user->nick, user->nick);
#else
		s_mode(whosends(chan->ci), chan->name, "-o", user->nick);
#endif
		return 0;
    }

    if (check_access(user, chan->ci, CA_AUTODEOP)) {
#ifdef IRC_UNREAL
		send_cmd(whosends(chan->ci), "MODE %s -aoq %s %s %s %s", chan->name, user->nick, user->nick, user->nick, user->nick);
#else
		s_mode(whosends(chan->ci), chan->name, "-o", user->nick);
#endif
		return 0;
    }

	//chan->ci->last_used(time(NULL));
    return 1;
}

/*************************************************************************/

/* Check whether a user should be opped on a channel, and if so, do it.
 * Return 1 if the user was opped, 0 otherwise.  (Updates the channel's
 * last used time if the user was opped.) */

int check_should_op(User *user, const char *chan)
{
    ChannelInfo *ci = channels.find(chan);

    if (!ci || *chan == '+')
	return 0;

    if ((ci->flags() & ChannelInfo::cfSecure) && !nick_identified(user))
	return 0;

    if (check_access(user, ci, CA_AUTOOP)) {
	s_mode(whosends(ci), chan, "+o", user->nick);
	return 1;
    }

    return 0;
}

/*************************************************************************/

/* Check whether a user should be voiced on a channel, and if so, do it.
 * Return 1 if the user was voiced, 0 otherwise. */

int check_should_voice(User *user, const char *chan)
{
    ChannelInfo *ci = channels.find(chan);

    if (!ci || *chan == '+')
	return 0;

    if ((ci->flags() & ChannelInfo::cfSecure) && !nick_identified(user))
	return 0;

    if (check_access(user, ci, CA_AUTOVOICE)) {
	s_mode(whosends(ci), chan, "+v", user->nick);
	return 1;
    }

    return 0;
}

/*************************************************************************/

#ifdef HAS_HALFOP

int check_should_halfop(User *user, const char *chan)
{
	ChannelInfo *ci = channels.find(chan);

    if (!ci || *chan == '+')
	return 0;

    if (check_access(user, ci, CA_AUTOHALFOP)) {
	s_mode(whosends(ci), chan, "+h", user->nick);
	return 1;
    }

    return 0;
}

#endif

/*************************************************************************/

#ifdef IRC_UNREAL

int check_should_owner(User *user, const char *chan)
{
	ChannelInfo *ci = channels.find(chan);

    if (!ci || *chan == '+')
		return 0;

    if (((ci->flags() & ChannelInfo::cfSecureFounder) && is_real_owner(user, ci))
    		|| (!(ci->flags() & ChannelInfo::cfSecureFounder) && is_owner(user, ci))) {
    		
    	if (check_access(user, ci, CA_AUTOOP))
			send_cmd(whosends(ci), "MODE %s +qo %s %s", chan, user->nick, user->nick);
		else
			s_mode(whosends(ci), chan, "+q", user->nick);
		return 1;
    }

    return 0;
}

/*************************************************************************/

int check_should_protect(User *user, const char *chan)
{
	ChannelInfo *ci = channels.find(chan);

    if (!ci || *chan == '+')
		return 0;

    if (check_access(user, ci, CA_AUTOPROTECT)) {
    	if (check_access(user, ci, CA_AUTOOP))
			send_cmd(whosends(ci), "MODE %s +ao %s %s", chan, user->nick, user->nick);
		else
			s_mode(whosends(ci), chan, "+a", user->nick);
		return 1;
    }

    return 0;
}

#endif

/*************************************************************************/

/* Tiny helper routine to get ChanServ out of a channel after it went in. */

static void timeout_leave(Timeout *to)
{
	char *chan = static_cast<char *>(to->data);
    ChannelInfo *ci = channels.find(chan);
	
	if (ci)	/* Check cos the channel may be dropped in the meantime */
		ci->flags_remove(ChannelInfo::cfInhabit);
	
    send_cmd(s_ChanServ, "PART %s", chan);
    delete [] chan;
}


/* Check whether a user is permitted to be on a channel.  If so, return 0;
 * else, kickban the user with an appropriate message (could be either
 * AKICK or restricted access) and return 1.  Note that this is called
 * _before_ the user is added to internal channel lists (so do_kick() is
 * not called).
 */

int check_kick(User *user, char *chan)
{
    ChannelInfo *ci = channels.find(chan);
    Channel *c;
    Akick *akick;
    NickGroup *ng;
    char const *av[3];
    char mask[BUFSIZE];
    const char *reason;
    Timeout *t;

    if (!ci)
		return 0;

    if (is_oper(user) || is_services_admin(user))
		return 0;

	
#ifdef HAS_EXEMPTS
	/* Avoid endless fight between services and ircd */
	if (ci->c && is_exempted(ci->c, user))
		return 0;
#endif

    if (nick_recognized(user))
		ng = user->na->ng();
    else
		ng = NULL;
	
	ChannelInfo::akicks_type::const_iterator it, end;
				
	for (it = ci->akicks().begin(), end = ci->akicks().end(); it != end; ++it)
	{
		akick = (Akick *) (*it);
		
		if (!akick)
			continue;
		
		if (akick->expires() && (time(0) >= akick->expires()))
		{
			if (akicks.writable())
				akicks.remove(akick);
			continue;
		}
		
	
		if ((akick->flags() & Akick::afIsGroup && akick->ng() == ng) ||
				(!(akick->flags() & Akick::afIsGroup) && 
				match_usermask(akick->mask(), user))) {
	    	if (epona->debuglevel() >= 2)
				epona->log(DebugChanServ(),
						"%s matched akick %s", user->nick, 
						(akick->flags() & Akick::afIsGroup) ? (akick->ng())->name() :
						akick->mask());
	    	if (akick->flags() & Akick::afIsGroup) 
	    		get_idealban(ci, user, mask, sizeof(mask));
	    	else 
	    		strcpy(mask, akick->mask());
	    	reason = akick->reason() ? akick->reason() :
	    			epona->cd().chanserv.akick_reason;
	    	goto kick;
		}
    }

    if (check_access(user, ci, CA_NOJOIN)) {
		get_idealban(ci, user, mask, sizeof(mask));
		reason = getstring(user->na, CHAN_NOT_ALLOWED_TO_JOIN);
		goto kick;
    }

    return 0;

kick:
    if (epona->debuglevel())
		epona->log(DebugChanServ(), "Autokicking %s!%s@%s from %s", user->nick,
				user->username, GetHost(user), chan);
		
    /* Remember that the user has not been added to our channel user list
     * yet, so we check whether the channel does not exist OR has no user
     * on it (before SJOIN would have created the channel structure, while 
     * JOIN would not). */
	/* Don't check for ChannelInfo::cfInhabit before for the Channel record cos else
	 * c may be NULL even if it exists */
    if ((!(c = findchan(chan)) || c->usercount == 0) && !(ci->flags() & ChannelInfo::cfInhabit)) {
#ifdef IRC_BAHAMUT
		send_cmd(s_ChanServ, "SJOIN %ld %s", (c ? (long) c->creation_time :
				(long) time(NULL)), chan);
#else
		send_cmd(s_ChanServ, "JOIN %s", chan);
#endif
		t = add_timeout(epona->cd().chanserv.inhabit, timeout_leave, 0);
		t->data = misc::nstrdup(chan);
		ci->flags_add(ChannelInfo::cfInhabit);
    }
    
	if (c) 
	{
		av[0] = chan;
		av[1] = "+b";
		av[2] = mask;
    	do_cmode(whosends(ci), 3, av);
	}
	
    s_mode(whosends(ci), chan, "+b", mask);
	send_cmd(whosends(ci), "KICK %s %s :%s", chan, user->nick, reason);
    
    return 1;
}

/*************************************************************************/

/* Return 1 if the user's access level on the given channel falls into the
 * given category, 0 otherwise.  Note that this may seem slightly confusing
 * in some cases: for example, check_access(..., CA_NOJOIN) returns true if
 * the user does _not_ have access to the channel (i.e. matches the NOJOIN
 * criterion). */

int check_access(User *user, ChannelInfo *ci, int what)
{
    int level = get_access(user, ci);
    int limit = ci->levels(what);
    
    /* Resetting the last used time */
	/* if (level > 0)
		ci->last_used = time(NULL); */

    if (level == ACCESS_FOUNDER)
	return (what==CA_AUTODEOP || what==CA_NOJOIN) ? 0 : 1;
    /* Hacks to make flags work */
    if (what == CA_AUTODEOP && (ci->flags() & ChannelInfo::cfSecureOps) && level == 0)
	return 1;
	if (what == CA_NOJOIN && (ci->flags() & ChannelInfo::cfRestricted) && level <= 0)
	return 1;
    if (limit == ACCESS_INVALID)
	return 0;
    if (what == CA_AUTODEOP || what == CA_NOJOIN)
	return (level <= ci->levels(what));
    else
	return (level >= ci->levels(what));
}

/*************************************************************************/
/*********************** ChanServ private routines ***********************/
/*************************************************************************/

/* Does the given user have founder access to the channel? */

int is_owner(User *user, ChannelInfo *ci)
{
    if (user->na && user->na->ng() == ci->owner()) {
		if ((nick_identified(user) || (nick_recognized(user) && !(ci->flags() & ChannelInfo::cfSecure))))
	    return 1;
    }
    if (is_identified(user, ci))
	return 1;
    return 0;
}

/*************************************************************************/

static int is_real_owner(User *user, ChannelInfo *ci)
{
    if (user->na && user->na->ng() == ci->owner()) {
		if ((nick_identified(user) || (nick_recognized(user) && !(ci->flags() & ChannelInfo::cfSecure))))
	    return 1;
    }
    return 0;
}

/*************************************************************************/

/* Has the given user password-identified as founder for the channel? */

static int is_identified(User *user, ChannelInfo *ci)
{
    User::u_chaninfolist *c;

    for (c = user->founder_chans; c; c = c->next) {
	if (c->chan == ci)
	    return 1;
    }
    return 0;
}

/*************************************************************************/

/* Returns the Access entry for an user */

Access *get_access_entry(NickGroup *ng, ChannelInfo *ci)
{
    Access *acc;
	ChannelInfo::accesses_type::const_iterator it, end;
				
	for (it = ci->accesses().begin(), end = ci->accesses().end();
			it != end; ++it)
	{
		acc = (Access *) (*it);

	   	if (acc && acc->ng() == ng)
			return acc;
	}
	
    return NULL;
}

/*************************************************************************/

/* Return the access level the given user has on the channel.  If the
 * channel doesn't exist, the user isn't on the access list, or the channel
 * is CS_SECURE and the user hasn't IDENTIFY'd with NickServ, return 0. */

int get_access(User *user, ChannelInfo *ci)
{
    Access *access;
    
    if (!ci)
		return 0;
	
    if (is_owner(user, ci))
		return ACCESS_FOUNDER;
	
	if (!user->na)
		return 0;
	
	if (nick_identified(user) || (nick_recognized(user) && !(ci->flags() & ChannelInfo::cfSecure)))
		if ((access = get_access_entry(user->na->ng(), ci))) 
			return access->level();
    
    return 0;	
}

/*************************************************************************/

void update_cs_lastseen(User *user, ChannelInfo *ci)
{
    Access *access;
    
    if (!ci || !user->na)
	return;
	
	if (is_owner(user, ci) || nick_identified(user) || 
			(nick_recognized(user) && !(ci->flags() & ChannelInfo::cfSecure)))
		if ((access = get_access_entry(user->na->ng(), ci))) 
			access->last_seen(time(NULL));
}

/*************************************************************************/

/* Returns the best ban possible for an user depending of the bantype 
   value. */

int get_idealban(ChannelInfo *ci, User *u, char *ret, int retlen) 
{
	char *mask;
	
	if (!ci || !u || !ret || retlen == 0) 
		return 0;	
	
	switch (ci->bantype()) {
		case 0: snprintf(ret, retlen, "*!%s@%s", u->username, GetHost(u));
				return 1;
		case 1: snprintf(ret, retlen, "*!%s%s@%s", 
					(strlen(u->username) < (*(u->username) == '~' ? USERMAX+1 : USERMAX) ? "*" : ""),
    				(*(u->username) == '~' ? u->username+1 : u->username),
					GetHost(u));
				return 1;
		case 2: snprintf(ret, retlen, "*!*@%s", GetHost(u));
				return 1;
		case 3: mask = create_mask(u); 
				snprintf(ret, retlen, "*!%s", mask);
				free(mask);
				return 1;
				
		default: return 0;
	}
}

/*************************************************************************/

static void make_unidentified(User *u, ChannelInfo *ci)
{
	User::u_chaninfolist *uci;
	
	if (!u || !ci) return;
	
	for (uci = u->founder_chans; uci; uci = uci->next) {
		if (uci->chan == ci) {
			if (uci->next)
			uci->next->prev = uci->prev;
			if (uci->prev)
			uci->prev->next = uci->next;
			else
			u->founder_chans = uci->next;
			free(uci);
			break;
		}
	}
}

/*************************************************************************/

#ifdef HAS_FMODE

char const *cs_get_flood(ChannelInfo::MLockData const *mld)
{
	return mld->flood();	
}

#endif

/*************************************************************************/

#ifdef HAS_JMODE

char const *cs_get_joinrate(ChannelInfo::MLockData const *mld)
{
	return mld->joinrate();	
}

#endif

/*************************************************************************/

char const *cs_get_key(ChannelInfo::MLockData const *mld)
{
	return mld->key();	
}

/*************************************************************************/

char const *cs_get_limit(ChannelInfo::MLockData const *mld)
{
	static char limit[16];
	
	if (mld->limit() == 0)
		return NULL;
	
	snprintf(limit, sizeof(limit), "%d", mld->limit());
	return limit;
}

/*************************************************************************/

#ifdef HAS_LMODE

char const *cs_get_redirect(ChannelInfo::MLockData const *mld)
{
	return mld->redirect();	
}

#endif

/*************************************************************************/

#ifdef HAS_FMODE

void cs_set_flood(ChannelInfo::MLockData *mld, char *value)
{
	/*char *dp, *end;*/
	
	/* This looks ugly, but it works ;) */
	if (value /*&& *value != ':' && (strtoul((*value == '*' ? value+1 : value), &dp, 10) > 0) && (*dp == ':') && (*(++dp) != 0) && (strtoul(dp, &end, 10) > 0) && (*end == 0)*/) {
		mld->flood(value);
	} else {
		mld->on_remove(CMODE_f);
		mld->flood(0);
	}
}

#endif

/*************************************************************************/

#ifdef HAS_JMODE

void cs_set_joinrate(ChannelInfo::MLockData *mld, char *value)
{
	char *dp, *end;
		
	/* This looks ugly, but it works ;) */
	if (value && *value != ':' && (strtol(value, &dp, 10) > 0) && 
			(*dp == ':') && (*(++dp) != 0) && (strtol(dp, &end, 10) > 0) && 
			(*end == 0)) {
		mld->joinrate(value);
	} else {
		mld->on_remove(CMODE_j);
		mld->joinrate(0);
	}
}

#endif

/*************************************************************************/

void cs_set_key(ChannelInfo::MLockData *mld, char *value)
{
	/* Don't allow keys with a coma */
	if (value && *value != ':' && !strchr(value, ',')) {
		mld->key(value);
	} else {
		mld->on_remove(CMODE_k);
		mld->key(0);
	}
}

/*************************************************************************/

void cs_set_limit(ChannelInfo::MLockData *mld, char *value)
{
	mld->limit(value ? atol(value) : 0);
	
	if (mld->limit() <= 0)
		mld->on_remove(CMODE_l);
}

/*************************************************************************/

#ifdef HAS_LMODE

void cs_set_redirect(ChannelInfo::MLockData *mld, char *value)
{
	if (value && *value == '#') {
		mld->redirect(value);
	} else {
		mld->on_remove(CMODE_L);
		mld->redirect(0);
	}
}

#endif

/*************************************************************************/
/*********************** ChanServ command routines ***********************/
/*************************************************************************/

static void do_help(User *u)
{
    char *cmd = strtok(NULL, "");

    if (!cmd) {
	notice_help(s_ChanServ, u, CHAN_HELP);
#ifdef IRC_UNREAL
	notice_help(s_ChanServ, u, CHAN_HELP_UNREAL);
#endif
#ifdef IRC_ULTIMATE
	notice_help(s_ChanServ, u, CHAN_HELP_ULTIMATE);
#endif
	if (epona->cd().chanserv.expire && 
			epona->cd().chanserv.expire.delay >= 86400)
	    notice_help(s_ChanServ, u, CHAN_HELP_EXPIRES,
	    		((time_t)epona->cd().chanserv.expire.delay)/86400);
	if (is_services_oper(u))
	    notice_help(s_ChanServ, u, CHAN_SERVADMIN_HELP);
    } else if (stricmp(cmd, "LEVELS DESC") == 0) {
	int i;
	notice_help(s_ChanServ, u, CHAN_HELP_LEVELS_DESC);
	if (!levelinfo_maxwidth) {
	    for (i = 0; levelinfo[i].what >= 0; i++) {
		int len = strlen(levelinfo[i].name);
		if (len > levelinfo_maxwidth)
		    levelinfo_maxwidth = len;
	    }
	}
	for (i = 0; levelinfo[i].what >= 0; i++) {
	    notice_help(s_ChanServ, u, CHAN_HELP_LEVELS_DESC_FORMAT,
			levelinfo_maxwidth, levelinfo[i].name,
			getstring(u->na, levelinfo[i].desc));
	}
    } else {
	help_cmd(s_ChanServ, u, cmds, cmd);
    }
}

/*************************************************************************/

static void do_register(User *u)
{
	char *chan = strtok(NULL, " ");
	char *pass = strtok(NULL, " ");
	char *desc = strtok(NULL, "");
	NickGroup *ng;
	Channel *c;
	ChannelInfo *ci;
	int is_servadmin = is_services_admin(u);
	
	if (!channels.writable()) 
	{
		notice_lang(s_ChanServ, u, CHAN_REGISTER_DISABLED);
		return;
	}
	
	if (!desc) {
		syntax_error(s_ChanServ, u, "REGISTER", CHAN_REGISTER_SYNTAX);
	} else if (*chan == '&') {
		notice_lang(s_ChanServ, u, CHAN_REGISTER_NOT_LOCAL);
	} else if (!u->na || !(ng = u->na->ng())) {
		notice_lang(s_ChanServ, u, CHAN_MUST_REGISTER_NICK, s_NickServ);
	} else if (!nick_recognized(u)) {
		notice_lang(s_ChanServ, u, CHAN_MUST_IDENTIFY_NICK, s_NickServ, s_NickServ);
	} else if (epona->cd().nickserv.auth &&
			epona->cd().chanserv.restrict_auth.reg() &&
			!(ng->flags() & NickGroup::gfAuth)) {
		notice_lang(s_ChanServ, u, CHAN_REGISTER_NOT_AUTH);
		if (!ng->auth_code())
			notice_lang(s_ChanServ, u,
					CHAN_REGISTER_NOT_AUTH_HOWTO, s_NickServ);
	} else if ((ci = channels.find(chan)) != NULL) {
		notice_lang(s_ChanServ, u, CHAN_ALREADY_REGISTERED, chan);
	} else if (!(c = findchan(chan)) || !chan_has_user_status(c, u, CUS_OP)) {
		notice_lang(s_ChanServ, u, CHAN_MUST_BE_CHANOP);
	} else if (!is_servadmin && (epona->cd().chanserv.max_reg > 0) &&
			ng->channelcount() >= epona->cd().chanserv.max_reg) {
		notice_lang(s_ChanServ, u, ng->channelcount() > epona->cd().chanserv.max_reg
					? CHAN_EXCEEDED_CHANNEL_LIMIT
					: CHAN_REACHED_CHANNEL_LIMIT,
		epona->cd().chanserv.max_reg.value());
	} else if (!(ci = (ChannelInfo *)channels.create())) {
		epona->log(ErrorChanServ(),
				"Creation of channel %s failed", chan);
		notice_lang(s_ChanServ, u, CHAN_REGISTRATION_FAILED);
	} else {
		ci->c = c;
		
		ci->name(chan);
		ci->owner(ng);
		ci->last_used(time(0));
		
		if (strlen(pass) > ChannelInfo::pass_rlen)
		{
			pass[ChannelInfo::pass_rlen] = 0;
			notice_lang(s_ChanServ, u, PASSWORD_TRUNCATED,
					ChannelInfo::pass_rlen);
		}
		
		if (!ci->change_password(pass))
		{
			memset(pass, 0, strlen(pass));
			notice_lang(s_ChanServ, u, CHAN_REGISTRATION_FAILED);
			delete ci;
			return;
		}
		
		if (ci->enc())
			memset(pass, 0, strlen(pass));
		
		ci->desc(desc);
		
		ci->bantype(epona->cd().chanserv.default_bantype);
		ci->mlock_modes("+ntr", 0);
		
		/*ci->mlock_on_add(CMODE_n | CMODE_t | CMODE_r);*/
		
		/* Set those there so we don't have to immediately update them
		 * after the channel is added. */
		if (c->topic)
		{
			ci->topic(c->topic);
			ci->topic_setter(c->topic_setter);
			ci->topic_time(c->topic_time);
		}
		
		ci->botflags_add(epona->cd().botserv.default_options);
		
		if (!channels.add(ci, 0))
		{
			epona->log(ErrorChanServ(),
					"Channel %s could not be added to collection", ci->name());
			notice_lang(s_ChanServ, u, CHAN_REGISTRATION_FAILED);
			delete ci;
		}
		
		epona->log(InfoChanServList(), "Channel %s registered by %s!%s@%s",
				ci->name(), u->nick, u->username, GetHost(u));
		notice_lang(s_ChanServ, u, CHAN_REGISTERED, ci->name(), u->nick);
		
		if (!ci->enc())
			notice_lang(s_ChanServ, u, CHAN_PASSWORD_IS, ci->pass());
	}
}

/*************************************************************************/

static void do_identify(User *u)
{
	char *chan = strtok(NULL, " ");
	char *pass = strtok(NULL, " ");
	ChannelInfo *ci;
	User::u_chaninfolist *uc;
	
	if (!pass) {
		syntax_error(s_ChanServ, u, "IDENTIFY", CHAN_IDENTIFY_SYNTAX);
	} else if (!(ci = channels.find(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
	} else if (epona->cd().nickserv.auth &&
			epona->cd().chanserv.restrict_auth.identify() &&
			(!u->na || !(u->na->ng()->flags() & NickGroup::gfAuth))) {
		notice_lang(s_ChanServ, u, CHAN_IDENTIFY_NOT_AUTH);
		if (!u->na || !u->na->ng()->auth_code())
			notice_lang(s_ChanServ, u,
					CHAN_IDENTIFY_NOT_AUTH_HOWTO, s_NickServ);
	} else {
		if (ci->check_password(pass)) 
		{
	    	if (!is_identified(u, ci)) {
				uc = static_cast<User::u_chaninfolist *>(scalloc(sizeof(*uc), 
						1));
				uc->next = u->founder_chans;
				if (u->founder_chans)
		    		u->founder_chans->prev = uc;
				u->founder_chans = uc;
				uc->chan = ci;
				epona->log(InfoChanServRights(), "%s!%s@%s identified for %s",
						u->nick, u->username, GetHost(u), ci->name());
	    	}
	
	    	notice_lang(s_ChanServ, u, CHAN_IDENTIFY_SUCCEEDED, chan);
		} else {
		    epona->log(InfoChanServRights(), "Failed IDENTIFY for %s by %s!%s@%s",
				ci->name(), u->nick, u->username, GetHost(u));
		    notice_lang(s_ChanServ, u, PASSWORD_INCORRECT);
		    bad_password(u);
		}
	
	}
}

/*************************************************************************/

static void do_logout(User *u)
{
	char *chan = strtok(NULL, " ");
	char *nick = strtok(NULL, " ");
	ChannelInfo *ci;
	User *u2 = NULL;
	int is_servadmin = is_services_admin(u);
	
	if (!chan || (!nick && !is_servadmin)) {
	syntax_error(s_ChanServ, u, "LOGOUT", (!is_servadmin ? CHAN_LOGOUT_SYNTAX : CHAN_LOGOUT_SERVADMIN_SYNTAX));
	} else if (!(ci = channels.find(chan))) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
	} else if (nick && !(u2 = finduser(nick))) {
	notice_lang(s_ChanServ, u, NICK_X_NOT_IN_USE, nick);
	} else if (!is_servadmin && u2 != u && !is_real_owner(u, ci)) {
	notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
		if (u2) {
			make_unidentified(u2, ci);
			notice_lang(s_ChanServ, u, CHAN_LOGOUT_SUCCEEDED, nick, chan);
		} else {
			int i;
			for (i = 0; i < 1024; i++)
				for (u2 = userlist[i]; u2; u2 = u2->next)
					make_unidentified(u2, ci);			
			notice_lang(s_ChanServ, u, CHAN_LOGOUT_ALL_SUCCEEDED, chan);
		}
	}
}

/*************************************************************************/

static void do_drop(User *u)
{
    char *chan = strtok(NULL, " ");
    ChannelInfo *ci;
    int is_servadmin = is_services_admin(u);

    if (!channels.writable()) 
    {
		notice_lang(s_ChanServ, u, CHAN_DROP_DISABLED);
		return;
    }

    if (!chan) {
		syntax_error(s_ChanServ, u, "DROP", CHAN_DROP_SYNTAX);
    } else if (!(ci = channels.find(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (!is_servadmin &&
    		(ci->flags() & ChannelInfo::cfSecureFounder ? !is_real_owner(u, ci) :
    		 !is_owner(u, ci))) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
		int level = get_access(u, ci);
		
		epona->log(InfoChanServList(),
				"Channel %s being dropped by %s!%s@%s (founder: %s)", 
				ci->name(), u->nick, u->username, GetHost(u),
				(ci->owner() ? (ci->owner())->name() : "(none)"));
		
		if (!channels.remove(ci))
		{
			epona->log(InfoChanServList(), "Removal of channel %s failed",
					ci->name());
			notice_lang(s_ChanServ, u, CHAN_DROP_FAILED);
			return;
		}
		
		/* We must make sure that the Services admin has not normally the right to
		 * drop the channel before issuing the wallops.
		 */
		if ((epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waDrop) &&
				is_servadmin && level < ACCESS_FOUNDER)
			wallops(s_ChanServ, "\2%s\2 used DROP on channel \2%s\2", u->nick, chan);
		
		notice_lang(s_ChanServ, u, CHAN_DROPPED, chan);
    }
}

/*************************************************************************/

/* Main SET routine.  Calls other routines as follows:
 *	do_set_command(User *command_sender, ChannelInfo *ci, char *param);
 * The parameter passed is the first space-delimited parameter after the
 * option name, except in the case of DESC, TOPIC, and ENTRYMSG, in which
 * it is the entire remainder of the line.  Additional parameters beyond
 * the first passed in the function call can be retrieved using
 * strtok(NULL, toks).
 */
static void do_set(User *u)
{
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *param;
    ChannelInfo *ci;
    int is_servadmin = is_services_admin(u);

    if (!channels.writable()) 
	{
		notice_lang(s_ChanServ, u, CHAN_SET_DISABLED);
		return;
    }

    if (cmd) {
		if (stricmp(cmd, "DESC") == 0 || stricmp(cmd, "ENTRYMSG") == 0)
	    	param = strtok(NULL, "");
		else
	    	param = strtok(NULL, " ");
    } else {
		param = NULL;
    }

    if (!param && (!cmd || (stricmp(cmd, "SUCCESSOR") != 0 &&
                            stricmp(cmd, "URL") != 0 &&
                            stricmp(cmd, "EMAIL") != 0 &&
                            stricmp(cmd, "ENTRYMSG") != 0))) {
	syntax_error(s_ChanServ, u, "SET", CHAN_SET_SYNTAX);
    } else if (!(ci = channels.find(chan))) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (!is_servadmin && !check_access(u, ci, CA_SET)) {
	notice_lang(s_ChanServ, u, ACCESS_DENIED);
    } else if (stricmp(cmd, "FOUNDER") == 0) {
	if (!is_servadmin && (ci->flags() & ChannelInfo::cfSecureFounder ? !is_real_owner(u, ci) : !is_owner(u, ci))) {
	    notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
	    do_set_owner(u, ci, param);
	}
    } else if (stricmp(cmd, "SUCCESSOR") == 0) {
	if (!is_servadmin && (ci->flags() & ChannelInfo::cfSecureFounder ? !is_real_owner(u, ci) : !is_owner(u, ci))) {
	    notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
	    do_set_successor(u, ci, param);
	}
    } else if (stricmp(cmd, "PASSWORD") == 0) {
	if (!is_servadmin && (ci->flags() & ChannelInfo::cfSecureFounder ? !is_real_owner(u, ci) : !is_owner(u, ci))) {
	    notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
	    do_set_password(u, ci, param);
	}
    } else if (stricmp(cmd, "DESC") == 0) {
	do_set_desc(u, ci, param);
    } else if (stricmp(cmd, "URL") == 0) {
	do_set_url(u, ci, param);
    } else if (stricmp(cmd, "EMAIL") == 0) {
	do_set_email(u, ci, param);
    } else if (stricmp(cmd, "ENTRYMSG") == 0) {
	do_set_entrymsg(u, ci, param);
    } else if (stricmp(cmd, "TOPIC") == 0) {
	notice_lang(s_ChanServ, u, OBSOLETE_COMMAND, "TOPIC");
	} else if (stricmp(cmd, "BANTYPE") == 0) {
	do_set_bantype(u, ci, param);
    } else if (stricmp(cmd, "MLOCK") == 0) {
	do_set_mlock(u, ci, param);
    } else if (stricmp(cmd, "KEEPTOPIC") == 0) {
	do_set_keeptopic(u, ci, param);
    } else if (stricmp(cmd, "TOPICLOCK") == 0) {
	do_set_topiclock(u, ci, param);
    } else if (stricmp(cmd, "PRIVATE") == 0) {
	do_set_private(u, ci, param);
    } else if (stricmp(cmd, "SECUREOPS") == 0) {
	do_set_secureops(u, ci, param);
	} else if (stricmp(cmd, "SECUREFOUNDER") == 0) {
	if (!is_servadmin && (ci->flags() & ChannelInfo::cfSecureFounder ? !is_real_owner(u, ci) : !is_owner(u, ci))) {
	    notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
	    do_set_securefounder(u, ci, param);
	}
    } else if (stricmp(cmd, "RESTRICTED") == 0) {
	do_set_restricted(u, ci, param);
    } else if (stricmp(cmd, "SECURE") == 0) {
	do_set_secure(u, ci, param);
	} else if (stricmp(cmd, "SIGNKICK") == 0) {
	do_set_signkick(u, ci, param);
    } else if (stricmp(cmd, "OPNOTICE") == 0) {
	do_set_opnotice(u, ci, param);
	} else if (stricmp(cmd, "PEACE") == 0) {
	do_set_peace(u, ci, param);
    } else if (stricmp(cmd, "NOEXPIRE") == 0) {
	do_set_noexpire(u, ci, param);
    } else {
	notice_lang(s_ChanServ, u, CHAN_SET_UNKNOWN_OPTION, cmd);
	notice_lang(s_ChanServ, u, MORE_INFO, s_ChanServ, "SET");
    }
}

/*************************************************************************/

static void do_set_owner(User *u, ChannelInfo *ci, char *param)
{
	NickAlias *na = aliases.find(param);
	NickGroup *ng;
	
	if (!na) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_REGISTERED, param);
		return;
	}
	
	ng = na->ng();
    
    if (epona->cd().nickserv.auth &&
			epona->cd().chanserv.restrict_auth.founder() &&
			!(ng->flags() & NickGroup::gfAuth)) 
	{
		notice_lang(s_ChanServ, u, CHAN_SET_FOUNDER_NOT_AUTH, na->nick());
		return;
	}
    
    if ((epona->cd().chanserv.max_reg > 0) &&
    		(ng->channelcount() >= epona->cd().chanserv.max_reg) &&
    		!is_services_admin(u)) 
    {
		notice_lang(s_ChanServ, u, CHAN_SET_FOUNDER_TOO_MANY_CHANS, param);
		return;
	}
	
	epona->log(InfoChanServRights(),
			"Changing founder of %s from %s to %s by %s!%s@%s",
			ci->name(), (ci->owner())->name(),
			ng->name(), u->nick, u->username, GetHost(u));
	
	/* Founder and successor must not be the same group */
	if (ng == ci->successor())
	ci->successor(NULL);
    
    ci->owner(ng);
    
    notice_lang(s_ChanServ, u, CHAN_FOUNDER_CHANGED, ci->name(), param);
}

/*************************************************************************/

static void do_set_successor(User *u, ChannelInfo *ci, char *param)
{
    NickAlias *na;
    NickGroup *ng;

    if (param) {
		na = aliases.find(param);
	
		if (!na) {
	    	notice_lang(s_ChanServ, u, NICK_X_NOT_REGISTERED, param);
	    	return;
		}
		
		ng = na->ng();
		
		if (ng == ci->owner()) 
		{
			notice_lang(s_ChanServ, u, CHAN_SUCCESSOR_IS_FOUNDER,
					na->nick(), ci->name());
	    	return;
		}
		
		if (epona->cd().nickserv.auth &&
				epona->cd().chanserv.restrict_auth.founder() &&
				!(ng->flags() & NickGroup::gfAuth)) 
		{
			notice_lang(s_ChanServ, u, CHAN_SET_SUCCESSOR_NOT_AUTH, na->nick());
			return;
		}
    } else {
		ng = NULL;
    }
    
    epona->log(InfoChanServRights(),
    		"Changing successor of %s from %s to %s by %s!%s@%s", ci->name(),
    		(ci->successor() ? (ci->successor())->name() : "none"), 
			(ng ? ng->name() : "none"), u->nick, u->username, 
			GetHost(u));
    
    ci->successor(ng);
    
    if (ng)
	notice_lang(s_ChanServ, u, CHAN_SUCCESSOR_CHANGED, ci->name(), param);
    else
	notice_lang(s_ChanServ, u, CHAN_SUCCESSOR_UNSET, ci->name());
}

/*************************************************************************/

static void do_set_password(User *u, ChannelInfo *ci, char *param)
{
	size_t len = strlen(param);
	
	if (len > ChannelInfo::pass_rlen)
	{
		param[ChannelInfo::pass_rlen] = 0;
		notice_lang(s_ChanServ, u, PASSWORD_TRUNCATED, ChannelInfo::pass_rlen);
	}
	
	if (!ci->change_password(param))
	{
		memset(param, 0, len);
		notice_lang(s_ChanServ, u, CHAN_SET_PASSWORD_FAILED);
		return;
	}
	
	if (ci->enc())
	{
		memset(param, 0, len);
		notice_lang(s_ChanServ, u, CHAN_PASSWORD_CHANGED, ci->name());
	} else {
		notice_lang(s_ChanServ, u, CHAN_PASSWORD_CHANGED_TO, ci->name(),
				ci->pass());
	}

    if (get_access(u, ci) < ACCESS_FOUNDER) 
    {
		epona->log(NoticeChanServPrivs(),
				"%s!%s@%s set password as Services admin for %s",
				u->nick, u->username, GetHost(u), ci->name());
		if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waSetpass)
	    	wallops(s_ChanServ, "\2%s\2 set password as Services admin for channel \2%s\2", u->nick, ci->name());
    } else {
    	epona->log(InfoChanServSettings(),
    			"%s!%s@%s changed password of %s (founder: %s)", 
    			u->nick, u->username, GetHost(u), 
    			ci->name(), (ci->owner())->name());
    }
}

/*************************************************************************/

static void do_set_desc(User *u, ChannelInfo *ci, char *param)
{
    ci->desc(param);
    notice_lang(s_ChanServ, u, CHAN_DESC_CHANGED, ci->name(), param);
}

/*************************************************************************/

static void do_set_url(User *u, ChannelInfo *ci, char *param)
{
    ci->url(param);
    if (param)
		notice_lang(s_ChanServ, u, CHAN_URL_CHANGED, ci->name(), param);
    else
		notice_lang(s_ChanServ, u, CHAN_URL_UNSET, ci->name());
}

/*************************************************************************/

static void do_set_email(User *u, ChannelInfo *ci, char *param)
{
	ci->email(param);
	if (param)
		notice_lang(s_ChanServ, u, CHAN_EMAIL_CHANGED, ci->name(), param);
	else
		notice_lang(s_ChanServ, u, CHAN_EMAIL_UNSET, ci->name());
}

/*************************************************************************/

static void do_set_entrymsg(User *u, ChannelInfo *ci, char *param)
{
	ci->entry_message(param);
	if (param)
		notice_lang(s_ChanServ, u, CHAN_ENTRY_MSG_CHANGED, ci->name(), param);
	else
		notice_lang(s_ChanServ, u, CHAN_ENTRY_MSG_UNSET, ci->name());
}

/*************************************************************************/

static void do_set_mlock(User *u, ChannelInfo *ci, char *param)
{
	char buf[BUFSIZE], *rem = strtok(NULL, "");
	
	snprintf(buf, sizeof(buf), "%s %s", param, rem);
	ci->mlock_modes(buf, u);
	
#if 0
	int add = -1; /* 1 if adding, 0 if deleting, -1 if neither */
    unsigned char mode;
    CBMode *cbm;

	/* Reinitialize everything */
	
	ci->mlock_on_clear();
	ci->mlock_on_add(CMODE_r);
	ci->mlock_off_clear(); 
	ci->mlock_limit(0);
	ci->mlock_key(0);
#ifdef HAS_FMODE
	ci->mlock_flood(0);
#endif
#ifdef HAS_JMODE
	ci->mlock_joinrate(0);
#endif
#ifdef HAS_LMODE
	ci->mlock_redirect(0);
#endif

    while ((mode = *param++)) 
    {
		switch (mode) 
		{
	  		case '+':
	    		add = 1;
	    		continue;
	  		case '-':
	    		add = 0;
	    		continue;
	    	default:
	    		if (add < 0)
	    			continue;
	    }
	    		
	  	if ((int)mode < 128 && (cbm = &cbmodes[(int)mode])->flag != 0) 
	  	{
	  		if ((cbm->flags & CBM_NO_MLOCK) ||
	  				((cbm->flags & CBM_NO_USER_MLOCK) && !is_oper(u))) 
	  		{
	  			notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_IMPOSSIBLE_CHAR, mode);
	  		} else if (add) 
	  		{
	  			ci->mlock_on_add(cbm->flag);
	  			ci->mlock_off_remove(cbm->flag);
	  			if (cbm->cssetvalue)
	  				cbm->cssetvalue(ci, strtok(NULL, " "));
	  		} else 
	  		{
	  			ci->mlock_off_add(cbm->flag);
	  			if (ci->mlock_on() & cbm->flag) 
	  			{
	  				ci->mlock_on_remove(cbm->flag);
	  				if (cbm->cssetvalue)
	  					cbm->cssetvalue(ci, NULL);
	  			}
	  		}
	  	} else {
	  		notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_UNKNOWN_CHAR, mode);
	  	}
	} /* while (*param) */

#ifdef HAS_LMODE
	/* We can't mlock +L if +l is not mlocked as well. */
	if ((ci->mlock_on() & CMODE_L) && !(ci->mlock_on() & CMODE_l)) 
	{
		ci->mlock_on_remove(CMODE_L);
		ci->mlock_redirect(NULL);
		notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_L_REQUIRED);
	}
#endif

#if defined(IRC_ULTIMATE) || defined(IRC_UNREAL)
	/* We can't mlock +K if +i is not mlocked as well. */
	if ((ci->mlock_on() & CMODE_K) && !(ci->mlock_on() & CMODE_i)) 
	{
		ci->mlock_on_remove(CMODE_K);
		notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_K_REQUIRED);
	}
#endif

	/* Since we always enforce mode r there is no way to have no
	 * mode lock at all.
     */
    notice_lang(s_ChanServ, u, CHAN_MLOCK_CHANGED, ci->name(),
    		ci->mlock_modes());

    /* Implement the new lock. */
    if (ci->c)
    	check_modes(ci->c);
#endif 
}

/*************************************************************************/

static void do_set_bantype(User *u, ChannelInfo *ci, char *param)
{
	char *endptr;
	
	int16_t bantype = strtol(param, &endptr, 10);
	
	if (*endptr != 0 || bantype < 0 || bantype > 3) {	
		notice_lang(s_ChanServ, u, CHAN_SET_BANTYPE_INVALID, param);
	} else {
		ci->bantype(bantype);
		notice_lang(s_ChanServ, u, CHAN_SET_BANTYPE_CHANGED, ci->name(), ci->bantype());
	}
}

/*************************************************************************/

static void do_set_keeptopic(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfKeepTopic);
	notice_lang(s_ChanServ, u, CHAN_SET_KEEPTOPIC_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfKeepTopic);
	notice_lang(s_ChanServ, u, CHAN_SET_KEEPTOPIC_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET KEEPTOPIC", CHAN_SET_KEEPTOPIC_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_topiclock(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfTopicLock);
	notice_lang(s_ChanServ, u, CHAN_SET_TOPICLOCK_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfTopicLock);
	notice_lang(s_ChanServ, u, CHAN_SET_TOPICLOCK_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET TOPICLOCK", CHAN_SET_TOPICLOCK_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_private(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfPrivate);
	notice_lang(s_ChanServ, u, CHAN_SET_PRIVATE_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfPrivate);
	notice_lang(s_ChanServ, u, CHAN_SET_PRIVATE_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET PRIVATE", CHAN_SET_PRIVATE_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_secureops(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfSecureOps);
	notice_lang(s_ChanServ, u, CHAN_SET_SECUREOPS_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfSecureOps);
	notice_lang(s_ChanServ, u, CHAN_SET_SECUREOPS_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET SECUREOPS", CHAN_SET_SECUREOPS_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_securefounder(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfSecureFounder);
	notice_lang(s_ChanServ, u, CHAN_SET_SECUREFOUNDER_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfSecureFounder);
	notice_lang(s_ChanServ, u, CHAN_SET_SECUREFOUNDER_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET SECUREFOUNDER", CHAN_SET_SECUREFOUNDER_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_restricted(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfRestricted);
	notice_lang(s_ChanServ, u, CHAN_SET_RESTRICTED_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfRestricted);
	notice_lang(s_ChanServ, u, CHAN_SET_RESTRICTED_OFF);
    } else {
	syntax_error(s_ChanServ,u,"SET RESTRICTED",CHAN_SET_RESTRICTED_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_secure(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfSecure);
	notice_lang(s_ChanServ, u, CHAN_SET_SECURE_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfSecure);
	notice_lang(s_ChanServ, u, CHAN_SET_SECURE_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET SECURE", CHAN_SET_SECURE_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_signkick(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfSignKick);
	ci->flags_remove(ChannelInfo::cfSignKickLevel);
	notice_lang(s_ChanServ, u, CHAN_SET_SIGNKICK_ON);
	} else if (stricmp(param, "LEVEL") == 0) {
	ci->flags_add(ChannelInfo::cfSignKickLevel);
	ci->flags_remove(ChannelInfo::cfSignKick);
	notice_lang(s_ChanServ, u, CHAN_SET_SIGNKICK_LEVEL);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfSignKick | ChannelInfo::cfSignKickLevel);
	notice_lang(s_ChanServ, u, CHAN_SET_SIGNKICK_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET SIGNKICK", CHAN_SET_SIGNKICK_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_opnotice(User *u, ChannelInfo *ci, char *param)
{
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfOpNotice);
	notice_lang(s_ChanServ, u, CHAN_SET_OPNOTICE_ON);
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfOpNotice);
	notice_lang(s_ChanServ, u, CHAN_SET_OPNOTICE_OFF);
    } else {
	syntax_error(s_ChanServ, u, "SET OPNOTICE", CHAN_SET_OPNOTICE_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_peace(User *u, ChannelInfo *ci, char *param)
{
	if (stricmp(param, "ON") == 0) {
		ci->flags_add(ChannelInfo::cfPeace);
		notice_lang(s_ChanServ, u, CHAN_SET_PEACE_ON);
    } else if (stricmp(param, "OFF") == 0) {
		ci->flags_remove(ChannelInfo::cfPeace);
		notice_lang(s_ChanServ, u, CHAN_SET_PEACE_OFF);
    } else {
		syntax_error(s_ChanServ, u, "SET PEACE", CHAN_SET_PEACE_SYNTAX);
    }
}

/*************************************************************************/

static void do_set_noexpire(User *u, ChannelInfo *ci, char *param)
{
    if (!is_services_admin(u)) {
	notice_lang(s_ChanServ, u, PERMISSION_DENIED);
	return;
    }
    if (stricmp(param, "ON") == 0) {
	ci->flags_add(ChannelInfo::cfNoExpire);
	notice_lang(s_ChanServ, u, CHAN_SET_NOEXPIRE_ON, ci->name());
    } else if (stricmp(param, "OFF") == 0) {
	ci->flags_remove(ChannelInfo::cfNoExpire);
	notice_lang(s_ChanServ, u, CHAN_SET_NOEXPIRE_OFF, ci->name());
    } else {
	syntax_error(s_ChanServ, u, "SET NOEXPIRE", CHAN_SET_NOEXPIRE_SYNTAX);
    }
}

/*************************************************************************/

AccessCmd::AccessCmd() : ListCmd("ACCESS", s_ChanServ, access_msgs, true),
		m_context(0), ci(0)
{ 

}

/*************************************************************************/

int AccessCmd::add(User *u, container_type &ctn, char const *sub)
{
	/* Get the nick and level */
	char *nick = strtok(NULL, " ");
	char *slevel = strtok(NULL, " ");
	
	if (!nick || !slevel)
	{
		syntax_error(m_service, u, m_name, CHAN_ACCESS_ADD_SYNTAX);
		return crStop;
	}

	/* Retrieve the nickname record */
	NickAlias *na = aliases.find(nick);
	
	if (!na) 
	{
		notice_lang(m_service, u, CHAN_ACCESS_NICKS_ONLY);
		return crStop;
	}
	
	if (epona->cd().nickserv.auth &&
			epona->cd().chanserv.restrict_auth.access() &&
			!(na->ng()->flags() & NickGroup::gfAuth)) 
	{
		notice_lang(s_ChanServ, u, CHAN_ACCESS_NOT_AUTH, na->nick());
		return crStop;
	}

	/* Compute a valid level */
	long level;
	
	if (!misc::strtol_range(&level, slevel, ACCESS_INVALID + 1,
				ACCESS_FOUNDER - 1))
	{
		if (errno != ERANGE)
		{
			notice_lang(m_service, u, CHAN_ACCESS_LEVEL_INVALID, slevel);
			return crStop;
		}
		
		notice_lang(m_service, u, CHAN_ACCESS_LEVEL_RANGE,
				ACCESS_INVALID + 1, ACCESS_FOUNDER - 1);
		return crStop;
	}
	
	if (level == 0) 
	{
		notice_lang(m_service, u, CHAN_ACCESS_LEVEL_NONZERO);
		return crStop;
	}
	
	/* Ensure the user is allowed to set that level. */

	int16_t ulevel = get_access(u, ci);
		
	if (ulevel <= level) 
	{
		notice_lang(m_service, u, PERMISSION_DENIED);
		return crStop;
	}
	
	/* Add the record */
	
	if (add_record(u, ulevel, ci, na->ng(), level) < 0)
		return crStop;

	return crStop;
}

/*************************************************************************/

int AccessCmd::add_record(User *u, int16_t ulevel, ChannelInfo *owner,
		NickGroup *ng, int16_t level)
{
	Access *acc;
	ChannelInfo::accesses_type::const_iterator it, end;
	
	/* First see if we need to exist an existing access entry */
	for (it = owner->accesses().begin(), end = owner->accesses().end();
			it != end; ++it)
	{
		acc = (Access *)*it;
		
		if (!acc)
			continue;
		
		if (acc->ng() == ng)
		{
			if (acc->level() >= ulevel) 
			{
				if (u)
					notice_lang(m_service, u, PERMISSION_DENIED);
				return -1;
			}
			
			if (acc->level() == level) 
			{
				if (u)
					notice_lang(m_service, u, CHAN_ACCESS_LEVEL_UNCHANGED,
							acc->ng()->name(), owner->name(), acc->level());
				return 0;
			}
			
			acc->level(level);
			
			/* Make sure we don't log if it wasn't updated */
			if (u && acc->level() == level)
				epona->log(InfoChanServRights(),
						"%s!%s@%s (level %d) set access level %d to %s "
						"on channel %s", u->nick, u->username,
						GetHost(u), ulevel, acc->level(), ng->name(),
						owner->name());
			
			if (u)
				notice_lang(m_service, u, CHAN_ACCESS_LEVEL_CHANGED,
						acc->ng()->name(), owner->name(), acc->level());
			return 0;
		}
	}
	
	/* Check whether we can add more entries. */
	if (owner->accesses().size() >= epona->cd().chanserv.max_access)
	{
		for (it = owner->accesses().begin(), end = owner->accesses().end();
				it != end; ++it)
		{
			if ((*it) == 0)
				break;
		}
		
		if (it == end)
		{
			if (u)
				notice_lang(m_service, u, CHAN_ACCESS_REACHED_LIMIT,
						epona->cd().chanserv.max_access.value());
			return -1;
		}
	}
	
	acc = (Access *)accesses.create();
	
	if (!acc)
	{
		if (u)
			notice_lang(m_service, u, CHAN_ACCESS_CREATION_FAILED);
		return -1;
	}
	
	acc->owner(owner);
	acc->ng(ng);
	acc->level(level);
	
	/* Try to add the access entry to the list */
	
	int error;
	
	if (!accesses.add(acc, &error))
	{
		if (u)
			notice_lang(m_service, u, m_msgs[cmDisabled]);
				
		delete acc;
		return -1;
	} else {
		if (u)
		{
			epona->log(InfoChanServRights(),
					"%s!%s@%s (level %d) set access level %d to %s "
					"on channel %s", u->nick, u->username, GetHost(u), ulevel,
					acc->level(), ng->name(), owner->name());

			notice_lang(m_service, u, CHAN_ACCESS_ADDED,
					acc->ng()->name(), owner->name(), acc->level());
		}
	}
	
	return 0;
}

/*************************************************************************/

bool AccessCmd::allow_del(User *u, container_type &ctn,
		char const *sub, databases::Record const *rec)
{
	if (is_services_admin(u))
		return true;
	
	return (get_access(u, ci) > ((Access *) rec)->level());
}

/*************************************************************************/

bool AccessCmd::allow_dispatch(User *u, container_type &ctn, char const *sub)
{
	/* Allow services admin access to DEL/LIST/CLEAR but not ADD. */
	if (stricmp(sub, "ADD") && is_services_admin(u))
		return true;
	
	if (!stricmp(sub, "ADD") || !stricmp(sub, "DEL"))
		return check_access(u, ci, CA_ACCESS_CHANGE);
	else if (!stricmp(sub, "LIST"))
		return check_access(u, ci, CA_ACCESS_LIST);
	else if (!stricmp(sub, "CLEAR"))
		return is_owner(u, ci);
	
	return false;
}

/*************************************************************************/

bool AccessCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return accesses.writable();
}

/*************************************************************************/

AccessCmd::container_type *AccessCmd::container(User *u)
{
	m_context = strtok(NULL, " ");
	
	if (!m_context)
	{
		syntax_error(m_service, u, m_name, m_msgs[cmSyntax]);
		return 0;
	}
	
	if (!(ci = channels.find(m_context)))
	{
		notice_lang(m_service, u, CHAN_X_NOT_REGISTERED, m_context);
		return 0;
	}
	
	return &ci->accesses();
}

/*************************************************************************/

bool AccessCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	return accesses.remove(rec);
}

/*************************************************************************/

bool AccessCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	Access *acc = (Access *)rec;
	
	if (count == 0)
		notice_lang(m_service, u, CHAN_ACCESS_LIST_COLHEAD);
	
	notice_lang(m_service, u, CHAN_ACCESS_LIST_FORMAT, num, acc->level(),
			acc->ng()->name());
	return true;
}

/*************************************************************************/

static void do_access(User *u)
{
	static AccessCmd access_cmd;
	access_cmd.execute(u);
}

/*************************************************************************/

/* Is the mask stuck? */

Akick *is_stuck(ChannelInfo *ci, char const *mask)
{
	Akick *akick;
	
	ChannelInfo::akicks_type::const_iterator it, end;
				
	for (it = ci->akicks().begin(), end = ci->akicks().end(); it != end; ++it)
	{
		akick = (Akick *) (*it);
	
		if (!akick || (akick->flags() & Akick::afIsGroup) ||
				!(akick->flags() & Akick::afStuck))
		    continue;
		
		/* Example: mask = *!*@*.org and akick->mask = *!*@*.epona.org */
		if (match_wild_nocase(mask, akick->mask()))
		    return akick;
#ifdef IRC_DREAMFORGE
        /* Example: mask = *!*@irc.epona.org and akick->mask = *!*@*.epona.org */
		if (match_wild_nocase(akick->mask(), mask))
			return akick;
#endif
	}
	
	return NULL;
}

/* Ban the stuck mask in a safe manner. */

void stick_all(ChannelInfo *ci) 
{
	char const *av[2];
	Akick *akick;

	ChannelInfo::akicks_type::const_iterator it, end;
				
	for (it = ci->akicks().begin(), end = ci->akicks().end(); it != end; ++it)
	{
		akick = (Akick *) (*it);
		
		if (!akick || (akick->flags() & Akick::afIsGroup) ||
				!(akick->flags() & Akick::afStuck))
		    continue;
		
		av[0] = "+b";
		av[1] = akick->mask();
		s_mode(whosends(ci), ci->c->name, "+b", akick->mask());
		chan_set_modes(s_ChanServ, ci->c, 2, av, 1);
	}
}

/*************************************************************************/

AkickCmd::AkickCmd() : ListCmd("AKICK", s_ChanServ, akick_msgs, true),
		m_context(0), ci(0)
{ 

}

/*************************************************************************/

int AkickCmd::add(User *u, container_type &ctn, char const *sub)
{
	/* Get the parameters */
	char *expiry = 0, *what = strtok(NULL, " ");
	
	if (what && *what == '+')
	{
		expiry = what;
		what = strtok(NULL, " ");
	}
	
	char *reason = strtok(NULL, "");
	
	if (!what)
	{
		syntax_error(m_service, u, m_name, CHAN_AKICK_ADD_SYNTAX);
		return crStop;
	}
	
	/* Computes the expiration time */
	time_t expires = 0;
	
	if (expiry && !misc::duration_scan(expiry, &expires))
	{
		notice_lang(m_service, u, BAD_EXPIRY_TIME);
		return crStop;
	}
	
	if (expires)
		expires += time(0);
	
	/* Determine whether we have a group or a mask */
	
	char *mask = 0, maskbuf[BUFSIZE];
	NickAlias *na = aliases.find(what);
	NickGroup *ng = (na ? na->ng() : 0);
	
	if (!na)
	{
		char *nick, *user, *host;
		
		split_usermask(what, &nick, &user, &host);
		snprintf(maskbuf, sizeof(maskbuf), "%s!%s@%s", nick, user, host);
		
		delete [] nick;
		delete [] user;
		delete [] host;
		
		mask = maskbuf;
	}
	
	/* Validates the mask, if any */
	
	if (mask && strlen(mask) > Akick::mask_len)
	{
		notice_lang(m_service, u, CHAN_AKICK_MASK_TOO_LONG,
				Akick::mask_len);
		return crStop;
	}
	
	/* Validates the reason */
	
	if (reason && strlen(reason) > Akick::reason_len)
	{
		notice_lang(m_service, u, CHAN_AKICK_REASON_TOO_LONG,
				Akick::reason_len);
		return crStop;
	}
	
	/* Add the record */
	
	if (add_record(u, ci, ng, mask, u->nick, time(0), expires, reason) < 0)
		return crStop;

	return crStop;
}

/*************************************************************************/

int AkickCmd::add_record(User *u, ChannelInfo *owner, NickGroup *ng,
		char const *mask, char const *creator, time_t since, time_t expires,
		char const *reason)
{
	/* Check whether we can add more entries. */
	if (owner->akicks().size() >= epona->cd().chanserv.max_akicks)
	{
		ChannelInfo::akicks_type::const_iterator it, end;
		
		for (it = owner->akicks().begin(), end = owner->akicks().end();
				it != end; ++it)
		{
			if ((*it) == 0)
				break;
		}
		
		if (it == end)
		{
			if (u)
				notice_lang(m_service, u, CHAN_AKICK_REACHED_LIMIT,
						epona->cd().chanserv.max_akicks.value());
			return -1;
		}
	}
	
	Akick *ak = (Akick *)akicks.create();
	
	if (!ak)
	{
		if (u)
			notice_lang(m_service, u, CHAN_AKICK_CREATION_FAILED);
		return -1;
	}
	
	ak->owner(owner);
	if (ng)
	{
		ak->flags_add(Akick::afIsGroup);
		ak->ng(ng);
	}
	if (mask)
		ak->mask(mask);
	ak->creator(creator);
	ak->reason(reason);
	ak->since(since);
	ak->expires(expires);
	
	/* Try to add the access entry to the list */
	
	int error;
	
	if (!akicks.add(ak, &error))
	{
		if (u)
		{
			if (error == Akicks::ceName ||
					error == databases::Collection::ceList)
				notice_lang(m_service, u, CHAN_AKICK_ALREADY_PRESENT,
				 		ak->name(), owner->name());
			else
				notice_lang(m_service, u, m_msgs[cmDisabled]);
		}
				
		delete ak;
		return -1;
	} else {
		if (u)
			notice_lang(m_service, u, CHAN_AKICK_ADDED,
					ak->name(), owner->name());
	}

	return 0;
}

/*************************************************************************/

bool AkickCmd::allow_dispatch(User *u, container_type &ctn, char const *sub)
{
	/* Allow services admin access to DEL/LIST/CLEAR but not ADD. */
	if (stricmp(sub, "ADD") && stricmp(sub, "STICK") &&
			stricmp(sub, "ENFORCE") && is_services_admin(u))
		return true;
	
	return check_access(u, ci, CA_AKICK);
}

/*************************************************************************/

bool AkickCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return akicks.writable();
}

/*************************************************************************/

AkickCmd::container_type *AkickCmd::container(User *u)
{
	m_context = strtok(NULL, " ");
	
	if (!m_context)
	{
		syntax_error(m_service, u, m_name, m_msgs[cmSyntax]);
		return 0;
	}
	
	if (!(ci = channels.find(m_context)))
	{
		notice_lang(m_service, u, CHAN_X_NOT_REGISTERED, m_context);
		return 0;
	}
	
	return &ci->akicks();
}

/*************************************************************************/

int AkickCmd::dispatch(User *u, container_type &ctn, char const *sub)
{
	if (!stricmp(sub, "STICK"))
		return stick(u, ctn, sub);
	else if (!stricmp(sub, "UNSTICK"))
		return unstick(u, ctn, sub);
	else if (!stricmp(sub, "VIEW"))
		return list(u, ctn, sub);
	else if (!stricmp(sub, "ENFORCE"))
		return enforce(u, ctn, sub);
	else
		return ListCmd::dispatch(u, ctn, sub);
}

/*************************************************************************/

bool AkickCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	return akicks.remove(rec);
}

/*************************************************************************/

int AkickCmd::enforce(User *u, container_type &ctn, char const *sub)
{
	if (!ci->c)
	{
		notice_lang(m_service, u, CHAN_X_NOT_IN_USE, ci->name());
		return crStop;
	}
	
	size_t count = 0;
	Channel::c_userlist *cu, *next;
		
	for (cu = ci->c->users; cu; cu = next)
	{
		next = cu->next;
		
		if (check_kick(cu->user, ci->c->name)) 
		{
			char const *av[3];
			
			av[0] = ci->c->name;
			av[1] = cu->user->nick;
			av[2] = epona->cd().chanserv.akick_reason;
		
		    do_kick(whosends(ci), 3, av);
			
		    count++;
		}
	}
	
	notice_lang(m_service, u, CHAN_AKICK_ENFORCE_DONE, ci->name(), count);
	
	return crStop;
}

/*************************************************************************/

bool AkickCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	Akick *ak = (Akick *)rec;
	
	if (ak->expires() && (time(0) >= ak->expires()))
	{
		if (akicks.writable())
			akicks.remove(ak);
		return false;
	}
	
	if (!stricmp(sub, "LIST"))
	{
		notice_lang(m_service, u, CHAN_AKICK_LIST_FORMAT, num, ak->name(),
				(ak->reason() ? ak->reason() : getstring(u->na, NO_REASON)));
	} else if (!stricmp(sub, "VIEW"))
	{
		char timebuf[64];
		
		if (ak->since())
		{
			struct tm tm;
			time_t t = ak->since();
			
			tm = *localtime(&t);
			strftime_lang(timebuf, sizeof(timebuf), u,
					STRFTIME_SHORT_DATE_FORMAT, &tm);
		} else 
		{
			misc::strncpyzt(timebuf, getstring(u->na, UNKNOWN),
					sizeof(timebuf) - 1);
		}
		
		char expirebuf[256];
		expire_left(u->na, expirebuf, sizeof(expirebuf), ak->expires());
		
		notice_lang(m_service, u, ((ak->flags() & Akick::afStuck) ?
				CHAN_AKICK_VIEW_FORMAT_STUCK : CHAN_AKICK_VIEW_FORMAT), num,
				ak->name(), (ak->creator() ? ak->creator() :
				getstring(u->na, UNKNOWN)), timebuf, expirebuf, (ak->reason() ?
				ak->reason() : getstring(u->na, NO_REASON)));
	}

	return true;
}

/*************************************************************************/

int AkickCmd::stick(User *u, container_type &ctn, char const *sub)
{
	/* Ensure the list is not read-only. */
	if (!changeable(u, ctn, sub))
	{
		notice_lang(m_service, u, m_msgs[cmDisabled]);
		return crStop;
	}
	
	/* Ensure the list is not empty. */
	if (ctn.empty())
	{
		notice_lang(m_service, u, m_msgs[cmEmpty], context(u));
		return crStop;
	}
	
	/* Get the mask */
	char *mask = strtok(NULL, " ");
	
	if (!mask)
	{
		syntax_error(m_service, u, m_name, CHAN_AKICK_STICK_SYNTAX, sub);
		return crStop;
	}
	
	/* Try to stick the mask */
	ChannelInfo::akicks_type::const_iterator it, end;
		
	for (it = ci->akicks().begin(), end = ci->akicks().end(); it != end; ++it)
	{
		Akick *ak = (Akick *)*it;
		
		if (!ak || (ak->flags() & Akick::afIsGroup))
			continue;
		
		if (!stricmp(ak->mask(), mask))
		{
			ak->flags_add(Akick::afStuck);
			notice_lang(m_service, u, CHAN_AKICK_STUCK, ak->mask(),
					ci->name());
			return crStop;
		}
	}
	
	/* Means the mask wasn't found. */
	
	notice_lang(m_service, u, CHAN_AKICK_NOT_FOUND, mask, ci->name());
	return crStop;
}

/*************************************************************************/

int AkickCmd::unstick(User *u, container_type &ctn, char const *sub)
{
	/* Ensure the list is not read-only. */
	if (!changeable(u, ctn, sub))
	{
		notice_lang(m_service, u, m_msgs[cmDisabled]);
		return crStop;
	}
	
	/* Ensure the list is not empty. */
	if (ctn.empty())
	{
		notice_lang(m_service, u, m_msgs[cmEmpty], context(u));
		return crStop;
	}
	
	/* Get the mask */
	char *mask = strtok(NULL, " ");
	
	if (!mask)
	{
		syntax_error(m_service, u, m_name, CHAN_AKICK_STICK_SYNTAX, sub);
		return crStop;
	}
	
	/* Try to stick the mask */
	ChannelInfo::akicks_type::const_iterator it, end;
		
	for (it = ci->akicks().begin(), end = ci->akicks().end(); it != end; ++it)
	{
		Akick *ak = (Akick *)*it;
		
		if (!ak || (ak->flags() & Akick::afIsGroup))
			continue;
		
		if (!stricmp(ak->mask(), mask))
		{
			ak->flags_remove(Akick::afStuck);
			notice_lang(m_service, u, CHAN_AKICK_UNSTUCK, ak->mask(),
					ci->name());
			return crStop;
		}
	}
	
	/* Means the mask wasn't found. */
	
	notice_lang(m_service, u, CHAN_AKICK_NOT_FOUND, mask, ci->name());
	return crStop;
}

/*************************************************************************/

static void do_akick(User *u)
{
	static AkickCmd akick_cmd;
	akick_cmd.execute(u);
}

/*************************************************************************/

static void do_levels(User *u)
{
    char *chan = strtok(NULL, " ");
    char *cmd  = strtok(NULL, " ");
    char *what = strtok(NULL, " ");
    char *s    = strtok(NULL, " ");
    char *error;
	
	ChannelInfo *ci;
    short level;
    int i;

    /* If SET, we want two extra parameters; if DIS[ABLE] or FOUNDER, we want only
     * one; else, we want none.
     */
    if (!cmd || ((stricmp(cmd,"SET")==0) ? !s : ((strnicmp(cmd,"DIS",3)==0)) ? (!what || s) : !!what)) {
		syntax_error(s_ChanServ, u, "LEVELS", CHAN_LEVELS_SYNTAX);
    } else if (!(ci = channels.find(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (!is_owner(u, ci) && !is_services_admin(u)) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
    } else if (stricmp(cmd, "SET") == 0) {
		level = strtol(s, &error, 10);
		
		if (*error != '\0') {
			syntax_error(s_ChanServ, u, "LEVELS", CHAN_LEVELS_SYNTAX);
			return;
		}
		
		if (level <= ACCESS_INVALID || level >= ACCESS_FOUNDER) {
	    	notice_lang(s_ChanServ, u, CHAN_LEVELS_RANGE, ACCESS_INVALID+1, ACCESS_FOUNDER-1);
	    	return;
		}
	
		for (i = 0; levelinfo[i].what >= 0; i++) {
	    	if (stricmp(levelinfo[i].name, what) == 0) {
				ci->levels(levelinfo[i].what, level);

				epona->log(InfoChanServRights(),
						"%s!%s@%s set level for %s on channel %s to %d",
						u->nick, u->username, GetHost(u), levelinfo[i].name,
						ci->name(), level);
				notice_lang(s_ChanServ, u, CHAN_LEVELS_CHANGED,
						levelinfo[i].name, chan, level);
				return;
	    	}
		}
	
		notice_lang(s_ChanServ, u, CHAN_LEVELS_UNKNOWN, what, s_ChanServ);
		
    } else if (stricmp(cmd, "DIS") == 0 || stricmp(cmd, "DISABLE") == 0) {
		for (i = 0; levelinfo[i].what >= 0; i++) {
	    	if (stricmp(levelinfo[i].name, what) == 0) {
				ci->levels(levelinfo[i].what, ACCESS_INVALID);
				
				epona->log(InfoChanServRights(),
						"%s!%s@%s disabled level %s on channel %s", u->nick,
						u->username, GetHost(u), levelinfo[i].name,
						ci->name());
				notice_lang(s_ChanServ, u, CHAN_LEVELS_DISABLED, levelinfo[i].name, chan);
				return;
	    	}
		}
	
		notice_lang(s_ChanServ, u, CHAN_LEVELS_UNKNOWN, what, s_ChanServ);
    } else if (stricmp(cmd, "LIST") == 0) {
		int i;
		
		notice_lang(s_ChanServ, u, CHAN_LEVELS_LIST_HEADER, chan);
		
		if (!levelinfo_maxwidth) {
	    	for (i = 0; levelinfo[i].what >= 0; i++) {
				int len = strlen(levelinfo[i].name);
				if (len > levelinfo_maxwidth)
		    		levelinfo_maxwidth = len;
	    	}
		}
		
		for (i = 0; levelinfo[i].what >= 0; i++) {
	    	int j = ci->levels(levelinfo[i].what);
	    	
	    	if (j == ACCESS_INVALID) {
				j = levelinfo[i].what;
				
				if (j == CA_AUTOOP || j == CA_AUTODEOP || j == CA_AUTOVOICE || j == CA_NOJOIN) {
		    		notice_lang(s_ChanServ, u, CHAN_LEVELS_LIST_DISABLED, levelinfo_maxwidth, levelinfo[i].name);
				} else {
					notice_lang(s_ChanServ, u, CHAN_LEVELS_LIST_DISABLED, levelinfo_maxwidth, levelinfo[i].name);
				}
			} else if (j == ACCESS_FOUNDER) {
	    		notice_lang(s_ChanServ, u, CHAN_LEVELS_LIST_FOUNDER, levelinfo_maxwidth, levelinfo[i].name);
	    	} else {
				notice_lang(s_ChanServ, u, CHAN_LEVELS_LIST_NORMAL, levelinfo_maxwidth, levelinfo[i].name, j);
	    	}
		}

    } else if (stricmp(cmd, "RESET") == 0) {
		ci->reset_levels();
		
		epona->log(InfoChanServRights(),
				"%s!%s@%s reset levels definitions on channel %s", u->nick,
				u->username, GetHost(u), ci->name()); 
		notice_lang(s_ChanServ, u, CHAN_LEVELS_RESET, chan);
    } else {
		syntax_error(s_ChanServ, u, "LEVELS", CHAN_LEVELS_SYNTAX);
    }
}

/*************************************************************************/

/* SADMINS and users, who have identified for a channel, can now cause it's
 * Enstry Message and Successor to be displayed by supplying the ALL parameter.
 * Syntax: INFO channel [ALL]
 * -TheShadow (29 Mar 1999)
 */

static void do_info(User *u)
{
    char *chan = strtok(NULL, " ");
    char *param = strtok(NULL, " ");
    ChannelInfo *ci;
    char buf[BUFSIZE], *end;
    time_t t;
    struct tm *tm;
    int need_comma = 0;
    const char *commastr = getstring(u->na, COMMA_SPACE);
    int is_servadmin = is_services_admin(u);
    int show_all = 0;

    if (!chan) {
        syntax_error(s_ChanServ, u, "INFO", CHAN_INFO_SYNTAX);
    } else if (!(ci = channels.find(chan))) {
        notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else {

        /* Should we show all fields? Only for sadmins and identified users */
        if (param && stricmp(param, "ALL") == 0 && 
	    				(check_access(u, ci, CA_INFO) || is_servadmin))
            show_all = 1;

        notice_lang(s_ChanServ, u, CHAN_INFO_HEADER, chan);
        notice_lang(s_ChanServ, u, CHAN_INFO_NO_FOUNDER, (ci->owner())->name());

		if (show_all && ci->successor())
		notice_lang(s_ChanServ, u, CHAN_INFO_NO_SUCCESSOR, (ci->successor())->name());

	notice_lang(s_ChanServ, u, CHAN_INFO_DESCRIPTION, ci->desc());
	t = ci->created();
	tm = localtime(&t);
	strftime_lang(buf, sizeof(buf), u, STRFTIME_DATE_TIME_FORMAT, tm);
	notice_lang(s_ChanServ, u, CHAN_INFO_TIME_REGGED, buf);
	t = ci->last_used();
	tm = localtime(&t);
	strftime_lang(buf, sizeof(buf), u, STRFTIME_DATE_TIME_FORMAT, tm);
	notice_lang(s_ChanServ, u, CHAN_INFO_LAST_USED, buf);
	if (ci->topic() && (show_all || (!(ci->mlock_on() & CMODE_s) &&
			(!ci->c || !(ci->c->mode & CMODE_s))))) {
	    notice_lang(s_ChanServ, u, CHAN_INFO_LAST_TOPIC, ci->topic());
	    notice_lang(s_ChanServ, u, CHAN_INFO_TOPIC_SET_BY, ci->topic_setter());
	}

	if (ci->entry_message() && show_all)
	    notice_lang(s_ChanServ, u, CHAN_INFO_ENTRYMSG, ci->entry_message());
	if (ci->url())
	    notice_lang(s_ChanServ, u, CHAN_INFO_URL, ci->url());
	if (ci->email())
	    notice_lang(s_ChanServ, u, CHAN_INFO_EMAIL, ci->email());
	    
	if (show_all) 
	{
		notice_lang(s_ChanServ, u, CHAN_INFO_BANTYPE, ci->bantype());
	
		end = buf;
		*end = 0;
		if (ci->flags() & ChannelInfo::cfKeepTopic) {
	    	end += snprintf(end, sizeof(buf)-(end-buf), "%s", 
	    		getstring(u->na, CHAN_INFO_OPT_KEEPTOPIC));
	    	need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfOpNotice) {
			end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_OPNOTICE));
			need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfPeace) {
	    	end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_PEACE));
	   		need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfPrivate) {
	    	end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_PRIVATE));
	   		need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfRestricted) {
			end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_RESTRICTED));
			need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfSecure) {
			end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_SECURE));
			need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfSecureOps) {
			end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_SECUREOPS));
			need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfSecureFounder) {
			end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_SECUREFOUNDER));
			need_comma = 1;
		}
		if ((ci->flags() & ChannelInfo::cfSignKick) || (ci->flags() & ChannelInfo::cfSignKickLevel)) {
			end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_SIGNKICK));
			need_comma = 1;
		}
		if (ci->flags() & ChannelInfo::cfTopicLock) {
			end += snprintf(end, sizeof(buf)-(end-buf), "%s%s",
				need_comma ? commastr : "",
				getstring(u->na, CHAN_INFO_OPT_TOPICLOCK));
			need_comma = 1;
		}
		notice_lang(s_ChanServ, u, CHAN_INFO_OPTIONS,
				(*buf ? buf : getstring(u->na, CHAN_INFO_OPT_NONE)));
	    notice_lang(s_ChanServ, u, CHAN_INFO_MODE_LOCK, ci->mlock_modes());
	
		if (epona->cd().chanserv.expire &&
				(ci->flags() & ChannelInfo::cfPendingExpire))
			notice_lang(s_ChanServ, u, CHAN_INFO_PENDING_EXPIRE);
	
		if (epona->cd().chanserv.expire && (ci->flags() & ChannelInfo::cfNoExpire))
			notice_lang(s_ChanServ, u, CHAN_INFO_NO_EXPIRE);
	}

	}
}

/*************************************************************************/

static void do_list(User *u)
{
	if (epona->cd().chanserv.list_opers && !is_oper(u)) 
	{
		notice_lang(s_ChanServ, u, PERMISSION_DENIED);
		return;
	}
	
	uint32_t flags = 0;
	bool nolimit = false;
	char *pattern = strtok(NULL, " "), *range = 0, *opt;
	
	if (pattern && isdigit(*pattern))
	{
		range = pattern;
		pattern = 0;
	}
	
	while ((pattern || range) && (opt = strtok(NULL, " ")))
	{
		if (isdigit(*opt) && !range && !flags)
		{
			range = opt;
			continue;
		}
		
		if (is_oper(u) && !stricmp(opt, "NOLIMIT") /*&& (pattern || range) implied */)
		{
			nolimit = true;
			continue;
		}
		
		if (is_services_admin(u) && !stricmp(opt, "NOEXPIRE"))
		{
			flags |= ChannelInfo::cfNoExpire;
			continue;
		} 
		
		syntax_error(s_ChanServ, u, "LIST",
				(is_oper(u) ? CHAN_LIST_SERVADMIN_SYNTAX : CHAN_LIST_SYNTAX));
		return;
	}
	
	unsigned long first = 0, last = 0, index = 0;
	int count = 0;
	
	if (!misc::simple_range(range, &first, &last)) 
	{
		notice_lang(s_ChanServ, u, CHAN_LIST_INVALID_RANGE, range);
		return;
	}
	
	bool header = false;
	char buf[BUFSIZE];
	
	NickGroup *myng = ((u->na && nick_identified(u)) ? u->na->ng() : 0);
	
	ChannelInfo *ci;
	Channels::const_niterator it, end;
	
	for (it = channels.nbegin(), end = channels.nend(); it != end; ++it)
	{
		ci = (ChannelInfo *)(*it);
		
		if (flags && !(ci->flags() & flags))
			continue;
		
		if ((ci->flags() & ChannelInfo::cfPrivate) &&
				!is_services_admin(u) && (ci->owner() != myng))
			continue;
		
		if (pattern && strcmp(pattern, "*"))
		{
			if (stricmp(ci->name(), pattern) &&
					!misc::irc_match(pattern, ci->name()))
				continue;
		}
		
		if (++index < first || index > last)
			continue;
		
		if (!header)
		{
			notice_lang(s_ChanServ, u,
					(pattern ? CHAN_LIST_HEADER : 
					CHAN_LIST_HEADER_NO_PATTERN), pattern);
			header = true;
		}
		
		if ((++count < epona->cd().chanserv.list_max) || nolimit)
		{
			char noexpire_char = ' ';
			if (is_services_admin(u) && (ci->flags() & ChannelInfo::cfNoExpire))
				noexpire_char = '!';
			
			snprintf(buf, sizeof(buf), "%-20s  %s", ci->name(),
					ci->desc() ? ci->desc() : "");
			notice_user(s_ChanServ, u, "   %c%s", noexpire_char, buf);
		}
	}
	
	if (!header && pattern)
	{
		notice_lang(s_ChanServ, u, CHAN_LIST_NO_NICKS_MATCH, pattern);
		return;
	}
	
	if (!nolimit && count > epona->cd().chanserv.list_max)
		count = epona->cd().chanserv.list_max;
	
	if (range && count)
	{
		if (first == last)
			notice_lang(s_ChanServ, u,
					CHAN_LIST_RESULTS_RANGE_ONE, first);
		else
			notice_lang(s_ChanServ, u, CHAN_LIST_RESULTS_RANGE,
					first, (first + count - 1));
	} else {
		notice_lang(s_ChanServ, u, CHAN_LIST_RESULTS, count, index);
	}
	
    /* char *pattern = strtok(NULL, " ");
    ChannelInfo *ci;
    int nchans;
    char buf[BUFSIZE];
    int is_servadmin = is_services_admin(u);

    if (epona->cd().chanserv.list_opers && (!u || !is_oper(u))) {
	notice_lang(s_ChanServ, u, PERMISSION_DENIED);
	return;
	}

    if (!pattern) {
		syntax_error(s_ChanServ, u, "LIST", is_servadmin ? CHAN_LIST_SERVADMIN_SYNTAX : CHAN_LIST_SYNTAX);
	} else {
		char *keyword;
		int32_t matchflags = 0;
	
		nchans = 0;
	
		while (is_servadmin && (keyword = strtok(NULL, " "))) 
		{
	    	if (stricmp(keyword, "NOEXPIRE") == 0)
				matchflags |= ChannelInfo::cfNoExpire;
		}
	
	notice_lang(s_ChanServ, u, CHAN_LIST_HEADER, pattern);
	
	Channels::const_niterator it, end;
	
	for (it = channels.nbegin(), end = channels.nend(); it != end; ++it)
	{
		ci = (ChannelInfo *)(*it);
		
		if (!is_servadmin && (ci->flags() & ChannelInfo::cfPrivate))
		    continue;
		if ((matchflags != 0) && !(ci->flags() & matchflags))
		    continue;
		if (stricmp(pattern, ci->name()) == 0 || match_wild_nocase(pattern, ci->name())) {
		    if (++nchans <= epona->cd().chanserv.list_max) {
			char noexpire_char = ' ';
			if (is_servadmin && (ci->flags() & ChannelInfo::cfNoExpire))
			    noexpire_char = '!';
			    
			snprintf(buf, sizeof(buf), "%-20s  %s", ci->name(),
					ci->desc() ? ci->desc() : "");
			    
			notice_user(s_ChanServ, u, "  %c%s",
						noexpire_char, buf);
		    }
		}
	}
	notice_lang(s_ChanServ, u, CHAN_LIST_END,
			nchans>epona->cd().chanserv.list_max ?
			(uint16_t) epona->cd().chanserv.list_max : nchans, nchans);
    }*/

}

/*************************************************************************/

static void do_clist(User *u)
{	
	if (epona->cd().chanserv.clist_opers && !is_oper(u)) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
		char *nick = strtok(NULL, " "), *privilege = strtok(NULL, " ");
		NickAlias *na = (nick ? aliases.find(nick) : NULL);
		
		if ((na || (nick && privilege)) && !is_services_admin(u)) {
			syntax_error(s_ChanServ, u, "CLIST", CHAN_CLIST_SYNTAX);
		} else if (nick && privilege && !na) {
			notice_lang(s_ChanServ, u, NICK_X_NOT_REGISTERED, nick);
		} else if (!na && !u->na) {
			notice_lang(s_ChanServ, u, NICK_NOT_REGISTERED);
		} else {
			int level = 0; 
			bool founder = false, minus = false, plus = false,
					successor = false;
			
			if (!na) 
			{
				if (!nick_identified(u))
				{
					notice_lang(s_NickServ, u, NICK_IDENTIFY_REQUIRED, s_ChanServ);
					return;
				}
				
				privilege = nick;
				na = u->na;
			}
			
			if (!privilege) {
				founder = minus = plus = successor = true;
			} else if (!stricmp(privilege, "FOUNDER")) {
				founder = true;
			} else if (!stricmp(privilege, "SUCCESSOR")) {
				successor = true;
			} else if ((level = atoi(privilege)) > 0) {
				founder = plus = true;
			} else if (level < 0) {
				minus = true;
			} else {
				 syntax_error(s_ChanServ, u, "CLIST", ((na == u->na) ? 
				 		CHAN_CLIST_SYNTAX : CHAN_CLIST_SYNTAX_X));
				 return;
			}
			
			int count = 0;
		
			notice_lang(s_ChanServ, u, ((na == u->na) ? CHAN_CLIST_HEADER :
					CHAN_CLIST_HEADER_X), na->nick());
		
			Channels::const_niterator it, end;
	
			for (it = channels.nbegin(), end = channels.nend(); it != end; ++it)
			{
				ChannelInfo *ci = (ChannelInfo *)(*it);
				
				int has_info = is_services_admin(u) || 
						check_access(u, ci, CA_INFO);
				
				if (founder && (ci->owner() == na->ng())) {
					notice_lang(s_ChanServ, u, CHAN_CLIST_FORMAT_STR,
							(has_info && (ci->flags() & ChannelInfo::cfNoExpire) ?
									'!' : ' '),
							ci->name(),
							getstring(u->na, CHAN_CLIST_FORMAT_STR_FOUNDER),
							ci->desc());
					count++;
					continue;
				}
				
				if (plus || minus) 
				{
					int found = 0;
					Access *acc;
					
					ChannelInfo::accesses_type::const_iterator it, end;
				
					for (it = ci->accesses().begin(),
							end = ci->accesses().end(); it != end; ++it)
					{
						acc = (Access *) (*it);
					
						if (!acc)
							continue;
						
						if (acc->ng() != na->ng())
							continue;
						
						if (!(plus && minus)) 
						{
							if (plus && (acc->level() < level))
								continue;
							
							if (minus && (acc->level() > level))
								continue;
						}
						
						
						notice_lang(s_ChanServ, u, CHAN_CLIST_FORMAT,
								(has_info && 
								(ci->flags() & ChannelInfo::cfNoExpire) ? '!' :
								' '),
								ci->name(), acc->level(), ci->desc());

						count++;
						
						found = 1;
						break;
					}
					
					if (found)
						continue;
				}
				
				if (successor && ci->successor() && (ci->successor() == na->ng())) 
				{
					notice_lang(s_ChanServ, u, CHAN_CLIST_FORMAT_STR,
							(has_info && (ci->flags() & ChannelInfo::cfNoExpire) ?
									'!' : ' '),
							ci->name(), 
							getstring(u->na, CHAN_CLIST_FORMAT_STR_SUCCESSOR),
							ci->desc());
					count++;
				}
			}
		
			notice_lang(s_ChanServ, u, CHAN_CLIST_FOOTER, count);
		}
	}
}

/*************************************************************************/

static void do_invite(User *u)
{
    char *chan = strtok(NULL, " ");
    Channel *c;
    ChannelInfo *ci;

    if (!chan) {
	syntax_error(s_ChanServ, u, "INVITE", CHAN_INVITE_SYNTAX);
    } else if (!(c = findchan(chan))) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (!u || !check_access(u, ci, CA_INVITE)) {
	notice_lang(s_ChanServ, u, PERMISSION_DENIED);
    } else {
	send_cmd(whosends(ci), "INVITE %s %s", u->nick, chan);
    }
}

/*************************************************************************/

/* do_util: not a command, but does the job of other */

static void do_util(User *u, CSModeUtil *util)
{
	char const *av[2];
	char *chan = strtok(NULL, " ");
    char *nick = strtok(NULL, " ");
    
    Channel *c;
    ChannelInfo *ci;
    User *u2;
    
    int is_same;
    
    if (!chan) {
    	User::u_chanlist *uc;
    	
    	av[0] = util->mode;
    	av[1] = u->nick;
    	
    	/* Sets the mode to the user on every channels he is on. */
    	
    	for (uc = u->chans; uc; uc = uc->next) {
    		if ((ci = uc->chan->ci) && check_access(u, ci, util->levelself)) {
				s_mode(whosends(ci), uc->chan->name, util->mode, u->nick);
				chan_set_modes(s_ChanServ, uc->chan, 2, av, 1);
				
				if (util->notice && ci->flags() & util->notice)
					notice(whosends(ci), chan, "%s command used for %s by %s", util->name, u->nick, u->nick);
    		}		
    	}
    	
    	return;
    } else if (!nick) {
    	nick = u->nick;
	}
    
    is_same = (nick == u->nick) ? 1 : (stricmp(nick, u->nick) == 0);
    
    if (!(c = findchan(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, c->name);
    } else if (is_same ? !(u2 = u) : !(u2 = finduser(nick))) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_IN_USE, nick);
	} else if (!is_on_chan(c, u2)) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_ON_CHAN, u2->nick, c->name);
    } else if (is_same ? !check_access(u, ci, util->levelself) : !check_access(u, ci, util->level)) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else if (*util->mode == '-' && !is_same && (ci->flags() & ChannelInfo::cfPeace) && (get_access(u2, ci) >= get_access(u, ci))) {
		notice_lang(s_ChanServ, u, PERMISSION_DENIED);
    } else {
		s_mode(whosends(ci), c->name, util->mode, u2->nick);
		
		av[0] = util->mode;
		av[1] = u2->nick;
		chan_set_modes(s_ChanServ, c, 2, av, 1);
		
		if (util->notice && ci->flags() & util->notice)
			notice(whosends(ci), c->name, "%s command used for %s by %s", util->name, u2->nick, u->nick);
    }
}

/*************************************************************************/

static void do_op(User *u)
{
    do_util(u, &csmodeutils[MUT_OP]);
}

/*************************************************************************/

static void do_deop(User *u)
{
    do_util(u, &csmodeutils[MUT_DEOP]);
}

/*************************************************************************/

static void do_voice(User *u)
{
    do_util(u, &csmodeutils[MUT_VOICE]);
}

/*************************************************************************/

static void do_devoice(User *u)
{
    do_util(u, &csmodeutils[MUT_DEVOICE]);
}

/*************************************************************************/

#ifdef HAS_HALFOP

static void do_halfop(User *u)
{
    do_util(u, &csmodeutils[MUT_HALFOP]);
}

/*************************************************************************/

static void do_dehalfop(User *u)
{
    do_util(u, &csmodeutils[MUT_DEHALFOP]);
}

#endif

/*************************************************************************/

#ifdef IRC_UNREAL

static void do_protect(User *u)
{
    do_util(u, &csmodeutils[MUT_PROTECT]);
}

/*************************************************************************/

static void do_deprotect(User *u)
{
    do_util(u, &csmodeutils[MUT_DEPROTECT]);
}

/*************************************************************************/

static void do_owner(User *u)
{
	char const *av[2];
	char *chan = strtok(NULL, " ");
    
    Channel *c;
    ChannelInfo *ci;
    
    if (!chan) {
    	User::u_chanlist *uc;
    	
    	av[0] = "+q";
    	av[1] = u->nick;
    	
    	/* Sets the mode to the user on every channels he is on. */
    	
    	for (uc = u->chans; uc; uc = uc->next) {
    		if ((ci = uc->chan->ci) && is_owner(u, ci)) {
				s_mode(whosends(ci), uc->chan->name, av[0], u->nick);
				chan_set_modes(s_ChanServ, uc->chan, 2, av, 1);
    		}		
    	}
    	return;
    }
    
    if (!(c = findchan(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, c->name);
    } else if (!is_on_chan(c, u)) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_ON_CHAN, u->nick, c->name);
    } else if (!is_owner(u, ci)) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
		s_mode(whosends(ci), c->name, "+q", u->nick);
		
		av[0] = "+q";
		av[1] = u->nick;
		chan_set_modes(s_ChanServ, c, 2, av, 1);
    }
}

/*************************************************************************/

static void do_deowner(User *u)
{
	char const *av[2];
	char *chan = strtok(NULL, " ");
    
    Channel *c;
    ChannelInfo *ci;
    
    if (!chan) {
    	User::u_chanlist *uc;
    	
    	av[0] = "-q";
    	av[1] = u->nick;
    	
    	/* Sets the mode to the user on every channels he is on. */
    	
    	for (uc = u->chans; uc; uc = uc->next) {
    		if ((ci = uc->chan->ci) && is_owner(u, ci)) {
				s_mode(whosends(ci), uc->chan->name, av[0], u->nick);
				chan_set_modes(s_ChanServ, uc->chan, 2, av, 1);
    		}		
    	}
    	return;
    }
    
    if (!(c = findchan(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, c->name);
    } else if (!is_on_chan(c, u)) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_ON_CHAN, u->nick, c->name);
    } else if (!is_owner(u, ci)) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else {
		s_mode(whosends(ci), c->name, "-q", u->nick);
		
		av[0] = "-q";
		av[1] = u->nick;
		chan_set_modes(s_ChanServ, c, 2, av, 1);
    }
}

#endif

/*************************************************************************/

static void do_cs_kick(User *u)
{
	char *chan = strtok(NULL, " ");
    char *params = strtok(NULL, " ");
    char *sreason = strtok(NULL, "");
    char const *reason = "Requested";
    
    Channel *c;
    ChannelInfo *ci;  
    User *u2;
    
    int is_same;
    
	if (sreason)
	{
		if (strlen(sreason) > 300)
			sreason[300] = '\0';
		reason = sreason;
	}
    
    if (!chan) 
    {
    	User::u_chanlist *uc, *next;
    	
    	/* Kicks the user on every channels he is on. */
    	
    	for (uc = u->chans; uc; uc = next) {
    		next = uc->next;
    		if ((ci = uc->chan->ci) && check_access(u, ci, CA_KICKME)) {
    			char const *av[3];
    			if ((ci->flags() & ChannelInfo::cfSignKick) || ((ci->flags() & ChannelInfo::cfSignKickLevel) && !check_access(u, ci, CA_SIGNKICK)))
					send_cmd(whosends(ci), "KICK %s %s :%s (%s)", ci->name(), u->nick, reason, u->nick);
				else
					send_cmd(whosends(ci), "KICK %s %s :%s", ci->name(), u->nick, reason);
				av[0] = ci->name();
				av[1] = u->nick;
				av[2] = reason;
				do_kick(s_ChanServ, 3, av);
    		}		
    	}
    	
    	return;
    } else if (!params) {
    	params = u->nick;
	}
	
	is_same = (params == u->nick) ? 1 : (stricmp(params, u->nick) == 0);
    
    if (!(c = findchan(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (is_same ? !(u2 = u) : !(u2 = finduser(params))) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_IN_USE, params);
	} else if (!is_on_chan(c, u2)) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_ON_CHAN, u2->nick, c->name);
    } else if (!is_same ? !check_access(u, ci, CA_KICK) : !check_access(u, ci, CA_KICKME)) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else if (!is_same && (ci->flags() & ChannelInfo::cfPeace) && (get_access(u2, ci) >= get_access(u, ci))) {
		notice_lang(s_ChanServ, u, PERMISSION_DENIED);
	} else {
		char const *av[3];
		if ((ci->flags() & ChannelInfo::cfSignKick) || ((ci->flags() & ChannelInfo::cfSignKickLevel) && !check_access(u, ci, CA_SIGNKICK)))
			send_cmd(whosends(ci), "KICK %s %s :%s (%s)", ci->name(), params, reason, u->nick);
		else
			send_cmd(whosends(ci), "KICK %s %s :%s", ci->name(), params, reason);
		av[0] = ci->name();
		av[1] = params;
		av[2] = reason;
		do_kick(s_ChanServ, 3, av);
	}
}

/*************************************************************************/

static void do_ban(User *u)
{
	char *chan = strtok(NULL, " ");
    char *params = strtok(NULL, " ");
    char *sreason = strtok(NULL, "");
    char const *reason = "Requested";
    
    Channel *c;
    ChannelInfo *ci;  
    User *u2;
    
    int is_same;
    
	if (sreason)
	{
		if (strlen(sreason) > 300)
			sreason[300] = '\0';
		reason = sreason;
	}
    
    if (!chan) {
    	User::u_chanlist *uc, *next;
    	
    	/* Bans the user on every channels he is on. */
    	
    	for (uc = u->chans; uc; uc = next) {
    		next = uc->next;
    		if ((ci = uc->chan->ci) && check_access(u, ci, CA_BANME)) {
    			char const *av[3];
    			char mask[BUFSIZE];
    			
				av[0] = "+b";
				get_idealban(ci, u, mask, sizeof(mask));
				av[1] = mask;
				s_mode(whosends(ci), uc->chan->name, "+b", av[1]);
				chan_set_modes(s_ChanServ, uc->chan, 2, av, 1);
    			
    			if ((ci->flags() & ChannelInfo::cfSignKick) || ((ci->flags() & ChannelInfo::cfSignKickLevel) && !check_access(u, ci, CA_SIGNKICK)))
					send_cmd(whosends(ci), "KICK %s %s :%s (%s)", ci->name(), u->nick, reason, u->nick);
				else
					send_cmd(whosends(ci), "KICK %s %s :%s", ci->name(), u->nick, reason);
				
				char const *av2[3];
				
				av2[0] = ci->name();
				av2[1] = u->nick;
				av2[2] = reason;
				do_kick(s_ChanServ, 3, av2);
    		}		
    	}
    	
    	return;
    } else if (!params) {
    	params = u->nick;
	} 
	
	is_same = (params == u->nick) ? 1 : (stricmp(params, u->nick) == 0);
    
    if (!(c = findchan(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (is_same ? !(u2 = u) : !(u2 = finduser(params))) {
		notice_lang(s_ChanServ, u, NICK_X_NOT_IN_USE, params);
    } else if (!is_same ? !check_access(u, ci, CA_BAN) : !check_access(u, ci, CA_BANME)) {
		notice_lang(s_ChanServ, u, ACCESS_DENIED);
	} else if (!is_same && (ci->flags() & ChannelInfo::cfPeace) && (get_access(u2, ci) >= get_access(u, ci))) {
		notice_lang(s_ChanServ, u, PERMISSION_DENIED);
	} else {
		char const *av[3];
		char mask[BUFSIZE];
    			
		av[0] = "+b";
		get_idealban(ci, u2, mask, sizeof(mask));
		av[1] = mask;
		s_mode(whosends(ci), c->name, "+b", av[1]);
		chan_set_modes(s_ChanServ, c, 2, av, 1);
		
		/* We still allow host banning while not allowing to kick */
		if (!is_on_chan(c, u2))
			return;
		
		if ((ci->flags() & ChannelInfo::cfSignKick) || ((ci->flags() & ChannelInfo::cfSignKickLevel) && !check_access(u, ci, CA_SIGNKICK)))
			send_cmd(whosends(ci), "KICK %s %s :%s (%s)", ci->name(), params, reason, u->nick);
		else
			send_cmd(whosends(ci), "KICK %s %s :%s", ci->name(), params, reason);
		
		char const *av2[3];
		
		av2[0] = ci->name();
		av2[1] = params;
		av2[2] = reason;
		do_kick(s_ChanServ, 3, av2);
	}
}

/*************************************************************************/

static void do_cs_topic(User *u)
{
	char *chan = strtok(NULL, " ");
	char *topic = strtok(NULL, "");
	
	ChannelInfo *ci;
	
	if (!chan) {
		syntax_error(s_ChanServ, u, "TOPIC", CHAN_TOPIC_SYNTAX);
	} else if (!channels.writable()) {
		notice_lang(s_ChanServ, u, CHAN_TOPIC_DISABLED);
	} else if (!(ci = channels.find(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
	} else if (!is_services_admin(u) && !check_access(u, ci, CA_TOPIC)) {
		notice_lang(s_ChanServ, u, PERMISSION_DENIED);
	} else {
		if (!ci->change_topic(topic, u->nick, time(0)))
		{
			notice_lang(s_ChanServ, u, CHAN_TOPIC_FAILED);
			return;
		}
		
		if (ci->c)
			set_topic(whosends(ci), ci->c, ci->topic(),
					ci->topic_setter(), ci->topic_time());
		
		if (is_services_admin(u))
			epona->log(NoticeChanServPrivs(),
					"%s!%s@%s changed topic of %s as services admin.",
					u->nick, u->username, u->host, ci->name());
	}
}

/*************************************************************************/

static void do_unban(User *u)
{
    char *chan = strtok(NULL, " ");
    Channel *c;
    ChannelInfo *ci;

    if (!chan) {
	syntax_error(s_ChanServ, u, "UNBAN", CHAN_UNBAN_SYNTAX);
    } else if (!(c = findchan(chan))) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (!check_access(u, ci, CA_UNBAN)) {
	notice_lang(s_ChanServ, u, PERMISSION_DENIED);
    } else {
#ifdef HAS_SVSMODE_UNBAN
    	send_cmd(epona->cd().services.sname, "SVSMODE %s -b %s",
    			c->name, u->nick);
#else
		int i;
    	char *av[3];
    	/* Save original ban info */
		int count = c->bancount;
		char **bans = static_cast<char **>(smalloc(sizeof(char *) * count));
		memcpy(bans, c->bans, sizeof(char *) * count);

		av[0] = chan;
		av[1] = "-b";
		for (i = 0; i < count; i++) {
	    	if (match_usermask(bans[i], u)) {
				s_mode(whosends(ci), chan, "-b", bans[i]);
				av[2] = misc::nstrdup(bans[i]);
				do_cmode(s_ChanServ, 3, av);
				delete [] av[2];
	    	}
		}
		free(bans);
#endif
		notice_lang(s_ChanServ, u, CHAN_UNBANNED, chan);
    }
}

/*************************************************************************/

static void do_clear(User *u)
{
    char *chan = strtok(NULL, " ");
    char *what = strtok(NULL, " ");
    Channel *c;
    ChannelInfo *ci;

    if (!what) {
	syntax_error(s_ChanServ, u, "CLEAR", CHAN_CLEAR_SYNTAX);
    } else if (!(c = findchan(chan))) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (!(ci = c->ci)) {
	notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
    } else if (!u || !check_access(u, ci, CA_CLEAR)) {
	notice_lang(s_ChanServ, u, PERMISSION_DENIED);
    } else if (stricmp(what, "bans") == 0) {
	char const *av[3];
	int i;

	/* Save original ban info */
	int count = c->bancount;
	char **bans = static_cast<char **>(smalloc(sizeof(char *) * count));
	for (i = 0; i < count; i++)
	    bans[i] = misc::nstrdup(c->bans[i]);

	for (i = 0; i < count; i++) {
	    av[0] = chan;
	    av[1] = "-b";
	    av[2] = bans[i];
	    s_mode(whosends(ci), av[0], av[1], av[2]);
	    do_cmode(s_ChanServ, 3, av);
	    delete [] bans[i];
	}
		notice_lang(s_ChanServ, u, CHAN_CLEARED_BANS, chan);
		free(bans);
	} else if (stricmp(what, "modes") == 0) {	
		char buf[BUFSIZE], *end = buf;

		if (c->mode) {
			CBModeInfo *cbmi = cbmodeinfos;
			CBMode *cbm;
	
			do {
				if (c->mode & cbmi->flag)
					*end++ = cbmi->mode;
			} while ((++cbmi)->flag != 0);
	
			cbmi = cbmodeinfos;
	
			do {
				if (cbmi->getvalue && (c->mode & cbmi->flag) && !(cbmi->flags & CBM_MINUS_NO_ARG)) {
					char *value = cbmi->getvalue(c);
			
					if (value) {
						*end++ = ' ';
						while (*value)
							*end++ = *value++;
						
						/* Free the value */
						cbm = &cbmodes[(int)cbmi->mode];
						cbm->setvalue(c, NULL);
					}
				}
			} while ((++cbmi)->flag != 0);
			
			*end = 0;

#ifdef IRC_BAHAMUT
			send_cmd(whosends(ci), "MODE %s 0 -%s", c->name, buf);
#else			
			send_cmd(whosends(ci), "MODE %s -%s", c->name, buf);
#endif
			c->mode = 0;
			check_modes(c);
		}
		
		notice_lang(s_ChanServ, u, CHAN_CLEARED_MODES, chan);
    } else if (stricmp(what, "ops") == 0) {
	char const *av[3];
	struct Channel::c_userlist *cu, *next;

	for (cu = c->users; cu; cu = next) {
	    next = cu->next;
	    if (!chan_has_user_status(c, cu->user, CUS_OP))
	    	continue;
	    av[0] = chan;
	    av[1] = "-o";
	    av[2] = cu->user->nick;
		s_mode(whosends(ci), av[0], av[1], av[2]);
	    do_cmode(s_ChanServ, 3, av);
	}
	notice_lang(s_ChanServ, u, CHAN_CLEARED_OPS, chan);
    } else if (stricmp(what, "voices") == 0) {
	char const *av[3];
	struct Channel::c_userlist *cu, *next;

	for (cu = c->users; cu; cu = next) {
	    next = cu->next;
	    if (!chan_has_user_status(c, cu->user, CUS_VOICE))
	    	continue;
	    av[0] = chan;
	    av[1] = "-v";
	    av[2] = cu->user->nick;
	    s_mode(whosends(ci), av[0], av[1], av[2]);
	    do_cmode(s_ChanServ, 3, av);
	}
	notice_lang(s_ChanServ, u, CHAN_CLEARED_VOICES, chan);
    } else if (stricmp(what, "users") == 0) {
	char *av[3];
	struct Channel::c_userlist *cu, *next;
	char buf[256];

	snprintf(buf, sizeof(buf), "CLEAR USERS command from %s", u->nick);

	for (cu = c->users; cu; cu = next) {
	    next = cu->next;
	    av[0] = chan;
	    av[1] = cu->user->nick;
	    av[2] = buf;
	    send_cmd(whosends(ci), "KICK %s %s :%s",
			av[0], av[1], av[2]);
	    do_kick(s_ChanServ, 3, av);
	}
	notice_lang(s_ChanServ, u, CHAN_CLEARED_USERS, chan);
    } else {
	syntax_error(s_ChanServ, u, "CLEAR", CHAN_CLEAR_SYNTAX);
    }
}

/*************************************************************************/

static void do_getkey(User *u)
{
    char *chan = strtok(NULL, " ");
    ChannelInfo *ci;

    if (chan && (ci = channels.find(chan)) && check_access(u, ci, CA_GETKEY)) {
		notice_user(s_ChanServ, u, "KEY %s %s", ci->name(),
				(ci->c ? (ci->c->key ? ci->c->key : "NO KEY") : "NO KEY"));
	} else {
		notice_user(s_ChanServ, u, "KEY %s ERROR", chan);	
	}
}

/*************************************************************************/

static void do_getpass(User *u)
{
	char *chan = strtok(NULL, " ");
	ChannelInfo *ci;

	if (!chan) {
		syntax_error(s_ChanServ, u, "GETPASS", CHAN_GETPASS_SYNTAX);
	} else if (!(ci = channels.find(chan))) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
	} else {
		char passbuf[13];
		char const *pass = passbuf;
		
		if (ci->enc() != 0)
		{
			if (!channels.writable())
			{
				notice_lang(s_ChanServ, u, CHAN_GETPASS_DISABLED);
				return;
			}
			
			if (!ci->generate_password(passbuf, sizeof(passbuf)))
			{
				notice_lang(s_ChanServ, u, CHAN_GETPASS_UNAVAILABLE);
				return;
			}
		} else {
			pass = ci->pass();
		}
		
		epona->log(NoticeChanServPrivs(), "%s!%s@%s used GETPASS on %s",
				u->nick, u->username, GetHost(u), ci->name());
		if (epona->cd().operserv.wall &
				Epona::Directives::OperServ::Wall::waGetpass) 
		{
	    	wallops(s_ChanServ, "\2%s\2 used GETPASS on channel \2%s\2",
					u->nick, ci->name());
		}
		notice_lang(s_ChanServ, u, CHAN_GETPASS_PASSWORD_IS,
				ci->name(), pass);
	}
}

/*************************************************************************/

static void do_sendpass(User *u)
{
	if (!mail::Mail::configured())
	{
		notice_lang(s_ChanServ, u, CHAN_SENDPASS_UNCONFIGURED);
		return;
	}
	
	char *chan = strtok(NULL, " ");
	ChannelInfo *ci;
	NickGroup *founder;
	
	if (!chan) {
		syntax_error(s_ChanServ, u, "SENDPASS", CHAN_SENDPASS_SYNTAX);
	} else if (epona->cd().chanserv.sendpass_opers && !is_oper(u)) {
		notice_lang(s_ChanServ, u, PERMISSION_DENIED);	
	} else if (!(ci = channels.find(chan)) || !(founder = ci->owner())) {
		notice_lang(s_ChanServ, u, CHAN_X_NOT_REGISTERED, chan);
	} else if (ci->enc()) {
		notice_lang(s_ChanServ, u, CHAN_SENDPASS_UNAVAILABLE);
	} else {
		char subject[BUFSIZE];
		snprintf(subject, sizeof(subject),
				getstring2(founder, CHAN_SENDPASS_SUBJECT), ci->name());
		
		mail::Mail mail;
		
		mail.subject(subject);
		mail.to_name(founder->name());
		mail.to_address(founder->email());
		mail.origin(u->host);
		
		mail.text_add(getstring2(founder, CHAN_SENDPASS_HEAD));
		mail.text_add("");
		mail.text_addf(getstring2(founder, CHAN_SENDPASS_LINE_1), ci->name());
		mail.text_add("");
		mail.text_addf(getstring2(founder, CHAN_SENDPASS_LINE_2), ci->pass());
		mail.text_add("");
		mail.text_add(getstring2(founder, CHAN_SENDPASS_LINE_3));
		mail.text_add("");
		mail.text_add(getstring2(founder, CHAN_SENDPASS_LINE_4));
		mail.text_add("");
		mail.text_addf(getstring2(founder, CHAN_SENDPASS_LINE_5),
				(const char *)epona->cd().services.network_name);
		
		uint32_t mflags = 0;
		
		if (is_oper(u))
			mflags |= mail::Mail::mfNoQuota;
		if (founder->flags() & NickGroup::gfAuth)
			mflags |= mail::Mail::mfAuthenticated;
		
		uint32_t err;
		std::string errdesc;
		
		if (!mail.send(mflags, &err, &errdesc))
		{
			notice_lang(s_ChanServ, u, ((err & mail::Mail::meTemporary) ?
					CHAN_SENDPASS_FAILURE_TEMPORARY : CHAN_SENDPASS_FAILURE),
					ci->name());
			
			if (err & mail::Mail::meQuota)
				epona->log(InfoChanServ(),
						"SENDPASS could not send an e-mail to %s: %s",
						founder->email(), errdesc.c_str());
			else
				epona->log(ErrorChanServ(),
						"SENDPASS could not send an e-mail to %s: %s",
						founder->email(), errdesc.c_str());
			return;
		}
		
		epona->log(InfoChanServSettings(), "%s!%s@%s used SENDPASS on %s",
				u->nick, u->username, GetHost(u), ci->name());
		notice_lang(s_ChanServ, u, CHAN_SENDPASS_OK, ci->name());
	}
}

/*************************************************************************/

static void do_forbid(User *u)
{
    notice_lang(s_ChanServ, u, REMOVED_COMMAND, s_OperServ, "SQLINE");
}

/*************************************************************************/

static void do_status(User *u)
{
    ChannelInfo *ci;
    User *u2;
    char *nick, *chan;

    chan = strtok(NULL, " ");
    nick = strtok(NULL, " ");
    if (!nick || strtok(NULL, " ")) {
	notice_user(s_ChanServ, u, "STATUS ERROR Syntax error");
	return;
    }
    if (!(ci = channels.find(chan))) {
	char *temp = chan;
	chan = nick;
	nick = temp;
	ci = channels.find(chan);
    }
    if (!ci) {
	notice_user(s_ChanServ, u, "STATUS ERROR Channel %s not registered",
		chan);
    } else if ((u2 = finduser(nick)) != NULL) {
	notice_user(s_ChanServ, u, "STATUS %s %s %d", chan, nick, get_access(u2, ci));
    } else { /* !u2 */
	notice_user(s_ChanServ, u, "STATUS ERROR Nick %s not online", nick);
    }
}

/*************************************************************************/
