/* ChanServ records management.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include <algorithm>
#include <map>
#if HAVE_OPENSSL
# include <openssl/rand.h>
#endif

#include <core/directives.h>
#include <core/epona.h>
#include <databases/database.h>
#include <databases/field.h>
#include <log/levels.h>
#include <modules/manager.h>
#include <password/password.h>

#include "botserv.h"
#include "channels.h"
#include "chanserv.h"
#include "epona.h"
#include "nickserv.h"
#include "operserv.h"

#include "language.h"
#include "services.h"

using namespace std;

using databases::Collection;
using databases::Database;
using databases::Field;
using databases::Record;

/*************************************************************************/

Channels channels;
Levels levels;
Accesses accesses;
Akicks akicks;

databases::Field *ChannelInfo::f_name = 0;
databases::Field *ChannelInfo::f_owner = 0;
databases::Field *ChannelInfo::f_successor = 0;
databases::Field *ChannelInfo::f_created = 0;
databases::Field *ChannelInfo::f_last_used = 0;
databases::Field *ChannelInfo::f_enc = 0;
databases::Field *ChannelInfo::f_pass = 0;
databases::Field *ChannelInfo::f_flags = 0;
databases::Field *ChannelInfo::f_desc = 0;
databases::Field *ChannelInfo::f_email = 0;
databases::Field *ChannelInfo::f_url = 0;
databases::Field *ChannelInfo::f_entry_message = 0;
databases::Field *ChannelInfo::f_bantype = 0;
databases::Field *ChannelInfo::f_mlock_ircd = 0;
databases::Field *ChannelInfo::f_mlock_modes = 0;
databases::Field *ChannelInfo::f_topic = 0;
databases::Field *ChannelInfo::f_topic_setter = 0;
databases::Field *ChannelInfo::f_topic_time = 0;
databases::Field *ChannelInfo::f_bot = 0;
databases::Field *ChannelInfo::f_botflags = 0;
databases::Field *ChannelInfo::f_caps_min = 0;
databases::Field *ChannelInfo::f_caps_percent = 0;
databases::Field *ChannelInfo::f_flood_lines = 0;
databases::Field *ChannelInfo::f_flood_secs = 0;
databases::Field *ChannelInfo::f_repeats = 0;

databases::Field *Level::f_owner = 0;
databases::Field *Level::f_name = 0;
databases::Field *Level::f_level = 0;

databases::Field *Access::f_owner = 0;
databases::Field *Access::f_group = 0;
databases::Field *Access::f_level = 0;
databases::Field *Access::f_last_seen = 0;

databases::Field *Akick::f_owner = 0;
databases::Field *Akick::f_flags = 0;
databases::Field *Akick::f_group = 0;
databases::Field *Akick::f_mask = 0;
databases::Field *Akick::f_creator = 0;
databases::Field *Akick::f_reason = 0;
databases::Field *Akick::f_since = 0;
databases::Field *Akick::f_expires = 0;

/*************************************************************************/

namespace
{

/*************************************************************************/

#define HASH(name)	((static_cast<unsigned char>(tolower((name)[0]))&31)<<5 | (static_cast<unsigned char>(tolower((name)[1]))&31))

class ChannelHash : public databases::NameCollection::name_hash
{
	public:
		virtual int compare(char const *const &key,
				databases::NameRecord *const &value) const 
				{ return stricmp(key, value->name()); }	
		virtual uint32_t hash(char const *const &key, uint32_t max) const
				{ return HASH(key); }
		virtual bool match(char const *const &key,
				databases::NameRecord *const &value) const
				{ return !stricmp(key, value->name()); }
};

#undef HASH

/*************************************************************************/

ChannelHash channel_hash;

typedef map<std::string, password::Encrypter *> EncMap;
static EncMap enc_map;

time_t const lu_sensibility = 600;
time_t const als_sensibility = 600;

/*************************************************************************/

DefEntry def_levels[CA_SIZE] = 
{
    DefEntry(CA_INVITE,         "INVITE",        5),
    DefEntry(CA_AKICK,          "AKICK",         10),
    DefEntry(CA_SET,     		"SET",           ACCESS_INVALID),
    DefEntry(CA_UNBAN,          "UNBAN",         5),
    DefEntry(CA_AUTOOP,         "AUTOOP",        5),
    DefEntry(CA_AUTODEOP,       "AUTODEOP",     -1),
    DefEntry(CA_AUTOVOICE,      "AUTOVOICE",     3),
    DefEntry(CA_OPDEOP,         "OPDEOP",        5),
    DefEntry(CA_ACCESS_LIST,    "ACC-LIST",      1),
    DefEntry(CA_CLEAR,          "CLEAR",         ACCESS_INVALID),
    DefEntry(CA_NOJOIN,         "NOJOIN",       -2),
    DefEntry(CA_ACCESS_CHANGE,  "ACC-CHANGE",    10),
    DefEntry(CA_MEMO,           "MEMO",          10),
    DefEntry(CA_ASSIGN,         "ASSIGN",        ACCESS_INVALID),
    DefEntry(CA_BADWORDS,       "BADWORDS",      10),
    DefEntry(CA_NOKICK,         "NOKICK",        1),
    DefEntry(CA_FANTASIA,       "FANTASIA",      3),
    DefEntry(CA_SAY,            "SAY",           5),
    DefEntry(CA_GREET,          "GREET",         5),
    DefEntry(CA_VOICEME,        "VOICEME",       3),
    DefEntry(CA_VOICE,          "VOICE",         5),
    DefEntry(CA_GETKEY,         "GETKEY",        5),
    DefEntry(CA_AUTOHALFOP,     "AUTOHALFOP",    4),
    DefEntry(CA_AUTOPROTECT,    "AUTOPROTECT",   10),
    DefEntry(CA_OPDEOPME,       "OPDEOPME",      5),
    DefEntry(CA_HALFOPME,       "HALFOPME",      4),
    DefEntry(CA_HALFOP,         "HALFOP",        5),
    DefEntry(CA_PROTECTME,      "PROTECTME",     10),
    DefEntry(CA_PROTECT,        "PROTECT",       ACCESS_INVALID),
    DefEntry(CA_KICKME,         "KICKME",        5),
    DefEntry(CA_KICK,           "KICK",          5),
    DefEntry(CA_SIGNKICK,       "SIGNKICK",      ACCESS_INVALID),
    DefEntry(CA_BANME,          "BANME",         5),
    DefEntry(CA_BAN,            "BAN",           5),
    DefEntry(CA_TOPIC,          "TOPIC",         ACCESS_INVALID),
    DefEntry(CA_INFO,           "INFO",          ACCESS_INVALID)
};

/*************************************************************************/

typedef std::map<std::string, DefEntry> levels_map;
levels_map levels_byname;

/*************************************************************************/

} /* anonymous namespace */

/*************************************************************************/

Access::Access() : databases::NameRecord(),
		m_owner(0), m_ng(0), m_level(ACCESS_INVALID), m_last_seen(0)
{
	
}

/*************************************************************************/

Access::Access(identifier_type id) : databases::NameRecord(id),
		m_owner(0), m_ng(0), m_level(ACCESS_INVALID), m_last_seen(0)
{
}

/*************************************************************************/

int32_t Access::get_i(Field const *field) const
{
	if (field == f_level)
		return level();
	else
		return NameRecord::get_i(field);
}

/*************************************************************************/

databases::Record::identifier_type Access::get_ri(Field const *field) const
{
	if (field == f_owner)
		return (m_owner ? m_owner->id() : 0);
	else if (field == f_group)
		return (m_ng ? m_ng->id() : 0);
	else
		return NameRecord::get_ri(field);
}

/*************************************************************************/

time_t Access::get_ut(Field const *field) const
{
	if (field == f_last_seen)
		return last_seen();
	else
		return NameRecord::get_ut(field);
}

/*************************************************************************/

time_t Access::last_seen(time_t last_seen)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept dates in the future */
		if (last_seen > time(0))
		{
			if (db_status() & dbsMember)
				return m_last_seen;
			
			last_seen = time(0);
		}
		
		if (m_last_seen && (last_seen < m_last_seen))
			return m_last_seen;
	}
	
	if (m_last_seen != last_seen)
	{
		if (((last_seen - m_last_seen) > als_sensibility) &&
				!accesses.change_ut(this, f_last_seen, last_seen))
			return m_last_seen;
		
		m_last_seen = last_seen;
	}
	
	return m_last_seen;
}

/*************************************************************************/

int16_t Access::level(int16_t level)
{
	if (db_status() & dbsUpdating)
	{
		/* Ensure the level is between -9999 and 9999 inclusive,
		 * but excluding zero. */
		if (level < -9999 || level > 9999 || level == 0)
		{
			/* If the access is already a member, return
			 * the previous value. */
			if (db_status() & dbsMember)
				return m_level;
			
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			
			/* Return the invalid value. */
			return level;
		}
	}
	
	if (m_level != level)
	{
		if (!accesses.change_i(this, f_level, level))
			return m_level;
		
		m_level = level;
	}
	
	return m_level;
}

/*************************************************************************/

size_t Access::memory_usage() const
{
	size_t result = NameRecord::memory_usage();
	
	result += sizeof(Access);
	
	return result;
}

/*************************************************************************/

char const *Access::name() const
{
	if (m_ng)
		return m_ng->name();
	else
		return 0;
}

/*************************************************************************/

NickGroup *Access::ng(NickGroup *ng)
{
	if (m_ng)
		return m_ng;
	
	return (m_ng = ng);
}

/*************************************************************************/

Record::identifier_type Access::ng_id(identifier_type ng_id)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	/* Don't allow changes. */
	if (m_ng)
		return m_ng->id();
	
	NickGroup *ng;
	
	/* If the group doesn't exist, make end_update() fail. */
	if (!ng_id || !(ng = (NickGroup *)groups.find(ng_id)))
	{
		epona->log(ErrorChanServList(), "Ignoring access entry #" RECORD_FORMAT
				" of channel %s with bad group #" RECORD_FORMAT, id(),
				m_owner ? m_owner->name() : "(unknown)", ng_id);
		
		db_status_add(dbsUpdateError);
		return ng_id;
	}
	
	this->ng(ng);
	return m_ng->id();
}

/*************************************************************************/

ChannelInfo *Access::owner(ChannelInfo *owner)
{
	if (m_owner)
		return m_owner;
	
	return (m_owner = owner);
}

/*************************************************************************/

databases::Record::identifier_type Access::owner_id(identifier_type owner_id)
{
	/* Assume (db_status() & dbsUpdating == true). */
	
	/* Don't allow owner changes. */
	if (m_owner)
		return m_owner->id();
	
	ChannelInfo *owner;
	
	/* Don't allow no ownership. */
	if (!owner_id || !(owner = (ChannelInfo *)channels.ifind(owner_id)))
	{
		epona->log(ErrorChanServList(), "Ignoring access entry #" RECORD_FORMAT
				" with bad owner #" RECORD_FORMAT, id(), owner_id);
		
		/* Update error. */
		db_status_add(dbsUpdateError);
		/* Don't need to change the value, the record will
		 * be deleted. */
		return owner_id;
	}
	
	m_owner = owner;
	return m_owner->id();
}

/*************************************************************************/

int32_t Access::set_i(Field const *field, int32_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_i(field, value);
	
	if (field == f_level)
		return level(value);
	else
		return NameRecord::set_i(field, value);
}

/*************************************************************************/

Record::identifier_type Access::set_ri(Field const *field,
		identifier_type value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ri(field, value);
	
	if (field == f_owner)
		return owner_id(value);
	else if (field == f_group)
		return ng_id(value);
	else
		return NameRecord::set_ri(field, value);
}

/*************************************************************************/

time_t Access::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ut(field, value);
	
	if (field == f_last_seen)
		return last_seen(value);
	else
		return NameRecord::set_ut(field, value);
}

/*************************************************************************/

bool Accesses::add(Record *record, int *err)
{
	Access *acc = (Access *)record;
	
	ChannelInfo::accesses_type::const_iterator it, end;
	
	for (it = acc->owner()->accesses().begin(),
			end = acc->owner()->accesses().end(); it != end; ++it)
	{
		Access const *acc2 = (Access const *)(*it);
		
		if (!acc2)
			continue;
		
		if (acc->ng() == acc2->ng())
		{
			if (err)
				*err = ceName;
			return false;
		}
	}
	
	if (!Collection::add(record, err))
		return false;
	
	return true;
}

/*************************************************************************/

bool Accesses::add_list(Record *record)
{
	Access *acc = (Access *)record;
	
	ChannelInfo::accesses_type::iterator it, end;
	
	for (it = acc->owner()->accesses().begin(),
			end = acc->owner()->accesses().end(); it != end; ++it)
	{
		if (*it == 0)
		{
			*it = acc;
			break;
		}
	}

	if (it == acc->owner()->accesses().end())
		it = acc->owner()->accesses().insert(it, acc);
	
	if (!Collection::add_list(record))
	{
		*it = 0;
		return false;
	}

	return true;
}

/*************************************************************************/

Record *Accesses::create(Record::identifier_type id) const
{
	if (id)
		return new Access(id);
	else
		return new Access;
}

/*************************************************************************/

databases::Field const *Accesses::owner_field(Collection const *coll) const
{
	if (coll == &channels)
		return Access::f_owner;
	else if (coll == &groups)
		return Access::f_group;
	else
		return 0;
}

/*************************************************************************/

void Accesses::remove_list(Record *record)
{
	Access *acc = (Access *)record;
	
	ChannelInfo::accesses_type::iterator it, end;
	
	for (it = acc->owner()->accesses().begin(),
			end = acc->owner()->accesses().end(); it != end; ++it)
	{
		if (acc == (*it))
		{
			*it = 0;
			break;
		}
	}
	
	Collection::remove_list(record);
}

/*************************************************************************/

bool Accesses::remove_owned_db(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels && coll != &groups)
		return true;
	
	if (coll == &channels)
	{
		ChannelInfo *ci = (ChannelInfo *)record;
	
		if (ci->accesses().empty())
			return true;
	}
	
	return Collection::remove_owned_db(coll, record, err);
}

/*************************************************************************/

bool Accesses::remove_owned_list(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll == &channels)
	{
		ChannelInfo *ci = (ChannelInfo *)record;
		ChannelInfo::accesses_type::iterator it, end;
		
		for (it = ci->accesses().begin(), end = ci->accesses().end();
				it != end; ++it)
		{
			Access *acc = (Access *)(*it);
			
			if (acc)
			{
				acc->db_status_remove(Record::dbsInsert);
			
				if (!remove(acc, err))
					return false;
			}
		}
	}
	
	if (coll == &groups)
	{
		NickGroup *ng = (NickGroup *)record;
		
		Channels::const_iterator chit, chend;
		
		for (chit = channels.begin(), chend = channels.end();
				chit != chend; ++chit)
		{
			ChannelInfo *ci = (ChannelInfo *) (*chit);
			ChannelInfo::accesses_type::iterator it, end;
		
			for (it = ci->accesses().begin(), end = ci->accesses().end();
					it != end; ++it)
			{
				Access *acc = (Access *)(*it);
				
				if (acc && acc->ng() == ng)
				{
					acc->db_status_remove(Record::dbsInsert);
				
					if (!remove(acc, err))
						return false;
				}
			}
		}
	}
	
	return true;
}

/*************************************************************************/

void Accesses::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("chanserv_access");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	Access::f_owner = db->field_create("owner", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 1);
	Access::f_group = db->field_create("group", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 2);
	Access::f_level = db->field_create("level", 1, Field::ftInteger,
			2, 0, "0", 3);
	Access::f_last_seen = db->field_create("last_seen", 1, Field::ftUnixTime,
			0, 0, "0", 4);
	
	if (!db->field_add(Access::f_owner) ||
			!db->field_add(Access::f_group) ||
			!db->field_add(Access::f_level) ||
			!db->field_add(Access::f_last_seen))
		epona->fatal("Can't add fields to database chanserv_access");
	
	this->db(db);
}

/*************************************************************************/

void Accesses::setup_owner()
{
	channels.set_owned(*this);
	groups.set_owned(*this);
}

/*************************************************************************/

Akick::Akick() : databases::NameRecord(),
		m_owner(0), m_flags(0), m_mask(), m_ng(0), m_creator(),
		m_reason(), m_since(time(0)), m_expires(0)
{
}

/*************************************************************************/

Akick::Akick(identifier_type id) : databases::NameRecord(id),
		m_owner(0), m_flags(0), m_mask(), m_ng(0), m_creator(),
		m_reason(), m_since(time(0)), m_expires(0)
{
}

/*************************************************************************/

char const *Akick::creator(char const *creator)
{
	/* We accept NULL creators for HISTORICAL reasons. */
	
	if (m_creator.compare(creator, creator_len) != 0)
	{
		misc::cstring tcreator(creator, creator_len);
			
		if (!akicks.change_s(this, f_creator, tcreator))
			return m_creator;
		
		m_creator.swap(tcreator);
	}
	
	return m_creator;
}

/*************************************************************************/

bool Akick::end_update()
{
	if (!NameRecord::end_update())
		return false;
	
	if ((m_flags & afIsGroup) && !m_ng)
	{
		epona->log(InfoChanServList(), "Ignoring bad group akick "
				RECORD_FORMAT " of channel %s with no group "
				"information", id(), (m_owner ? m_owner->name() : "(unknown)"));
		return false;
	}
	
	if (!(m_flags & afIsGroup) && !m_mask)
	{
		epona->log(InfoChanServList(), "Ignoring bad mask akick "
				RECORD_FORMAT " of channel %s with no mask",
				id(), (m_owner ? m_owner->name() : "(unknown)"));
		return false;
	}
	
	if ((m_flags & afIsGroup) && m_mask)
	{
		/* Try to change this, but since it's not that essential,
		 * don't generate an error if it fails. */
		akicks.change_s(this, f_mask, 0);
		m_mask = 0;
	}
	
	if (!(m_flags & afIsGroup) && m_ng)
	{
		/* Try to change this, but since it's not that essential,
		 * don't generate an error if it fails. */
		akicks.change_ri(this, f_group, 0);
		m_ng = 0;
	}
	
	return true;
}

/*************************************************************************/

time_t Akick::expires(time_t expires)
{
	if (db_status() & dbsUpdating)
	{
		/* Reject exceptions with expiration time in the past. They have no
		 * sense. */
		if (expires != 0 && (time(0) >= expires))
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_expires;
			/* Else accept the value anyway, but make end_update() fail
			 * so that the record is deleted. */
			db_status_add(dbsUpdateError);
			
			epona->log(InfoChanServList(),
					"Ignoring akick %s (#" RECORD_FORMAT ") with expiration"
					" time in the past", (name() ? name() : "<unknown>"),
					id());
		}
	}
	
	if (m_expires != expires) 
	{
		if (!akicks.change_ut(this, f_expires, expires))
			return m_expires;
		m_expires = expires;
	}

	return m_expires;
}

/*************************************************************************/

uint8_t Akick::flags(uint8_t flags)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	flags &= ~afUnused;
	
	if (db_status() & dbsMember)
	{
		if (m_flags & afIsGroup)
			flags |= afIsGroup;
		else
			flags &= ~afIsGroup;
	}
	
	if ((flags & afIsGroup) && (flags & afStuck))
		flags &= ~afStuck;
	
	if ((db_status() & dbsMember) &&
			!(m_flags & afStuck) && (flags & afStuck))
		stick();
	
	m_flags = flags;
	
	return m_flags;
}

/*************************************************************************/

uint8_t Akick::flags_add(uint8_t flags)
{
	if ((m_flags & flags) != flags) 
	{
		uint8_t tflags = (m_flags | flags);
		
		if (!akicks.change_ui(this, f_flags, tflags))
			return m_flags;
		
		if ((db_status() & dbsMember) && !(m_flags & afStuck) &&
				(flags & afStuck))
			stick();
		
		m_flags = tflags;
	}
	
	return m_flags;
}

/*************************************************************************/

uint8_t Akick::flags_remove(uint8_t flags)
{
	if ((m_flags & flags) != 0) 
	{
		uint8_t tflags = (m_flags & ~flags);
		
		if (!akicks.change_ui(this, f_flags, tflags))
			return m_flags;
		
		m_flags = tflags;
	}
	
	return m_flags;
}

/*************************************************************************/

databases::Record::identifier_type Akick::get_ri(Field const *field) const
{
	if (field == f_owner)
		return (m_owner ? m_owner->id() : 0);
	else if (field == f_group)
		return (m_ng ? m_ng->id() : 0);
	else
		return NameRecord::get_ri(field);
}

/*************************************************************************/

char const *Akick::get_s(Field const *field) const
{
	if (field == f_mask)
		return mask();
	else if (field == f_creator)
		return creator();
	else if (field == f_reason)
		return reason();
	else
		return NameRecord::get_s(field);
}

/*************************************************************************/

uint32_t Akick::get_ui(Field const *field) const
{
	if (field == f_flags)
		return flags();
	else
		return NameRecord::get_ui(field);
}

/*************************************************************************/

time_t Akick::get_ut(Field const *field) const
{
	if (field == f_since)
		return since();
	else if (field == f_expires)
		return expires();
	else
		return NameRecord::get_ut(field);
}

/*************************************************************************/

char const *Akick::mask(char const *mask)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow mask changes if we're already a member
		 * (except for case changes) */
		if (db_status() & dbsMember)
		{
			if (!m_mask || stricmp(m_mask, mask) != 0)
				return m_mask;
		}
	}
	
	if (m_mask.compare(mask, mask_len) != 0)
	{
		misc::cstring tmask(mask, mask_len);
			
		if (!akicks.change_s(this, f_mask, tmask))
			return m_mask;
		
		m_mask.swap(tmask);
	}
	
	return m_mask;
}

/*************************************************************************/

size_t Akick::memory_usage() const
{
	size_t result = NameRecord::memory_usage();
	
	result += sizeof(Akick);
	
	if (m_mask)
		result += strlen(m_mask) + 1;
	if (m_creator)
		result += strlen(m_creator) + 1;
	if (m_reason)
		result += strlen(m_reason) + 1;
	
	return result;
}

/*************************************************************************/

char const *Akick::name() const
{
	if ((m_flags & Akick::afIsGroup) && m_ng)
		return m_ng->name();
	else
		return m_mask;
}

/*************************************************************************/

NickGroup *Akick::ng(NickGroup *ng)
{
	if (db_status() & dbsMember)
		return m_ng;
	
	return (m_ng = ng);
}

/*************************************************************************/

Record::identifier_type Akick::ng_id(identifier_type ng_id)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	/* Don't allow changes. */
	if (db_status() & dbsMember)
		return m_ng->id();
	
	NickGroup *ng = (ng_id ? (NickGroup *)groups.find(ng_id) : 0);
	
	this->ng(ng);
	return (m_ng ? m_ng->id() : 0);
}

/*************************************************************************/

ChannelInfo *Akick::owner(ChannelInfo *owner)
{
	if (m_owner)
		return m_owner;
	
	return (m_owner = owner);
}

/*************************************************************************/

databases::Record::identifier_type Akick::owner_id(identifier_type owner_id)
{
	/* Assume (db_status() & dbsUpdating == true). */
	
	/* Don't allow owner changes. */
	if (m_owner)
		return m_owner->id();
	
	ChannelInfo *owner;
	
	/* Don't allow no ownership. */
	if (!owner_id || !(owner = (ChannelInfo *)channels.ifind(owner_id)))
	{
		epona->log(ErrorChanServList(), "Ignoring access entry #" RECORD_FORMAT
				" with bad owner #" RECORD_FORMAT, id(), owner_id);
		
		/* Update error. */
		db_status_add(dbsUpdateError);
		/* Don't need to change the value, the record will
		 * be deleted. */
		return owner_id;
	}
	
	m_owner = owner;
	return m_owner->id();
}

/*************************************************************************/

char const *Akick::reason(char const *reason)
{
	/* NULL reasons are accepted. */
	
	if (m_reason.compare(reason, reason_len) != 0)
	{
		misc::cstring treason(reason, reason_len);
			
		if (!akicks.change_s(this, f_reason, treason))
			return m_reason;
		
		m_reason.swap(treason);
	}
	
	return m_reason;
}

/*************************************************************************/

Record::identifier_type Akick::set_ri(Field const *field,
		identifier_type value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ri(field, value);
	
	if (field == f_owner)
		return owner_id(value);
	else if (field == f_group)
		return ng_id(value);
	else
		return NameRecord::set_ri(field, value);
}

/*************************************************************************/

char const *Akick::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_s(field, value);
	
	if (field == f_mask)
		return mask(value);
	else if (field == f_creator)
		return creator(value);
	else if (field == f_reason)
		return reason(value);
	else
		return NameRecord::set_s(field, value);
}

/*************************************************************************/

uint32_t Akick::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ui(field, value);
	
	if (field == f_flags)
		return flags(value);
	else
		return NameRecord::set_ui(field, value);
}

/*************************************************************************/

time_t Akick::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ut(field, value);
	
	if (field == f_since)
		return since(value);
	else if (field == f_expires)
		return expires(value);
	else
		return NameRecord::set_ut(field, value);
}

/*************************************************************************/

time_t Akick::since(time_t since)
{
	if (db_status() & dbsUpdating)
	{
		/* We don't allow time in the future. */
		if (since > time(0))
		{
			/* If it's already a member, just return the old time. */
			if (db_status() & dbsMember)
				return m_since;
			/* Else, assign a correct time. */
			since = time(0);
		}
	}
	
	if (m_since != since)
	{
		if (!akicks.change_ut(this, f_since, since))
			return m_since;
		m_since = since;
	}
	
	return m_since;
}

/*************************************************************************/

void Akick::stick()
{
	int i;
	char const *av[2];
	
	if (!m_owner->c)
		return;
	
	for (i = 0; i < m_owner->c->bancount; i++) 
	{
		/* If akick is already covered by a wider ban.
		   Example: c->bans[i] = *!*@*.org and mask = *!*@*.epona.org */
		if (match_wild_nocase(m_owner->c->bans[i], m_mask))
			return;
		
#ifdef IRC_DREAMFORGE
        /* If akick is wider than a ban already in place.
		   Example: c->bans[i] = *!*@irc.epona.org and mask = *!*@*.epona.org */
		if (match_wild_nocase(m_mask, m_owner->c->bans[i]))
			return;
#endif
	}

	/* Falling there means we can set the ban */
	
	av[0] = "+b";
	av[1] = m_mask;
	s_mode(whosends(m_owner), m_owner->c->name, "+b", m_mask);
	chan_set_modes(s_ChanServ, m_owner->c, 2, av, 1);
}

/*************************************************************************/

bool Akicks::add(Record *record, int *err)
{
	Akick *ak = (Akick *)record;
	
	ChannelInfo::akicks_type::const_iterator it, end;
	
	for (it = ak->owner()->akicks().begin(),
			end = ak->owner()->akicks().end(); it != end; ++it)
	{
		Akick const *ak2 = (Akick const *)(*it);
		
		if (!ak2)
			continue;
		
		if ((ak->flags() & Akick::afIsGroup) && ak->ng() == ak2->ng())
		{
			if (err)
				*err = ceName;
			return false;
		}
		
		if (!(ak->flags() & Akick::afIsGroup) && *ak == ak2->name())
		{
			if (err)
				*err = ceName;
			return false;
		}
	}
	
	if (!Collection::add(record, err))
		return false;
	
	if (ak->flags() & Akick::afStuck)
		ak->stick();
	
	return true;
}

/*************************************************************************/

bool Akicks::add_list(Record *record)
{
	Akick *ak = (Akick *)record;
	
	ChannelInfo::akicks_type::iterator it, end;
	
	for (it = ak->owner()->akicks().begin(),
			end = ak->owner()->akicks().end(); it != end; ++it)
	{
		if (*it == 0)
		{
			*it = ak;
			break;
		}
	}

	if (it == ak->owner()->akicks().end())
		it = ak->owner()->akicks().insert(it, ak);
	
	if (!Collection::add_list(record))
	{
		*it = 0;
		return false;
	}

	return true;
}

/*************************************************************************/

Record *Akicks::create(Record::identifier_type id) const
{
	if (id)
		return new Akick(id);
	else
		return new Akick;
}

/*************************************************************************/

databases::Field const *Akicks::owner_field(Collection const *coll) const
{
	if (coll == &channels)
		return Akick::f_owner;
	else if (coll == &groups)
		return Akick::f_group;
	else
		return 0;
}

/*************************************************************************/

void Akicks::remove_list(Record *record)
{
	Akick *ak = (Akick *)record;
	
	ChannelInfo::akicks_type::iterator it, end;
	
	for (it = ak->owner()->akicks().begin(),
			end = ak->owner()->akicks().end(); it != end; ++it)
	{
		if (ak == (*it))
		{
			*it = 0;
			break;
		}
	}
	
	Collection::remove_list(record);
}

/*************************************************************************/

bool Akicks::remove_owned_db(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels && coll != &groups)
		return true;
	
	if (coll == &channels)
	{
		ChannelInfo *ci = (ChannelInfo *)record;
	
		if (ci->akicks().empty())
			return true;
	}
	
	return Collection::remove_owned_db(coll, record, err);
}

/*************************************************************************/

bool Akicks::remove_owned_list(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll == &channels)
	{
		ChannelInfo *ci = (ChannelInfo *)record;
		ChannelInfo::akicks_type::iterator it, end;
		
		for (it = ci->akicks().begin(), end = ci->akicks().end();
				it != end; ++it)
		{
			Akick *ak = (Akick *)(*it);
			
			if (ak)
			{
				ak->db_status_remove(Record::dbsInsert);
			
				if (!remove(ak, err))
					return false;
			}
		}
	}
	
	if (coll == &groups)
	{
		NickGroup *ng = (NickGroup *)record;
		
		Channels::const_iterator chit, chend;
		
		for (chit = channels.begin(), chend = channels.end();
				chit != chend; ++chit)
		{
			ChannelInfo *ci = (ChannelInfo *) (*chit);
			ChannelInfo::akicks_type::iterator it, end;
		
			for (it = ci->akicks().begin(), end = ci->akicks().end();
					it != end; ++it)
			{
				Akick *ak = (Akick *)(*it);
				
				if (ak && (ak->flags() & Akick::afIsGroup) && ak->ng() == ng)
				{
					ak->db_status_remove(Record::dbsInsert);
				
					if (!remove(ak, err))
						return false;
				}
			}
		}
	}
	
	return true;
}

/*************************************************************************/

void Akicks::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("chanserv_akicks");
	db->version(Database::vtExport, 2);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	Akick::f_owner = db->field_create("owner", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 1);
	Akick::f_flags = db->field_create("flags", 1, Field::ftUnsigned,
			1, 0, "0", 2);
	Akick::f_mask = db->field_create("mask", 1, Field::ftString,
			Akick::mask_len, 0, 0, 3);
	Akick::f_group = db->field_create("group", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 4);
	Akick::f_creator = db->field_create("creator", 1, Field::ftString,
			Akick::creator_len, 0, 0, 5);
	Akick::f_reason = db->field_create("reason", 1, Field::ftString,
			Akick::reason_len, 0, 0, 6);
	Akick::f_since = db->field_create("since", 1, Field::ftUnixTime,
			0, 0, "0", 7);
	Akick::f_expires = db->field_create("expires", 2, Field::ftUnixTime,
			0, 0, "0", 8);
	
	if (!db->field_add(Akick::f_owner) ||
			!db->field_add(Akick::f_flags) ||
			!db->field_add(Akick::f_mask) ||
			!db->field_add(Akick::f_group) ||
			!db->field_add(Akick::f_creator) ||
			!db->field_add(Akick::f_reason) ||
			!db->field_add(Akick::f_since) ||
			!db->field_add(Akick::f_expires))
		epona->fatal("Can't add fields to database chanserv_akicks");
	
	this->db(db);
}

/*************************************************************************/

void Akicks::setup_owner()
{
	channels.set_owned(*this);
	groups.set_owned(*this);
}

/*************************************************************************/

ChannelInfo::ChannelInfo() : databases::NameRecord(),
		c(0), m_name(), m_owner(0), m_successor(0),
		m_created(time(0)), m_last_used(0), m_enc(), m_pass(),
		m_flags(cfDefaults | epona->cd().chanserv.default_options), m_desc(),
		m_email(), m_url(), m_entry_message(), m_bantype(0), m_mlock_modes(),
		m_mlock_data(), m_topic(), m_topic_setter(), m_topic_time(0),
		m_levels(), m_accesses(), m_akicks()
		, m_bi(0), m_botflags(0), m_caps_min(10), m_caps_percent(25),
		m_flood_lines(6), m_flood_secs(10), m_repeats(3), m_kbb(),
		m_badwords()
{
}

/*************************************************************************/

ChannelInfo::ChannelInfo(identifier_type id) : databases::NameRecord(id),
		c(0), m_name(), m_owner(0), m_successor(0),
		m_created(time(0)), m_last_used(0), m_enc(), m_pass(),
		m_flags(cfDefaults | epona->cd().chanserv.default_options), m_desc(),
		m_email(), m_url(), m_entry_message(), m_bantype(0), m_mlock_modes(),
		m_mlock_data(), m_topic(), m_topic_setter(), m_topic_time(0),
		m_levels(), m_accesses(), m_akicks()
		, m_bi(0), m_botflags(0), m_caps_min(0), m_caps_percent(0),
		m_flood_lines(0), m_flood_secs(0), m_repeats(0), m_kbb(),
		m_badwords()
{
}

/*************************************************************************/

ChannelInfo::~ChannelInfo()
{
	channels.release_encrypter(m_enc);
}

/*************************************************************************/

uint8_t ChannelInfo::bantype(uint8_t bantype)
{
	if (db_status() & dbsUpdating)
	{
		if (bantype > 3)
			bantype = epona->cd().chanserv.default_bantype;
	}
	
	if (m_bantype != bantype) 
	{
		if (!channels.change_ui(this, f_bantype, bantype))
			return m_bantype;
		
		m_bantype = bantype;
	}
	
	return m_bantype;
}

/*************************************************************************/

BotInfo *ChannelInfo::bi(BotInfo *bi)
{
	if (m_bi != bi)
	{
		if (!channels.change_ri(this, f_bot, (bi ? bi->id() : 0)))
			return m_bi;
		
		if (m_bi)
		{
			if (db_status() & dbsMember)
			{
				if (c && c->usercount >= epona->cd().botserv.min_users)
					send_cmd(m_bi->nick(), "PART %s", c->name);
		
				m_bi->chancount(m_bi->chancount() - 1);
			}
		}
		
		m_bi = bi;
		
		if (m_bi)
		{
			if (db_status() & dbsMember)
			{
				m_bi->chancount(m_bi->chancount() + 1);
			
				if (c && c->usercount >= epona->cd().botserv.min_users)
					bot_join(this);
			}
		}
	}
	
	return m_bi;
}

/*************************************************************************/

Record::identifier_type ChannelInfo::bot_id(identifier_type bot_id)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	BotInfo *bi = 0;
	
	if (bot_id && !(bi = (BotInfo *)bots.ifind(bot_id)))
		return (m_bi ? m_bi->id() : 0);
	
	this->bi(bi);
	
	return (m_bi ? m_bi->id() : 0);
}

/*************************************************************************/

uint32_t ChannelInfo::botflags(uint32_t botflags)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	botflags &= ~bfUnused;
	
	if (botflags & bfDefaults)
		botflags = (bfDefaults | epona->cd().botserv.default_options);
	
	if (m_botflags != botflags)
		m_botflags = botflags;
	
	return m_botflags;
}

/*************************************************************************/

uint32_t ChannelInfo::botflags_add(uint32_t botflags)
{
	if ((m_botflags & botflags) != botflags) 
	{
		uint32_t tbotflags = (m_botflags | botflags);
		tbotflags &= ~bfDefaults;
		
		if (!channels.change_ui(this, f_botflags, tbotflags))
			return m_botflags;
		
		m_botflags = tbotflags;
	}
	
	return m_botflags;
}

/*************************************************************************/

uint32_t ChannelInfo::botflags_remove(uint32_t botflags)
{
	if ((m_botflags & botflags) != 0) 
	{
		uint32_t tbotflags = (m_botflags & ~(botflags | bfDefaults));
		
		if (!channels.change_ui(this, f_botflags, tbotflags))
			return m_botflags;
		
		m_botflags = tbotflags;
	}
	
	return m_botflags;
}

/*************************************************************************/

uint8_t ChannelInfo::caps_min(uint8_t caps_min)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept invalid values */
		if (caps_min == 0)
		{
			/* If it's already a member, return the previous value. */
			if (db_status() & dbsMember)
				return m_caps_min;
			/* Else, set it to the default value */
			caps_min = 10;
		}
	}
	
	if (m_caps_min != caps_min) 
	{
		if (!channels.change_ui(this, f_caps_min, caps_min))
			return m_caps_min;
		
		m_caps_min = caps_min;
	}
	
	return m_caps_min;
}

/*************************************************************************/

uint8_t ChannelInfo::caps_percent(uint8_t caps_percent)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept invalid values */
		if (caps_percent == 0 || caps_percent > 100)
		{
			/* If it's already a member, return the previous value. */
			if (db_status() & dbsMember)
				return m_caps_percent;
			/* Else, set it to the default value */
			caps_percent = 25;
		}
	}
	
	if (m_caps_percent != caps_percent) 
	{
		if (!channels.change_ui(this, f_caps_percent, caps_percent))
			return m_caps_percent;
		
		m_caps_percent = caps_percent;
	}
	
	return m_caps_percent;
}

/*************************************************************************/

bool ChannelInfo::change_caps(uint8_t caps_min, uint8_t caps_percent)
{
	if (m_caps_min != caps_min && m_caps_percent != caps_percent) 
	{
		if (!channels.change_multi(this, 2, f_caps_min, caps_min,
				f_caps_percent, caps_percent))
			return false;
		
		m_caps_min = caps_min;
		m_caps_percent = caps_percent;
	} else if (m_caps_min != caps_min) {
		this->caps_min(caps_min);
		
		if (m_caps_min != caps_min)
			return false;
	} else if (m_caps_percent != caps_percent) {
 		this->caps_percent(caps_percent);
 		
 		if (m_caps_percent != caps_percent)
			return false;
 	}
 	
 	return true;
}

/*************************************************************************/

bool ChannelInfo::change_flood(uint8_t flood_lines, uint8_t flood_secs)
{
	if (m_flood_lines != flood_lines && m_flood_secs != flood_secs) 
	{
		if (!channels.change_multi(this, 2, f_flood_lines, flood_lines,
				f_flood_secs, flood_secs))
			return false;
		
		m_flood_lines = flood_lines;
		m_flood_secs = flood_secs;
	} else if (m_flood_lines != flood_lines) {
		this->flood_lines(flood_lines);
		
		if (m_flood_lines != flood_lines)
			return false;
	} else if (m_flood_secs != flood_secs) {
 		this->flood_secs(flood_secs);
 		
 		if (m_flood_secs != flood_secs)
			return false;
 	}
 	
 	return true;
}

/*************************************************************************/

bool ChannelInfo::change_password(char const *pass)
{
	if (!pass)
		return false;
	
	if (channels.encrypter())
	{
		/* Make sure we can store the password. */
		if (channels.encrypter()->length() > pass_len)
		{
			epona->log(ErrorChanServ(),
					"Encrypted passwords of module %s are too long "
					"(%u characters, max %u)",
					channels.encrypter()->module()->name().c_str(),
					channels.encrypter()->length(), pass_len);
			return false;
		}
		
		misc::cstring tpass;
		tpass.reserve(channels.encrypter()->length() * 2 + 1);
		
		if (!channels.encrypter()->encrypt(pass, tpass,
				channels.encrypter()->length() + 1))
		{
			epona->log(ErrorChanServ(), "Can't encrypt password of "
					"channel %s", (m_name ? m_name.str() : ""));
			return false;
		}
				
		if (m_pass.compare(tpass, pass_len) != 0)
		{
			if (!channels.change_multi(this, 2, f_enc, 
					channels.encrypter()->module()->name().c_str(),
					f_pass, tpass.str()))
				return false;
			
			if (m_enc != channels.encrypter())
			{
				channels.release_encrypter(m_enc);
				m_enc = channels.encrypter();
				m_enc->module()->references_add();
			}
			
			m_pass.swap(tpass);
		}
	} else
	{
		if (m_pass.compare(pass, pass_len) != 0)
		{
			misc::cstring tpass(pass, pass_len);
		
			if (!channels.change_multi(this, 2, f_enc, 0,
					f_pass, tpass.str()))
				return false;
			
			if (m_enc)
			{
				channels.release_encrypter(m_enc);
				m_enc = 0;
			}
			
			m_pass.swap(tpass);
		}
	}
	
	return true;
}

/*************************************************************************/

bool ChannelInfo::change_topic(char const *topic, char const *topic_setter,
		time_t topic_time)
{
	if (m_topic.compare(topic, topic_len) != 0 ||
		m_topic_setter.compare(topic_setter, topic_setter_len) != 0 ||
		m_topic_time != topic_time)
	{
		misc::cstring ttopic(topic, topic_len);
		misc::cstring ttopic_setter(topic_setter, topic_setter_len);
		
		if (!channels.change_multi(this, 3, f_topic, ttopic.str(),
				f_topic_setter, ttopic_setter.str(), f_topic_time, topic_time))
			return false;
		
		m_topic.swap(ttopic);
		m_topic_setter.swap(ttopic_setter);
		m_topic_time = topic_time;
	}
	
	return true;
}

/*************************************************************************/

bool ChannelInfo::check_password(char const *pass)
{
	/* Paranoia */
	if (!m_pass)
		return false;
	
	bool result;
	
	if (m_enc)
		result = m_enc->check(pass, m_pass);
	else
		result = (m_pass.compare(pass, pass_rlen) == 0);
	
	if (result && m_enc != channels.encrypter())
	{
		epona->log(InfoChanServ(), "Changing password encryption scheme "
				"of channel %s to %s", (m_name ? m_name.str() : ""),
				(channels.encrypter() ?
				 channels.encrypter()->module()->name().c_str() : "cleartext"));
		change_password(pass);
	}
	
	return result;
}

/*************************************************************************/

bool ChannelInfo::check_topiclock() const
{
	if (!c || !(m_flags & cfTopicLock))
		return false;
	
	if ((m_topic != c->topic) || (m_topic_setter != c->topic_setter) ||
			(m_topic_time != c->topic_time))
		set_topic(whosends(this), c, m_topic.str(),
    			(m_topic_setter ? m_topic_setter.str() : whosends(this)),
    			m_topic_time);
    
    return true;
}

/*************************************************************************/

void ChannelInfo::clear_bot()
{
	/* No need to send a PART, as the bot will soon leave. */
	m_bi = 0;
}

/*************************************************************************/

void ChannelInfo::clear_successor()
{
	m_successor = 0;
}

/*************************************************************************/

time_t ChannelInfo::created(time_t created)
{
	if (db_status() & dbsUpdating)
	{
		/* We don't allow time in the future. */
		if (created > time(0))
		{
			/* If it's already a member, just return the old time. */
			if (db_status() & dbsMember)
				return m_created;
			/* Else, assign a correct time. */
			created = time(0);
		}
	}
	
	if (m_created != created)
	{
		if (!channels.change_ut(this, f_created, created))
			return m_created;
		
		m_created = created;
	}
	
	return m_created;
}

/*************************************************************************/

char const *ChannelInfo::desc(char const *desc)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL descriptions. */
		if (!desc)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_desc;
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL description... =) */
			return m_desc;
		}
	}
	
	if (m_desc.compare(desc, desc_len) != 0)
	{	
		misc::cstring tdesc(desc, desc_len);
		
		if (!channels.change_s(this, f_desc, tdesc))
			return m_desc;
		
		m_desc.swap(tdesc);
	}
	
	return m_desc;
}

/*************************************************************************/

char const *ChannelInfo::email(char const *email)
{
	if (m_email.compare(email, email_len) != 0)
	{	
		misc::cstring temail(email, email_len);
		
		if (!channels.change_s(this, f_email, temail))
			return m_email;
		
		m_email.swap(temail);
	}
	
	return m_email;
}

/*************************************************************************/

char const *ChannelInfo::enc() const
{
	return (m_enc ? m_enc->module()->name().c_str() : 0);
}

/*************************************************************************/

char const *ChannelInfo::enc(char const *enc_)
{
	if (!enc_)
	{
		if (m_enc)
		{
			channels.release_encrypter(m_enc);
			m_enc = 0;
		}
	} else 
	{
		if (!m_enc || strcmp(m_enc->module()->name().c_str(), enc_))
		{
			password::Encrypter *enc = channels.load_encrypter(enc_);
		
			if (!enc)
				epona->fatalf("Can't load password module %s required by "
						" channel %s", enc_,
						(m_name ? m_name.str() : ""));
		
			channels.release_encrypter(m_enc);
			m_enc = enc;
		}
	}
	
	return this->enc();
}

/*************************************************************************/

bool ChannelInfo::end_update()
{
	if (!NameRecord::end_update())
		return false;
	
	/* Don't allow same owner and successor */
	if (m_successor && (m_owner == m_successor))
	{
		channels.change_ri(this, f_successor, 0);
		m_successor = 0;
	}
	
	/* Clear the pending expire if needed */
	if ((m_flags & cfPendingExpire) &&
			(time(0) - last_used()) < epona->cd().chanserv.expire.delay)
	{
		if (channels.change_ui(this, f_flags, (m_flags & ~cfPendingExpire)))
			m_flags &= ~cfPendingExpire;
	}

	/* If the IRCd changed, update mlock info */
	if (db_status() & dbsIRCdChange)
	{
		if (channels.change_multi(this, 2, f_mlock_ircd, IRCD_NAME,
				f_mlock_modes, mlock_modes()))
			db_status_remove(dbsIRCdChange);
	}
	
	/* Make sure the topic is valid */
	if (m_topic && (!m_topic_setter || !m_topic_time))
	{
		epona->log(InfoChanServList(), "Ignoring bad topic information "
				"for channel %s", name());
		change_topic(0, 0, 0);
		
		if (db_status() & dbsMember)
			db_status_add(dbsTopicChange);
	}
	
	/* If the topic changed, update the topic on the channel */
	if (db_status() & dbsTopicChange)
	{
		if (c)
			set_topic(whosends(this), c, m_topic.str(),
					(m_topic_setter ? m_topic_setter.str() : whosends(this)),
					m_topic_time);
		db_status_remove(dbsTopicChange);
	}
	
	return true;
}

/*************************************************************************/

char const *ChannelInfo::entry_message(char const *entry_message)
{	
	if (m_entry_message.compare(entry_message, entry_message_len) != 0)
	{	
		misc::cstring tentry_message(entry_message, entry_message_len);
		
		if (!channels.change_s(this, f_entry_message, tentry_message))
			return m_entry_message;
		
		m_entry_message.swap(tentry_message);
	}
	
	return m_entry_message;
}

/*************************************************************************/

uint32_t ChannelInfo::flags(uint32_t flags)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	if ((flags & cfPendingExpire) &&
			(!epona->cd().chanserv.expire || (flags & cfNoExpire)))
		flags &= ~cfPendingExpire;
	
	flags &= ~(cfTemporary | cfUnused);
	
	if (flags & cfDefaults)
		flags = (cfDefaults | epona->cd().chanserv.default_options);
	
	if ((m_flags & ~cfTemporary) != flags)
	{
		uint32_t tflags = (m_flags & cfTemporary);
		
		m_flags = flags;
		m_flags |= tflags;
	}
	
	return m_flags;
}

/*************************************************************************/

uint32_t ChannelInfo::flags_add(uint32_t flags)
{
	if ((m_flags & flags) != flags) 
	{
		if ((flags & cfTemporary) != flags)
		{
			uint32_t tflags = ((m_flags | flags) & ~(cfTemporary | cfDefaults));
		
			if (flags & cfNoExpire)
				tflags &= ~cfPendingExpire;
		
			if (!channels.change_ui(this, f_flags, tflags))
				return m_flags;
		}
		
		m_flags |= flags;
		
		m_flags &= ~cfDefaults;
		if (flags & cfNoExpire)
			m_flags &= ~cfPendingExpire;
	}
	
	return m_flags;
}

/*************************************************************************/

uint32_t ChannelInfo::flags_remove(uint32_t flags)
{
	if ((m_flags & flags) != 0) 
	{
		if ((flags & cfTemporary) != flags)
		{
			uint32_t tflags = (m_flags & ~cfTemporary);
			tflags &= ~flags;
			
			if (!channels.change_ui(this, f_flags, tflags))
				return m_flags;
		}
		
		m_flags &= ~(flags | cfDefaults);
	}
	
	return m_flags;
}

/*************************************************************************/

uint8_t ChannelInfo::flood_lines(uint8_t flood_lines)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept invalid values */
		if (flood_lines < 2)
		{
			/* If it's already a member, return the previous value. */
			if (db_status() & dbsMember)
				return m_flood_lines;
			/* Else, set it to the default value */
			flood_lines = 6;
		}
	}
	
	if (m_flood_lines != flood_lines) 
	{
		if (!channels.change_ui(this, f_flood_lines, flood_lines))
			return m_flood_lines;
		
		m_flood_lines = flood_lines;
	}
	
	return m_flood_lines;
}

/*************************************************************************/

uint8_t ChannelInfo::flood_secs(uint8_t flood_secs)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept invalid values */
		if (flood_secs == 0 || flood_secs > epona->cd().botserv.keep_data)
		{
			/* If it's already a member, return the previous value. */
			if (db_status() & dbsMember)
				return m_flood_secs;
			/* Else, set it to the default value */
			flood_secs = 10;
		}
	}
	
	if (m_flood_secs != flood_secs) 
	{
		if (!channels.change_ui(this, f_flood_secs, flood_secs))
			return m_flood_secs;
		
		m_flood_secs = flood_secs;
	}
	
	return m_flood_secs;
}

/*************************************************************************/

bool ChannelInfo::generate_password(char *passbuf, size_t size)
{
#if HAVE_OPENSSL
	size_t rsize = (size - 1) - ((size - 1) % 4);
	rsize *= 3;
	rsize /= 4;
	
	misc::cstring rbuf;
	rbuf.reserve(rsize);
	
	if (RAND_bytes((unsigned char *)rbuf.str(), rsize) != 1 &&
			RAND_pseudo_bytes((unsigned char *)rbuf.str(), rsize) == -1)
		return false;
	
	size = base64_encode(rbuf, rsize, passbuf);
	passbuf[size] = 0;
	
	return change_password(passbuf);
#else
	return false;
#endif
}

/*************************************************************************/

Record::identifier_type ChannelInfo::get_ri(Field const *field) const
{
	if (field == f_owner)
		return (m_owner ? m_owner->id() : 0);
	else if (field == f_successor)
		return (m_successor ? m_successor->id() : 0);
	else if (field == f_bot)
		return (m_bi ? m_bi->id() : 0);
	else
		return NameRecord::get_ri(field);
}

/*************************************************************************/

char const *ChannelInfo::get_s(Field const *field) const
{
	if (field == f_name)
		return name();
	else if (field == f_enc)
		return enc();
	else if (field == f_pass)
		return pass();
	else if (field == f_desc)
		return desc();
	else if (field == f_email)
		return email();
	else if (field == f_url)
		return url();
	else if (field == f_entry_message)
		return entry_message();
	else if (field == f_mlock_ircd)
		return mlock_ircd();
	else if (field == f_mlock_modes)
		return mlock_modes();
	else if (field == f_topic)
		return topic();
	else if (field == f_topic_setter)
		return topic_setter();
	else
		return NameRecord::get_s(field);
}

/*************************************************************************/

uint32_t ChannelInfo::get_ui(Field const *field) const
{
	if (field == f_flags)
		return flags();
	else if (field == f_bantype)
		return bantype();
	else if (field == f_botflags)
		return botflags();
	else if (field == f_caps_min)
		return caps_min();
	else if (field == f_caps_percent)
		return caps_percent();
	else if (field == f_flood_lines)
		return flood_lines();
	else if (field == f_flood_secs)
		return flood_secs();
	else if (field == f_repeats)
		return repeats();
	else
		return NameRecord::get_ui(field);
}

/*************************************************************************/

time_t ChannelInfo::get_ut(Field const *field) const
{
	if (field == f_created)
		return created();
	else if (field == f_last_used)
		return last_used();
	else if (field == f_topic_time)
		return topic_time();
	else
		return NameRecord::get_ut(field);
}

/*************************************************************************/

uint8_t ChannelInfo::kbb(uint8_t index) const
{
	if (m_kbb.size() <= index || !m_kbb[index])
		return def_kbb[index].defval();
	
	KBB *kbb = (KBB *)m_kbb[index];
	return kbb->count();
}

/*************************************************************************/

uint8_t ChannelInfo::kbb(uint8_t index, uint8_t count)
{
	/* If it's the default value, remove the record if it exists. */
	
	if (def_kbb[index].defval() == count)
	{
		if (m_kbb.size() <= index || !m_kbb[index])
			return count;
		
		KBB *kbb = (KBB *)m_kbb[index];
		
		if (!kbbs.remove(kbb))
			return kbb->count();
		
		return count;
	}
	
	/* If the record doesn't exist, create it. */
	
	if (m_kbb.size() <= index || !m_kbb[index])
	{
		KBB *kbb = (KBB *)kbbs.create();
		
		if (!kbb)
			return def_kbb[index].defval();
		
		kbb->owner(this);
		kbb->index(index);
		kbb->count(count);
		
		if (!kbbs.add(kbb, 0))
		{
			delete kbb;
			return def_kbb[index].defval();
		}
		
		return kbb->count();
	}
	
	/* Just update the KBB */
	KBB *kbb = (KBB *)m_kbb[index];
	return kbb->count(count);
}

/*************************************************************************/

time_t ChannelInfo::last_used(time_t last_used)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept dates in the future */
		if (last_used > time(0))
		{
			if (db_status() & dbsMember)
				return m_last_used;
			
			last_used = time(0);
		}
		
		if (m_last_used && (last_used < m_last_used))
			return m_last_used;
	}
	
	if (m_last_used != last_used)
	{
		if (((last_used - m_last_used) > lu_sensibility) &&
				!channels.change_ut(this, f_last_used, last_used))
			return m_last_used;
		
		m_last_used = last_used;
	 	
	 	/* Clear pending expire if this is NOT updated by a
	 	 * database module. */
	 	if (!(db_status() & dbsUpdating))
	 		flags_remove(ChannelInfo::cfPendingExpire);
	}
	
	return m_last_used;
}

/*************************************************************************/

int16_t ChannelInfo::levels(uint16_t index) const
{
	if (m_levels.size() <= index || !m_levels[index])
		return def_levels[index].defval();
	
	Level *lv = (Level *)m_levels[index];
	return lv->level();
}

/*************************************************************************/

int16_t ChannelInfo::levels(uint16_t index, int16_t level)
{
	Level *lv = 0;
	
	if (m_levels.size() > index)
		lv = (Level *)m_levels[index];
	
	/* If the record doesn't exist and the new level is
	 * the default value, do nothing. */
	if (!lv && (level == def_levels[index].defval()))
		return level;
	
	/* If the record exists and the new level is the default
	 * value, delete the record. */
	if (lv && (level == def_levels[index].defval()))
	{
		if (!::levels.remove(lv, 0))
			return lv->level();
	}
	
	/* If the record doesn't exist, create it. */
	
	if (!lv)
	{
		lv = (Level *)::levels.create();
		
		if (!lv)
			return def_levels[index].defval();
		
		lv->owner(this);
		lv->index(index);
		lv->level(level);
		
		if (!::levels.add(lv, 0))
		{
			delete lv;
			return def_levels[index].defval();
		}
		
		return lv->level();
	}
	
	/* Just update the Level */
	return lv->level(level);
}

/*************************************************************************/

size_t ChannelInfo::memory_usage() const
{
	size_t result = NameRecord::memory_usage();
	
	result += sizeof(ChannelInfo);
	if (m_name)
		result += strlen(m_name) + 1;
	if (m_pass)
		result += strlen(m_pass) + 1;
	if (m_desc)
		result += strlen(m_desc) + 1;
	if (m_email)
		result += strlen(m_email) + 1;
	if (m_url)
		result += strlen(m_url) + 1;
	if (m_entry_message)
		result += strlen(m_entry_message) + 1;
	if (m_mlock_modes)
		result += strlen(m_mlock_modes) + 1;
	if (m_topic)
		result += strlen(m_topic) + 1;
	if (m_topic_setter)
		result += strlen(m_topic_setter) + 1;
	
	result += m_levels.capacity() * sizeof(Level *);
	result += m_accesses.capacity() * sizeof(Access *);
	result += m_akicks.capacity() * sizeof(Akick *);
	
	result += m_kbb.capacity() * sizeof(KBB *);
	result += m_badwords.capacity() * sizeof(BadWord *);
	
	return result;
}

/*************************************************************************/

char const *ChannelInfo::mlock_ircd(char const *mlock_ircd)
{
	if (stricmp(mlock_ircd, IRCD_NAME))
		db_status_add(dbsIRCdChange);
	return IRCD_NAME;
}

/*************************************************************************/

char const *ChannelInfo::mlock_modes() const
{
	return m_mlock_modes;
}

/*************************************************************************/

char const *ChannelInfo::mlock_modes(char const *mlock_modes, User *u)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept NULL mlock modes */
		if (!mlock_modes)
		{
			/* Convert them to just a +... */
			mlock_modes = "+";
		}
	}
	
	misc::cstring tmlock_modes(mlock_modes, mlock_modes_len);
	char *param = strtok(tmlock_modes, " ");
	
	int add = -1; /* 1 if adding, 0 if deleting, -1 if neither */
	unsigned char mode;
	CBMode *cbm;
	
	MLockData mld;
	mld.on_add(CMODE_r);
	
	while ((mode = *param++)) 
	{
		switch (mode) 
		{
			case '+':
				add = 1;
				continue;
			case '-':
				add = 0;
				continue;
			default:
				if (add < 0)
					continue;
		}
				
		if ((int)mode < 128 && (cbm = &cbmodes[(int)mode])->flag != 0) 
		{
			if ((cbm->flags & CBM_NO_MLOCK) ||
					((cbm->flags & CBM_NO_USER_MLOCK) && u && !is_oper(u))) 
			{
				if (u)
					notice_lang(s_ChanServ, u,
							CHAN_SET_MLOCK_IMPOSSIBLE_CHAR, mode);
			} else if (add) 
			{
				mld.on_add(cbm->flag);
				mld.off_remove(cbm->flag);
				if (cbm->cssetvalue)
					cbm->cssetvalue(&mld, strtok(NULL, " "));
			} else 
			{
				mld.off_add(cbm->flag);
				if (mld.on() & cbm->flag) 
				{
					mld.on_remove(cbm->flag);
					if (cbm->cssetvalue)
						cbm->cssetvalue(&mld, NULL);
				}
			}
		} else {
			if (u)
				notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_UNKNOWN_CHAR, mode);
		}
	} /* while (*param) */

#ifdef HAS_LMODE
	/* We can't mlock +L if +l is not mlocked as well. */
	if ((mld.on() & CMODE_L) && !(mld.on() & CMODE_l)) 
	{
		mld.on_remove(CMODE_L);
		mld.redirect(0);
		if (u)
			notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_L_REQUIRED);
	}
#endif

#if defined(IRC_ULTIMATE) || defined(IRC_UNREAL)
	/* We can't mlock +K if +i is not mlocked as well. */
	if ((mld.on() & CMODE_K) && !(mld.on() & CMODE_i)) 
	{
		mld.on_remove(CMODE_K);
		if (u)
			notice_lang(s_ChanServ, u, CHAN_SET_MLOCK_K_REQUIRED);
	}
#endif

	if (m_mlock_data != mld)
	{
		tmlock_modes = mld.compute();
		
		if (!channels.change_s(this, f_mlock_modes, tmlock_modes))
			return false;
		
		/* Swap the data */
		m_mlock_data.swap(mld);
		m_mlock_modes.swap(tmlock_modes);

		/* Implement the new lock. */
		if (c)
			check_modes(c);
	}
	
	/* Since we always enforce mode r there is no way to have no
	 * mode lock at all. */
	if (u)
		notice_lang(s_ChanServ, u, CHAN_MLOCK_CHANGED, m_name.str(),
				m_mlock_modes.str());
	
	/* Don't return the new modes if dbsIRCdChange is set; let
	 * end_update() update everything... */
	if (db_status() & dbsIRCdChange)
		return (*mlock_modes ? mlock_modes : 0);
	
	return m_mlock_modes;
}

/*************************************************************************/

char const *ChannelInfo::name(char const *name)
{
	if (db_status() & dbsUpdating)
	{
		/* Do not accept NULL names */
		if (!name)
		{
			/* If we're already a member, just return the current name */
			if (db_status() & dbsMember)
				return m_name;
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* And return the NULL name */
			return 0;
		}
		
		/* Do not accept name changes (but allow case changes)  */
		if (m_name && stricmp(m_name, name) != 0)
			return m_name;
	}
	
	if (m_name.compare(name, name_len) != 0)
	{
		misc::cstring tname(name, name_len);
			
		if (!channels.change_s(this, f_name, tname))
			return m_name;
		
		m_name.swap(tname);
	}
	
	return m_name;
}

/*************************************************************************/

bool ChannelInfo::owned(Collection const *coll, Record const *record)
{	
	if (coll == &groups)
		return (m_owner == record);
	else if (coll == &bots)
		return (m_bi == record);
	else
		return NameRecord::owned(coll, record);
}


/*************************************************************************/

NickGroup *ChannelInfo::owner(NickGroup *owner)
{
	if (m_owner != owner) 
	{
		if (!channels.change_ri(this, f_owner, (owner ? owner->id() : 0)))
			return m_owner;
		
		if ((db_status() & dbsMember) && m_owner)
			m_owner->channelcount(m_owner->channelcount() - 1);
		m_owner = owner;
		if ((db_status() & dbsMember) && m_owner)
			m_owner->channelcount(m_owner->channelcount() + 1);
	}
	
	return m_owner;
}

/*************************************************************************/

Record::identifier_type ChannelInfo::owner_id(identifier_type owner_id)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	NickGroup *ng;
	
	/* Set the owner only if the group is valid... However, don't
	 * generate an error if it is invalid and no owner was set
	 * previously: let Channels::add() deal with it (and possibly
	 * transfer to successor).
	 *
	 * If the group is invalid, we return the current owner ID
	 * if it is set or the invalid value we got if not (as
	 * that will be taken care of in Channels::add(), as was
	 * said above). */
	
	if (!owner_id || !(ng = (NickGroup *)groups.find(owner_id)))
		return (m_owner ? m_owner->id() : owner_id);
	
	owner(ng);
	return (owner() ? owner()->id() : 0);
}

/*************************************************************************/

char const *ChannelInfo::pass(char const *pass)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL passwords. */
		if (!pass)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_pass;
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL reason... =) */
			return m_pass;
		}
	}
	
	if (m_pass.compare(pass, pass_len) != 0)
		m_pass.assign(pass, pass_len);
	
	return m_pass;
}

/*************************************************************************/

void ChannelInfo::post_registered()
{
	if (c)
	{
		/* Set the topic */
		if (channels.status() & (Collection::csInitial | Collection::csImport))
			restore_topic(); 
		else if (c->topic)
			change_topic(c->topic, c->topic_setter, c->topic_time);
		
		/* Enforce modes */
		check_modes(c);
		
		/* Make the bot join if needed */
    	if (m_bi && c->usercount >= epona->cd().botserv.min_users)
    		bot_join(this);
	}
}

/*************************************************************************/

void ChannelInfo::post_unregistered()
{
    if (c) 
    {
    	if (m_bi && c->usercount >= epona->cd().botserv.min_users)
    		send_cmd(m_bi->nick(), "PART %s", c->name);

		c->mode &= ~CMODE_r;
	    s_mode(whosends(this), m_name, "-r", NULL);
	}
}

/*************************************************************************/

uint8_t ChannelInfo::repeats(uint8_t repeats)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't accept invalid values */
		if (repeats < 2)
		{
			/* If it's already a member, return the previous value. */
			if (db_status() & dbsMember)
				return m_repeats;
			/* Else, set it to the default value */
			repeats = 3;
		}
	}
	
	if (m_repeats != repeats) 
	{
		if (!channels.change_ui(this, f_repeats, repeats))
			return m_repeats;
		
		m_repeats = repeats;
	}
	
	return m_repeats;
}

/*************************************************************************/

bool ChannelInfo::reset_levels()
{
	return ::levels.remove_owned(&channels, this, 0);
}

/*************************************************************************/

void ChannelInfo::restore_topic() const
{
	if (!c || !(m_flags & cfKeepTopic))
		return;
	
	if (m_topic || m_topic_setter || m_topic_time)
		set_topic(whosends(this), c, m_topic.str(),
				(m_topic_setter ? m_topic_setter.str() : whosends(this)),
				m_topic_time);
}

/*************************************************************************/

Record::identifier_type ChannelInfo::set_ri(Field const *field,
		identifier_type value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ri(field, value);
	
	if (field == f_owner)
		return owner_id(value);
	else if (field == f_successor)
		return successor_id(value);
	else if (field == f_bot)
		return bot_id(value);
	else
		return NameRecord::set_ri(field, value);
}

/*************************************************************************/

char const *ChannelInfo::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_s(field, value);
	
	if (field == f_name)
		return name(value);
	else if (field == f_enc)
		return enc(value);
	else if (field == f_pass)
		return pass(value);
	else if (field == f_desc)
		return desc(value);
	else if (field == f_email)
		return email(value);
	else if (field == f_url)
		return url(value);
	else if (field == f_entry_message)
		return entry_message(value);
	else if (field == f_mlock_ircd)
		return mlock_ircd(value);
	else if (field == f_mlock_modes)
		return mlock_modes(value);
	else if (field == f_topic)
		return topic(value);
	else if (field == f_topic_setter)
		return topic_setter(value);
	else
		return NameRecord::set_s(field, value);
}

/*************************************************************************/

uint32_t ChannelInfo::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ui(field, value);
	
	if (field == f_flags)
		return flags(value);
	else if (field == f_bantype)
		return bantype(value);
	else if (field == f_botflags)
		return botflags(value);
	else if (field == f_caps_min)
		return caps_min(value);
	else if (field == f_caps_percent)
		return caps_percent(value);
	else if (field == f_flood_lines)
		return flood_lines(value);
	else if (field == f_flood_secs)
		return flood_secs(value);
	else if (field == f_repeats)
		return repeats(value);
	else
		return NameRecord::set_ui(field, value);
}

/*************************************************************************/

time_t ChannelInfo::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ut(field, value);
	
	if (field == f_created)
		return created(value);
	else if (field == f_last_used)
		return last_used(value);
	else if (field == f_topic_time)
		return topic_time(value);
	else
		return NameRecord::set_ut(field, value);
}

/*************************************************************************/

NickGroup *ChannelInfo::successor(NickGroup *successor)
{
	if (m_successor != successor) 
	{
		if (!channels.change_ri(this, f_successor,
				(successor ? successor->id() : 0)))
			return m_successor;
		
		m_successor = successor;
	}
	
	return m_successor;
}

/*************************************************************************/

Record::identifier_type ChannelInfo::successor_id(identifier_type successor_id)
{
	/* Assume (db_status() & dbsUpdating) == true */
	
	NickGroup *ng = 0;
	
	if (successor_id && !(ng = (NickGroup *)groups.find(successor_id)))
		return (m_successor ? m_successor->id() : 0);
	
	/* Don't allow the successor to be the same value as the owner */
	if (ng && (m_owner == ng))
		return (m_successor ? m_successor->id() : 0);
	
	successor(ng);
	return (m_successor ? m_successor->id() : 0);
}

/*************************************************************************/

char const *ChannelInfo::topic(char const *topic)
{
	if (m_topic.compare(topic, topic_len) != 0)
	{	
		misc::cstring ttopic(topic, topic_len);
		
		if (!channels.change_s(this, f_topic, ttopic))
			return m_topic;
		
		m_topic.swap(ttopic);
		
		if (db_status() & dbsUpdating)
			db_status_add(dbsTopicChange);	
	}
	
	return m_topic;
}

/*************************************************************************/

char const *ChannelInfo::topic_setter(char const *topic_setter)
{
	if (m_topic_setter.compare(topic_setter, topic_setter_len) != 0)
	{	
		misc::cstring ttopic_setter(topic_setter, topic_setter_len);
		
		if (!channels.change_s(this, f_topic_setter, ttopic_setter))
			return m_topic_setter;
		
		m_topic_setter.swap(ttopic_setter);
		
		if (db_status() & dbsUpdating)
			db_status_add(dbsTopicChange);
	}
	
	return m_topic_setter;
}

/*************************************************************************/

time_t ChannelInfo::topic_time(time_t topic_time)
{
	if (m_topic_time != topic_time)
	{
		if (!channels.change_ut(this, f_topic_time, topic_time))
			return m_topic_time;
		
		m_topic_time = topic_time;
		
		if (db_status() & dbsUpdating)
			db_status_add(dbsTopicChange);
	}
	
	return m_topic_time;
}

/*************************************************************************/

bool ChannelInfo::transfer()
{
	if (!channels.writable())
		return false;
	
	if (!m_successor)
		return false;
	
	/* WARNING: the founder may be NULL. */
	
	if (!nick_is_services_admin(m_successor) && 
			(epona->cd().chanserv.max_reg > 0) && 
			(m_successor->channelcount() >= epona->cd().chanserv.max_reg)) 
	{
		epona->log(InfoChanServList(),
				"Can't transfer channel %s to successor %s: "
				"too many channels owned", name(), m_successor->name());
		return false;
	}
	
	epona->log(InfoChanServRights(),
			"Transferring foundership of %s to successor %s",
			name(), m_successor->name());
	
	if (!channels.change_multi(this, 2, f_owner, m_successor->id(),
			f_successor, 0))
	{
		epona->log(InfoChanServList(),
				"Can't transfer channel %s to successor %s: "
				"owner change failed", name(), m_successor->name());
		return false;
	}
	
	m_owner = m_successor;
	m_successor = 0;
	
	return true;
}

/*************************************************************************/

char const *ChannelInfo::url(char const *url)
{
	if (m_url.compare(url, url_len) != 0)
	{	
		misc::cstring turl(url, url_len);
		
		if (!channels.change_s(this, f_url, turl))
			return m_url;
		
		m_url.swap(turl);
	}
	
	return m_url;
}

/*************************************************************************/

ChannelInfo::MLockData &ChannelInfo::MLockData::operator=(MLockData const &right)
{
	if (this != &right)
	{
		m_on = right.m_on;
		m_off = right.m_off;
		
		m_limit = right.m_limit;
		m_key = right.m_key;

#ifdef HAS_FMODE
		m_flood = right.m_flood;
#endif
#ifdef HAS_JMODE
		m_joinrate = right.m_joinrate;
#endif
#ifdef HAS_LMODE
		m_redirect = right.m_redirect;
#endif
	}
	
	return *this;
}

/*************************************************************************/

bool ChannelInfo::MLockData::operator==(MLockData const &right) const
{
	if (this == &right)
		return true;
	
	if (m_on != right.m_on || m_off != right.m_off || m_limit != right.m_limit
			|| m_key != right.m_key)
		return false;
	
#ifdef HAS_FMODE
	if (m_flood != right.m_flood)
		return false;
#endif
#ifdef HAS_JMODE
	if (m_joinrate != right.m_joinrate)
		return false;
#endif
#ifdef HAS_LMODE
	if (m_redirect != right.m_redirect)
		return false;
#endif

	return true;
}

/*************************************************************************/

char const *ChannelInfo::MLockData::compute() const
{
	static char res[BUFSIZE];
	char *end = res;

	if (m_on || m_off) 
	{
		CBModeInfo *cbmi = cbmodeinfos;
		
		if (m_on) 
		{
			end += snprintf(end, sizeof(res) - (end - res), "+");
	
			do 
			{
				if (m_on & cbmi->flag)
					end += snprintf(end, sizeof(res) - (end - res),
							"%c", cbmi->mode);
			} while ((++cbmi)->flag != 0);
			
			cbmi = cbmodeinfos;
		}
		
		if (m_off) 
		{
			end += snprintf(end, sizeof(res) - (end - res), "-");
	
			do 
			{
				if (m_off & cbmi->flag)
					end += snprintf(end, sizeof(res) - (end - res),
							"%c", cbmi->mode);
			} while ((++cbmi)->flag != 0);
			
			cbmi = cbmodeinfos;
		}
		
		if (m_on) 
		{
			do 
			{
				if (cbmi->csgetvalue && (m_on & cbmi->flag)) 
				{
					char const *value = cbmi->csgetvalue(this);

					if (value)
						end += snprintf(end, sizeof(res) - (end - res),
								" %s", value);
				}
			} while ((++cbmi)->flag != 0);
		}
	}
	
	*end = 0;
	
	return res;
}

/*************************************************************************/

#ifdef HAS_FMODE

char const *ChannelInfo::MLockData::flood(char const *flood)
{
	if (m_flood.compare(flood) != 0)
		m_flood.assign(flood);
	
	return m_flood;
}

#endif

/*************************************************************************/

#ifdef HAS_JMODE

char const *ChannelInfo::MLockData::joinrate(char const *joinrate)
{
	if (m_joinrate.compare(joinrate) != 0)
		m_joinrate.assign(joinrate);
	
	return m_joinrate;
}

#endif

/*************************************************************************/

char const *ChannelInfo::MLockData::key(char const *key)
{
	if (m_key.compare(key) != 0)
		m_key.assign(key);
	
	return m_key;
}

/*************************************************************************/

int32_t ChannelInfo::MLockData::limit(int32_t limit)
{
	return (m_limit = limit);
}

/*************************************************************************/

uint32_t ChannelInfo::MLockData::on_add(uint32_t on)
{	
	return (m_on |= on);
}

/*************************************************************************/

uint32_t ChannelInfo::MLockData::on_clear()
{	
	return (m_on = 0);
}

/*************************************************************************/

uint32_t ChannelInfo::MLockData::on_remove(uint32_t on)
{	
	return (m_on &= ~on);
}

/*************************************************************************/

uint32_t ChannelInfo::MLockData::off_add(uint32_t off)
{	
	return (m_off |= off);
}

/*************************************************************************/

uint32_t ChannelInfo::MLockData::off_clear()
{	
	return (m_off = 0);
}

/*************************************************************************/

uint32_t ChannelInfo::MLockData::off_remove(uint32_t off)
{
	return (m_off &= ~off);
}

/*************************************************************************/

#ifdef HAS_LMODE

char const *ChannelInfo::MLockData::redirect(char const *redirect)
{
	if (m_redirect.compare(redirect) != 0)
		m_redirect.assign(redirect);
	
	return m_redirect;
}

#endif

/*************************************************************************/

void ChannelInfo::MLockData::swap(MLockData &mld)
{
	::swap(m_on, mld.m_on);
	::swap(m_off, mld.m_off);
	
	::swap(m_limit, mld.m_limit);
	m_key.swap(mld.m_key);

#ifdef HAS_FMODE
	m_flood.swap(mld.m_flood);
#endif
#ifdef HAS_JMODE
	m_joinrate.swap(mld.m_joinrate);
#endif
#ifdef HAS_LMODE
	m_redirect.swap(mld.m_redirect);
#endif
}

/*************************************************************************/

Channels::Channels() : databases::NameCollection(channel_hash), m_encrypter(0)
{
	
}

/*************************************************************************/

bool Channels::add(databases::Record *record, int *err)
{
	ChannelInfo *ci = (ChannelInfo *)record;
	
	/* Ensure there is a founder... Else, fail. */
	if (!ci->owner() && !ci->transfer())
		return false;
	
	if (!NameCollection::add(record, err))
		return false;
	
	/* Set the channel record */
	if (ci->c || (ci->c = findchan(ci->name())))
		ci->c->ci = ci;
	
	/* Increment the channel count of the owner */
	if (ci->owner())
		ci->owner()->channelcount(ci->owner()->channelcount() + 1);

	/* Increment the channel count of the bot */
	if (ci->bi())
		ci->bi()->chancount(ci->bi()->chancount() + 1);

	ci->post_registered();
	
	if (status() & csCreation)
		epona->log(InfoChanServList(),
				"Channel %s has been registered", ci->name());
	
	return true;
}

/*************************************************************************/

databases::Record *Channels::create(databases::Record::identifier_type id) const
{
	if (id)
		return new ChannelInfo(id);
	else
		return new ChannelInfo;
}

/*************************************************************************/

void Channels::expire()
{
	if (!epona->cd().chanserv.expire)
		return;
	
	if (!writable())
		return;
	
    ChannelInfo *ci;
    time_t now = time(NULL);

    const_iterator it, end;
	
	for (it = begin(), end = this->end(); it != end; )
	{
		ci = (ChannelInfo *)*it++;
		
    	if ((ci->flags() & ChannelInfo::cfNoExpire) || ci->c)
    		continue;
    	
    	if ((now - ci->last_used()) >= (epona->cd().chanserv.expire.delay +
				epona->cd().chanserv.expire.grace))
    	{
			epona->log(InfoChanServList(),
					"Expiring channel %s (founder: %s)", ci->name(), 
					(ci->owner() ? (ci->owner())->name() : "(none)"));
			
			if (!remove(ci))
			{
				epona->log(ErrorChanServList(),
						"Expiration of channel %s failed", ci->name());
				
				/* If it fails, we may have switched to read-only,
				 * so check for that. */
				if (!writable())
					return;
			}
			
			continue; /* Important */
    	}
    	
    	if ((now - ci->last_used()) >= epona->cd().chanserv.expire.delay)
    		ci->flags_add(ChannelInfo::cfPendingExpire);
    }
}

/*************************************************************************/

password::Encrypter *Channels::load_encrypter(char const *name)
{
	EncMap::const_iterator it = enc_map.find(name);
		
	if (it != enc_map.end())
	{
		it->second->module()->references_add();
		return it->second;
	}
	
	modules::Module *m = 0;
	
	if (!epona->modules().load(name) || !(m = epona->modules().find(name)))
	{
		epona->log(ErrorChanServ(), "Can't load password module %s: %s",
				name, epona->modules().error().c_str());
		return false;
	}
	
	password::Encrypter *enc;
	
	if (!(enc = (password::Encrypter *)m->interface(password::Encrypter::IID)))
	{
		epona->log(ErrorChanServ(), "%s is not a password module", name);
		return false;
	}
	
	enc_map[m->name()] = enc;
	enc->module()->references_add();
	
	return enc;
}

/*************************************************************************/

databases::Field const *Channels::owner_field(Collection const *coll) const
{
	if (coll == &groups)
		return ChannelInfo::f_owner;
	else if (coll == &bots)
		return ChannelInfo::f_bot;
	else
		return 0;
}

/*************************************************************************/

void Channels::release_encrypter(password::Encrypter *enc)
{
	if (enc)
		enc->module()->references_remove();
}

/*************************************************************************/

bool Channels::remove(databases::Record *record, int *err)
{
	ChannelInfo *ci = (ChannelInfo *)record;
	
	Channel *c = ci->c;
	NickGroup *owner = ci->owner();
	BotInfo *bi = ci->bi();
	
	misc::cstring name(ci->name());
	
	ci->post_unregistered();
	
	if (!NameCollection::remove(record, err))
	{
		ci->post_registered();
		return false;
	}
	
	/* The channel record is INVALID from this point */

	if (c)
		c->ci = 0;
	
	/* Decrement the channel count of the owner */
	if (owner)
		owner->channelcount(owner->channelcount() - 1);

	/* Decrement the channel count of the bot */
	if (bi)
		bi->chancount(bi->chancount() - 1);
	
	if (status() & csDeletion)
		epona->log(InfoChanServList(),
				"Channel %s has been deleted", name.str());
	
	return true;
}

/*************************************************************************/

bool Channels::remove_owned(databases::Collection const *coll,
		databases::Record const *record, int *err)
{
	if (coll == &groups)
	{
		ChannelInfo *ci;
		id_hashlist::iterator it, end;
	
		for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
		{
			ci = (ChannelInfo *) (*it);
			
			if (ci->owned(coll, record))
				ci->transfer();
		}
	}
	
	return NameCollection::remove_owned(coll, record, err);
}

/*************************************************************************/

bool Channels::remove_owned_db(databases::Collection const *coll,
		databases::Record const *record, int *err)
{
	if (coll == &groups)
	{
		/* Just update the successors, let the inherited function
		 * clear the owned records */
		if (!change_owned(ChannelInfo::f_successor, record->id(), 0))
		{
			if (err)
				*err = ceDatabase;
			return false;
		}
	}
	
	if (coll == &bots)
	{
		if (!change_owned(ChannelInfo::f_bot, record->id(), 0))
		{
			if (err)
				*err = ceDatabase;
			return false;
		}
		
		return true;
	}
	
	return NameCollection::remove_owned_db(coll, record, err);
}

/*************************************************************************/

bool Channels::remove_owned_list(databases::Collection const *coll,
		databases::Record const *record, int *err)
{
	if (coll == &groups)
	{
		ChannelInfo *ci;
		id_hashlist::iterator it, end;
		
		for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
		{
			ci = (ChannelInfo *) (*it);
			
			if (ci->successor() == record)
				ci->clear_successor();
		}
	}
	
	if (coll == &bots)
	{
		ChannelInfo *ci;
		id_hashlist::iterator it, end;
	
		for (it = m_members.begin(), end = m_members.end(); it != end; ++it)
		{
			ci = (ChannelInfo *) (*it);
			
			if (ci->owned(coll, record))
				ci->clear_bot();
		}
		
		return true;
	}
	
	return NameCollection::remove_owned_list(coll, record, err);
}

/*************************************************************************/

void Channels::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("chanserv_channels");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	ChannelInfo::f_name = db->field_create("name", 1, Field::ftString,
			ChannelInfo::name_len, 0, 0, 1);
	ChannelInfo::f_owner = db->field_create("owner", 1, Field::ftIdentifier,
			0, Field::faOwner, "0", 2);
	ChannelInfo::f_successor = db->field_create("successor", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 3);
	ChannelInfo::f_created = db->field_create("created", 1, Field::ftUnixTime,
			0, 0, "0", 4);
	ChannelInfo::f_last_used = db->field_create("last_used", 1, Field::ftUnixTime,
			0, 0, "0", 5);
	ChannelInfo::f_enc = db->field_create("enc", 1, Field::ftString,
			255, 0, 0, 6);
	ChannelInfo::f_pass = db->field_create("pass", 1, Field::ftString,
			ChannelInfo::pass_len, Field::faCase, 0, 7);
	ChannelInfo::f_flags = db->field_create("flags", 1, Field::ftUnsigned,
			4, 0, "1073741824", 8);
	ChannelInfo::f_desc = db->field_create("desc", 1, Field::ftString,
			ChannelInfo::desc_len, 0, 0, 9);
	ChannelInfo::f_email = db->field_create("email", 1, Field::ftString,
			ChannelInfo::email_len, 0, 0, 10);
	ChannelInfo::f_url = db->field_create("url", 1, Field::ftString,
			ChannelInfo::url_len, 0, 0, 11);
	ChannelInfo::f_entry_message = db->field_create("entry_message", 1,
			Field::ftString, ChannelInfo::entry_message_len, 0, 0, 12);
	ChannelInfo::f_bantype = db->field_create("bantype", 1, Field::ftUnsigned,
			1, 0, "255", 13);
	ChannelInfo::f_mlock_ircd = db->field_create("mlock_ircd", 1,
			Field::ftString, 63, 0, 0, 14);
	ChannelInfo::f_mlock_modes = db->field_create("mlock_modes", 1,
			Field::ftString, ChannelInfo::mlock_modes_len, 0, 0, 15);
	ChannelInfo::f_topic = db->field_create("topic", 1, Field::ftString,
			ChannelInfo::topic_len, 0, 0, 16);
	ChannelInfo::f_topic_setter = db->field_create("topic_setter", 1,
			Field::ftString, ChannelInfo::topic_setter_len, 0, 0, 17);
	ChannelInfo::f_topic_time = db->field_create("topic_time", 1,
			Field::ftUnixTime, 0, 0, "0", 18);
	ChannelInfo::f_bot = db->field_create("bot", 1, Field::ftIdentifier,
			0, Field::faOwner, "0", 19);
	ChannelInfo::f_botflags = db->field_create("botflags", 1,
			Field::ftUnsigned, 4, 0, "64", 20);
	ChannelInfo::f_caps_min = db->field_create("caps_min", 1,
			Field::ftUnsigned, 1, 0, "10", 21);
	ChannelInfo::f_caps_percent = db->field_create("caps_percent", 1,
			Field::ftUnsigned, 1, 0, "25", 22);
	ChannelInfo::f_flood_lines = db->field_create("flood_lines", 1,
			Field::ftUnsigned, 1, 0, "6", 23);
	ChannelInfo::f_flood_secs = db->field_create("flood_secs", 1,
			Field::ftUnsigned, 1, 0, "10", 24);
	ChannelInfo::f_repeats = db->field_create("repeats", 1,
			Field::ftUnsigned, 1, 0, "3", 25);
	
	
	if (!db->field_add(ChannelInfo::f_name) || 
			!db->field_add(ChannelInfo::f_owner) ||
			!db->field_add(ChannelInfo::f_successor) ||
			!db->field_add(ChannelInfo::f_created) || 
			!db->field_add(ChannelInfo::f_last_used) || 
			!db->field_add(ChannelInfo::f_enc) ||
			!db->field_add(ChannelInfo::f_pass) || 
			!db->field_add(ChannelInfo::f_flags) || 
			!db->field_add(ChannelInfo::f_desc) ||
			!db->field_add(ChannelInfo::f_email) ||
			!db->field_add(ChannelInfo::f_url) ||
			!db->field_add(ChannelInfo::f_entry_message) ||
			!db->field_add(ChannelInfo::f_bantype) ||
			!db->field_add(ChannelInfo::f_mlock_ircd) ||
			!db->field_add(ChannelInfo::f_mlock_modes) ||  
			!db->field_add(ChannelInfo::f_topic) ||
			!db->field_add(ChannelInfo::f_topic_setter) || 
			!db->field_add(ChannelInfo::f_topic_time) || 
			!db->field_add(ChannelInfo::f_bot) ||
			!db->field_add(ChannelInfo::f_botflags) ||
			!db->field_add(ChannelInfo::f_caps_min) ||
			!db->field_add(ChannelInfo::f_caps_percent) ||
			!db->field_add(ChannelInfo::f_flood_lines) ||
			!db->field_add(ChannelInfo::f_flood_secs) ||
			!db->field_add(ChannelInfo::f_repeats))
		epona->fatal("Can't add fields to database chanserv_channels");
	
	this->db(db);
}

/*************************************************************************/

void Channels::setup_owner()
{
	groups.set_owned(*this);
	bots.set_owned(*this);
}

/*************************************************************************/

DefEntry &DefEntry::operator=(DefEntry const &right)
{
	if (this != &right)
	{
		m_index = right.m_index;
		m_name = right.m_name;
		m_defval = right.m_defval;
	}
	
	return *this;
}

/*************************************************************************/

Level::Level() : databases::Record(),
		m_owner(0), m_index(0), m_level(0)
{
}

/*************************************************************************/

Level::Level(identifier_type id) : databases::Record(id),
		m_owner(0), m_index(0), m_level(0)
{
}

/*************************************************************************/

bool Level::end_update()
{
	if (!Record::end_update())
		return false;
	
	if (def_levels[m_index].defval() == level())
	{
		epona->log(InfoChanServList(), "Ignoring level "
				RECORD_FORMAT " of channel %s where level == defval ",
				id(), (m_owner ? m_owner->name() : "(unknown)"));
		return false;
	}
	
	return true;
}

/*************************************************************************/

int32_t Level::get_i(Field const *field) const
{
	if (field == f_level)
		return level();
	else
		return Record::get_i(field);
}

/*************************************************************************/

databases::Record::identifier_type Level::get_ri(Field const *field) const
{
	if (field == f_owner)
		return (m_owner ? m_owner->id() : 0);
	else
		return Record::get_ri(field);
}

/*************************************************************************/

char const *Level::get_s(Field const *field) const
{
	if (field == f_name)
		return def_levels[m_index].name();
	else
		return Record::get_s(field);
}

/*************************************************************************/

uint16_t Level::index(uint16_t index)
{
	if (db_status() & dbsMember)
		return m_index;
	
	return (m_index = index);
}

/*************************************************************************/

int16_t Level::level(int16_t level)
{	
	if (db_status() & dbsUpdating)
	{
		/* Ensure the level is between ACCESS_INVALID and 9999 inclusive. */
		if (level < ACCESS_INVALID || level > 9999)
		{
			/* If the level is already a member, return
			 * the previous value. */
			if (db_status() & dbsMember)
				return m_level;
			
			/* Else, make end_update() fail. */
			db_status_add(dbsUpdateError);
			
			/* Return the invalid value. */
			return level;
		}
	}
	
	if (m_level != level)
	{
		if (!levels.change_i(this, f_level, level))
			return m_level;
		
		m_level = level;
	}
	
	return m_level;
}

/*************************************************************************/

size_t Level::memory_usage() const
{
	size_t result = Record::memory_usage();
	
	result += sizeof(Level);
	
	return result;
}

/*************************************************************************/

char const *Level::name(char const *name)
{
	/* Assume db_status() & dbsUpdating */
	
	/* Do not allow changes */
	if (db_status() & dbsMember)
		return def_levels[m_index].name();
	
	levels_map::const_iterator it = levels_byname.find(name);
	
	/* If the name is not found, make end_update() error */
	if (it == levels_byname.end())
	{
		db_status_add(dbsUpdateError);
		return name;
	}
	
	m_index = it->second.index();
	return it->second.name();
}

/*************************************************************************/

ChannelInfo *Level::owner(ChannelInfo *owner)
{
	if (m_owner)
		return m_owner;
	
	return (m_owner = owner);
}

/*************************************************************************/

databases::Record::identifier_type Level::owner_id(identifier_type owner_id)
{
	/* Assume (db_status() & dbsUpdating == true). */
	
	/* Don't allow owner changes. */
	if (m_owner)
		return m_owner->id();
	
	ChannelInfo *owner;
	
	/* Don't allow no ownership. */
	if (!owner_id || !(owner = (ChannelInfo *)channels.ifind(owner_id)))
	{
		epona->log(ErrorBotServList(), "Ignoring level entry #" RECORD_FORMAT
				" with bad owner #" RECORD_FORMAT, id(), owner_id);
		
		/* Update error. */
		db_status_add(dbsUpdateError);
		/* Don't need to change the value, the record will
		 * be deleted. */
		return owner_id;
	}
	
	m_owner = owner;
	return m_owner->id();
}

/*************************************************************************/

Record::identifier_type Level::set_ri(Field const *field,
		identifier_type value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ri(field, value);
	
	if (field == f_owner)
		return owner_id(value);
	else
		return Record::set_ri(field, value);
}

/*************************************************************************/

int32_t Level::set_i(Field const *field, int32_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_i(field, value);
	
	if (field == f_level)
		return level(value);
	else
		return Record::set_i(field, value);
}

/*************************************************************************/

char const *Level::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_s(field, value);
	
	if (field == f_name)
		return name(value);
	else
		return Record::set_s(field, value);
}

/*************************************************************************/

bool Levels::add(Record *record, int *err)
{
	Level *lv = (Level *)record;
	
	if ((lv->owner()->levels().size() > lv->index()) &&
			lv->owner()->levels()[lv->index()])
	{
		epona->log(ErrorBotServList(), "Ignoring duplicate level entry #"
				RECORD_FORMAT " from channel %s", lv->id(),
				lv->owner()->name());
		return false;
	}
	
	if (!Collection::add(record, err))
		return false;
	
	return true;
}

/*************************************************************************/

bool Levels::add_list(Record *record)
{
	if (!Collection::add_list(record))
		return false;
	
	Level *lv = (Level *)record;
	
	if (lv->owner()->levels().size() <= lv->index())
		lv->owner()->levels().resize(lv->index() + 1, 0);
	
	lv->owner()->levels()[lv->index()] = lv;

	return true;
}

/*************************************************************************/

Record *Levels::create(Record::identifier_type id) const
{
	if (id)
		return new Level(id);
	else
		return new Level;
}

/*************************************************************************/

void Levels::remove_list(Record *record)
{
	Level *lv = (Level *)record;
	
	lv->owner()->levels()[lv->index()] = 0;
	
	Collection::remove_list(record);
}

/*************************************************************************/

bool Levels::remove_owned_db(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels)
		return true;
	
	ChannelInfo *ci = (ChannelInfo *)record;
	
	if (ci->levels().empty())
		return true;
	
	return Collection::remove_owned_db(coll, record, err);
}

/*************************************************************************/

bool Levels::remove_owned_list(databases::Collection const *coll,
				databases::Record const *record, int *err)
{
	if (coll != &channels)
		return true;
	
	ChannelInfo *ci = (ChannelInfo *)record;
	ChannelInfo::levels_type::iterator it, end;
	
	for (it = ci->levels().begin(), end = ci->levels().end();
			it != end; ++it)
	{
		Level *lv = (Level *)*it;
		
		if (lv)
		{
			lv->db_status_remove(Record::dbsInsert);
		
			if (!remove(lv, err))
				return false;
		}
	}
	
	return true;
}

/*************************************************************************/

void Levels::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("chanserv_levels");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	Level::f_owner = db->field_create("owner", 1,
			Field::ftIdentifier, 0, Field::faOwner, "0", 1);
	Level::f_name = db->field_create("name", 1, Field::ftString,
			Level::name_len, 0, 0, 2);
	Level::f_level = db->field_create("level", 1, Field::ftInteger,
			2, 0, "0", 3);
	
	if (!db->field_add(Level::f_owner) ||
			!db->field_add(Level::f_name) ||
			!db->field_add(Level::f_level))
		epona->fatal("Can't add fields to database chanserv_levels");
	
	this->db(db);
}

/*************************************************************************/

void Levels::setup_map()
{
	for (int i = 0; i < CA_SIZE; ++i)
		levels_byname.insert(make_pair(def_levels[i].name(), def_levels[i]));
}

/*************************************************************************/

void Levels::setup_owner()
{
	channels.set_owned(*this);
}

/*************************************************************************/
