/* Prototypes and external variable declarations.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#ifndef EXTERN_H
#define EXTERN_H

#define E extern


/**** actions.cc ****/

E void change_user_mode(const char *source, User *u, char const *modes,
		char const *arg);
E void kill_user(const char *source, const char *user, const char *reason);
E void bad_password(User *u);
E void set_topic(char const *source, Channel *c, char const *topic,
		char const *setter, time_t when);

/**** channels.cc ****/

extern Channel *chanlist[1024];

E void get_channel_stats(long *nrec, long *memuse);
E Channel *findchan(const char *chan);
E Channel *firstchan(void);
E Channel *nextchan(void);

E void chan_deluser(User *user, Channel *c);

E int is_on_chan(Channel *c, User *u);
E User *ng_on_chan(Channel *c, NickGroup *nc);

E char *chan_get_modes(Channel *chan, int complete, int plus);
E void chan_set_modes(char const *source, Channel *chan, int ac,
		char const *const *av, int check);

E int chan_get_user_status(Channel *chan, User *user);
E int chan_has_user_status(Channel *chan, User *user, int16_t status);
E void chan_remove_user_status(Channel *chan, User *user, int16_t status);
E void chan_set_user_status(Channel *chan, User *user, int16_t status);

#ifdef HAS_EXEMPTS
E int is_exempted(Channel *chan, User *user);
#endif

E void do_cmode(char const *source, int ac, char const *const *av);
E void do_join(const char *source, int ac, char **av);
E void do_kick(char const *source, int ac, char const *const *av);
E void do_part(const char *source, int ac, char **av);
E void do_sjoin(const char *source, int ac, char **av);
E void do_topic(const char *source, int ac, char **av);

#define whosends(ci) ((!(ci) || !((ci)->botflags() & ChannelInfo::bfSymbiosis) || \
		!(ci)->bi() || !(ci)->c || (ci)->c->usercount < \
		(epona->cd().botserv.min_users) ? s_ChanServ : ((ci)->bi())->nick()))

/**** epona.cc ****/

E char  inbuf[BUFSIZE];
E int   servsock;

E void introduce_user(const char *user);

/**** language.cc ****/

E char **langtexts[NUM_LANGS];
E char *langnames[NUM_LANGS];
E int langlist[NUM_LANGS];

E void lang_init(void);
#define getstring(na,index) \
	(langtexts[((na)&&((NickAlias*)na)->ng()?(((NickAlias*)na)->ng())->language():epona->cd().nickserv.default_language)][(index)])
#define getstring2(ng,index) \
	(langtexts[((ng)?((NickGroup*)ng)->language():epona->cd().nickserv.default_language)][(index)])
#define getstring3(index) \
	(langtexts[epona->cd().nickserv.default_language][(index)])
	
E int strftime_lang(char *buf, int size, User *u, int format, struct tm *tm);
E void syntax_error(const char *service, User *u, char const *command,
		int msgnum, ...);


/**** memory.cc ****/

E void *smalloc(size_t size);
E void *scalloc(size_t elsize, size_t els);
E void *srealloc(void *oldptr, size_t newsize);
E char *sstrdup(char const *s);

/**** memoserv.cc ****/

E void ms_init(void);
E void memoserv(User *u, char *buf);
E void check_memos(User *u);


/**** misc.cc ****/

E char *strscpy(char *d, const char *s, size_t len);
E char const *stristr(char const *s1, char const *s2);
E char *stristr(char *s1, char *s2);
E char *strnrepl(char *s, size_t size, const char *old, const char *newstr);

E char *merge_args(int argc, char const *const *argv);

E int match_wild(const char *pattern, const char *str);
E int match_wild_nocase(const char *pattern, const char *str);

E int dotime(const char *s);
E char *duration(NickAlias *na, char *buf, int bufsize, time_t seconds);
E char *expire_left(NickAlias *na, char *buf, int len, time_t expires);

typedef int (*range_callback_t)(User *u, int num, va_list args);
E int process_numlist(const char *numstr, int *count_ret,
		range_callback_t callback, User *u, ...);

E void clean_buffer(char *str);


/**** process.cc ****/

E int allow_ignore;
E IgnoreData *ignore[];

E void add_ignore(const char *nick, time_t delta);
E IgnoreData *get_ignore(const char *nick);

E int split_buf(char *buf, char ***argv, int colon_special);
E void process(void);

/**** protocol.cc ****/

#ifdef HAS_CHGHOST
E void s_chghost(char const *nick, char const *host, time_t ts);
#endif
E void s_linkneg(char const *pass, char const *name, char const *desc);
E void s_mode(char const *source, char const *chan, char const *mode, char const *target);
E void s_mode_plus(char const *source, char const *chan, char mode, char const *target);
E void s_mode_minus(char const *source, char const *chan, char mode, char const *target);
E void s_nick(char const *nick, char const *user, char const *host,
		char const *realname, char const *modes, bool qline);
E void s_nick_change(char const *nick, char const *newnick, bool qlined);
E void s_quit(char const *nick, char const *reason, bool qlined);
E void s_squit(char const *server, char const *reason);
E void s_svsmode_user(char const *source, char const *nick, time_t timestamp,
		char const *modes, char const *arg);
E void s_svsnick(char const *service, User *u);
E void s_svsnoop(char *server, int set);

/**** send.cc ****/

E void send_cmd(const char *source, const char *fmt, ...)
	FORMAT(printf,2,3);
E void vsend_cmd(const char *source, const char *fmt, va_list args)
	FORMAT(printf,2,0);

E void wallops(const char *source, const char *fmt, ...)
	FORMAT(printf,2,3);

E void notice(const char *source, const char *dest, const char *fmt, ...)
	FORMAT(printf,3,4);
E void notice_user(const char *source, User *u, const char *fmt, ...)
	FORMAT(printf,3,4);

E void notice_list(const char *source, const char *dest, const char **text);
E void notice_lang(const char *source, User *dest, int message, ...);
E void notice_help(const char *source, User *dest, int message, ...);

E void privmsg(const char *source, const char *dest, const char *fmt, ...)
	FORMAT(printf,3,4);

/**** servers.cc ****/

E Server *server_list;

E Server *server_find(char const *name);
E Server *server_uplink();

E void do_server(char const *source, int ac, char **av);
E void do_squit(char const *source, int ac, char **av);

/**** sockutil.cc ****/

E int32_t total_read, total_written;
E int32_t read_buffer_len(void);
E int32_t write_buffer_len(void);

E int sgetc(int s);
E char *sgets(char *buf, int len, int s);
E char *sgets2(char *buf, int len, int s);
E int sread(int s, char *buf, int len);
E int sputs(char *str, int s);
E int sockprintf(int s, char const *fmt,...);
E int conn(struct in_addr const &addr, uint16_t port, 
		struct in_addr const &laddr, uint16_t lport);
E void disconn(int s);

/**** users.cc ****/

E User *userlist[1024];

E uint32_t usercnt, opcnt;

E void set_umode(User *user, int ac, char const *const *av);

E void get_user_stats(long *nusers, long *memuse);
E User *finduser(const char *nick);
E User *firstuser(void);
E User *nextuser(void);

#ifdef HAS_CHGHOST
E void change_user_host(User *user, const char *host);
#endif
#if defined(IRC_ULTIMATE) || defined(IRC_UNREAL)
E void change_user_username(User *user, const char *username);
E void change_user_realname(User *user, const char *realname);
#endif

E User *do_nick(const char *source, char *nick, char *username, char *host,
		char *server, char *realname, time_t ts, uint32_t svid,
		char const *modes, ...);
E void do_umode(const char *source, int ac, char **av);
E void do_quit(const char *source, int ac, char **av);
E void do_kill(const char *source, int ac, char const *const *av);

E int is_oper(User *user);

E int match_usermask(const char *mask, User *user, bool restrict = false);
E void split_usermask(const char *mask, char **nick, char **user, char **host);
E char *create_mask(User *u, bool vhost = true);


#endif	/* EXTERN_H */
