# United States English language file.
#
# Epona (c) 2000-2002, 2004 PegSoft
# Contact us at epona@pegsoft.net
#
# This program is free but copyrighted software; see the file COPYING for
# details.
#
# Based on the original code of Services by Andy Church.

# When translating this file to another language, keep in mind that the
# order of parameters for sprintf() is fixed in the source code, so your
# messages need to take the same parameters in the same order as the
# English messages do.  (Obviously, this doesn't hold for the strftime()
# format lines immediately below.)  If you can't get a natural translation
# of a message without changing the order of the parameters, let me know
# (achurch@dragonfire.net) which message is causing a problem and I'll see
# what I can do.
#
# In help messages, "%S" (capital S, not lowercase) refers to the name of
# the service sending the message; for example, in NickServ help messages,
# "%S" is replaced by "NickServ" (or whatever it is renamed to in
# config.h).  The %S's do not count as sprintf() parameters, so they can be
# rearranged, removed, or added as necessary.
#
# Also in help messages, please try to limit line lengths to 60 characters
# of text (not including the leading tab).  This length was chosen because
# it does not cause line wrap under default settings on most current IRC
# clients.  Remember that format characters (control-B, control-_) are not
# included in that 60-character limit (since they don't show on the user's
# screen).  Also remember that format specifiers (%S, etc.) will expand
# when displayed, so remember to take this into account; you can assume
# that the length of a pseudoclient name (%S replacement) will be eight
# characters, so reduce the maximum line length by 6 for every %S on a
# line.
#
# Finally, remember to put a tab at the beginning of every line of text
# (including empty lines).  This has to be a tab, not spaces.

###########################################################################
#
# Name of this language
#
###########################################################################

# For languages other than English, this string should have the following
# format:
#     language-name-in-language (language-name-in-English)
# For example, "Espaol (Spanish)" or "Franais (French)".

LANG_NAME
	English

###########################################################################
#
# General messages
#
###########################################################################

# strftime() format strings.  man 3 strftime for information on the
# meanings of the format specifiers.  Short version:
#	%a = weekday name (short)	%H = hour
#	%b = month name (short)		%M = minute
#	%d = day of month		%S = second
#	%Y = year			%Z = time zone

# This is used as the format string for strftime() for a date and time
# together.
STRFTIME_DATE_TIME_FORMAT
	%b %d %H:%M:%S %Y %Z
# This is used as the format string for strftime() for a date alone in long
# format (English: including weekday).
STRFTIME_LONG_DATE_FORMAT
	%a %b %d %Y
# This is used as the format string for strftime() for a date alone in
# short format (English: without weekday).
STRFTIME_SHORT_DATE_FORMAT
	%b %d %Y

# These tell our strftime() what the names of months and days are.  If you
# don't use %a, %A, %b, or %B in your strftime() strings above, you can
# leave these empty.  However, if you enter names, they MUST stay in order,
# one per line, and the list MUST be complete!

# %a
STRFTIME_DAYS_SHORT
	Sun
	Mon
	Tue
	Wed
	Thu
	Fri
	Sat
# %A
STRFTIME_DAYS_LONG
	Sunday
	Monday
	Tuesday
	Wednesday
	Thursday
	Friday
	Saturday
# %b
STRFTIME_MONTHS_SHORT
	Jan
	Feb
	Mar
	Apr
	May
	Jun
	Jul
	Aug
	Sep
	Oct
	Nov
	Dec
# %B
STRFTIME_MONTHS_LONG
	January
	February
	March
	April
	May
	June
	July
	August
	September
	October
	November
	December


# This is used in ChanServ/NickServ INFO displays.
COMMA_SPACE
	, 


# Various error messages.
USER_RECORD_NOT_FOUND
	Internal error - unable to process request.
UNKNOWN_COMMAND
	Unknown command %s.
UNKNOWN_COMMAND_HELP
	Unknown command %s.  "/msg %s HELP" for help.
SYNTAX_ERROR
	Syntax: %s
MORE_INFO
	/msg %s HELP %s for more information.
NO_HELP_AVAILABLE
	No help available for %s.
OBSOLETE_COMMAND
	This command is obsolete; use %s from now on.
REMOVED_COMMAND
	This command has been removed; use %s %s from now on.

BAD_USERHOST_MASK
	Mask must be in the form user@host.
BAD_EXPIRY_TIME
	Invalid expiry time.
USERHOST_MASK_TOO_WIDE
	%s coverage is too wide; Please use a more specific mask.

SERVICE_OFFLINE
	%s is currently offline.
READ_ONLY_MODE
	Notice: Services is in read-only mode; changes will not be saved!
PASSWORD_INCORRECT
	Password incorrect.

# What's the difference between "Access denied" and "Permission denied"?
# Very little; you can safely make them the same message with no real loss
# of meaning.  If you want to make a distinction, "Access denied" is
# usually used more generally; for example, a non-oper trying to access
# OperServ gets "Access denied", while a non-Services admin trying to use
# NickServ SET NOEXPIRE gets "Permission denied".
ACCESS_DENIED
	Access denied.
PERMISSION_DENIED
	Permission denied.

MORE_OBSCURE_PASSWORD
	Please try again with a more obscure password.  Passwords should be at least five characters long, should not be something easily guessed (e.g. your real name or your nick), and cannot contain the space or tab characters.
PASSWORD_TRUNCATED
	Notice:  Your password has been truncated to %d characters.

NICK_NOT_REGISTERED
	Your nick isn't registered.
NICK_NOT_REGISTERED_HELP
	Your nick isn't registered.  Type /msg %s HELP for information on registering your nickname.
NICK_X_NOT_REGISTERED
	Nick %s isn't registered.
NICK_X_NOT_IN_USE
	Nick %s isn't currently in use.
NICK_X_NOT_ON_CHAN
	%s is not currently on channel %s.
CHAN_X_NOT_REGISTERED
	Channel %s isn't registered.
CHAN_X_NOT_IN_USE
	Channel %s doesn't exist.
NICK_IDENTIFY_REQUIRED
	Password authentication required for that command.
	Retry after typing /msg %s IDENTIFY password.
CHAN_IDENTIFY_REQUIRED
	Password authentication required for that command.
	Retry after typing /msg %s IDENTIFY %s password.
	
MAIL_DISABLED
	Services have been configured to not send mail.
MAIL_INVALID
	E-mail for %s is invalid.
MAIL_X_INVALID
	%s is not a valid e-mail address.
MAIL_LATER
	Cannot send mail now; please retry a little later.
MAIL_DELAYED
	Please wait %d seconds and retry.
	
NO_REASON
	No reason
UNKNOWN
	<unknown>

# Duration system
DURATION_DAY
	1 day
DURATION_DAYS
	%d days
DURATION_HOUR
	1 hour
DURATION_HOURS
	%d hours
DURATION_MINUTE
	1 minute
DURATION_MINUTES
	%d minutes
DURATION_SECOND
	1 second
DURATION_SECONDS
	%d seconds
	
# Human readable expiration
NO_EXPIRE
	does not expire
EXPIRES_SOON
	expires at next database update
EXPIRES_M
	expires in %d minutes
EXPIRES_1M
	expires in %d minute
EXPIRES_HM
	expires in %d hours, %d minutes
EXPIRES_H1M
	expires in %d hours, %d minute
EXPIRES_1HM
	expires in %d hour, %d minutes
EXPIRES_1H1M
	expires in %d hour, %d minute
EXPIRES_D
	expires in %d days
EXPIRES_1D
	expires in %d day

###########################################################################
#
# NickServ messages
#
###########################################################################

# Automatic messages
NICK_IS_REGISTERED
	This nick is owned by someone else.  Please choose another.
	(If this is your nick, type /msg %s IDENTIFY password.)
NICK_IS_SECURE
	This nickname is registered and protected.  If it is your
	nick, type /msg %s IDENTIFY password.  Otherwise,
	please choose a different nick.
FORCENICKCHANGE_IN_1_MINUTE
	If you do not change within one minute, I will change your nick.
FORCENICKCHANGE_IN_20_SECONDS
	If you do not change within 20 seconds, I will change your nick.
FORCENICKCHANGE_NOW
	This nickname has been registered; you may not use it.
FORCENICKCHANGE_CHANGING
	You are not allowed to use this nickname; it is now being changed to %s.

# REGISTER responses
NICK_REGISTER_SYNTAX
	REGISTER password [email]
NICK_REGISTER_SYNTAX_EMAIL
	REGISTER password email
NICK_REGISTRATION_DISABLED
	Sorry, nickname registration is temporarily disabled.
NICK_REGISTRATION_FAILED
	Sorry, registration failed.
NICK_REG_PLEASE_WAIT
	Please wait %d seconds before using the REGISTER command again.
NICK_CANNOT_BE_REGISTERED
	Nickname %s may not be registered.
NICK_ALREADY_REGISTERED
	Nickname %s is already registered!
NICK_REGISTERED
	Nickname %s registered under your account: %s
NICK_PASSWORD_IS
	Your password is %s - remember this for later use.

# GROUP responses
NICK_GROUP_SYNTAX
	GROUP target password
NICK_GROUP_DISABLED
	Sorry, nickname grouping is temporarily disabled.
NICK_GROUP_PLEASE_WAIT
	Please wait %d seconds before using the GROUP command again.
NICK_GROUP_CHANGE_DISABLED
	Your nick is already registered; type /msg %s DROP first.
NICK_GROUP_SAME
	You are already a member of the group of %s.
NICK_GROUP_NOT_AUTH
	You can't join a group whose e-mail address is not authenticated.
NICK_GROUP_TOO_MANY
	There are too many nicks in %s's group; list them and drop some.
	Type /msg %s HELP GLIST and /msg %s HELP DROP 
	for more information.
NICK_GROUP_FAILED
	Sorry, nickname grouping failed.
NICK_GROUP_JOINED
	You are now in the group of %s.

# IDENTIFY responses
NICK_IDENTIFY_SYNTAX
	IDENTIFY password
NICK_IDENTIFY_NOT_AUTH
	You can't identify to a nick whose e-mail address is not authenticated.
NICK_IDENTIFY_NOT_AUTH_HOWTO
	Type /msg %s HELP AUTH for more information on the authentication process.
NICK_IDENTIFY_SUCCEEDED
	Password accepted - you are now recognized.
NICK_IDENTIFY_EMAIL_REQUIRED
	You must now supply an e-mail for your nick.
	This e-mail will allow you to retrieve your password in
	case you forget it.
NICK_IDENTIFY_EMAIL_HOWTO
	Type /msg %s SET EMAIL e-mail in order to set your e-mail.
	Your privacy is respected; this e-mail won't be given to
	any third-party person.
	
# LOGOUT responses
NICK_LOGOUT_SYNTAX
	LOGOUT
NICK_LOGOUT_SUCCEEDED
	Your nick has been logged out.
NICK_LOGOUT_X_SUCCEEDED
	Nick %s has been logged out.

# AUTH responses
NICK_AUTH_SYNTAX
	AUTH [code | PASS | CANCEL] [code new-password]
NICK_AUTH_UNAVAILABLE
	Sorry, e-mail authentication is disabled.
NICK_AUTH_DISABLED
	Sorry, e-mail authentication is temporarily disabled.
NICK_AUTH_NOT_IN_PROGRESS
	There is no authentication process in progress.
NICK_AUTH_START_EMAIL_MISSING
	You need to set an e-mail address before using this command.
	Type /msg %s HELP SET EMAIL for more information.
NICK_AUTH_START_AUTHENTIFIED
	Your e-mail address is already authenticated.
NICK_AUTH_START_ALREADY
	There is already an authentication process in progress.
NICK_AUTH_START_FAILED
	Sorry, starting the authentication process failed. Please retry later.
NICK_AUTH_START_REQUIRED
	We would like to verify the validity of your e-mail address.
NICK_AUTH_START_REQUIRED_NO_IDENTIFY
	We would like to verify the validity of your e-mail address before allowing you to identify for this nick.
NICK_AUTH_START_DONE
	An e-mail has been sent to %s with instructions on how to finish the authentication process.
NICK_AUTH_START_DONE_PRIVATE
	An e-mail has been sent to you with instructions on how to finish the authentication process.
NICK_AUTH_END_BAD_CODE
	%s is not the correct authentication code.
NICK_AUTH_END_TOO_MANY
	The authentication process has been cancelled due to too many attempts.
NICK_AUTH_END_FAILED
	Sorry, finishing the authentication process failed. Please retry later.
NICK_AUTH_END_DONE
	Your e-mail address is now authenticated.
NICK_AUTH_PASS_UNAVAILABLE
	The AUTH PASS command is not available for your nickname.
NICK_AUTH_CANCEL_DONE
	The authentication process has been cancelled.
NICK_AUTH_MAIL_SUBJECT
	Validation of the e-mail address of your nickname %s
NICK_AUTH_MAIL_HEAD
	Hi,
NICK_AUTH_MAIL_LINE_1
	This message is sent to you as part of the e-mail authentication process used by %s to verify the validity of your e-mail address and reduce the risks of abuses.
NICK_AUTH_MAIL_LINE_2
	To finish the process, please type the following command: /msg %s AUTH %X
NICK_AUTH_MAIL_LINE_2_WEB
	To finish the process, please click on that link: <%s>
NICK_AUTH_MAIL_LINE_3
	If you are not the owner of the nick %s or don't know what this e-mail is about, please ignore it.
NICK_AUTH_MAIL_LINE_4
	Feel free to join channel %s should you have any questions about our privacy and security policies.
NICK_AUTH_MAIL_LINE_5
	Your %s administrators.

# DROP responses
NICK_DROP_DISABLED
	Sorry, nickname de-registration is temporarily disabled.
NICK_DROP_FAILED
	Sorry, nickname de-registration failed.
NICK_DROPPED
	Your nickname has been dropped.
NICK_X_DROPPED
	Nickname %s has been dropped.

# SET responses
NICK_SET_SYNTAX
	SET option parameters
NICK_SET_SERVADMIN_SYNTAX
	SET [=nick] option parameters
NICK_SET_DISABLED
	Sorry, nickname option setting is temporarily disabled.
NICK_SET_UNKNOWN_OPTION
	Unknown SET option %s.
NICK_SET_OPTION_DISABLED
	Option %s is disabled and cannot be set.

# SET GNAME responses
NICK_SET_GNAME_INVALID
	The new name MUST be one of the nicknames in your group!
NICK_SET_GNAME_CHANGED
	The group name has been changed to %s.
NICK_SET_SERVADMIN_GNAME_CHANGED
	The group name of %s has been changed to %s.

# SET PASSWORD responses
NICK_SET_PASSWORD_FAILED
	Sorry, couldn't change password.
NICK_SET_PASSWORD_CHANGED
	Password changed.
NICK_SET_SERVADMIN_PASSWORD_CHANGED
	Password of nickname %s changed.
NICK_SET_PASSWORD_CHANGED_TO
	Password changed to %s.
NICK_SET_SERVADMIN_PASSWORD_CHANGED_TO
	Password of nickname %s changed to %s.

# SET LANGUAGE responses
NICK_SET_LANGUAGE_SYNTAX
	SET LANGUAGE number
NICK_SET_LANGUAGE_UNKNOWN
	Unknown language number %d.  Type /msg %s HELP SET LANGUAGE for a list of languages.
NICK_SET_LANGUAGE_CHANGED
	Language changed to English.
NICK_SET_SERVADMIN_LANGUAGE_CHANGED
	Language of nickname %s changed to English.

# SET URL responses
NICK_SET_URL_CHANGED
	URL changed to %s.
NICK_SET_SERVADMIN_URL_CHANGED
	URL of nickname %s changed to %s.
NICK_SET_URL_UNSET
	URL unset.
NICK_SET_SERVADMIN_URL_UNSET
	URL of nickname %s unset.

# SET EMAIL responses
NICK_SET_EMAIL_CHANGED
	E-mail address changed to %s.
NICK_SET_SERVADMIN_EMAIL_CHANGED
	E-mail address of nickname %s changed to %s.
NICK_SET_EMAIL_UNSET
	E-mail address unset.
NICK_SET_SERVADMIN_EMAIL_UNSET
	E-mail address of nickname %s unset.
NICK_SET_EMAIL_UNSET_IMPOSSIBLE
	Unsetting the e-mail address is disabled on this network.

# SET IM responses
NICK_SET_IM_SYNTAX
	SET IM [program identifier]
NICK_SET_IM_UNSUPPORTED_TYPE
	%s is not a valid instant messenger. Supported programs are: %s.
NICK_SET_IM_CHANGED
	Instant messaging info changed to %s (%s).
NICK_SET_SERVADMIN_IM_CHANGED
	Instant messaging info of nickname %s changed to %s (%s).
NICK_SET_IM_UNSET
	Instant messaging info unset.
NICK_SET_SERVADMIN_IM_UNSET
	Instant messaging info of nickname %s unset.

# SET GREET responses
NICK_SET_GREET_CHANGED
	Greet message changed to %s.
NICK_SET_SERVADMIN_GREET_CHANGED
	Greet message of nickname %s changed to %s.
NICK_SET_GREET_UNSET
	Greet message unset.
NICK_SET_SERVADMIN_GREET_UNSET
	Greet message of nickname %s unset.

# SET PROTECT responses
NICK_SET_KILL_SYNTAX
	SET KILL {ON | QUICK | OFF}
NICK_SET_KILL_IMMED_SYNTAX
	SET KILL {ON | QUICK | IMMED | OFF}
NICK_SET_KILL_ON
	Protection is now ON.
NICK_SET_SERVADMIN_KILL_ON
	Protection of nickname %s is now ON.
NICK_SET_KILL_QUICK
	Protection is now ON, with a reduced delay.
NICK_SET_SERVADMIN_KILL_QUICK
	Protection of nickname %s is now ON, with a reduced delay.
NICK_SET_KILL_IMMED
	Protection is now ON, with no delay.
NICK_SET_SERVADMIN_KILL_IMMED
	Protection of nickname %s is now ON, with no delay.
NICK_SET_KILL_IMMED_DISABLED
	The IMMED option is not available on this network.
NICK_SET_KILL_OFF
	Protection is now OFF.
NICK_SET_SERVADMIN_KILL_OFF
	Protection of nickname %s is now OFF.

# SET SECURE responses
NICK_SET_SECURE_SYNTAX
	SET SECURE {ON | OFF}
NICK_SET_SECURE_ON
	Secure option is now ON.
NICK_SET_SERVADMIN_SECURE_ON
	Secure option of nickname %s is now ON.
NICK_SET_SECURE_OFF
	Secure option is now OFF.
NICK_SET_SERVADMIN_SECURE_OFF
	Secure option of nickname %s is now OFF.

# SET PRIVATE responses
NICK_SET_PRIVATE_SYNTAX
	SET PRIVATE {ON | OFF}
NICK_SET_PRIVATE_ON
	Private option is now ON.
NICK_SET_SERVADMIN_PRIVATE_ON
	Private option of nickname %s is now ON.
NICK_SET_PRIVATE_OFF
	Private option is now OFF.
NICK_SET_SERVADMIN_PRIVATE_OFF
	Private option of nickname %s is now OFF.

# SET HIDE responses
NICK_SET_HIDE_SYNTAX
	SET HIDE {EMAIL | USERMASK | QUIT} {ON | OFF}
NICK_SET_HIDE_EMAIL_ON
	Your E-mail address will now be hidden from %s INFO displays.
NICK_SET_SERVADMIN_HIDE_EMAIL_ON
	E-mail address of nickname %s will now be hidden from %s INFO displays.
NICK_SET_HIDE_EMAIL_OFF
	Your E-mail address will now be shown in %s INFO displays.
NICK_SET_SERVADMIN_HIDE_EMAIL_OFF
	E-mail address of nickname %s will now be shown in %s INFO displays.
NICK_SET_HIDE_MASK_ON
	Your last seen user@host mask will now be hidden from %s INFO displays.
NICK_SET_SERVADMIN_HIDE_MASK_ON
	Last seen user@host mask of nickname %s will now be hidden from %s INFO displays.
NICK_SET_HIDE_MASK_OFF
	Your last seen user@host mask will now be shown in %s INFO displays.
NICK_SET_SERVADMIN_HIDE_MASK_OFF
	Last seen user@host mask of nickname %s will now be shown in %s INFO displays.
NICK_SET_HIDE_QUIT_ON
	Your last quit message will now be hidden from %s INFO displays.
NICK_SET_SERVADMIN_HIDE_QUIT_ON
	Last quit message of nickname %s will now be hidden from %s INFO displays.
NICK_SET_HIDE_QUIT_OFF
	Your last quit message will now be shown in %s INFO displays.
NICK_SET_SERVADMIN_HIDE_QUIT_OFF
	Last quit message of nickname %s will now be shown in %s INFO displays.
	
# SET HOST responses
NICK_SET_HOST_SYNTAX
	SET HOST {ON | OFF}
NICK_SET_HOST_UNSET
	No custom hostname is currently assigned.
NICK_SET_HOST_ON
	Custom hostname is now ON.
NICK_SET_SERVADMIN_HOST_ON
	Custom hostname of nickname %s is now ON.
NICK_SET_HOST_OFF
	Custom hostname is now OFF.
NICK_SET_SERVADMIN_HOST_OFF
	Custom hostname of nickname %s is now OFF.

# SET NOEXPIRE responses
NICK_SET_NOEXPIRE_SYNTAX
	SET [nickname] NOEXPIRE {ON | OFF}
NICK_SET_NOEXPIRE_ON
	Nick %s will not expire.
NICK_SET_NOEXPIRE_OFF
	Nick %s will expire.

# ACCESS responses
NICK_ACCESS_SYNTAX
	ACCESS {ADD | DEL | LIST | CLEAR} {mask | entry-list}
NICK_ACCESS_ADD_SYNTAX
	ACCESS ADD mask
NICK_ACCESS_DEL_SYNTAX
	ACCESS %s {mask | entry-list}
NICK_ACCESS_DISABLED
	Sorry, access list modification is temporarily disabled.
NICK_ACCESS_EMPTY
	The access list is empty.
NICK_ACCESS_MASK_INVALID
	%s is not a valid access mask.
NICK_ACCESS_MASK_NO_NICK
	Reminder: Access masks cannot contain nicknames; make sure you have not included a nick portion in your mask.
NICK_ACCESS_MASK_TOO_LONG
	The mask can't be longer than %u characters.
NICK_ACCESS_ALREADY_PRESENT
	The mask %s is already present on the access list.
NICK_ACCESS_REACHED_LIMIT
	Sorry, you can only have %d access entries for a nickname.
NICK_ACCESS_CREATION_FAILED
	Sorry, creation of a new access failed.
NICK_ACCESS_ADDED
	%s added to the access list.
NICK_ACCESS_NO_SUCH_ENTRY
	No such entry (#%d) on the access list.
NICK_ACCESS_NOT_FOUND
	%s not found on the access list.
NICK_ACCESS_NO_MATCH
	No matching entries on the access list.
NICK_ACCESS_DELETED
	%s deleted from the access list.
NICK_ACCESS_DELETED_ONE
	Deleted 1 entry from the access list.
NICK_ACCESS_DELETED_SEVERAL
	Deleted %d entries from the access list.
NICK_ACCESS_LIST_HEADER
	Current access list:
NICK_ACCESS_LIST_COLHEAD
	  Num   Mask
# mask, reason
NICK_ACCESS_LIST_FORMAT
	  %3d   %s
NICK_ACCESS_LISTED_ONE
	1 entry listed.
NICK_ACCESS_LISTED_SEVERAL
	%d entries listed.
NICK_ACCESS_CLEAR_PARTIALLY
	The access list has been partially cleared.
NICK_ACCESS_CLEAR
	The access list has been cleared.

# INFO responses
NICK_INFO_SYNTAX
	INFO nick [ALL]
NICK_INFO_REALNAME
	%s is %s
NICK_INFO_SERVICES_OPER
	%s is a services operator.
NICK_INFO_SERVICES_ADMIN
	%s is a services admin.
NICK_INFO_ADDRESS
	Last seen address: %s
NICK_INFO_ADDRESS_ONLINE
	   Is online from: %s
NICK_INFO_ADDRESS_ONLINE_NOHOST
	%s is currently online.
NICK_INFO_TIME_REGGED
	  Time registered: %s
NICK_INFO_GROUP_CREATED
	    Group created: %s
NICK_INFO_LAST_SEEN
	   Last seen time: %s
NICK_INFO_LAST_QUIT
	Last quit message: %s
NICK_INFO_URL
	              URL: %s
NICK_INFO_EMAIL
	   E-mail address: %s
NICK_INFO_EMAIL_AUTH
	   E-mail address: %s (authenticated)
NICK_INFO_IM
	%17s: %s
NICK_INFO_GREET
	    Greet message: %s
NICK_INFO_CUSTOM_HOST
	  Custom hostname: %s
NICK_INFO_CUSTOM_HOST_WHO
	  Custom hostname: %s (set by %s on %s)
NICK_INFO_OPTIONS
	          Options: %s
# These strings MUST NOT be empty
NICK_INFO_OPT_KILL
	Protection
NICK_INFO_OPT_SECURE
	Security
NICK_INFO_OPT_PRIVATE
	Private
NICK_INFO_OPT_CUSTOM_HOST
	Custom hostname
NICK_INFO_OPT_NONE
	None
NICK_INFO_PENDING_EXPIRE
	This nickname may expire soon.
NICK_INFO_NO_EXPIRE
	This nickname will not expire.
NICK_INFO_FOR_MORE
	For more verbose information, type /msg %s INFO %s ALL.

# LIST responses
NICK_LIST_SYNTAX
	LIST [[pattern] [range]]
NICK_LIST_SERVADMIN_SYNTAX
	LIST [[pattern] [range] [NOLIMIT] [NOEXPIRE]]
NICK_LIST_INVALID_RANGE
	%s is not a valid results range.
NICK_LIST_NO_NICKS_MATCH
	There are no registered nicknames matching %s.
NICK_LIST_HEADER
	List of registered nicknames matching %s:
NICK_LIST_HEADER_NO_PATTERN
	List of registered nicknames:
NICK_LIST_RESULTS
	End of list - %d/%d results shown.
NICK_LIST_RESULTS_RANGE
	End of list - Results %d to %d shown.
NICK_LIST_RESULTS_RANGE_ONE
	End of list - Result %d shown.

# GLIST responses
NICK_GLIST_HEADER
	List of nicknames in your group %s:
	  Nickname             Expiration date
NICK_GLIST_HEADER_X
	List of nicknames in the group of %s:
	  Nickname             Expiration date
NICK_GLIST_ENTRY
	  %-20s %s
NICK_GLIST_ENTRY_PENDING
	  %-20s Pending
NICK_GLIST_ENTRY_NOEXPIRE
	  %-20s Never
NICK_GLIST_FOOTER
	%d nicknames in the group.

# RECOVER responses
NICK_RECOVER_SYNTAX
	RECOVER nickname [password]
NICK_NO_RECOVER_SELF
	You can't recover yourself!
NICK_RECOVERED
	User claiming your nick has been killed.
	/msg %s RELEASE %s to get it back before the one-minute timeout.

# RELEASE responses
NICK_RELEASE_SYNTAX
	RELEASE nickname [password]
NICK_RELEASE_NOT_HELD
	Nick %s isn't being held.
NICK_RELEASED
	Services' hold on your nick has been released.

# GHOST responses
NICK_GHOST_SYNTAX
	GHOST nickname [password]
NICK_NO_GHOST_SELF
	You can't ghost yourself!
NICK_GHOST_KILLED
	Ghost with your nick has been killed.

# HOST responses
NICK_HOST_SYNTAX
	HOST {SET | LIST} [nick | mask] [custom-host | range]
NICK_HOST_UNAVAILABLE
	Sorry, the HOST command is not available on this network.
NICK_HOST_SET_DISABLED
	Sorry, custom hostname modification is temporarily disabled.
NICK_HOST_TOO_LONG
	Custom hostname is too long (limited to %d characters).
NICK_HOST_INVALID
	Custom hostnames can contain only alphanumeric characters, dots and hyphens and must include at least one dot.
NICK_HOST_TOO_YOUNG
	This group was created too recently. You can't set custom hostname yet.
NICK_HOST_TOO_FAST
	The custom hostname was changed too recently. You can't change it again.
NICK_HOST_UNCHANGED
	Your custom hostname was already %s.
NICK_HOST_UNCHANGED_X
	The custom hostname of %s was already %s.
NICK_HOST_SET_FAILED
	Sorry, the custom hostname could not be changed.
NICK_HOST_CHANGED
	Your custom hostname has been changed to %s.
NICK_HOST_CHANGED_X
	The custom hostname of %s has been changed to %s.
NICK_HOST_UNSET
	Your custom hostname has been unset.
NICK_HOST_UNSET_X
	The custom hostname of %s has been unset.
NICK_HOST_INVALID_RANGE
	%s is not a valid results range.
NICK_HOST_LIST_HEADER
	List of custom hostnames:
NICK_HOST_LIST_HEADER_MATCH
	List of custom hostnames matching %s:
NICK_HOST_LIST_ENTRY
	%-15s  %s (by %s on %s)
NICK_HOST_LIST_RESULTS
	End of list - %d/%d results shown.
NICK_HOST_LIST_RESULTS_RANGE
	End of list - Results %d to %d shown.
NICK_HOST_LIST_RESULTS_RANGE_ONE
	End of list - Result %d shown.

# GETPASS responses
NICK_GETPASS_SYNTAX
	GETPASS nickname
NICK_GETPASS_DISABLED
	Sorry, nickname password retrieval is temporarily disabled.
NICK_GETPASS_UNAVAILABLE
	GETPASS command unavailable because encryption is in use.
NICK_GETPASS_PASSWORD_IS
	Password for %s is %s.
	
# SENDPASS responses
NICK_SENDPASS_SYNTAX
	SENDPASS nickname
NICK_SENDPASS_DISABLED
	Sorry, nickname password retrieval is temporarily disabled.
NICK_SENDPASS_UNAVAILABLE
	The SENDPASS command is unavailable because encryption is in use.
NICK_SENDPASS_UNCONFIGURED
	Sorry, the SENDPASS command is permanently disabled.
NICK_SENDPASS_SUBJECT
	Nickname password (%s)
NICK_SENDPASS_HEAD
	Hi,
NICK_SENDPASS_LINE_1
	You have requested to receive the password of nickname %s by e-mail.
NICK_SENDPASS_LINE_2
	The password is %s. For security purposes, you should change it as soon as you receive this mail.
NICK_SENDPASS_LINE_2_ENCRYPTION
	Unfortunately, for security purposes, we cannot retrieve the password you set. However, you may change it by typing the following command: /msg %s AUTH PASS %X new-password
NICK_SENDPASS_LINE_3
	If you don't know why this mail is sent to you, please ignore it silently.
NICK_SENDPASS_LINE_4
	PLEASE DON'T ANSWER TO THIS MAIL!
NICK_SENDPASS_LINE_5
	%s administrators.
NICK_SENDPASS_FAILURE
	Password of nickname %s could not be sent.
NICK_SENDPASS_FAILURE_TEMPORARY
	Password of nickname %s could not be sent; please retry later.
NICK_SENDPASS_OK
	Password of nickname %s has been sent.

###########################################################################
#
# ChanServ messages
#
###########################################################################

# Access level descriptions
CHAN_LEVEL_AUTOOP
	Automatic channel operator status
CHAN_LEVEL_AUTOVOICE
	Automatic mode +v
CHAN_LEVEL_AUTOHALFOP
	Automatic mode +h
CHAN_LEVEL_AUTOPROTECT
	Automatic mode +a
CHAN_LEVEL_AUTODEOP
	Channel operator status disallowed
CHAN_LEVEL_NOJOIN
	Not allowed to join channel if RESTRICTED
CHAN_LEVEL_INVITE
	Allowed to use INVITE command
CHAN_LEVEL_AKICK
	Allowed to use AKICK command
CHAN_LEVEL_SET
	Allowed to use SET command (not FOUNDER/PASSWORD)
CHAN_LEVEL_CLEAR
	Allowed to use CLEAR command
CHAN_LEVEL_UNBAN
	Allowed to use UNBAN command
CHAN_LEVEL_OPDEOP
	Allowed to use OP/DEOP commands
CHAN_LEVEL_ACCESS_LIST
	Allowed to view the access list
CHAN_LEVEL_ACCESS_CHANGE
	Allowed to modify the access list
CHAN_LEVEL_MEMO
	Allowed to list/read channel memos
CHAN_LEVEL_ASSIGN
	Allowed to assign/unassign a bot
CHAN_LEVEL_BADWORDS
	Allowed to use BADWORDS command
CHAN_LEVEL_NOKICK
	Never kicked by the bot's kickers
CHAN_LEVEL_FANTASIA
	Allowed to use fantaisist commands
CHAN_LEVEL_SAY
	Allowed to use SAY and ACT commands
CHAN_LEVEL_GREET
	Greet message displayed
CHAN_LEVEL_VOICEME
	Allowed to (de)voice him/herself
CHAN_LEVEL_VOICE
	Allowed to use VOICE/DEVOICE commands
CHAN_LEVEL_GETKEY
	Allowed to use GETKEY command
CHAN_LEVEL_OPDEOPME
	Allowed to (de)op him/herself
CHAN_LEVEL_HALFOPME
	Allowed to (de)halfop him/herself
CHAN_LEVEL_HALFOP
	Allowed to use HALFOP/DEHALFOP commands
CHAN_LEVEL_PROTECTME
	Allowed to (de)protect him/herself
CHAN_LEVEL_PROTECT
	Allowed to use PROTECT/DEPROTECT commands
CHAN_LEVEL_KICKME
	Allowed to kick him/herself
CHAN_LEVEL_KICK
	Allowed to use KICK command
CHAN_LEVEL_SIGNKICK
	No signed kick when SIGNKICK LEVEL is used
CHAN_LEVEL_BANME
	Allowed to ban him/herself
CHAN_LEVEL_BAN
	Allowed to use BAN command
CHAN_LEVEL_TOPIC
	Allowed to use TOPIC command
CHAN_LEVEL_INFO
	Allowed to use INFO command with ALL option

# Automatic responses
CHAN_IS_REGISTERED
	This channel has been registered with %s.
CHAN_NOT_ALLOWED_OP
	You are not allowed chanop status on channel %s.
CHAN_NOT_ALLOWED_TO_JOIN
	You are not permitted to be on this channel.

# REGISTER responses
CHAN_REGISTER_SYNTAX
	REGISTER channel password description
CHAN_REGISTER_DISABLED
	Sorry, channel registration is temporarily disabled.
CHAN_REGISTER_NOT_LOCAL
	Local channels cannot be registered.
CHAN_MUST_REGISTER_NICK
	You must register your nickname first.  Type /msg %s HELP for information on registering nicknames.
CHAN_MUST_IDENTIFY_NICK
	Please identify with %s first, using the command:
	/msg %s IDENTIFY password
CHAN_REGISTER_NOT_AUTH
	You need to authenticate your e-mail address to register a channel.
CHAN_REGISTER_NOT_AUTH_HOWTO
	Type /msg %s HELP AUTH for more information on the authentication process.
CHAN_ALREADY_REGISTERED
	Channel %s is already registered!
CHAN_MUST_BE_CHANOP
	You must be a channel operator to register the channel.
CHAN_REACHED_CHANNEL_LIMIT
	Sorry, you have already reached your limit of %d channels.
CHAN_EXCEEDED_CHANNEL_LIMIT
	Sorry, you have already exceeded your limit of %d channels.
CHAN_REGISTRATION_FAILED
	Sorry, registration failed.
CHAN_REGISTERED
	Channel %s registered under your nickname: %s
CHAN_PASSWORD_IS
	Your channel password is %s - remember it for later use.

# IDENTIFY responses
CHAN_IDENTIFY_SYNTAX
	IDENTIFY channel password
CHAN_IDENTIFY_NOT_AUTH
	You need to authenticate your e-mail address to identify for a channel.
CHAN_IDENTIFY_NOT_AUTH_HOWTO
	Type /msg %s HELP AUTH for more information on the authentication process.
CHAN_IDENTIFY_SUCCEEDED
	Password accepted - you now have founder-level access to %s.

# LOGOUT responses
CHAN_LOGOUT_SYNTAX
	LOGOUT channel nickname
CHAN_LOGOUT_SERVADMIN_SYNTAX
	LOGOUT channel [nickname]
CHAN_LOGOUT_SUCCEEDED
	User %s has been logged out of channel %s.
CHAN_LOGOUT_ALL_SUCCEEDED
	All users identified have been logged out of channel %s.

# DROP responses
CHAN_DROP_SYNTAX
	DROP channel
CHAN_DROP_DISABLED
	Sorry, channel de-registration is temporarily disabled.
CHAN_DROP_FAILED
	Sorry, de-registration failed.
CHAN_DROPPED
	Channel %s has been dropped.

# SET responses
CHAN_SET_SYNTAX
	SET channel option parameters
CHAN_SET_DISABLED
	Sorry, channel option setting is temporarily disabled.
CHAN_SET_UNKNOWN_OPTION
	Unknown SET option %s.

# SET FOUNDER responses
CHAN_SET_FOUNDER_NOT_AUTH
	%s is not allowed to be founder because his/her e-mail address is not authenticated.
CHAN_SET_FOUNDER_TOO_MANY_CHANS
	%s has too many channels registered.
CHAN_FOUNDER_CHANGED
	Founder of %s changed to %s.

# SET SUCCESSOR responses
CHAN_SUCCESSOR_IS_FOUNDER
	%s cannot be the successor of channel %s because he's the founder.
CHAN_SET_SUCCESSOR_NOT_AUTH
	%s is not allowed to be successor because his/her e-mail address is not authenticated.
CHAN_SUCCESSOR_CHANGED
	Successor for %s changed to %s.
CHAN_SUCCESSOR_UNSET
	Successor for %s unset.

# SET PASSWORD responses
CHAN_SET_PASSWORD_FAILED
	Sorry, couldn't set password.
CHAN_PASSWORD_CHANGED
	%s password changed.
CHAN_PASSWORD_CHANGED_TO
	%s password changed to %s.

# SET DESC responses
CHAN_DESC_CHANGED
	Description of %s changed to %s.

# SET URL responses
CHAN_URL_CHANGED
	URL for %s changed to %s.
CHAN_URL_UNSET
	URL for %s unset.

# SET EMAIL responses
CHAN_EMAIL_CHANGED
	E-mail address for %s changed to %s.
CHAN_EMAIL_UNSET
	E-mail address for %s unset.

# SET ENTRYMSG responses
CHAN_ENTRY_MSG_CHANGED
	Entry message for %s changed.
CHAN_ENTRY_MSG_UNSET
	Entry message for %s unset.
	
# SET BANTYPE responses
CHAN_SET_BANTYPE_INVALID
	%s is not a valid ban type.
CHAN_SET_BANTYPE_CHANGED
	Ban type for channel %s is now #%d.

# SET MLOCK responses
CHAN_SET_MLOCK_UNKNOWN_CHAR
	Unknown mode character %c ignored.
CHAN_SET_MLOCK_IMPOSSIBLE_CHAR
	Mode %c ignored because you can't lock it.
CHAN_SET_MLOCK_L_REQUIRED
	You must lock mode +l as well to lock mode +L.
CHAN_SET_MLOCK_K_REQUIRED
	You must lock mode +i as well to lock mode +K.
CHAN_MLOCK_CHANGED
	Mode lock on channel %s changed to %s.

# SET KEEPTOPIC responses
CHAN_SET_KEEPTOPIC_SYNTAX
	SET channel KEEPTOPIC {ON | OFF}
CHAN_SET_KEEPTOPIC_ON
	Topic retention option is now ON.
CHAN_SET_KEEPTOPIC_OFF
	Topic retention option is now OFF.

# SET TOPICLOCK responses
CHAN_SET_TOPICLOCK_SYNTAX
	SET channel TOPICLOCK {ON | OFF}
CHAN_SET_TOPICLOCK_ON
	Topic lock option is now ON.
CHAN_SET_TOPICLOCK_OFF
	Topic lock option is now OFF.

# SET PEACE responses
CHAN_SET_PEACE_SYNTAX
	SET channel PEACE {ON | OFF}
CHAN_SET_PEACE_ON
	Peace option is now ON.
CHAN_SET_PEACE_OFF
	Peace option is now OFF.

# SET PRIVATE responses
CHAN_SET_PRIVATE_SYNTAX
	SET channel PRIVATE {ON | OFF}
CHAN_SET_PRIVATE_ON
	Private option is now ON.
CHAN_SET_PRIVATE_OFF
	Private option is now OFF.

# SET SECUREOPS responses
CHAN_SET_SECUREOPS_SYNTAX
	SET channel SECUREOPS {ON | OFF}
CHAN_SET_SECUREOPS_ON
	Secure ops option is now ON.
CHAN_SET_SECUREOPS_OFF
	Secure ops option is now OFF.
	
# SET SECUREFOUNDER responses
CHAN_SET_SECUREFOUNDER_SYNTAX
	SET channel SECUREFOUNDER {ON | OFF}
CHAN_SET_SECUREFOUNDER_ON
	Secure founder option is now ON.
CHAN_SET_SECUREFOUNDER_OFF
	Secure founder option is now OFF.

# SET RESTRICTED responses
CHAN_SET_RESTRICTED_SYNTAX
	SET channel RESTRICTED {ON | OFF}
CHAN_SET_RESTRICTED_ON
	Restricted access option is now ON.
CHAN_SET_RESTRICTED_OFF
	Restricted access option is now OFF.

# SET SECURE responses
CHAN_SET_SECURE_SYNTAX
	SET channel SECURE {ON | OFF}
CHAN_SET_SECURE_ON
	Secure option is now ON.
CHAN_SET_SECURE_OFF
	Secure option is now OFF.
	
# SET SIGNKICK responses
CHAN_SET_SIGNKICK_SYNTAX
	SET channel SIGNKICK {ON | LEVEL | OFF}
CHAN_SET_SIGNKICK_ON
	Signed kick option is now ON.
CHAN_SET_SIGNKICK_LEVEL
	Signed kick option is now ON, but depends of the level of the 
	user that is using the command.
CHAN_SET_SIGNKICK_OFF
	Signed kick option is now OFF.

# SET OPNOTICE responses
CHAN_SET_OPNOTICE_SYNTAX
	SET channel OPNOTICE {ON | OFF}
CHAN_SET_OPNOTICE_ON
	Op-notice option is now ON.
CHAN_SET_OPNOTICE_OFF
	Op-notice option is now OFF.

# SET NOEXPIRE responses
CHAN_SET_NOEXPIRE_SYNTAX
	SET channel NOEXPIRE {ON | OFF}
CHAN_SET_NOEXPIRE_ON
	Channel %s will not expire.
CHAN_SET_NOEXPIRE_OFF
	Channel %s will expire.

# ACCESS messages
CHAN_ACCESS_SYNTAX
	ACCESS channel {ADD | DEL | LIST | CLEAR} [nick [level] | entry-list]
CHAN_ACCESS_ADD_SYNTAX
	ACCESS channel ADD nick level
CHAN_ACCESS_DEL_SYNTAX
	ACCESS channel %s {nick | entry-list}
CHAN_ACCESS_DISABLED
	Sorry, channel access list modification is temporarily disabled.
CHAN_ACCESS_EMPTY
	%s access list is empty.
CHAN_ACCESS_NOT_AUTH
	%s is not allowed to be in the access list because his/her e-mail address is not authenticated.
CHAN_ACCESS_LEVEL_INVALID
	%s is not a valid access level.
CHAN_ACCESS_LEVEL_NONZERO
	Access level must be non-zero.
CHAN_ACCESS_LEVEL_RANGE
	Access level must be between %d and %d inclusive.
CHAN_ACCESS_NICKS_ONLY
	Channel access lists may only contain registered nicknames.
CHAN_ACCESS_REACHED_LIMIT
	Sorry, you can only have %d access entries on a channel.
CHAN_ACCESS_CREATION_FAILED
	Sorry, creation of a new bad word entry failed.
CHAN_ACCESS_LEVEL_UNCHANGED
	Access level for %s on %s unchanged from %d.
CHAN_ACCESS_LEVEL_CHANGED
	Access level for %s on %s changed to %d.
CHAN_ACCESS_ADDED
	%s added to %s access list at level %d.
CHAN_ACCESS_NO_SUCH_ENTRY
	No such entry (#%d) on %s access list.
CHAN_ACCESS_NOT_FOUND
	%s not found on %s access list.
CHAN_ACCESS_NO_MATCH
	No matching entries on %s access list.
CHAN_ACCESS_DELETED
	%s deleted from %s access list.
CHAN_ACCESS_DELETED_ONE
	Deleted 1 entry from %s access list.
CHAN_ACCESS_DELETED_SEVERAL
	Deleted %d entries from %s access list.
CHAN_ACCESS_LIST_HEADER
	Access list for %s:
CHAN_ACCESS_LIST_COLHEAD
	  Num   Lev  Nick
CHAN_ACCESS_LIST_FORMAT
	  %3d  %4d  %s
CHAN_ACCESS_LISTED_ONE
	1 entry listed.
CHAN_ACCESS_LISTED_SEVERAL
	%d entries listed.
CHAN_ACCESS_CLEAR_PARTIALLY
	%s access list has been partially cleared.
CHAN_ACCESS_CLEAR
	%s access list has been cleared.

# AKICK responses
CHAN_AKICK_SYNTAX
	AKICK channel {ADD | STICK | UNSTICK | DEL | LIST | VIEW | ENFORCE | CLEAR} [nick-or-usermask [reason] | entry-list]
CHAN_AKICK_ADD_SYNTAX
	AKICK channel ADD nick-or-usermask [reason]
CHAN_AKICK_DEL_SYNTAX
	AKICK channel %s {mask | entry-list}
CHAN_AKICK_STICK_SYNTAX
	AKICK channel %s mask
CHAN_AKICK_DISABLED
	Sorry, channel autokick list modification is temporarily disabled.
CHAN_AKICK_EMPTY
	%s autokick list is empty.
CHAN_AKICK_MASK_TOO_LONG
	The mask can't be longer than %u characters.
CHAN_AKICK_REASON_TOO_LONG
	The reason can't be longer than %u characters.
CHAN_AKICK_ALREADY_PRESENT
	%s is already present on %s autokick list.
CHAN_AKICK_REACHED_LIMIT
	Sorry, you can only have %d entries on a channel autokick list.
CHAN_AKICK_CREATION_FAILED
	Sorry, creation of a new autokick entry failed.
CHAN_AKICK_ADDED
	%s added to %s autokick list.
CHAN_AKICK_NO_SUCH_ENTRY
	No such entry (#%d) on %s autokick list.
CHAN_AKICK_NOT_FOUND
	%s not found on %s autokick list.
CHAN_AKICK_NO_MATCH
	No matching entries on %s autokick list.
CHAN_AKICK_STUCK
	%s is now always active on channel %s.
CHAN_AKICK_UNSTUCK
	%s is not always active anymore on channel %s.
CHAN_AKICK_DELETED
	%s deleted from %s autokick list.
CHAN_AKICK_DELETED_ONE
	Deleted 1 entry from %s autokick list.
CHAN_AKICK_DELETED_SEVERAL
	Deleted %d entries from %s autokick list.
CHAN_AKICK_LIST_HEADER
	Autokick list for %s:
CHAN_AKICK_LIST_FORMAT
	  %3d %s (%s)
CHAN_AKICK_VIEW_FORMAT
	%3d %s (by %s on %s; %s)
	    %s
CHAN_AKICK_VIEW_FORMAT_STUCK
	%3d %s (by %s on %s; stuck)
	    %s
CHAN_AKICK_LISTED_ONE
	1 entry listed.
CHAN_AKICK_LISTED_SEVERAL
	%d entries listed.
CHAN_AKICK_ENFORCE_DONE
	AKICK ENFORCE for %s complete; %d users were affected.
CHAN_AKICK_CLEAR_PARTIALLY
	%s autokick list has been partially cleared.
CHAN_AKICK_CLEAR
	%s autokick list has been cleared.

# LEVELS responses
CHAN_LEVELS_SYNTAX
	LEVELS channel {SET | DIS[ABLE] | LIST | RESET} [item [level]]
CHAN_LEVELS_RANGE
	Level must be between %d and %d inclusive.
CHAN_LEVELS_CHANGED
	Level for %s on channel %s changed to %d.
CHAN_LEVELS_UNKNOWN
	Setting %s not known.  Type /msg %s HELP LEVELS DESC for a list of valid settings.
CHAN_LEVELS_DISABLED
	%s disabled on channel %s.
CHAN_LEVELS_LIST_HEADER
	Access level settings for channel %s:
CHAN_LEVELS_LIST_DISABLED
	    %-*s  (disabled)
CHAN_LEVELS_LIST_FOUNDER
	    %-*s  (founder only)
CHAN_LEVELS_LIST_NORMAL
	    %-*s  %d
CHAN_LEVELS_RESET
	Access levels for %s reset to defaults.

# INFO responses
CHAN_INFO_SYNTAX
	INFO channel [ALL]
CHAN_INFO_HEADER
	Information for channel %s:
CHAN_INFO_FOUNDER
	        Founder: %s (%s)
CHAN_INFO_NO_FOUNDER
	        Founder: %s
CHAN_INFO_SUCCESSOR
	      Successor: %s (%s)
CHAN_INFO_NO_SUCCESSOR
	      Successor: %s
CHAN_INFO_DESCRIPTION
	    Description: %s
CHAN_INFO_ENTRYMSG
	  Entry message: %s
CHAN_INFO_TIME_REGGED
	     Registered: %s
CHAN_INFO_LAST_USED
	      Last used: %s
CHAN_INFO_LAST_TOPIC
	     Last topic: %s
CHAN_INFO_TOPIC_SET_BY
	   Topic set by: %s
CHAN_INFO_URL
	            URL: %s
CHAN_INFO_EMAIL
	 E-mail address: %s
CHAN_INFO_BANTYPE
	       Ban type: %d
CHAN_INFO_OPTIONS
	        Options: %s
CHAN_INFO_OPT_KEEPTOPIC
	Topic Retention
CHAN_INFO_OPT_OPNOTICE
	OP Notice
CHAN_INFO_OPT_PEACE
	Peace
CHAN_INFO_OPT_PRIVATE
	Private
CHAN_INFO_OPT_RESTRICTED
	Restricted Access
CHAN_INFO_OPT_SECURE
	Secure
CHAN_INFO_OPT_SECUREOPS
	Secure Ops
CHAN_INFO_OPT_SECUREFOUNDER
	Secure Founder
CHAN_INFO_OPT_SIGNKICK
	Signed kicks
CHAN_INFO_OPT_TOPICLOCK
	Topic Lock
CHAN_INFO_OPT_NONE
	None
CHAN_INFO_MODE_LOCK
	      Mode lock: %s
CHAN_INFO_PENDING_EXPIRE
	This channel may expire soon.
CHAN_INFO_NO_EXPIRE
	This channel will not expire.

# LIST responses
CHAN_LIST_SYNTAX
	LIST [[pattern] [range]]
CHAN_LIST_SERVADMIN_SYNTAX
	LIST [[pattern] [range] [NOLIMIT] [NOEXPIRE]]
CHAN_LIST_INVALID_RANGE
	%s is not a valid results range.
CHAN_LIST_NO_NICKS_MATCH
	There are no registered channels matching %s.
CHAN_LIST_HEADER
	List of registered channels matching %s:
CHAN_LIST_HEADER_NO_PATTERN
	List of registered channels:
CHAN_LIST_RESULTS
	End of list - %d/%d results shown.
CHAN_LIST_RESULTS_RANGE
	End of list - Results %d to %d shown.
CHAN_LIST_RESULTS_RANGE_ONE
	End of list - Result %d shown.
	
# CLIST responses
CHAN_CLIST_SYNTAX
	CLIST [privilege]
CHAN_CLIST_SYNTAX_X
	CLIST [nickname] [privilege]
CHAN_CLIST_HEADER
	Channels privileges list:
	   Channel               Level      Description
CHAN_CLIST_HEADER_X
	Channels privileges list of %s:
	   Channel               Level      Description
CHAN_CLIST_FORMAT
	  %c%-20s  %-9d  %s
CHAN_CLIST_FORMAT_STR
	  %c%-20s  %-9s  %s
CHAN_CLIST_FORMAT_STR_FOUNDER
	Founder
CHAN_CLIST_FORMAT_STR_SUCCESSOR
	Successor
CHAN_CLIST_FOOTER
	%d channels listed.

# INVITE responses
CHAN_INVITE_SYNTAX
	INVITE channel

# UNBAN responses
CHAN_UNBAN_SYNTAX
	UNBAN channel
CHAN_UNBANNED
	You have been unbanned from %s.

# TOPIC responses
CHAN_TOPIC_SYNTAX
	TOPIC channel [topic]
CHAN_TOPIC_DISABLED
	Sorry, topic modification is temporarily disabled.
CHAN_TOPIC_FAILED
	Sorry, topic modification failed.

# CLEAR responses
CHAN_CLEAR_SYNTAX
	CLEAR channel what
CHAN_CLEARED_BANS
	All bans on channel %s have been removed.
CHAN_CLEARED_MODES
	All modes on channel %s have been reset.
CHAN_CLEARED_OPS
	Mode +o has been cleared from channel %s.
CHAN_CLEARED_VOICES
	Mode +v has been cleared from channel %s.
CHAN_CLEARED_USERS
	All users have been kicked from channel %s.

# GETPASS responses
CHAN_GETPASS_SYNTAX
	GETPASS channel
CHAN_GETPASS_DISABLED
	Sorry, channel password retrieval is temporarily disabled.
CHAN_GETPASS_UNAVAILABLE
	GETPASS command unavailable because encryption is in use.
CHAN_GETPASS_PASSWORD_IS
	Password for channel %s is %s.
	
# SENDPASS responses
CHAN_SENDPASS_SYNTAX
	SENDPASS channel
CHAN_SENDPASS_UNAVAILABLE
	SENDPASS command unavailable because encryption is in use.
CHAN_SENDPASS_UNCONFIGURED
	Sorry, the SENDPASS command is permanently disabled.
CHAN_SENDPASS_SUBJECT
	Channel password (%s)
CHAN_SENDPASS_HEAD
	Hi,
CHAN_SENDPASS_LINE_1
	You have requested to receive the password of channel %s by e-mail.
CHAN_SENDPASS_LINE_2
	The password is %s. For security reasons, you should change it as soon as you receive this mail.
CHAN_SENDPASS_LINE_3
	If you don't know why this mail is sent to you, please ignore it silently.
CHAN_SENDPASS_LINE_4
	PLEASE DON'T ANSWER TO THIS MAIL!
CHAN_SENDPASS_LINE_5
	%s administrators.
CHAN_SENDPASS_FAILURE
	Password of channel %s could not be sent.
CHAN_SENDPASS_FAILURE_TEMPORARY
	Password of channel %s could not be sent; please retry later.
CHAN_SENDPASS_OK
	Password of %s has been sent.

###########################################################################
#
# MemoServ messages
#
###########################################################################

# Automatic messages
MEMO_HAVE_NEW_MEMO
	You have 1 new memo.
MEMO_HAVE_NEW_MEMOS
	You have %d new memos.
MEMO_TYPE_READ_LAST
	Type /msg %s READ LAST to read it.
MEMO_TYPE_READ_NUM
	Type /msg %s READ %d to read it.
MEMO_TYPE_LIST_NEW
	Type /msg %s LIST NEW to list them.
MEMO_AT_LIMIT
	Warning: You have reached your maximum number of memos (%d).  You will be unable to receive any new memos until you delete some of your current ones.
MEMO_OVER_LIMIT
	Warning: You are over your maximum number of memos (%d).  You will be unable to receive any new memos until you delete some of your current ones.
MEMO_NEW_MEMO_ARRIVED
	You have a new memo from %s.
	Type /msg %s READ %d to read it.

# Multi-use responses
MEMO_HAVE_NO_MEMOS
	You have no memos.
MEMO_DOES_NOT_EXIST
	Memo %d does not exist!
MEMO_LIST_NOT_FOUND
	No matching memos found.

# SEND responses
MEMO_SEND_SYNTAX
	SEND nick memo-text
MEMO_SEND_DISABLED
	Sorry, memo sending is temporarily disabled.
MEMO_SEND_NOT_AUTH
	You need to authenticate your e-mail address to send a memo.
MEMO_SEND_NOT_AUTH_HOWTO
	Type /msg %s HELP AUTH for more information on the authentication process.
MEMO_SEND_TARGET_NOT_AUTH
	%s cannot receive memos because his/her e-mail address is not authenticated.
MEMO_SEND_PLEASE_WAIT
	Please wait %d seconds before using the SEND command again.
MEMO_SEND_CHANNEL_TARGET_DISABLED
	Sorry, memo sending to channel target is disabled.
MEMO_X_GETS_NO_MEMOS
	%s cannot receive memos.
MEMO_X_HAS_TOO_MANY_MEMOS
	%s currently has too many memos and cannot receive more.
MEMO_SEND_FAILED
	Sorry, memo sending failed.
MEMO_SENT
	Memo sent to %s.
	
# CANCEL responses
MEMO_CANCEL_SYNTAX
	CANCEL nick
MEMO_CANCEL_DISABLED
	Sorry, memo cancellation is temporarily disabled.
MEMO_CANCEL_NONE
	No memo was cancellable.
MEMO_CANCEL_FAILED
	Sorry, memo cancellation failed.
MEMO_CANCELLED
	Last memo to %s has been cancelled.

# LIST responses
MEMO_LIST_SYNTAX
	LIST [list | NEW]
MEMO_HAVE_NO_NEW_MEMOS
	You have no new memos.
MEMO_LIST_MEMOS
	Memos for %s.  To read, type: /msg %s READ num
MEMO_LIST_NEW_MEMOS
	New memos for %s.  To read, type: /msg %s READ num
MEMO_LIST_HEADER
	 Num  Sender            Date/Time
MEMO_LIST_FORMAT
	%c%3d  %-16s  %s

# READ responses
MEMO_READ_SYNTAX
	READ {list | LAST | NEW}
MEMO_HEADER
	Memo %d from %s (%s).  To delete, type: /msg %s DEL %d
MEMO_HEADER_TARGET
	Memo %d from %s to %s (%s).  To delete, type: /msg %s DEL %d
MEMO_TEXT
	%s

# DEL responses
MEMO_DEL_SYNTAX
	DEL {num | list | ALL}
MEMO_DEL_DISABLED
	Sorry, memo removal is temporarily disabled.
MEMO_DEL_FAILED
	Sorry, memo removal failed.
MEMO_DELETED_NONE
	No memos were deleted.
MEMO_DELETED_ONE
	Memo %d has been deleted.
MEMO_DELETED_SEVERAL
	Memos %s have been deleted.
MEMO_DELETED_ALL
	All of your memos have been deleted.

# SET responses
MEMO_SET_SYNTAX
	SET option parameters
MEMO_SET_DISABLED
	Sorry, memo option setting is temporarily disabled.
MEMO_SET_UNKNOWN_OPTION
	Unknown SET option %s.

# SET NOTIFY responses
MEMO_SET_NOTIFY_SYNTAX
	SET NOTIFY {ON | LOGON | NEW | OFF}
MEMO_SET_NOTIFY_ON
	%s will now notify you of memos when you log on and when they are sent to you.
MEMO_SET_NOTIFY_LOGON
	%s will now notify you of memos when you log on or unset /AWAY.
MEMO_SET_NOTIFY_NEW
	%s will now notify you of memos when they are sent to you.
MEMO_SET_NOTIFY_OFF
	%s will not send you any notification of memos.

# SET LIMIT responses
MEMO_SET_LIMIT_SYNTAX
	SET LIMITlimit
MEMO_SET_LIMIT_SERVADMIN_SYNTAX
	SET LIMIT [user] {limit | NONE} [HARD]
MEMO_SET_YOUR_LIMIT_FORBIDDEN
	You are not permitted to change your memo limit.
MEMO_SET_YOUR_LIMIT_TOO_HIGH
	You cannot set your memo limit higher than %d.
MEMO_SET_LIMIT_OVERFLOW
	Memo limit too large; limiting to %d instead.
MEMO_SET_YOUR_LIMIT
	Your memo limit has been set to %d.
MEMO_SET_YOUR_LIMIT_ZERO
	You will no longer be able to receive memos.
MEMO_UNSET_YOUR_LIMIT
	Your memo limit has been disabled.
MEMO_SET_LIMIT
	Memo limit for %s set to %d.
MEMO_SET_LIMIT_ZERO
	Memo limit for %s set to 0.
MEMO_UNSET_LIMIT
	Memo limit disabled for %s.

# SET FORWARD responses
MEMO_SET_FORWARD_SYNTAX
	SET FORWARD {ON | OFF}
MEMO_SET_FORWARD_UNAVAILABLE
	Sorry, memo forwarding is disabled.
MEMO_SET_FORWARD_NO_EMAIL
	You have no e-mail address configured.
	Type /msg %s HELP SET EMAIL for more information.
MEMO_SET_FORWARD_NOT_AUTH
	Your e-mail address needs to be authenticated.
	Type /msg %s HELP AUTH for more information.
MEMO_SET_FORWARD_ON
	You will now get a copy of your new memos by e-mail.
MEMO_SET_FORWARD_OFF
	You will no longer get a copy of your new memos by e-mail.
MEMO_SET_FORWARD_SUBJECT
	You got a new memo on %s !
MEMO_SET_FORWARD_SUBJECT_TARGET
	You got a new %s memo on %s !

# INFO responses
MEMO_INFO_SYNTAX
	INFO
MEMO_INFO_SERVADMIN_SYNTAX
	INFO [nick]
MEMO_INFO_NO_MEMOS
	You currently have no memos.
MEMO_INFO_MEMO
	You currently have 1 memo.
MEMO_INFO_MEMO_UNREAD
	You currently have 1 memo, and it has not yet been read.
MEMO_INFO_MEMOS
	You currently have %d memos.
MEMO_INFO_MEMOS_ONE_UNREAD
	You currently have %d memos, of which 1 is unread.
MEMO_INFO_MEMOS_SOME_UNREAD
	You currently have %d memos, of which %d are unread.
MEMO_INFO_MEMOS_ALL_UNREAD
	You currently have %d memos; all of them are unread.
MEMO_INFO_LIMIT
	Your memo limit is %d.
MEMO_INFO_HARD_LIMIT
	Your memo limit is %d, and may not be changed.
MEMO_INFO_LIMIT_ZERO
	Your memo limit is 0; you will not receive any new memos.
MEMO_INFO_HARD_LIMIT_ZERO
	Your memo limit is 0; you will not receive any new memos.  You cannot change this limit.
MEMO_INFO_NO_LIMIT
	You have no limit on the number of memos you may keep.
MEMO_INFO_NOTIFY_OFF
	You will not be notified of new memos.
MEMO_INFO_NOTIFY_ON
	You will be notified of new memos at logon and when they arrive.
MEMO_INFO_NOTIFY_RECEIVE
	You will be notified when new memos arrive.
MEMO_INFO_NOTIFY_SIGNON
	You will be notified of new memos at logon.
MEMO_INFO_FORWARD_ON
	You will get a copy of new memos by e-mail.
MEMO_INFO_FORWARD_OFF
	You will not get a copy of new memos by e-mail.
MEMO_INFO_X_NO_MEMOS
	%s currently has no memos.
MEMO_INFO_X_MEMO
	%s currently has 1 memo.
MEMO_INFO_X_MEMO_UNREAD
	%s currently has 1 memo, and it has not yet been read.
MEMO_INFO_X_MEMOS
	%s currently has %d memos.
MEMO_INFO_X_MEMOS_ONE_UNREAD
	%s currently has %d memos, of which 1 is unread.
MEMO_INFO_X_MEMOS_SOME_UNREAD
	%s currently has %d memos, of which %d are unread.
MEMO_INFO_X_MEMOS_ALL_UNREAD
	%s currently has %d memos; all of them are unread.
MEMO_INFO_X_LIMIT
	%s's memo limit is %d.
MEMO_INFO_X_HARD_LIMIT
	%s's memo limit is %d, and may not be changed.
MEMO_INFO_X_NO_LIMIT
	%s has no memo limit.
MEMO_INFO_X_NOTIFY_OFF
	%s is not notified of new memos.
MEMO_INFO_X_NOTIFY_ON
	%s is notified of new memos at logon and when they arrive.
MEMO_INFO_X_NOTIFY_RECEIVE
	%s is notified when new memos arrive.
MEMO_INFO_X_NOTIFY_SIGNON
	%s is notified of news memos at logon.
MEMO_INFO_X_FORWARD_ON
	%s gets a copy of new memos by e-mail.
MEMO_INFO_X_FORWARD_OFF
	%s does not get a copy of new memos by e-mail.
	
###########################################################################
#
# BotServ messages
#
###########################################################################
	
# Standard responses
BOT_DOES_NOT_EXIST
	Bot %s does not exist.
BOT_NOT_ASSIGNED
	You must assign a bot to the channel before using this command.
	Type /msg %S HELP ASSIGN for more information.
BOT_NOT_ON_CHANNEL
	Bot is not on channel %s.
	
# Kick reasons (must be a single line)
BOT_REASON_BADWORD
	Don't use word %s on this channel!
BOT_REASON_BADWORD_GENTLE
	Watch your language!
BOT_REASON_BOLD
	Don't use bolds on this channel!
BOT_REASON_CAPS
	Turn caps lock OFF!
BOT_REASON_COLOR
	Don't use colors on this channel!
BOT_REASON_FLOOD
	Stop flooding!
BOT_REASON_REPEAT
	Stop repeating yourself!
BOT_REASON_REVERSE
	Don't use reverses on this channel!
BOT_REASON_UNDERLINE
	Don't use underlines on this channel!
	
# !seen replies
BOT_SEEN_BOT
	You found me, %s!
BOT_SEEN_YOU
	Looking for yourself, eh %s?
BOT_SEEN_ON_CHANNEL
	%s is on the channel right now!
BOT_SEEN_ON_CHANNEL_AS
	%s is on the channel right now (as %s) ! 
BOT_SEEN_ON
	%s was last seen here %s ago.
BOT_SEEN_NEVER
	I've never seen %s on this channel.
BOT_SEEN_UNKNOWN
	I don't know who %s is.
	
# BOT responses
BOT_BOT_SYNTAX
	BOT {ADD | CHANGE | DEL | LIST} [[oldnick] nick [username [hostname [realname]]]]
BOT_BOT_ALREADY_EXISTS
	Bot %s already exists.
BOT_BOT_CREATION_FAILED
	Sorry, bot creation failed.
BOT_BOT_READONLY
	Sorry, bot modification is temporarily disabled.
BOT_BOT_ADDED
	%s!%s@%s (%s) added to the bot list.
BOT_BOT_ANY_CHANGES
	Old info is equal to the new one.
BOT_BOT_CHANGED
	Bot %s has been changed to %s!%s@%s (%s)
BOT_BOT_DELETION_FAILED
	Sorry, bot deletion failed.
BOT_BOT_DELETED
	Bot %s has been deleted.
	
# BOTLIST responses
BOT_BOTLIST_HEADER
	Bot list:
BOT_BOTLIST_PRIVATE_HEADER
	Bots reserved to IRC operators:
BOT_BOTLIST_FOOTER
	%d bots available.
BOT_BOTLIST_EMPTY
	There are no bots available at this time.
	Ask a services admin to create one!
	
# ASSIGN responses
BOT_ASSIGN_SYNTAX
	ASSIGN channel nick
BOT_ASSIGN_READONLY
	Sorry, bot assignment is temporarily disabled.
BOT_ASSIGN_ALREADY
	Channel %s has already a bot assigned.
BOT_ASSIGN_ASSIGNED
	Bot %s has been assigned to %s.
	
# UNASSIGN responses
BOT_UNASSIGN_SYNTAX
	UNASSIGN channel
BOT_UNASSIGN_UNASSIGNED
	There is no bot assigned to %s anymore.
	
# INFO responses
BOT_INFO_SYNTAX
	INFO {channel | nick}
BOT_INFO_NOT_FOUND
	%s is not a valid bot or registered channel.
BOT_INFO_BOT_HEADER
	Information for bot %s:
BOT_INFO_BOT_MASK
	       Mask : %s@%s
BOT_INFO_BOT_REALNAME
	  Real name : %s
BOT_INFO_BOT_CREATED
	    Created : %s
BOT_INFO_BOT_USAGE
	    Used on : %d channel(s)
BOT_INFO_BOT_OPTIONS
	    Options : %s
BOT_INFO_OPT_PRIVATE
	Private
BOT_INFO_CHAN_HEADER
	Information for channel %s:
BOT_INFO_CHAN_BOT
	           Bot nick : %s
BOT_INFO_CHAN_BOT_NONE
	           Bot nick : not assigned yet.
BOT_INFO_CHAN_KICK_BADWORDS
	   Bad words kicker : %s
BOT_INFO_CHAN_KICK_BADWORDS_BAN
	   Bad words kicker : %s (%d kick(s) before ban)
BOT_INFO_CHAN_KICK_BOLDS
	       Bolds kicker : %s
BOT_INFO_CHAN_KICK_BOLDS_BAN
	       Bolds kicker : %s (%d kick(s) before ban)
BOT_INFO_CHAN_KICK_CAPS_ON
	        Caps kicker : %s (minimum %d/%d%%)
BOT_INFO_CHAN_KICK_CAPS_BAN
	        Caps kicker : %s (%d kick(s) before ban; minimum %d/%d%%)
BOT_INFO_CHAN_KICK_CAPS_OFF
	        Caps kicker : %s
BOT_INFO_CHAN_KICK_COLORS
	      Colors kicker : %s
BOT_INFO_CHAN_KICK_COLORS_BAN
	      Colors kicker : %s (%d kick(s) before ban)
BOT_INFO_CHAN_KICK_FLOOD_ON
	       Flood kicker : %s (%d lines in %ds)
BOT_INFO_CHAN_KICK_FLOOD_BAN
	       Flood kicker : %s (%d kick(s) before ban; %d lines in %ds)
BOT_INFO_CHAN_KICK_FLOOD_OFF
	       Flood kicker : %s
BOT_INFO_CHAN_KICK_REPEAT_ON
	      Repeat kicker : %s (%d times)
BOT_INFO_CHAN_KICK_REPEAT_BAN
	      Repeat kicker : %s (%d kick(s) before ban; %d times)
BOT_INFO_CHAN_KICK_REPEAT_OFF
	      Repeat kicker : %s
BOT_INFO_CHAN_KICK_REVERSES
	    Reverses kicker : %s
BOT_INFO_CHAN_KICK_REVERSES_BAN
	    Reverses kicker : %s (%d kick(s) before ban)
BOT_INFO_CHAN_KICK_UNDERLINES
	  Underlines kicker : %s
BOT_INFO_CHAN_KICK_UNDERLINES_BAN
	  Underlines kicker : %s (%d kick(s) before ban)
BOT_INFO_ACTIVE
	enabled
BOT_INFO_INACTIVE
	disabled
BOT_INFO_CHAN_OPTIONS
	            Options : %s
BOT_INFO_OPT_DONTKICKOPS
	Ops protection
BOT_INFO_OPT_DONTKICKVOICES
	Voices protection
BOT_INFO_OPT_FANTASY
	Fantasy
BOT_INFO_OPT_GREET
	Greet
BOT_INFO_OPT_NOBOT
	No bot
BOT_INFO_OPT_SYMBIOSIS
	Symbiosis
BOT_INFO_OPT_NONE
	None
	
# SET responses
BOT_SET_SYNTAX
	SET channel option settings
BOT_SET_DISABLED
	Sorry, bot option setting is temporarily disabled.
BOT_SET_UNKNOWN
	Unknown option %s.
	Type /msg %S HELP SET for more information.

# SET DONTKICKOPS messages
BOT_SET_DONTKICKOPS_SYNTAX
	SET channel DONTKICKOPS {ON|OFF}
BOT_SET_DONTKICKOPS_ON
	Bot won't kick ops on channel %s.
BOT_SET_DONTKICKOPS_OFF
	Bot will kick ops on channel %s.
	
# SET DONTKICKVOICES messages
BOT_SET_DONTKICKVOICES_SYNTAX
	SET channel DONTKICKVOICES {ON|OFF}
BOT_SET_DONTKICKVOICES_ON
	Bot won't kick voices on channel %s.
BOT_SET_DONTKICKVOICES_OFF
	Bot will kick voices on channel %s.

# SET FANTASY messages
BOT_SET_FANTASY_SYNTAX
	SET channel FANTASY {ON|OFF}
BOT_SET_FANTASY_ON
	Fantasy mode is now ON on channel %s.
BOT_SET_FANTASY_OFF
	Fantasy mode is now OFF on channel %s.
	
# SET GREET messages
BOT_SET_GREET_SYNTAX
	SET channel GREET {ON|OFF}
BOT_SET_GREET_ON
	Greet mode is now ON on channel %s.
BOT_SET_GREET_OFF
	Greet mode is now OFF on channel %s.
	
# SET NOBOT
BOT_SET_NOBOT_SYNTAX
	SET channel NOBOT {ON|OFF}
BOT_SET_NOBOT_ON
	No Bot mode is now ON on channel %s.
BOT_SET_NOBOT_OFF
	No Bot mode is now OFF on channel %s.

# SET PRIVATE
BOT_SET_PRIVATE_SYNTAX
	SET channel PRIVATE {ON|OFF}
BOT_SET_PRIVATE_ON
	Private mode of bot %s is now ON.
BOT_SET_PRIVATE_OFF
	Private mode of bot %s is now OFF.

# SET SYMBIOSIS
BOT_SET_SYMBIOSIS_SYNTAX
	SET channel SYMBIOSIS {ON|OFF}
BOT_SET_SYMBIOSIS_ON
	Symbiosis mode is now ON on channel %s.
BOT_SET_SYMBIOSIS_OFF
	Symbiosis mode is now OFF on channel %s.
	
# KICK responses
BOT_KICK_SYNTAX
	KICK channel option {ON|OFF} [settings]
BOT_KICK_DISABLED
	Sorry, kicker configuration is temporarily disabled.
BOT_KICK_UNKNOWN
	Unknown option %s.
	Type /msg %S HELP KICK for more information.
BOT_KICK_BAD_TTB
	%s is not a valid number of kicks before ban.

BOT_KICK_BADWORDS_ON
	Bot will now kick bad words. Use the BADWORDS command
	to add or remove a bad word.
BOT_KICK_BADWORDS_ON_BAN
	Bot will now kick bad words, and will make a ban after 
	%d kicks for the same user. Use the BADWORDS command
	to add or remove a bad word.
BOT_KICK_BADWORDS_OFF
	Bot won't kick bad words anymore.

BOT_KICK_BOLDS_ON
	Bot will now kick bolds.
BOT_KICK_BOLDS_ON_BAN
	Bot will now kick bolds, and will make a ban after 
	%d kicks for the same user.
BOT_KICK_BOLDS_OFF
	Bot won't kick bolds anymore.

BOT_KICK_CAPS_ON
	Bot will now kick caps (they must constitute at least
	%d characters and %d%% of the entire message).
BOT_KICK_CAPS_ON_BAN
	Bot will now kick caps (they must constitute at least
	%d characters and %d%% of the entire message), and will 
	make a ban after %d kicks for the same user.
BOT_KICK_CAPS_ON_INVALID_MIN
	The minimum number of caps must be between %d and %d inclusive.
BOT_KICK_CAPS_ON_INVALID_PERCENT
	The percentage of caps must be between %d and %d inclusive.
BOT_KICK_CAPS_OFF
	Bot won't kick caps anymore.

BOT_KICK_COLORS_ON
	Bot will now kick colors.
BOT_KICK_COLORS_ON_BAN
	Bot will now kick colors, and will make a ban after %d 
	kicks for the same user.
BOT_KICK_COLORS_OFF
	Bot won't kick colors anymore.

BOT_KICK_FLOOD_ON
	Bot will now kick flood (%d lines in %d seconds).
BOT_KICK_FLOOD_ON_BAN
	Bot will now kick flood (%d lines in %d seconds), and 
	will make a ban after %d kicks for the same user.
BOT_KICK_FLOOD_ON_INVALID_LINES
	The number of lines must be between %d and %d inclusive.
BOT_KICK_FLOOD_ON_INVALID_SECONDS
	The number of seconds must be between %d and %d inclusive.
BOT_KICK_FLOOD_OFF
	Bot won't kick flood anymore.

BOT_KICK_REPEAT_ON
	Bot will now kick repeats (users that say %d times
	the same thing).
BOT_KICK_REPEAT_ON_BAN
	Bot will now kick repeats (users that say %d times
	the same thing), and will make a ban after %d 
	kicks for the same user.
BOT_KICK_REPEAT_ON_INVALID_TIMES
	The number of times must be between %d and %d inclusive.
BOT_KICK_REPEAT_OFF
	Bot won't kick repeats anymore.

BOT_KICK_REVERSES_ON
	Bot will now kick reverses.
BOT_KICK_REVERSES_ON_BAN
	Bot will now kick reverses, and will make a ban after %d 
	kicks for the same user.
BOT_KICK_REVERSES_OFF
	Bot won't kick reverses anymore.

BOT_KICK_UNDERLINES_ON
	Bot will now kick underlines.
BOT_KICK_UNDERLINES_ON_BAN
	Bot will now kick underlines, and will make a ban after %d 
	kicks for the same user.
BOT_KICK_UNDERLINES_OFF
	Bot won't kick underlines anymore.

# BADWORDS responses
BOT_BADWORDS_SYNTAX
	BADWORDS channel {ADD | DEL | LIST | CLEAR} [expr | entry-list] [SINGLE | START | END]
BOT_BADWORDS_ADD_SYNTAX
	BADWORDS channel ADD expr [SINGLE | START | END]
BOT_BADWORDS_DEL_SYNTAX
	BADWORDS channel %s {expr | entry-list}
BOT_BADWORDS_DISABLED
	Sorry, channel bad words list modification is temporarily disabled.
BOT_BADWORDS_EMPTY
	%s bad words list is empty.
BOT_BADWORDS_EXPR_TOO_SHORT
	%s is too short and thus would match too many words.
BOT_BADWORDS_EXPR_TOO_LONG
	The expression can't be longer than %u characters.
BOT_BADWORDS_ALREADY_PRESENT
	The expression %s is already present on %s bad words list.
BOT_BADWORDS_REACHED_LIMIT
	Sorry, you can only have %d entries on a channel bad words list.
BOT_BADWORDS_CREATION_FAILED
	Sorry, creation of a new bad word entry failed.
BOT_BADWORDS_ADDED
	%s added to %s bad words list.
BOT_BADWORDS_NO_SUCH_ENTRY
	No such entry (#%d) on %s bad words list.
BOT_BADWORDS_NOT_FOUND
	%s not found on %s bad words list.
BOT_BADWORDS_NO_MATCH
	No matching entries on %s bad words list.
BOT_BADWORDS_DELETED
	%s deleted from %s bad words list.
BOT_BADWORDS_DELETED_ONE
	Deleted 1 entry from %s bad words list.
BOT_BADWORDS_DELETED_SEVERAL
	Deleted %d entries from %s bad words list.
BOT_BADWORDS_LIST_HEADER
	Bad words list for %s:
BOT_BADWORDS_LIST_COLHEAD
	  Num   Expr                            Type
# mask, reason
BOT_BADWORDS_LIST_FORMAT
	  %3d   %-30s  %s
BOT_BADWORDS_LISTED_ONE
	1 entry listed.
BOT_BADWORDS_LISTED_SEVERAL
	%d entries listed.
BOT_BADWORDS_CLEAR_PARTIALLY
	%s bad words list has been partially cleared.
BOT_BADWORDS_CLEAR
	%s bad words list has been cleared.

# SAY responses
BOT_SAY_SYNTAX
	SAY channel text

# ACT responses
BOT_ACT_SYNTAX
	ACT channel text

###########################################################################
#
# OperServ messages
#
###########################################################################

# General messages
OPER_BOUNCY_MODES
	Services is unable to change modes.  Are your servers configured correctly?
OPER_BOUNCY_MODES_U_LINE
	Services is unable to change modes.  Are your servers' U:lines configured correctly?

# GLOBAL responses
OPER_GLOBAL_SYNTAX
	GLOBAL message

# STATS responses
OPER_STATS_UNKNOWN_OPTION
	Unknown STATS option %s.
OPER_STATS_CURRENT_USERS
	Current users: %d (%d ops)
OPER_STATS_MAX_USERS
	Maximum users: %d (%s)
# This is ugly, but at least it's language-portable...
# Note that you can include seconds in any of these--my personal preference
# is to only give resolution to the nearest minute when uptime is one hour
# or more, but just add a %d to get seconds as well.
OPER_STATS_UPTIME_DHMS
	Services up %d days, %02d:%02d
OPER_STATS_UPTIME_1DHMS
	Services up %d day, %02d:%02d
OPER_STATS_UPTIME_HMS
	Services up %d hours, %d minutes
OPER_STATS_UPTIME_HM1S
	Services up %d hours, %d minutes
OPER_STATS_UPTIME_H1MS
	Services up %d hours, %d minute
OPER_STATS_UPTIME_H1M1S
	Services up %d hours, %d minute
OPER_STATS_UPTIME_1HMS
	Services up %d hour, %d minutes
OPER_STATS_UPTIME_1HM1S
	Services up %d hour, %d minutes
OPER_STATS_UPTIME_1H1MS
	Services up %d hour, %d minute
OPER_STATS_UPTIME_1H1M1S
	Services up %d hour, %d minute
OPER_STATS_UPTIME_MS
	Services up %d minutes, %d seconds
OPER_STATS_UPTIME_M1S
	Services up %d minutes, %d second
OPER_STATS_UPTIME_1MS
	Services up %d minute, %d seconds
OPER_STATS_UPTIME_1M1S
	Services up %d minute, %d second
OPER_STATS_BYTES_READ
	Bytes read           : %5d kB
OPER_STATS_BYTES_WRITTEN
	Bytes written        : %5d kB
OPER_STATS_USER_MEM
	User                 : %6d records, %5d kB
OPER_STATS_CHANNEL_MEM
	Channel              : %6d records, %5d kB
OPER_STATS_SESSIONS_MEM
	Sessions             : %6d records, %5d kB
OPER_STATS_BLACKLIST_MEM
	Blacklist            : %6d records, %5d kB
OPER_STATS_MEM
	%-20s : %6d records, %5d kB
OPER_STATS_MEM_ONE
	%-20s : %6d record,  %5d kB
	
OPER_STATS_AKILL_COUNT
	Current number of AKILLs: %d
OPER_STATS_AKILL_EXPIRE_DAYS
	Default AKILL expiry time: %d days
OPER_STATS_AKILL_EXPIRE_DAY
	Default AKILL expiry time: 1 day
OPER_STATS_AKILL_EXPIRE_HOURS
	Default AKILL expiry time: %d hours
OPER_STATS_AKILL_EXPIRE_HOUR
	Default AKILL expiry time: 1 hour
OPER_STATS_AKILL_EXPIRE_MINS
	Default AKILL expiry time: %d minutes
OPER_STATS_AKILL_EXPIRE_MIN
	Default AKILL expiry time: 1 minute
OPER_STATS_AKILL_EXPIRE_NONE
	Default AKILL expiry time: No expiration
OPER_STATS_SGLINE_COUNT
	Current number of SGLINEs: %d
OPER_STATS_SGLINE_EXPIRE_DAYS
	Default SGLINE expiry time: %d days
OPER_STATS_SGLINE_EXPIRE_DAY
	Default SGLINE expiry time: 1 day
OPER_STATS_SGLINE_EXPIRE_HOURS
	Default SGLINE expiry time: %d hours
OPER_STATS_SGLINE_EXPIRE_HOUR
	Default SGLINE expiry time: 1 hour
OPER_STATS_SGLINE_EXPIRE_MINS
	Default SGLINE expiry time: %d minutes
OPER_STATS_SGLINE_EXPIRE_MIN
	Default SGLINE expiry time: 1 minute
OPER_STATS_SGLINE_EXPIRE_NONE
	Default SGLINE expiry time: No expiration
OPER_STATS_SQLINE_COUNT
	Current number of SQLINEs: %d
OPER_STATS_SQLINE_EXPIRE_DAYS
	Default SQLINE expiry time: %d days
OPER_STATS_SQLINE_EXPIRE_DAY
	Default SQLINE expiry time: 1 day
OPER_STATS_SQLINE_EXPIRE_HOURS
	Default SQLINE expiry time: %d hours
OPER_STATS_SQLINE_EXPIRE_HOUR
	Default SQLINE expiry time: 1 hour
OPER_STATS_SQLINE_EXPIRE_MINS
	Default SQLINE expiry time: %d minutes
OPER_STATS_SQLINE_EXPIRE_MIN
	Default SQLINE expiry time: 1 minute
OPER_STATS_SQLINE_EXPIRE_NONE
	Default SQLINE expiry time: No expiration
OPER_STATS_SZLINE_COUNT
	Current number of SZLINEs: %d
OPER_STATS_SZLINE_EXPIRE_DAYS
	Default SZLINE expiry time: %d days
OPER_STATS_SZLINE_EXPIRE_DAY
	Default SZLINE expiry time: 1 day
OPER_STATS_SZLINE_EXPIRE_HOURS
	Default SZLINE expiry time: %d hours
OPER_STATS_SZLINE_EXPIRE_HOUR
	Default SZLINE expiry time: 1 hour
OPER_STATS_SZLINE_EXPIRE_MINS
	Default SZLINE expiry time: %d minutes
OPER_STATS_SZLINE_EXPIRE_MIN
	Default SZLINE expiry time: 1 minute
OPER_STATS_SZLINE_EXPIRE_NONE
	Default SZLINE expiry time: No expiration
OPER_STATS_RESET
	Statistics reset.
OPER_STATS_RESET_DISABLED
	Sorry, resetting statistics is temporarily disabled.

# MODE responses
OPER_MODE_SYNTAX
	MODE channel modes

# CLEARMODES responses
OPER_CLEARMODES_SYNTAX
	CLEARMODES channel [ALL]
OPER_CLEARMODES_DONE
	Binary modes and bans cleared from channel %s.
OPER_CLEARMODES_ALL_DONE
	All modes cleared from channel %s.


# KICK responses
OPER_KICK_SYNTAX
	KICK channel user reason

# ADMIN responses
OPER_ADMIN_SYNTAX
	ADMIN {ADD | DEL | LIST | VIEW | CLEAR} [nick | entry-list]
OPER_ADMIN_ADD_SYNTAX
	ADMIN ADD nick
OPER_ADMIN_DEL_SYNTAX
	ADMIN %s {nick | entry-list}
OPER_ADMIN_DISABLED
	Sorry, services admin list modification is temporarily disabled.
OPER_ADMIN_EMPTY
	The services admin list is empty.
OPER_ADMIN_ALREADY_PRESENT
	%s is already present on the services admin list.
OPER_ADMIN_TOO_MANY
	The services admin list is full!
OPER_ADMIN_ADDED
	%s added to the services admin list.
OPER_ADMIN_NO_SUCH_ENTRY
	No such entry (#%d) on the services admin list.
OPER_ADMIN_NOT_FOUND
	%s not found on the services admin list.
OPER_ADMIN_NO_MATCH
	No matching entries on the services admin list.
OPER_ADMIN_DELETED
	%s deleted from the services admin list.
OPER_ADMIN_DELETED_ONE
	Deleted 1 entry from the services admin list.
OPER_ADMIN_DELETED_SEVERAL
	Deleted %d entries from the services admin list.
OPER_ADMIN_LIST_HEADER
	Services admin list:
OPER_ADMIN_LIST_COLHEAD
	  Num   Nick
OPER_ADMIN_LIST_FORMAT
	  %3d   %s
OPER_ADMIN_LISTED_ONE
	1 entry listed.
OPER_ADMIN_LISTED_SEVERAL
	%d entries listed.
OPER_ADMIN_CLEAR_PARTIALLY
	The services admin list has been partially cleared.
OPER_ADMIN_CLEAR
	The services admin list has been cleared.

# OPER responses
OPER_OPER_SYNTAX
	OPER {ADD | DEL | LIST | VIEW | CLEAR} [nick | entry-list]
OPER_OPER_ADD_SYNTAX
	OPER ADD nick
OPER_OPER_DEL_SYNTAX
	OPER %s {nick | entry-list}
OPER_OPER_DISABLED
	Sorry, services operator list modification is temporarily disabled.
OPER_OPER_EMPTY
	The services operator list is empty.
OPER_OPER_ALREADY_PRESENT
	%s is already present on the services operator list.
OPER_OPER_TOO_MANY
	The services operator list is full!
OPER_OPER_ADDED
	%s added to the services operator list.
OPER_OPER_NO_SUCH_ENTRY
	No such entry (#%d) on the services operator list.
OPER_OPER_NOT_FOUND
	%s not found on the services operator list.
OPER_OPER_NO_MATCH
	No matching entries on the services operator list.
OPER_OPER_DELETED
	%s deleted from the services operator list.
OPER_OPER_DELETED_ONE
	Deleted 1 entry from the services operator list.
OPER_OPER_DELETED_SEVERAL
	Deleted %d entries from the services operator list.
OPER_OPER_LIST_HEADER
	Services operator list:
OPER_OPER_LIST_COLHEAD
	  Num   Nick
OPER_OPER_LIST_FORMAT
	  %3d   %s
OPER_OPER_LISTED_ONE
	1 entry listed.
OPER_OPER_LISTED_SEVERAL
	%d entries listed.
OPER_OPER_CLEAR_PARTIALLY
	The services operator list has been partially cleared.
OPER_OPER_CLEAR
	The services operator list has been cleared.

# AKILL responses
OPER_AKILL_SYNTAX
	AKILL {ADD | DEL | LIST | VIEW | CLEAR} [[+expiry] {mask | nick[:type] | entry-list} [reason]]
OPER_AKILL_ADD_SYNTAX
	AKILL ADD [+expiry] {mask | nick[:type]} reason
OPER_AKILL_DEL_SYNTAX
	AKILL %s {mask | entry-list}
OPER_AKILL_UNSUPPORTED
	Sorry, the AKILL command is not available on this network.
OPER_AKILL_DISABLED
	Sorry, AKILL list modification is temporarily disabled.
OPER_AKILL_EMPTY
	The AKILL list is empty.
OPER_AKILL_MASK_INVALID
	%s is not a valid AKILL mask.
OPER_AKILL_MASK_NO_NICK
	Reminder: AKILL masks cannot contain nicknames; make sure you have not included a nick portion in your mask.
OPER_AKILL_MASK_TOO_LONG
	The mask can't be longer than %u characters.
OPER_AKILL_REASON_TOO_LONG
	The reason can't be longer than %u characters.
OPER_AKILL_ALREADY_PRESENT
	The mask %s is already present on the AKILL list.
OPER_AKILL_ALREADY_COVERED
	The mask %s is already covered by AKILL %s.
OPER_AKILL_TOO_MANY
	The AKILL list is full!
OPER_AKILL_CREATION_FAILED
	Sorry, creation of a new AKILL failed.
OPER_AKILL_ADDED
	%s added to the AKILL list.
OPER_AKILL_CHANGED
	Expiry time of AKILL %s has been changed.
OPER_AKILL_NO_SUCH_ENTRY
	No such entry (#%d) on the AKILL list.
OPER_AKILL_NOT_FOUND
	%s not found on the AKILL list.
OPER_AKILL_NO_MATCH
	No matching entries on the AKILL list.
OPER_AKILL_DELETED
	%s deleted from the AKILL list.
OPER_AKILL_DELETED_ONE
	Deleted 1 entry from the AKILL list.
OPER_AKILL_DELETED_SEVERAL
	Deleted %d entries from the AKILL list.
OPER_AKILL_LIST_HEADER
	Current AKILL list:
OPER_AKILL_LIST_COLHEAD
	  Num   Mask                              Reason
# mask, reason
OPER_AKILL_LIST_FORMAT
	  %3d   %-32s  %s
OPER_AKILL_LISTED_ONE
	1 entry listed.
OPER_AKILL_LISTED_SEVERAL
	%d entries listed.
# number, mask, set-by, set-time, expires, reason
OPER_AKILL_VIEW_FORMAT
	%3d  %s (by %s on %s; %s)
	      %s
OPER_AKILL_CLEAR_PARTIALLY
	The AKILL list has been partially cleared.
OPER_AKILL_CLEAR
	The AKILL list has been cleared.

# SGLINE responses
OPER_SGLINE_SYNTAX
	SGLINE {ADD | DEL | LIST | VIEW | CLEAR} [[+expiry] {mask | entry-list}[:reason]]
OPER_SGLINE_ADD_SYNTAX
	SGLINE ADD [+expiry] mask:reason
OPER_SGLINE_DEL_SYNTAX
	SGLINE %s {mask | entry-list}
OPER_SGLINE_UNSUPPORTED
	Sorry, the SGLINE command is not available on this network.
OPER_SGLINE_DISABLED
	Sorry, SGLINE list modification is temporarily disabled.
OPER_SGLINE_EMPTY
	The SGLINE list is empty.
OPER_SGLINE_MASK_INVALID
	%s is not a valid SGLINE mask.
OPER_SGLINE_MASK_TOO_LONG
	The mask can't be longer than %u characters.
OPER_SGLINE_REASON_TOO_LONG
	The reason can't be longer than %u characters.
OPER_SGLINE_ALREADY_PRESENT
	The mask %s is already present on the SGLINE list.
OPER_SGLINE_ALREADY_COVERED
	The mask %s is already covered by SGLINE %s.
OPER_SGLINE_TOO_MANY
	The SGLINE list is full!
OPER_SGLINE_CREATION_FAILED
	Sorry, creation of a new SGLINE failed.
OPER_SGLINE_ADDED
	%s added to the SGLINE list.
OPER_SGLINE_CHANGED
	Expiry time of SGLINE %s has been changed.
OPER_SGLINE_NO_SUCH_ENTRY
	No such entry (#%d) on the SGLINE list.
OPER_SGLINE_NOT_FOUND
	%s not found on the SGLINE list.
OPER_SGLINE_NO_MATCH
	No matching entries on the SGLINE list.
OPER_SGLINE_DELETED
	%s deleted from the SGLINE list.
OPER_SGLINE_DELETED_ONE
	Deleted 1 entry from the SGLINE list.
OPER_SGLINE_DELETED_SEVERAL
	Deleted %d entries from the SGLINE list.
OPER_SGLINE_LIST_HEADER
	Current SGLINE list:
OPER_SGLINE_LIST_COLHEAD
	  Num   Mask                              Reason
# mask, reason
OPER_SGLINE_LIST_FORMAT
	  %3d   %-32s  %s
OPER_SGLINE_LISTED_ONE
	1 entry listed.
OPER_SGLINE_LISTED_SEVERAL
	%d entries listed.
# number, mask, set-by, set-time, expires, reason
OPER_SGLINE_VIEW_FORMAT
	%3d  %s (by %s on %s; %s)
	      %s
OPER_SGLINE_CLEAR_PARTIALLY
	The SGLINE list has been partially cleared.
OPER_SGLINE_CLEAR
	The SGLINE list has been cleared.

# SQLINE responses
OPER_SQLINE_SYNTAX
	SQLINE {ADD | DEL | LIST | VIEW | CLEAR} [[+expiry] {mask | entry-list} [reason]]
OPER_SQLINE_ADD_SYNTAX
	SQLINE ADD [+expiry] {mask | nick[:type]} reason
OPER_SQLINE_DEL_SYNTAX
	SQLINE %s {mask | entry-list}
OPER_SQLINE_DISABLED
	Sorry, SQLINE list modification is temporarily disabled.
OPER_SQLINE_EMPTY
	The SQLINE list is empty.
OPER_SQLINE_MASK_INVALID
	%s is not a valid SQLINE mask.
OPER_SQLINE_MASK_TOO_LONG
	The mask can't be longer than %u characters.
OPER_SQLINE_REASON_TOO_LONG
	The reason can't be longer than %u characters.
OPER_SQLINE_ALREADY_PRESENT
	The mask %s is already present on the SQLINE list.
OPER_SQLINE_ALREADY_COVERED
	The mask %s is already covered by SQLINE %s.
OPER_SQLINE_TOO_MANY
	The SQLINE list is full!
OPER_SQLINE_CREATION_FAILED
	Sorry, creation of a new SQLINE failed.
OPER_SQLINE_ADDED
	%s added to the SQLINE list.
OPER_SQLINE_CHANGED
	Expiry time of SQLINE %s has been changed.
OPER_SQLINE_NO_SUCH_ENTRY
	No such entry (#%d) on the SQLINE list.
OPER_SQLINE_NOT_FOUND
	%s not found on the SQLINE list.
OPER_SQLINE_NO_MATCH
	No matching entries on the SQLINE list.
OPER_SQLINE_DELETED
	%s deleted from the SQLINE list.
OPER_SQLINE_DELETED_ONE
	Deleted 1 entry from the SQLINE list.
OPER_SQLINE_DELETED_SEVERAL
	Deleted %d entries from the SQLINE list.
OPER_SQLINE_LIST_HEADER
	Current SQLINE list:
OPER_SQLINE_LIST_COLHEAD
	  Num   Mask                              Reason
# mask, reason
OPER_SQLINE_LIST_FORMAT
	  %3d   %-32s  %s
OPER_SQLINE_LISTED_ONE
	1 entry listed.
OPER_SQLINE_LISTED_SEVERAL
	%d entries listed.
# number, mask, set-by, set-time, expires, reason
OPER_SQLINE_VIEW_FORMAT
	%3d  %s (by %s on %s; %s)
	      %s
OPER_SQLINE_CLEAR_PARTIALLY
	The SQLINE list has been partially cleared.
OPER_SQLINE_CLEAR
	The SQLINE list has been cleared.

# SZLINE responses
OPER_SZLINE_SYNTAX
	SZLINE {ADD | DEL | LIST | VIEW | CLEAR} [[+expiry] {mask | entry-list} [reason]]
OPER_SZLINE_ADD_SYNTAX
	SZLINE ADD [+expiry] mask reason
OPER_SZLINE_DEL_SYNTAX
	SZLINE %s {mask | entry-list}
OPER_SZLINE_UNSUPPORTED
	Sorry, the SZLINE command is not available on this network.
OPER_SZLINE_DISABLED
	Sorry, SZLINE list modification is temporarily disabled.
OPER_SZLINE_EMPTY
	The SZLINE list is empty.
OPER_SZLINE_MASK_INVALID
	%s is not a valid SZLINE mask.
OPER_SZLINE_MASK_ONLY_IPS
	Reminder: SZLINE masks cannot contain nicknames or usernames; make sure you have not included a nick or user portion in your mask.
OPER_SZLINE_MASK_TOO_LONG
	The mask can't be longer than %u characters.
OPER_SZLINE_REASON_TOO_LONG
	The reason can't be longer than %u characters.
OPER_SZLINE_ALREADY_PRESENT
	The mask %s is already present on the SZLINE list.
OPER_SZLINE_ALREADY_COVERED
	The mask %s is already covered by SZLINE %s.
OPER_SZLINE_TOO_MANY
	The SZLINE list is full!
OPER_SZLINE_CREATION_FAILED
	Sorry, creation of a new SZLINE failed.
OPER_SZLINE_ADDED
	%s added to the SZLINE list.
OPER_SZLINE_CHANGED
	Expiry time of SZLINE %s has been changed.
OPER_SZLINE_NO_SUCH_ENTRY
	No such entry (#%d) on the SZLINE list.
OPER_SZLINE_NOT_FOUND
	%s not found on the SZLINE list.
OPER_SZLINE_NO_MATCH
	No matching entries on the SZLINE list.
OPER_SZLINE_DELETED
	%s deleted from the SZLINE list.
OPER_SZLINE_DELETED_ONE
	Deleted 1 entry from the SZLINE list.
OPER_SZLINE_DELETED_SEVERAL
	Deleted %d entries from the SZLINE list.
OPER_SZLINE_LIST_HEADER
	Current SZLINE list:
OPER_SZLINE_LIST_COLHEAD
	  Num   Mask                  Reason
# mask, reason
OPER_SZLINE_LIST_FORMAT
	  %3d   %-20s  %s
OPER_SZLINE_LISTED_ONE
	1 entry listed.
OPER_SZLINE_LISTED_SEVERAL
	%d entries listed.
# number, mask, set-by, set-time, expires, reason
OPER_SZLINE_VIEW_FORMAT
	%3d  %s (by %s on %s; %s)
	      %s
OPER_SZLINE_CLEAR_PARTIALLY
	The SZLINE list has been partially cleared.
OPER_SZLINE_CLEAR
	The SZLINE list has been cleared.

# SET responses
OPER_SET_SYNTAX
	SET option setting
OPER_SET_IGNORE_ON
	Ignore code will be used.
OPER_SET_IGNORE_OFF
	Ignore code will not be used.
OPER_SET_IGNORE_ERROR
	Setting for IGNORE must be ON or OFF.
OPER_SET_READONLY_ON
	Services is now in read-only mode.
OPER_SET_READONLY_OFF
	Services is now in read-write mode.
OPER_SET_READONLY_ERROR
	Setting for READONLY must be ON or OFF.
OPER_SET_DEBUG_ON
	Services is now in debug mode.
OPER_SET_DEBUG_OFF
	Services is now in non-debug mode.
OPER_SET_DEBUG_LEVEL
	Services is now in debug mode (level %d).
OPER_SET_DEBUG_ERROR
	Setting for DEBUG must be ON, OFF, or a positive number.
OPER_SET_NOEXPIRE_ON
	Services is now in no expire mode.
OPER_SET_NOEXPIRE_OFF
	Services is now in expire mode.
OPER_SET_NOEXPIRE_ERROR
	Setting for NOEXPIRE must be ON or OFF.
OPER_SET_UNKNOWN_OPTION
	Unknown option %s.

# NOOP responses
OPER_NOOP_SYNTAX
	NOOP {SET|REVOKE} server
OPER_NOOP_SET
	All O:lines of %s have been removed.
OPER_NOOP_REVOKE
	All O:lines of %s have been reset.

# JUPE responses
OPER_JUPE_SYNTAX
	JUPE servername [reason]
OPER_JUPE_USE_SZLINE
	Use the SZLINE command to ban the server's IP address instead.

# RAW responses
OPER_RAW_SYNTAX
	RAW text

# SYNC responses
OPER_SYNC_IN_PROGRESS
	Synchronization request has been sent.
	
# RELOAD responses
OPER_RELOAD
	Configuration has been reloaded.

# LISTIGNORE responses
OPER_IGNORE_LIST
	Services ignore list:
OPER_IGNORE_LIST_EMPTY
	Ignore list is empty.
	
# CHANLIST responses
OPER_CHANLIST_SYNTAX
	CHANLIST [{[pattern] [range] [NOLIMIT] | nick} [SECRET]]
OPER_CHANLIST_INVALID_RANGE
	%s is not a valid results range.
OPER_CHANLIST_NO_CHANNELS
	There are currently no channels.
OPER_CHANLIST_NO_CHANNELS_MATCH
	There are no channels matching %s.
OPER_CHANLIST_NO_CHANNELS_USER
	%s is not present on any channel.
OPER_CHANLIST_HEADER
	Channel list:
	Name                  Users  Modes    Topic
OPER_CHANLIST_HEADER_MATCH
	Channels matching %s:
	Name                  Users  Modes    Topic
OPER_CHANLIST_HEADER_USER
	%s channel list:
	Name                  Users  Modes    Topic
OPER_CHANLIST_ENTRY
	%-20s   %4d  +%-6s  %s
OPER_CHANLIST_RESULTS
	End of list - %d/%d results shown.
OPER_CHANLIST_RESULTS_RANGE
	End of list - Results %d to %d shown.
OPER_CHANLIST_RESULTS_RANGE_ONE
	End of list - Result %d shown.
	
# USERLIST responses
OPER_USERLIST_SYNTAX
	USERLIST [{[pattern] [range] [NOLIMIT] | channel} [INVISIBLE]]
OPER_USERLIST_INVALID_RANGE
	%s is not a valid results range.
OPER_USERLIST_NO_USERS_MATCH
	There are no users matching %s.
OPER_USERLIST_HEADER
	Users list:
	Nick                  Mask
OPER_USERLIST_HEADER_MATCH
	Users matching %s:
	Nick                  Mask
OPER_USERLIST_HEADER_CHAN
	%s users list:
	Nick                  Mask
OPER_USERLIST_ENTRY
	%-20s  %s@%s
OPER_USERLIST_RESULTS
	End of list - %d/%d results shown.
OPER_USERLIST_RESULTS_RANGE
	End of list - Results %d to %d shown.
OPER_USERLIST_RESULTS_RANGE_ONE
	End of list - Result %d shown.

# BLACKLIST responses
OPER_BLACKLIST_SYNTAX
	BLACKLIST {DEL | LIST} {hostname | pattern}
OPER_BLACKLIST_DISABLED
	Black list is disabled.
OPER_BLACKLIST_NOT_FOUND
	%s was not found in the black list.
OPER_BLACKLIST_REMOVED
	%s has been removed from the black list.
OPER_BLACKLIST_HEADER
	Hostnames in the black list:
OPER_BLACKLIST_LIST
	%s
OPER_BLACKLIST_FOOTER
	End of list - %d/%d matches shown.

# MODULES responses
OPER_MODULES_SYNTAX
	MODULES {LOAD | UNLOAD | LIST | INFO | QUERY} [name] [query]
OPER_MODULES_QUERY_SYNTAX
	MODULES QUERY [name] [query]
OPER_MODULES_DISABLED
	The MODULES %s command is disabled.
OPER_MODULES_NOT_LOADED
	Module %s is not loaded.
OPER_MODULES_LOAD_ALREADY
	Module %s is already loaded!
OPER_MODULES_LOAD_ERROR
	Module %s can't be loaded.
OPER_MODULES_LOADED
	Module %s has been loaded.
OPER_MODULES_UNLOAD_ERROR
	Module %s can't be unloaded.
OPER_MODULES_UNLOADED
	Module %s has been unloaded.
OPER_MODULES_HEADER
	List of loaded modules:
	Name              Ref.  Description
OPER_MODULES_LIST
	%-16s  %4d  %s
OPER_MODULES_FOOTER
	%d modules loaded.
OPER_MODULES_INFO_HEADER
	Information about module %s:
OPER_MODULES_INFO_VERSION
	     Version: %d.%d.%d
OPER_MODULES_INFO_DESC
	 Description: %s
OPER_MODULES_INFO_AUTHOR
	      Author: %s
OPER_MODULES_INFO_URL
	         URL: %s
OPER_MODULES_INFO_MEMORY_USAGE
	Memory usage: %d bytes
OPER_MODULES_INFO_MEMORY_USAGE_K
	Memory usage: %d kB
OPER_MODULES_QUERY_UNSUPPORTED
	Module %s does not support this query.
OPER_MODULES_QUERY_UNANSWERED
	Module %s executed the query; no answer provided.
OPER_MODULES_QUERY_ANSWERED
	Module %s executed the query and answered:

###########################################################################
#
# Session Limiting and Exception messages
#
###########################################################################

# EXCEPTION responses
OPER_EXCEPTION_SYNTAX
	EXCEPTION {ADD | DEL | LIST | VIEW | CLEAR} [hostmask | entry-list]
OPER_EXCEPTION_ADD_SYNTAX
	EXCEPTION ADD [+expiry] hostmask limit reason
OPER_EXCEPTION_DEL_SYNTAX
	EXCEPTION %s {hostmask | list}
OPER_EXCEPTION_SESSIONS_DISABLED
	Session limiting is disabled.
OPER_EXCEPTION_DISABLED
	Sorry, session-limit exception list modification is temporarily disabled.
OPER_EXCEPTION_EMPTY
	The session-limit exception list is empty.
OPER_EXCEPTION_INVALID_LIMIT
	Invalid session limit. It must be a valid integer greater than or equal to zero and less than %d.
OPER_EXCEPTION_INVALID_HOSTMASK
	Invalid hostmask. Only real hostmasks are valid as exceptions are not matched against nicks or usernames.
OPER_EXCEPTION_HOSTMASK_TOO_LONG
	The hostmask can't be longer than %u characters.
OPER_EXCEPTION_REASON_TOO_LONG
	The reason can't be longer than %u characters.
OPER_EXCEPTION_ALREADY_PRESENT
	Mask %s already present on exception list.
OPER_EXCEPTION_TOO_MANY
	Session-limit exception list is full!
OPER_EXCEPTION_CREATION_FAILED
	Sorry, creation of a new exception failed.
OPER_EXCEPTION_ADDED
	Session limit for %s set to %d.
OPER_EXCEPTION_NO_SUCH_ENTRY
	No such entry (#%d) on session-limit exception list.
OPER_EXCEPTION_NOT_FOUND
	%s not found on session-limit exception list.
OPER_EXCEPTION_NO_MATCH
	No matching entries on session-limit exception list.
OPER_EXCEPTION_DELETED
	%s deleted from session-limit exception list.
OPER_EXCEPTION_DELETED_ONE
	Deleted 1 entry from session-limit exception list.
OPER_EXCEPTION_DELETED_SEVERAL
	Deleted %d entries from session-limit exception list.
OPER_EXCEPTION_LIST_HEADER
	Current session limit exception list:
OPER_EXCEPTION_LIST_COLHEAD
	Num  Limit  Host
# mask, limit
OPER_EXCEPTION_LIST_FORMAT
	%3d  %4d   %s
OPER_EXCEPTION_LISTED_ONE
	1 entry listed.
OPER_EXCEPTION_LISTED_SEVERAL
	%d entries listed.
# mask, set-by, set-time, expires, limit, reason
OPER_EXCEPTION_VIEW_FORMAT
	%3d.  %s  (by %s on %s; %s)
	    Limit: %-4d  - %s
OPER_EXCEPTION_CLEAR_PARTIALLY
	Session limit exception list has been partially cleared.
OPER_EXCEPTION_CLEAR
	Session limit exception list has been cleared.

# SESSION responses
OPER_SESSION_SYNTAX
	SESSION {LIST limit | VIEW host}
OPER_SESSION_LIST_SYNTAX
	SESSION LIST limit
OPER_SESSION_VIEW_SYNTAX
	SESSION VIEW host
OPER_SESSION_DISABLED
	Session limiting is disabled.
OPER_SESSION_INVALID_THRESHOLD
	Invalid threshold value. It must be a valid integer greater than 1.
OPER_SESSION_NOT_FOUND
	%s not found on session list.
OPER_SESSION_LIST_HEADER
	Hosts with atleast %d sessions:
OPER_SESSION_LIST_COLHEAD
	Sessions  Host
# limit, host
OPER_SESSION_LIST_FORMAT
	%6d    %s
# host, sessions, limit
OPER_SESSION_VIEW_FORMAT
	The host %s currently has %d sessions with a limit of %d.

# EXCEPTION help

OPER_HELP_EXCEPTION
	Syntax: EXCEPTION ADD [+expiry] mask limit reason
	        EXCEPTION DEL {mask | list}
	        EXCEPTION LIST [mask | list]
	        EXCEPTION VIEW [mask | list]
	        EXCEPTION CLEAR
	
	Allows services admins to manipulate the list of hosts that
	have specific session limits - allowing certain machines, 
	such as shell servers, to carry more than the default number
	of clients at a time. Once a host reaches its session limit,
	all clients attempting to connect from that host will be 
	killed. Before the user is killed, they are notified, via a
	/NOTICE from %S, of a source of help regarding session
	limiting. The contents of this notice is a config setting.
	
	EXCEPTION ADD adds the given host mask to the exception list.
	Note that nick!user@host and user@host masks are invalid! 
	Only real host masks, such as box.host.dom and *.host.dom, 
	are allowed because sessions limiting does not take nick or 
	user names into account. limit must be a number greater than
	or equal to zero. This determines how many sessions this host
	may carry at a time. A value of zero means the host has an
	unlimited session limit. See the AKILL help for details about
	the format of the optional expiry parameter.
	EXCEPTION DEL removes the given mask from the exception list.
	EXCEPTION LIST and EXCEPTION VIEW show all current 
	exceptions; if the optional mask is given, the list is limited
	to those exceptions matching the mask. The difference is that
	EXCEPTION VIEW is more verbose, displaying the name of the
	person who added the exception, its session limit, reason, 
	host mask and the expiry date and time.
	EXCEPTION CLEAR removes all entries from the exception list.
	
	Note that a connecting client will "use" the first exception
	their host matches. Large exception lists and widely matching
	exception masks are likely to degrade the performance.
	
	Limited to services admins.

OPER_HELP_SESSION
	Syntax: SESSION LIST threshold
	        SESSION VIEW host
	
	Allows services admins to view the session list.
	
	SESSION LIST lists hosts with atleast threshold sessions.
	The threshold must be a number greater than 1. This is to 
	prevent accidental listing of the large number of single 
	session hosts.
	SESSION VIEW displays detailed information about a specific
	host - including the current session count and session limit.
	The host value may not include wildcards.
	
	See the EXCEPTION help for more information about session
	limiting and how to set session limits specific to certain
	hosts and groups thereof.
	
	Limited to services admins.

###########################################################################
#
# News system messages
#
###########################################################################

OPER_NEWS_LOGON
	[Logon News - %s] %s
OPER_NEWS_OPER
	[Oper News - %s] %s
OPER_NEWS_RANDOM
	[Random News - %s] %s

# NEWS responses
OPER_NEWS_SYNTAX
	NEWS {LOGON | OPER | RANDOM} {ADD | DEL | LIST | CLEAR} [text | num | entry-list]
OPER_NEWS_ADD_SYNTAX
	NEWS {LOGON | OPER | RANDOM} ADD text
OPER_NEWS_DEL_SYNTAX
	NEWS {LOGON | OPER | RANDOM} %s {hostmask | list}
OPER_NEWS_LIST_SYNTAX
	NEWS {LOGON | OPER | RANDOM} %s {list}
OPER_NEWS_DISABLED
	Sorry, news list modification is temporarily disabled.
OPER_NEWS_EMPTY
	The %s news list is empty.
OPER_NEWS_TEXT_TOO_LONG
	The text can't be longer than %u characters.
OPER_NEWS_TOO_MANY
	%s news list is full!
OPER_NEWS_ADDED
	News added to the %s news list.
OPER_NEWS_NO_SUCH_ENTRY
	No such entry (#%d) on %s news list.
OPER_NEWS_NO_MATCH
	No matching entries on %s news list.
OPER_NEWS_DELETED_ONE
	Deleted 1 entry from %s news list.
OPER_NEWS_DELETED_SEVERAL
	Deleted %d entries from %s news list.
OPER_NEWS_LIST_HEADER
	Current %s news list:
OPER_NEWS_LIST_COLHEAD
	Num  Text
# num, text
OPER_NEWS_LIST_FORMAT
	%3d  %s
OPER_NEWS_LISTED_ONE
	1 entry listed.
OPER_NEWS_LISTED_SEVERAL
	%d entries listed.
OPER_NEWS_CLEAR_PARTIALLY
	%s news list has been partially cleared.
OPER_NEWS_CLEAR
	%s news list has been cleared.

OPER_HELP_NEWS
	Syntax: NEWS {LOGON | OPER | RANDOM} ADD text
	        NEWS {LOGON | OPER | RANDOM} DEL {num | list}
	        NEWS {LOGON | OPER | RANDOM} LIST [list]
	        NEWS {LOGON | OPER | RANDOM} CLEAR
	
	Edits or displays the list of LOGON news (up to three
	sent to a user connecting to the network), OPER news (up
	to three sent to a user opering up -- through the /OPER
	command) and RANDOM news (one randomly chosen sent to
	a user connecting to the network).
	
	LIST may be used by any IRC operator to list the current
	news messages. ADD, DEL and CLEAR may only be used by
	services admins.

###########################################################################
#
# HelpServ help message
#
###########################################################################

HELP_HELP
	%S is a service designed to give out information on
	Services.  Help topics are accessible via 
	the HELP commands of the other Services clients:

	/msg %s HELP
	     for information on registering nicknames

	/msg %s HELP
	     for information on registering and controlling
	     channels

	/msg %s HELP
	     for information on sending messages to off-line users

HELP_HELP_BOT
	
	/msg %s HELP
	     for information on setting up a bot on your channel

###########################################################################
#
# NickServ help messages
#
###########################################################################

NICK_HELP
	%S allows you to "register" a nickname and
	prevent others from using it. The following
	commands allow for registration and maintenance of
	nicknames; to use them, type /msg %S command.
	For more information on a specific command, type
	/msg %S HELP command.
	
	    REGISTER   Register a nickname
	    GROUP      Join a group
	    IDENTIFY   Identify yourself with your password
	    ACCESS     Modify the list of authorized addresses
	    SET        Set options, including kill protection
	    DROP       Cancel the registration of a nickname
	    RECOVER    Kill another user who has taken your nick
	    RELEASE    Regain custody of your nick after RECOVER
	    SENDPASS   Forgot your password? Try this
	
	Other commands: AUTH, GHOST, GLIST, HOST, INFO, LIST, 
	                LOGOUT, STATUS
	
	NOTICE: This service is intended to provide a way for
	IRC users to ensure their identity is not compromised.
	It is NOT intended to facilitate "stealing" of
	nicknames or other malicious actions.  Abuse of %S
	will result in, at minimum, loss of the abused
	nickname(s).
	
NICK_HELP_EXPIRES
	
	Nicknames that are not used anymore are subject to 
	the automatic expiration, i.e. they will be deleted
	after %d days if not used.

NICK_HELP_REGISTER
	Syntax: REGISTER password [email]
	
	Registers your nickname in the %S database.  Once
	your nick is registered, you can use the SET and ACCESS
	commands to configure your nick's settings as you like
	them.  Make sure you remember the password you use when
	registering - you'll need it to make changes to your nick
	later.  (Note that case matters!  EPONA, Epona, and 
	epona are all different passwords!)
	
	Guidelines on choosing passwords:
	
	Passwords should not be easily guessable.  For example,
	using your real name or your birth date as a password is
	a bad idea.  Using your nickname as a password is a much
	worse idea ;) and, in fact, %S will not allow it.
	Also, short passwords are vulnerable to trial-and-error
	searches, so you should choose a password at least 5
	characters long. Finally, the space character cannot be
	used in passwords.
	
	The parameter email is optional and will set the email
	for your nick immediately. However, it may be required
	on certain networks.
	Your privacy is respected; this e-mail won't be given to
	any third-party person.
	
	This command also creates a new group for your nickname,
	that will allow you to register other nicks later sharing
	the same configuration, the same set of memos and the 
	same channel privileges. For more information on this
	feature, type /msg %S HELP GROUP.
	
NICK_HELP_GROUP
	Syntax: GROUP target password
	
	This command makes your nickname join the target nickname's 
	group. password is the password of the target nickname.
	
	Joining a group will allow you to share your configuration,
	memos, and channel privileges with all nicknames in the
	group, and much more!
	 
	A group exists as long as it is useful. This means that even
	if a nick of the group is dropped, you won't lose the
	shared things described above, as long as there is at
	least one nick remaining in the group.
	
	You can use this command even if you have not registered
	your nick yet. If your nick is already registered, you'll
	need to identify yourself before using this command. Type 
	/msg %S HELP IDENTIFY for more information. This
	last may be not possible on your IRC network.
	
	It is recommended to use this command with a non-registered
	nick because it will be registered automatically when 
	using this command. You may use it with a registered nick (to 
	change your group) only if your network administrators allowed 
	it.
	
	You can only be in one group at a time. Group merging is
	not possible.
	
	Note: all the nicknames of a group have the same password.

NICK_HELP_IDENTIFY
	Syntax: IDENTIFY password
	
	Tells %S that you are really the owner of this
	nick.  Many commands require you to authenticate yourself
	with this command before you use them.  The password
	should be the same one you sent with the REGISTER
	command.
	
NICK_HELP_LOGOUT
	Syntax: LOGOUT
	
	Reverses the effect of the IDENTIFY command, i.e. makes
	you not recognized as the real owner of the nick anymore.
	Note, however, that you won't have to reidentify yourself.

NICK_HELP_AUTH
	Syntax: AUTH
	        AUTH code
	        AUTH PASS code new-password
	        AUTH CANCEL
	
	With no parameter, starts an authentication process, which
	consists of an e-mail sent to the e-mail address associated
	with your group and containing a code used to verify the
	validity of your e-mail address.
	
	With a code parameter, finishes the authentication process
	currently in progress. The code is the one that was sent
	to you by e-mail at the beginning of the authentication
	process. Once your e-mail is fully authenticated, you are
	given access to all commands and features that your
	administrators chose to reserve to groups with
	authenticated e-mail addresses.
	
	The AUTH PASS command finishes the authentication process
	as well, but allows you to additionally change your
	nickname password as well. This command may be unavailable.
	
	The AUTH CANCEL command cancels the authentication
	process in progress.
	
	This command may be disabled on some networks.

NICK_HELP_DROP
	Syntax: DROP [nickname]
	
	Drops your nickname from the %S database.  A nick
	that has been dropped is free for anyone to re-register.
	
	You may drop a nick within your group by passing it
	as the nick parameter.
	
	In order to use this command, you must first identify
	with your password (/msg %S HELP IDENTIFY for more
	information).

NICK_HELP_ACCESS
	Syntax: ACCESS ADD mask
	        ACCESS DEL {mask | entry-num | list}
	        ACCESS LIST [mask | list]
	        ACCESS CLEAR
	
	Modifies or displays the access list for your nickname
	group.  This is the list of addresses which will be
	automatically recognized by %S as allowed to use
	the nicks.  If you want to use the nick from a different
	address, you need to send an IDENTIFY command to
	make %S recognize you.
	
	ACCESS ADD adds the given user@host mask to the access
	list; for example:
	
	  ACCESS ADD epona@*.epona.org
	        Allows access to user epona from any machine in
	        the epona.org domain.
	
	The ACCESS DEL command removes the given mask from the
	ACCESS list if it is present.  If a list of entry numbers is 
	given, those entries are deleted.  (See the example for LIST 
	below.)
	
	The ACCESS LIST command displays the access list.  
	If a wildcard mask is given, only those entries matching the
	mask are displayed. If a list of entry numbers is
	given, only those entries are shown; for example:

	   ACCESS LIST 2,7-9
	      Lists access list entries numbered 2 and 7 
	      through 9.
	
	ACCESS CLEAR clears all entries of the access list.

NICK_HELP_SET
	Syntax: SET option parameters
	
	Sets various nickname options.  option can be one of:
	
	    GNAME      Set the name of your group
	    PASSWORD   Set your nickname password
	    LANGUAGE   Set the language services will use when
	                   sending messages to you
	    URL        Associate a URL with your nickname
	    EMAIL      Associate an e-mail address with your nickname
	    IM         Associate instant messaging contact info
	                   with your nickname
	    GREET      Associate a greet message with your nickname
	    KILL       Turn protection on or off
	    SECURE     Turn nickname security on or off
	    PRIVATE    Prevent your nickname from appearing in a
	                   /msg %S LIST
	    HIDE       Hide certain pieces of nickname information
	    HOST       Customize your IRC hostname
	
	In order to use this command, you must first identify
	with your password (/msg %S HELP IDENTIFY for more
	information).
	
	Type /msg %S HELP SET option for more information
	on a specific option.

NICK_HELP_SET_GNAME
	Syntax: SET GNAME new-name
	
	Changes the name used by %S to refer to your group.
	The new-name MUST be one of the nicks in your group.

NICK_HELP_SET_PASSWORD
	Syntax: SET PASSWORD new-password
	
	Changes the password used to identify you as the nick's
	owner.

NICK_HELP_SET_LANGUAGE
	Syntax: SET LANGUAGE number
	
	Changes the language Services uses when sending messages to
	you (for example, when responding to a command you send).
	number should be chosen from the following list of
	supported languages:
	

NICK_HELP_SET_URL
	Syntax: SET URL url
	
	Associates the given URL with your nickname.  This URL
	will be displayed whenever someone requests information
	on your nick with the INFO command.

NICK_HELP_SET_EMAIL
	Syntax: SET EMAIL address
	
	Associates the given E-mail address with your nickname.
	This address will be displayed whenever someone requests
	information on the channel with the INFO command.

NICK_HELP_SET_IM
	Syntax: SET IM program identifier

	Associates the given instant messaging contact info with
	your nickname. program is the instant messaging program
	and identifier is your instant messaging identifier.
	This will be displayed whenever someone requests
	information on your nick with the INFO command.

NICK_HELP_SET_GREET
	Syntax: SET GREET message
	
	Associates the given greet message with your nickname. It
	will be displayed when joining a channel having the GREET
	option enabled, provided you have the required access level
	on it.
	
NICK_HELP_SET_KILL
	Syntax: SET KILL {ON | QUICK | IMMED | OFF}
	
	Turns the automatic protection option for your nick
	on or off.  With protection on, if another user
	tries to take your nick, they will be given one minute to
	change to another nick, after which %S will forcibly change
	their nick. The protection is disabled for users taking
	the nickname whose user mask matches an entry of the
	nickname access list; type /msg %S HELP ACCESS for
	more information.
	
	If you select QUICK, the user will be given only 20 seconds
	to change nicks instead of the usual 60.  If you select
	IMMED, user's nick will be changed immediately without being
	warned first or given a chance to change their nick; please
	do not use this option unless necessary, and make sure your
	nickname access list is up-to-date, or you won't be able to
	use your nickname again!  However, your network's
	administrators may have disabled this option.

NICK_HELP_SET_SECURE
	Syntax: SET SECURE {ON | OFF}
	
	Turns %S's security features on or off for your
	nick.  With SECURE set, you must enter your password
	before you will be recognized as the owner of the nick
	and thus getting access to your privileges, memos and
	options, regardless of whether your address is on the
	access list.

NICK_HELP_SET_PRIVATE
	Syntax: SET PRIVATE {ON | OFF}
	
	Turns %S's privacy option on or off for your nick.
	With PRIVATE set, your nickname will not appear in
	nickname lists generated with %S's LIST command.
	(However, anyone who knows your nickname can still get
	information on it using the INFO command.)

NICK_HELP_SET_HIDE
	Syntax: SET HIDE {EMAIL | USERMASK | QUIT} {ON | OFF}
	
	Allows you to prevent certain pieces of information from
	being displayed when someone does a %S INFO on your
	nick.  You can hide your E-mail address (EMAIL), last seen
	user@host mask (USERMASK), and last quit message (QUIT).
	The second parameter specifies whether the information should
	be displayed (OFF) or hidden (ON).
	
NICK_HELP_SET_HOST
	Syntax: SET HOST {ON | OFF}
	
	Allows you to enable (ON) or disable (OFF) the
	custom hostname (visible in /WHOIS) that may be
	assigned to your nickname.
	
	Note: this option may be unavailable.

NICK_HELP_RECOVER
	Syntax: RECOVER nickname [password]
	
	Allows you to recover your nickname if someone else has
	taken it; this does the same thing that %S does
	automatically if someone tries to use a kill-protected
	nick.
	
	When you give this command, %S will bring a fake
	user online with the same nickname as the user you're
	trying to recover your nick from.  This causes the IRC
	servers to disconnect the other user.  This fake user will
	will remain online for one minute to ensure that the other
	user does not immediately reconnect; after that minute, you
	can reclaim your nick.  Alternatively, use the RELEASE
	command (/msg %S HELP RELEASE) to get the nick
	back sooner.
	
	In order to use the RECOVER command for a nick, your
	current address as shown in /WHOIS must be on that nick's
	access list, you must be identified and in the group of
	that nick, or you must supply the correct password for
	the nickname.

NICK_HELP_RELEASE
	Syntax: RELEASE nickname [password]
	
	Instructs %S to remove any hold on your nickname
	caused by automatic kill protection or use of the RECOVER
	command.  By default, such holds last for one minute;
	this command gets rid of them sooner.
	
	In order to use the RELEASE command for a nick, your
	current address as shown in /WHOIS must be on that nick's
	access list, you must be identified and in the group of
	that nick, or you must supply the correct password for
	the nickname.

NICK_HELP_GHOST
	Syntax: GHOST nickname [password]
	
	Terminates a "ghost" IRC session using your nick.  A
	"ghost" session is one which is not actually connected,
	but which the IRC server believes is still online for one
	reason or another.  Typically, this happens if your
	computer crashes or your Internet or modem connection
	goes down while you're on IRC.
	
	In order to use the GHOST command for a nick, your
	current address as shown in /WHOIS must be on that nick's
	access list, you must be identified and in the group of
	that nick, or you must supply the correct password for
	the nickname.

NICK_HELP_INFO
	Syntax: INFO nickname [ALL]
	
	Displays information about the given nickname, such as
	the nick's owner, last seen address and time, and nick
	options. If you are identified for the nick you're 
	getting information for and ALL is specified, you will
	be shown all the information; regardless of whether 
	it's hidden or not.

NICK_HELP_LIST
	Syntax: LIST [[pattern] [range]]
	
	Lists all registered nicknames.  Nicks with the PRIVATE
	option set will not be displayed.
	
	If pattern is given, only users with a matching
	usermask (in nick!user@host format) are displayed. The
	number of results shown at once is limited; you may specify
	a range to see results past this limit. For example:
	
	   LIST 51-100
	       Shows results 51 to 100.
	   LIST *Bot*!* 15-20
	       Shows results 15 to 20 whose nick contains "Bot".
	   LIST *!joeuser@foo.com
	        Lists all registered nicks owned by joeuser@foo.com.
	
	(Note: unlike numbered lists, it is not possible to use
	extended syntax such as 51-60,75-80).
	
	On some networks, this command is reserved to IRC
	operators.

NICK_HELP_GLIST
	Syntax: GLIST
	
	Lists all nicks in your group, with their expiration dates.

NICK_HELP_STATUS
	Syntax: STATUS nickname...
	
	Returns whether the user using the given nickname is
	recognized as the owner of the nickname.  The response has
	this format:
	
	    nickname status-code
	
	where nickname is the nickname sent with the command, and
	status-code is one of the following:
	
	    0 - no such user online or nickname not registered
	    1 - user not recognized as nickname's owner
	    2 - user recognized as owner via access list only
	    3 - user recognized as owner via password identification
	
	Up to sixteen nicknames may be sent with each command; the
	rest will be ignored.  No error message is generated if no
	nickname is given.
	
NICK_HELP_SENDPASS
	Syntax: SENDPASS nickname
	
	Send the password of the given nickname to the e-mail
	address set in the nickname record. This command is really
	useful to deal with lost passwords.
	
	May be limited to IRC operators on certain networks.
	
	This command is unavailable when encryption is enabled and
	it is impossible to generate a password through OpenSSL.

NICK_HELP_HOST
	Syntax: HOST SET [custom-hostname]
	
	HOST SET sets the custom hostname of your nickname. 
	It is unset if no parameter is given.
	
	Note: on some networks, this command may be disabled,
	unavailable or resticted to services operators.

NICK_SERVADMIN_HELP
	
	The following commands are available to services admins:
	
	    GETPASS    Retrieve the password for a nickname
	                   (only if encryption is disabled)
	
	services admins can also drop any nickname without needing
	to identify for the nick.

NICK_SERVADMIN_HELP_LOGOUT
	Syntax: LOGOUT [nickname [REVALIDATE]]
	
	Without a parameter, reverses the effect of the IDENTIFY 
	command, i.e. make you not recognized as the real owner of the nick
	anymore. Note, however, that you won't be asked to reidentify
	yourself.
	
	With a parameter, does the same for the given nick. If you 
	specify REVALIDATE as well, Services will ask the given nick
	to re-identify. This use limited to services admins.

NICK_SERVADMIN_HELP_DROP
	Syntax: DROP [nickname]
	
	Without a parameter, drops your nickname from the
	%S database.
	
	With a parameter, drops the named nick from the database.
	You may drop any nick within your group without any 
	special privileges. Dropping any nick is limited to 
	services admins.

NICK_SERVADMIN_HELP_SET
	
	services admins can also set the option NOEXPIRE, with
	which nicknames can be prevented from expiring.
	Additionally, services admins can set options for any
	nickname without entering a password, using the format
	SET =nickname option parameters.

NICK_SERVADMIN_HELP_SET_NOEXPIRE
	Syntax: SET [=nickname] NOEXPIRE {ON | OFF}
	
	Sets whether the given nickname will expire.  Setting this
	to ON prevents the nickname from expiring.  If no
	nickname is given, sets the no-expire flag for your nick.
	
	Limited to services admins.

NICK_SERVADMIN_HELP_INFO
	
	services admins may use the ALL parameter with any nick.

NICK_SERVADMIN_HELP_LIST
	Syntax: LIST [[pattern] [range] [NOLIMIT] [NOEXPIRE]]
	
	Lists all registered nicknames.  Nicks with the PRIVATE
	option set will only be displayed to services admins. Nicks
	with the NOEXPIRE option set will have a ! prepended
	to the nickname for services admins.
	
	If pattern is given, only users with a matching
	usermask (in nick!user@host format) are displayed. The
	number of results shown at once is limited; you may specify
	a range to see results past this limit. For example:
	
	   LIST 51-100
	       Shows results 51 to 100.
	   LIST *Bot*!* 15-20
	       Shows results 15 to 20 whose nick contains "Bot".
	   LIST *!joeuser@foo.com
	        Lists all registered nicks owned by joeuser@foo.com.
	
	(Note: unlike numbered lists, it is not possible to use
	extended syntax such as 51-60,75-80).
	
	If a pattern or range is given, specifying the NOLIMIT
	option shows an unlimited number of results.
	
	You may also specify the NOEXPIRE option to display only
	users having the NOEXPIRE flag set. This option is limited
	to services admins.

NICK_SERVADMIN_HELP_GLIST
	Syntax: GLIST [nickname]
	
	Without a parameter, lists all nicknames that are in
	your group, with their expiration dates.
	
	With a parameter, lists all nicknames that are in the
	group of the given nick.
	This use limited to services admins.

NICK_SERVADMIN_HELP_GETPASS
	Syntax: GETPASS nickname
	
	Returns the password for the given nickname.  Note that
	whenever this command is used, a message including the
	person who issued the command and the nickname it was used
	on will be logged and sent out as a WALLOPS/GLOBOPS.
	
	Limited to services admins.
	
	This command is unavailable when encryption is enabled and
	it is impossible to generate a password through OpenSSL.

NICK_SERVADMIN_HELP_HOST
	Syntax: HOST SET [nickname] [custom-hostname]
	        HOST LIST [mask] [range]
	
	HOST SET sets the custom hostname of either your
	nickname or the specified one. The custom hostname is
	unset if not given.
	
	HOST LIST lists groups that have a custom
	hostname set. If mask is given, only groups with a
	matching custom hostname will be listed. The number of
	results shown at once is limited; you may use range
	to see results past this limit. For example:
	
	   HOST LIST 51-100
	       Shows results 51 to 100.
	
	(Note: unlike numbered lists, it is not possible to use
	extended syntax such as 51-60,75-80).
	
	Limited to services operators, except for HOST SET
	that may be configured to be available to everyone.
	
	Note: this feature may be disabled or unavailable on
	your network.

###########################################################################
#
# ChanServ help messages
#
###########################################################################

CHAN_HELP
	%S allows you to register and control various
	aspects of channels.  %S can often prevent
	malicious users from "taking over" channels by limiting
	who is allowed channel operator priviliges.  Available
	commands are listed below; to use them, type
	/msg %S command.  For more information on a
	specific command, type /msg %S HELP command.
	
	    REGISTER   Register a channel
	    IDENTIFY   Identify yourself with your password
	    SET        Set channel options and information
	    ACCESS     Modify the list of privileged users
	    LEVELS     Redefine the meanings of access levels
	    AKICK      Maintain the AutoKick list
	    DROP       Cancel the registration of a channel
	    SENDPASS   Help retrive lost passwords
	
	Other commands: BAN, CLEAR, CLIST, DEOP, DEVOICE, 
	                GETKEY, INFO, INVITE, KICK, LIST,
	                LOGOUT, OP, TOPIC, UNBAN, VOICE

CHAN_HELP_UNREAL
	
	These commands are also available on this network:
	    DEHALFOP, DEOWNER, DEPROTECT, HALFOP, OWNER, PROTECT

CHAN_HELP_ULTIMATE
	
	These commands are also available on this network:
	    DEHALFOP, HALFOP

CHAN_HELP_EXPIRES
	
	Note that any channel which is not used for %d days
	(i.e. which no user on the channel's access list enters
	for that period of time) will be automatically dropped.

CHAN_HELP_REGISTER
	Syntax: REGISTER channel password description
	
	Registers a channel in the %S database.  In order
	to use this command, you must first be a channel operator
	on the channel you're trying to register.  The password
	is used with the IDENTIFY command to allow others to
	make changes to the channel settings at a later time.
	The last parameter, which must be included, is a
	general description of the channel's purpose.
	
	When you register a channel, you are recorded as the
	"founder" of the channel.  The channel founder is allowed
	to change all of the channel settings for the channel;
	%S will also automatically give the founder
	channel-operator privileges when s/he enters the channel.
	See the ACCESS command (/msg %S HELP ACCESS) for
	information on giving a subset of these privileges to
	other channel users.
	
	NOTICE: In order to register a channel, you must have
	first registered your nickname.  If you haven't,
	/msg %s HELP for information on how to do so.

CHAN_HELP_IDENTIFY
	Syntax: IDENTIFY channel password
	
	Authenticates you to %S as the founder of the given
	channel.  Many commands require you to use this command
	before using them.  The password should be the same one
	you sent with the REGISTER command.
	
CHAN_HELP_LOGOUT
	Syntax: LOGOUT channel nickname
	
	This command makes the selected nickname not identified
	anymore for the given channel.
	
	If you are the founder of the channel, you can log out anybody,
	else you can only log out yourself.

CHAN_HELP_DROP
	Syntax: DROP channel
	
	Unregisters the named channel.  Can only be used by
	channel founder.

CHAN_HELP_SET
	Syntax: SET channel option parameters
	
	Allows the channel founder to set various channel options
	and other information.
	
	Available options:
	
	    FOUNDER       Set the founder of a channel
	    SUCCESSOR     Set the successor for a channel
	    PASSWORD      Set the founder password
	    DESC          Set the channel description
	    URL           Associate a URL with the channel
	    EMAIL         Associate an E-mail address with the channel
	    ENTRYMSG      Set a message to be sent to users when they
	                     enter the channel
	    BANTYPE       Set how Services make bans on the channel
	    MLOCK         Lock channel modes on or off
	    KEEPTOPIC     Retain topic when channel is not in use
	    OPNOTICE      Send a notice when OP/DEOP commands are used
	    PEACE         Regulate the use of critical commands
	    PRIVATE       Hide channel from LIST command
	    RESTRICTED    Restrict access to the channel
	    SECURE        Activate %S security features
	    SECUREOPS     Stricter control of chanop status
	    SECUREFOUNDER Stricter control of channel founder status
	    SIGNKICK      Sign kicks that are done with KICK command
	    TOPICLOCK     Topic can only be changed with TOPIC
	
	Type /msg %S HELP option for more information on a
	particular option.

CHAN_HELP_SET_FOUNDER
	Syntax: SET channel FOUNDER nick
	
	Changes the founder of a channel.  The new nickname must
	be a registered one.

CHAN_HELP_SET_SUCCESSOR
	Syntax: SET channel SUCCESSOR nick
	
	Changes the successor of a channel.  If the founder's
	nickname expires or is dropped while the channel is still
	registered, the successor will become the new founder of the
	channel.  However, if the successor already has too many
	channels registered (%d), the channel will be dropped
	instead, just as if no successor had been set.  The new
	nickname must be a registered one.

CHAN_HELP_SET_PASSWORD
	Syntax: SET channel PASSWORD password
	
	Sets the password used to identify as the founder of the
	channel.

CHAN_HELP_SET_DESC
	Syntax: SET channel DESC description
	
	Sets the description for the channel, which shows up with
	the LIST and INFO commands.

CHAN_HELP_SET_URL
	Syntax: SET channel URL [url]
	
	Associates the given URL with the channel.  This URL will
	be displayed whenever someone requests information on the
	channel with the INFO command.  If no parameter is given,
	deletes any current URL for the channel.

CHAN_HELP_SET_EMAIL
	Syntax: SET channel EMAIL [address]
	
	Associates the given E-mail address with the channel.
	This address will be displayed whenever someone requests
	information on the channel with the INFO command.  If no
	parameter is given, deletes any current E-mail address for
	the channel.

CHAN_HELP_SET_ENTRYMSG
	Syntax: SET channel ENTRYMSG [message]
	
	Sets the message which will be sent via /notice to users
	when they enter the channel.  If no parameter is given,
	causes no message to be sent upon entering the channel.

CHAN_HELP_SET_BANTYPE
	Syntax: SET channel BANTYPE bantype
	
	Sets the ban type that will be used by services whenever
	they need to ban someone from your channel.
	
	bantype is a number between 0 and 3 that means:
	
	0: ban in the form *!user@host
	1: ban in the form *!*user@host
	2: ban in the form *!*@host
	3: ban in the form *!*user@*.domain

CHAN_HELP_SET_KEEPTOPIC
	Syntax: SET channel KEEPTOPIC {ON | OFF}
	
	Enables or disables the topic retention option for a
	channel.  When topic retention is set, the topic for the
	channel will be remembered by %S even after the
	last user leaves the channel, and will be restored the
	next time the channel is created.

CHAN_HELP_SET_TOPICLOCK
	Syntax: SET channel TOPICLOCK {ON | OFF}
	
	Enables or disables the topic lock option for a channel.
	When topic lock is set, %S will not allow the
	channel topic to be changed except via the TOPIC
	command.

CHAN_HELP_SET_MLOCK
	Syntax: SET channel MLOCK modes
	
	Sets the mode-lock parameter for the channel. %S
	allows you to define certain channel modes to be always
	on, off or free to be either on or off.
	
	The modes parameter is constructed exactly the same way 
	as a /MODE command; that is, modes followed by a + are 
	locked on, and modes followed by a - are locked off. Note,
	however, that unlike the /MODE command, each use of
	SET MLOCK will remove all modes previously locked before
	setting the new!
	
	Warning:  If you set a mode-locked key, as in the second
	example below, you should also set the RESTRICTED option for
	the channel (see HELP SET RESTRICTED), or anyone entering
	the channel when it is empty will be able to see the key!
	
	Examples:
	
	    SET #channel MLOCK +nt-iklps
	        Forces modes n and t on, and modes i, k, l, p, and
	        s off.  Mode m is left free to be either on or off.
	
	    SET #channel MLOCK +knst-ilmp my-key
	        Forces modes k, n, s, and t on, and modes i, l, m,
	        and p off.  Also forces the channel key to be
	        "my-key".
	
	    SET #channel MLOCK +
	        Removes the mode lock; all channel modes are free
	        to be either on or off.

CHAN_HELP_SET_PEACE
	Syntax: SET channel PEACE {ON | OFF}
	
	Enables or disables the peace option for a channel.
	When peace is set, an user won't be able to kick,
	ban or remove a channel status of an user that has
	a level superior or equal to his via %S commands.

CHAN_HELP_SET_PRIVATE
	Syntax: SET channel PRIVATE {ON | OFF}
	
	Enables or disables the private option for a channel.
	When private is set, a /msg %S LIST will not
	include the channel in any lists.

CHAN_HELP_SET_RESTRICTED
	Syntax: SET channel RESTRICTED {ON | OFF}
	
	Enables or disables the restricted access option for a
	channel.  When restricted access is set, users who would
	normally be disallowed from having channel operator
	privileges (users with negative access levels and, if
	secure ops is set, users not on the access list) will
	instead be kicked and banned from the channel.

CHAN_HELP_SET_SECURE
	Syntax: SET channel SECURE {ON | OFF}
	
	Enables or disables %S's security features for a
	channel.  When SECURE is set, only users who have
	registered their nicknames with %s and IDENTIFY'd
	with their password will be given access to the channel
	as controlled by the access list.

CHAN_HELP_SET_SECUREOPS
	Syntax: SET channel SECUREOPS {ON | OFF}
	
	Enables or disables the secure ops option for a channel.
	When secure ops is set, users who are not on the userlist
	will not be allowed chanop status.
	
CHAN_HELP_SET_SECUREFOUNDER
	Syntax: SET channel SECUREFOUNDER {ON | OFF}
	
	Enables or disables the secure founder option for a channel.
	When secure founder is set, only the real founder will be
	able to drop the channel, change its password, its founder and its
	successor, and not those who are IDENTIFY'd with %S.
	
CHAN_HELP_SET_SIGNKICK
	Syntax: SET channel SIGNKICK {ON | LEVEL | OFF}
	
	Enables or disables signed kicks for a
	channel.  When SIGNKICK is set, kicks issued with
	%S KICK command will have the nick that used the
	command in their reason.
	
	If you use LEVEL, those who have a level that is superior 
	or equal to the SIGNKICK level on the channel won't have their 
	kicks signed. See /msg %S HELP LEVELS for more information. 

CHAN_HELP_SET_OPNOTICE
	Syntax: SET channel OPNOTICE {ON | OFF}
	
	Enables or disables the op-notice option for a channel.
	When op-notice is set, %S will send a notice to the
	channel whenever the OP or DEOP commands are used for a user
	in the channel.

CHAN_HELP_ACCESS
	Syntax: ACCESS channel ADD nick level
	        ACCESS channel DEL {nick | entry-num | list}
	        ACCESS channel LIST [mask | list]
	        ACCESS channel CLEAR
	
	Maintains the access list for a channel.  The access
	list specifies which users are allowed chanop status or
	access to %S commands on the channel.  Different
	user levels allow for access to different subsets of
	privileges; /msg %S HELP ACCESS LEVELS for more
	specific information.  Any nick not on the access list has
	a user level of 0.
	
	The ACCESS ADD command adds the given nickname to the
	access list with the given user level; if the nick is
	already present on the list, its access level is changed to
	the level specified in the command.  The level specified
	must be less than that of the user giving the command, and
	if the nick is already on the access list, the current
	access level of that nick must be less than the access level
	of the user giving the command.
	
	The ACCESS DEL command removes the given nick from the
	access list.  If a list of entry numbers is given, those
	entries are deleted.  (See the example for LIST below.)
	
	The ACCESS LIST command displays the access list.  If
	a wildcard mask is given, only those entries matching the
	mask are displayed.  If a list of entry numbers is given,
	only those entries are shown; for example:

	   ACCESS #channel LIST 2-5,7-9
	      Lists access entries numbered 2 through 5 and
	      7 through 9.
	      
	The ACCESS CLEAR command clears all entries of the
	access list.

CHAN_HELP_ACCESS_LEVELS
	User access levels
	
	By default, the following access levels are defined:
	
	   Founder   Full access to %S functions; automatic
	                     opping upon entering channel.  Note
	                     that only one person may have founder
	                     status (it cannot be given using the
	                     ACCESS command).
	        10   Access to AKICK command; automatic opping.
	         5   Automatic opping.
	         3   Automatic voicing.
	         0   No special privileges; can be opped by other
	                     ops (unless secure-ops is set).
	        <0   May not be opped.
	
	These levels may be changed, or new ones added, using the
	LEVELS command; type /msg %S HELP LEVELS for
	information.

CHAN_HELP_AKICK
	Syntax: AKICK channel ADD [+expiry] {mask | nick} [reason]
	        AKICK channel STICK mask
	        AKICK channel UNSTICK mask
	        AKICK channel DEL {mask| entry-num | list}
	        AKICK channel LIST [mask | list]
	        AKICK channel VIEW [mask | list]
	        AKICK channel ENFORCE
	        AKICK channel CLEAR
	
	Maintains the AutoKick list for a channel.  If a user
	on the AutoKick list attempts to join the channel,
	%S will ban that user from the channel, then kick
	the user.
	
	The AKICK ADD command adds the given nick or usermask
	to the AutoKick list. expiry is specified as an integer
	followed by one of w (weeks), d (days), h (hours),
	or m (minutes). Combinations (such as 1h30m) are
	allowed. To add an AutoKick which does not expire, leave
	that parameter out. If a reason is given with the
	command, that reason will be used when the user is kicked;
	if not, a default reason message is used.
	
	The AKICK STICK command permanently bans the given mask 
	on the channel. If someone tries to remove the ban, %S
	will automatically set it again. You can stick usermasks
	only, not registered nicks.
	
	The AKICK UNSTICK command cancels the effect of the
	AKICK STICK command, letting any operator on the channel
	remove the ban.
	
	The AKICK DEL command removes the given nick or mask
	from the AutoKick list.  It does not, however, remove any
	bans placed by an AutoKick; those must be removed
	manually. If a list of entry numbers is given, those
	entries are deleted.  (See the example for LIST below.)
	
	The AKICK LIST command displays the AutoKick list. If
	a wildcard mask is given, only those entries matching the
	mask are displayed.  If a list of entry numbers is given,
	only those entries are shown; for example:

	   AKICK #channel LIST 2-5,7-9
	      Lists AutoKick entries numbered 2 through 5 and
	      7 through 9.
	
	The AKICK VIEW command is a more verbose version of
	AKICK LIST command.
	
	The AKICK ENFORCE command causes %S to enforce the
	current AKICK list by removing those users who match an
	AKICK mask.
	
	The AKICK CLEAR command clears all entries of the
	akick list.

CHAN_HELP_LEVELS
	Syntax: LEVELS channel SET type level
	        LEVELS channel {DIS | DISABLE} type
	        LEVELS channel LIST
	        LEVELS channel RESET
	
	The LEVELS command allows fine control over the meaning of
	the numeric access levels used for channels.  With this
	command, you can define the access level required for most
	of %S's functions.  (The SET FOUNDER and SET PASSWORD
	commands, as well as this command, are always restricted to
	the channel founder.)
	
	LEVELS SET allows the access level for a function or group of
	functions to be changed.  LEVELS DISABLE (or DIS for short)
	disables an automatic feature or disallows access to a
	function by anyone other than the channel founder.
	LEVELS LIST shows the current levels for each function or
	group of functions.  LEVELS RESET resets the levels to the
	default levels of a newly-created channel (see
	HELP ACCESS LEVELS).
	
	For a list of the features and functions whose levels can be
	set, see HELP LEVELS DESC.

CHAN_HELP_LEVELS_DESC
	The following feature/function names are understood.  Note
	that the levels for AUTODEOP and NOJOIN are maximum levels,
	while all others are minimum levels.

CHAN_HELP_LEVELS_DESC_FORMAT
	    %-*s  %s

CHAN_HELP_INFO
	Syntax: INFO channel [ALL]
	
	Lists information about the named registered channel,
	including its founder, time of registration, last time
	used, description, and mode lock, if any. If ALL is 
	specified, the entry message and successor will also 
	be displayed.
	
	By default, the ALL option is limited to those with
	founder access on the channel.

CHAN_HELP_LIST
	Syntax: LIST [[pattern] [range]]
	
	Lists all registered channels. Channels with the PRIVATE
	option set are not displayed.
	
	If pattern is given, only channels with a matching
	mask are displayed. The number of results shown at once is
	limited; you may specify a range to see results past
	this limit. For example:
	
	   LIST 51-100
	       Shows results 51 to 100.
	   LIST *help* 15-20
	       Shows results 15 to 20 whose nick contains "help".
	
	(Note: unlike numbered lists, it is not possible to use
	extended syntax such as 51-60,75-80).
	
	On some networks, this command is reserved to IRC
	operators.

CHAN_HELP_CLIST
	Syntax: CLIST [privilege]
	
	Lists channels where you have the given privilege.
	
	Usage examples:
	
	  CLIST
	  
	    Lists all channels where you have privileges.
	  
	  CLIST FOUNDER
	  
	    Lists all channels where are the founder.
	    
	  CLIST SUCCESSOR
	  
	    Lists all channels where you are the successor. 
	
	  CLIST 3
	  
	    Lists all channels where you have access level 3
	    or more.
	  
	  CLIST -1
	  
	    Lists all channels where you have access level -1
	    or less.

CHAN_HELP_OP
	Syntax: OP [#channel [nick]]
	
	Ops a selected nick on a channel. If nick is not given,
	it will op you. If channel and nick are not given,
	it will op you on all channels you're on, provided you
	have the rights to.
	
	By default, limited those with level 5 access and above
	on the channel.

CHAN_HELP_DEOP
	Syntax: DEOP [#channel [nick]]
	
	Deops a selected nick on a channel. If nick is not given,
	it will deop you. If channel and nick are not given,
	it will deop you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to those with level 5 access and above
	on the channel.
	
CHAN_HELP_VOICE
	Syntax: VOICE [#channel [nick]]
	
	Voices a selected nick on a channel. If nick is not given,
	it will voice you. If channel and nick are not given,
	it will voice you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to those with level 5 access and above
	on the channel, or to those with level 3 and above for self
	voicing.

CHAN_HELP_DEVOICE
	Syntax: DEVOICE [#channel [nick]]
	
	Devoices a selected nick on a channel. If nick is not given,
	it will devoice you. If channel and nick are not given,
	it will devoice you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to those with level 5 access and above
	on the channel, or to those with level 3 and above for self
	devoicing.
	
CHAN_HELP_HALFOP
	Syntax: HALFOP [#channel [nick]]
	
	Halfops a selected nick on a channel. If nick is not given,
	it will halfop you. If channel and nick are not given,
	it will halfop you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to those with level 5 access and above
	on the channel, or to those with level 4 and above for self
	halfopping.

CHAN_HELP_DEHALFOP
	Syntax: DEHALFOP [#channel [nick]]
	
	Dehalfops a selected nick on a channel. If nick is not given,
	it will dehalfop you. If channel and nick are not given,
	it will dehalfop you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to those with level 5 access and above
	on the channel, or to those with level 4 and above for self
	dehalfopping.
	
CHAN_HELP_PROTECT
	Syntax: PROTECT [#channel [nick]]
	
	Protects a selected nick on a channel. If nick is not given,
	it will protect you. If channel and nick are not given,
	it will protect you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to the founder, or to those with 
	level 10 and above on the channel for self protecting.

CHAN_HELP_DEPROTECT
	Syntax: DEPROTECT [#channel [nick]]
	
	Deprotects a selected nick on a channel. If nick is not given,
	it will deprotect you. If channel and nick are not given,
	it will deprotect you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to the founder, or to those with 
	level 10 and above on the channel for self deprotecting.

CHAN_HELP_OWNER
	Syntax: OWNER [#channel]
	
	Gives you owner status on channel. If channel is not 
	given, it will give you owner status on all channels you're 
	on, provided you have the rights to.
	
	Limited to those with founder access on the channel.

CHAN_HELP_DEOWNER
	Syntax: DEOWNER [#channel]
	
	Removes your owner status on channel. If channel is 
	not given, it will remove your owner status on all channels 
	you're on, provided you have the rights to.
	
	Limited to those with founder access on the channel.

CHAN_HELP_INVITE
	Syntax: INVITE channel
	
	Tells %S to invite you into the given channel.  
	
	By default, limited to those with level 5 and above
	on the channel.

CHAN_HELP_UNBAN
	Syntax: UNBAN channel
	
	Tells %S to remove all bans preventing you from
	entering the given channel.  
	
	By default, limited to those with level 5 and above
	on the channel.
	
CHAN_HELP_KICK
	Syntax: KICK [#channel [nick [reason]]]
	
	Kicks a selected nick on a channel. If nick is not given,
	it will kick you. If channel and nick are not given,
	it will kick you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to those with level 5 and above
	on the channel.

CHAN_HELP_BAN
	Syntax: BAN [#channel [nick [reason]]]
	
	Bans a selected nick on a channel. If nick is not given,
	it will ban you. If channel and nick are not given,
	it will ban you on all channels you're on, provided you
	have the rights to.
	
	By default, limited to those with level 5 and above
	on the channel.

CHAN_HELP_TOPIC
	Syntax: TOPIC channel [topic]
	
	Causes %S to set the channel topic to the one
	specified. If topic is not given, then an empty topic
	is set. This command is most useful in conjunction
	with SET TOPICLOCK. See /msg %S HELP SET TOPICLOCK
	for more information.
	
	By default, limited to those with founder access on the
	channel. 

CHAN_HELP_CLEAR
	Syntax: CLEAR channel what
	
	Tells %S to clear certain settings on a channel.  what
	can be any of the following:
	
	     MODES    Resets all modes on the channel (i.e. clears
	                  modes i,k,l,m,n,p,s,t).
	     BANS     Clears all bans on the channel.
	     OPS      Removes channel-operator status (mode +o) from
	                  all channel operators.
	     VOICES   Removes "voice" status (mode +v) from anyone
	                  with that mode set.
	     USERS    Removes (kicks) all users from the channel.
	
	By default, limited to those with founder access on the
	channel.
	
CHAN_HELP_GETKEY
	Syntax: GETKEY channel
	
	Returns the key of the given channel. This is a command
	mainly intended to be used by bots and/or scripts, so
	the output is in the following way:
	
	KEY <channel> <key>
	
	key is "NO KEY" if no key is set.
	
CHAN_HELP_SENDPASS
	Syntax: SENDPASS channel
	
	Send the password of the given channel to the e-mail
	address set in the founder's nickname record. This
	command is really useful to deal with lost passwords.
	
	May be limited to IRC operators on certain networks.
	
	This command is unavailable when encryption is enabled and
	it is impossible to generate a password through OpenSSL.

CHAN_SERVADMIN_HELP
	
	The following commands are available to services admins:
	
	    GETPASS    Retrieve the founder password for a channel
	                   (only if encryption is disabled)
	    STATUS     Returns the current access level of a user
	                   on a channel
	
	services admins can also drop any channel without needing
	to identify via password, and may view the access, AKICK,
	and level setting lists for any channel and the channel
	privileges list for any nick.
	
CHAN_SERVADMIN_HELP_LOGOUT
	Syntax: LOGOUT channel [nickname]
	
	This command makes the selected nickname not identified
	anymore for the given channel.
	
	If you are the founder of the channel, you can log out anybody,
	else you can only log out yourself.
	
	If you are a Services admin, you can log out 
	anybody of any channel without having to be the founder
	of the channel. Also, you can omit the nickname parameter;
	this will log out all identified users from the channel.

CHAN_SERVADMIN_HELP_DROP
	Syntax: DROP channel
	
	Unregisters the named channel.  Only services admins
	can drop a channel for which they have not identified.

CHAN_SERVADMIN_HELP_SET
	
	services admins can also set the option NOEXPIRE, with
	which channels can be prevented from expiring.
	Additionally, services admins can set options for any
	channel without identifying by password for the channel.

CHAN_SERVADMIN_HELP_SET_NOEXPIRE
	Syntax: SET channel NOEXPIRE {ON | OFF}
	
	Sets whether the given channel will expire.  Setting this
	to ON prevents the channel from expiring.
	
	Limited to services admins.

CHAN_SERVADMIN_HELP_INFO
	
	services admins can use the ALL parameter with any channel.

CHAN_SERVADMIN_HELP_LIST
	Syntax: LIST [[pattern] [range] [NOLIMIT] [NOEXPIRE]]
	
	Lists all registered channels.  Channels with the PRIVATE
	option set are only displayed to services admins. Channels
	with the NOEXPIRE option set have a ! prepended
	to the nickname for services admins.
	
	If pattern is given, only channels with a matching
	mask are displayed. The number of results shown at once is
	limited; you may specify a range to see results past this
	limit. For example:
	
	   LIST 51-100
	       Shows results 51 to 100.
	   LIST *help* 15-20
	       Shows results 15 to 20 whose name contains "help".
	
	(Note: unlike numbered lists, it is not possible to use
	extended syntax such as 51-60,75-80).
	
	If a pattern or range is given, specifying the NOLIMIT
	option shows an unlimited number of results.
	
	You may also specify the NOEXPIRE option to display only
	channels having the NOEXPIRE flag set. This option is limited
	to services admins.

CHAN_SERVADMIN_HELP_CLIST
	Syntax: CLIST [nickname] [privilege]
	
	Lists channels where you have the given privilege. If a
	nickname is given, lists the privileges for this user
	instead (this use is limited to services admins).
	Channels that have the NOEXPIRE option set will be
	prefixed by an exclamation mark.
	
	Usage examples:
	
	  CLIST Epona
	  
	    Lists all channels where Epona has privileges.
	  
	  CLIST Pegasus FOUNDER
	  
	    Lists all channels where Pegasus is the founder.
	    
	  CLIST SUCCESSOR
	  
	    Lists all channels where you are the successor. 
	
	  CLIST 3
	  
	    Lists all channels where you have access level 3
	    or more.
	  
	  CLIST Unicorn-Lover -1
	  
	    Lists all channels where Unicorn-Lover has access
	    level -1 or less.

CHAN_SERVADMIN_HELP_GETPASS
	Syntax: GETPASS channel
	
	Returns the password for the given channel.  Note that
	whenever this command is used, a message including the
	person who issued the command and the channel it was used
	on will be logged and sent out as a WALLOPS/GLOBOPS.
	
	Limited to services admins.
	
	This command is unavailable when encryption is enabled and
	it is impossible to generate a password through OpenSSL.

CHAN_SERVADMIN_HELP_STATUS
	Syntax: STATUS channel nickname
	
	Returns the current access level of the given nick on the
	given channel.  The reply is of the form:
	
	    STATUS channel nickname access-level
	
	If an error occurs, the reply will be in the form:
	
	    STATUS ERROR error-message
	
	Limited to services admins.

###########################################################################
#
# MemoServ help messages
#
###########################################################################

MEMO_HELP
	%S is a utility allowing IRC users to send short
	messages to other IRC users, whether they are online at
	the time or not.  Both the sender's nickname and the target
	nickname must be registered in order to send a memo.
	
	%S's commands include:
	
	    SEND   Send a memo to a nick or channel
	    CANCEL Cancel the last memo you sent
	    LIST   List your memos
	    READ   Read a memo or memos
	    DEL    Delete a memo or memos
	    SET    Set options related to memos
	
	Type /msg %S HELP command for help on any of the
	above commands.

MEMO_HELP_SEND
	Syntax: SEND {nick | channel} memo-text
	
	Sends the named nick a memo containing memo-text. If
	a channel is specified instead of a nick, the memo is
	sent to all users on its access list with at least level 10
	(by default; this can be changed with the %s
	LEVELS command.)
	
	The recipient(s) will receive a notice that they have a new
	memo.  The target nickname/channel must be registered.
	
MEMO_HELP_CANCEL
	Syntax: CANCEL nick
	
	Cancels the last memo you sent to the given nick, provided
	it has not been read at the time you use the command.

MEMO_HELP_LIST
	Syntax: LIST [list | NEW]
	
	Lists any memos you currently have.  With NEW, lists only
	new (unread) memos.  Unread memos are marked with a "*"
	to the left of the memo number.  You can also specify a list
	of numbers, as in the example below:

	   LIST 2-5,7-9
	      Lists memos numbered 2 through 5 and 7 through 9.

MEMO_HELP_READ
	Syntax: READ {num | list | LAST | NEW}
	
	Sends you the text of the memos specified.  If LAST is
	given, sends you the memo you most recently received.  If
	NEW is given, sends you all of your new memos.  Otherwise,
	sends you memo number num.  You can also give a list of
	numbers, as in this example:
	
	   READ 2-5,7-9
	      Displays memos numbered 2 through 5 and 7 through 9.

MEMO_HELP_DEL
	Syntax: DEL {num | list | ALL}
	
	Deletes the specified memo or memos.  You can supply
	multiple memo numbers or ranges of numbers instead of a
	single number, as in the second example below.
	
	If ALL is given, deletes all of your memos.
	
	Examples:
	
	   DEL 1
	      Deletes your first memo.
	
	   DEL 2-5,7-9
	      Deletes memos numbered 2 through 5 and 7 through 9.

MEMO_HELP_SET
	Syntax: SET option parameters
	
	Sets various memo options.  option can be one of:
	
	    NOTIFY      Changes when you will be notified about
	                   new memos
	    LIMIT       Sets the maximum number of memos you can
	                   receive
	    FORWARD     Sets whether you will get a copy
	                   of new memos to your e-mail address
	
	Type /msg %S HELP SET option for more information
	on a specific option.

MEMO_HELP_SET_NOTIFY
	Syntax: SET NOTIFY {ON | LOGON | NEW | OFF}
	
	Changes when you will be notified about new memos:
	
	    ON      You will be notified of memos when you log on,
	               when you unset /AWAY, and when they are sent
	               to you.
	    LOGON   You will only be notified of memos when you log
	               on or when you unset /AWAY.
	    NEW     You will only be notified of memos when they
	               are sent to you.
	    OFF     You will not receive any notification of memos.
	
	ON is essentially LOGON and NEW combined.

MEMO_HELP_SET_LIMIT
	Syntax: SET LIMIT limit
	
	Sets the maximum number of memos you are allowed to have.
	If you set this to 0, no one will be able to send any
	memos to you.  However, you cannot set this any higher
	than %d.

MEMO_HELP_SET_FORWARD
	Syntax: SET FORWARD {ON | OFF}
	
	Sets whether you will get a copy of new memos by
	e-mail (ON) or not (OFF).
	
	You can't enable this option unless you have an
	authenticated e-mail address associated with your nick.
	Type /msg %s HELP SET EMAIL and /msg %s HELP AUTH for
	more information.
	
	This option may be disabled on some networks.

MEMO_HELP_INFO
	Syntax: INFO
	
	Displays information on the number of memos you have, how
	many of them are unread, and how many total memos you can
	receive.

MEMO_SERVADMIN_HELP_SET_LIMIT
	Syntax: SET LIMIT [nick] {limit | NONE} [HARD]
	
	Sets the maximum number of memos a user is allowed to have.
	Setting the limit to 0 prevents the user from receiving any
	memos; setting it to NONE allows the user to receive and
	keep as many memos as they want.  If you do not give a
	nickname, your own limit is set.
	
	Adding HARD prevents the user from changing the limit.
	Not adding HARD has the opposite effect, allowing the
	user to change the limit (even if a previous limit was set
	with HARD).
	
	This use of the SET LIMIT command is limited to
	services admins. Other users may only enter a limit for
	themselves, may not remove their limit, may not set a limit
	above %d, and may not set a hard limit.

MEMO_SERVADMIN_HELP_INFO
	Syntax: INFO [nick]
	
	Without a parameter, displays information on the number of
	memos you have, how many of them are unread, and how many
	total memos you can receive.
	
	With a channel parameter, displays the same information for
	the given channel.
	
	With a nickname parameter, displays the same information
	for the given nickname.  This use limited to Services
	admins.

###########################################################################
#
# OperServ help messages
#
###########################################################################

OPER_HELP
	%S commands:
	    GLOBAL      Send a message to all users
	    STATS       Show status of services and network
	    OPER LIST   List all services operators
	    ADMIN LIST  List all services admins
	
	Commands available to services operators:
	    MODE        Change a channel's modes
	    KICK        Kick a user from a channel
	    CLEARMODES  Clear modes of a channel
	    AKILL       Manipulate the AKILL list
	    SGLINE      Manipulate the SGLINE list
	    SQLINE      Manipulate the SQLINE list
	    SZLINE      Manipulate the SZLINE list
	
	Commands available to services admins:
	    OPER        Modify the services operator list
	    CHANLIST    List all existing channels on the network
	    USERLIST    List all users connected to the network
	    NEWS        Define news messages shown on various occasions
	    SESSION     View the list of host sessions
	    EXCEPTION   Modify the session-limit exception list
	    BLACKLIST   Modify blacklisted hosts
	    NOOP        Temporarily remove all O:lines of a server 
	                    remotely
	    JUPE        "Jupiter" a server
	    SET         Set various global Epona options
	    RELOAD      Reload Epona configuration
	    SYNC        Synchronize modules now
	    RESTART     Restart Epona
	    STOP        Stop Epona
	    FATAL       Stop Epona abruptly
	
	Commands available to the services super-users:
	    ADMIN       Modify the services admin list
	    MODULES     Manages loaded modules
	
	Notice: All commands sent to %S are logged!

OPER_HELP_GLOBAL
	Syntax: GLOBAL message
	
	Allows IRCops to send messages to all users on the network.
	The message will be sent from the nick %s.

OPER_HELP_STATS
	Syntax: STATS [AKILL | ALL | RESET]
	
	Without any option, shows the current number of users and
	IRCops online (excluding services), the highest number of
	users online since services was started, and the length of
	time services has been running.
	
	With the AKILL option, displays the current size of the
	AKILL, SGLINE, SQLINE and SZLINE lists and their current
	default expiry time.
	
	The ALL option is available only to services admins, and
	displays information on services' memory usage.  Using this
	option can freeze services for a short period of time on
	large networks, so don't overuse it!
	
	The RESET option currently resets the maximum user count
	to the number of users currently present on the network.
	
	UPTIME may be used as a synonym for STATS.

OPER_HELP_OPER
	Syntax: OPER ADD nick
	        OPER DEL {nick | entry-num | list}
	        OPER LIST [mask | list]
	        OPER CLEAR
	
	Allows the services admins to add or remove nicknames
	to or from the services operator list.  A user whose nickname
	is on the services operator list and who has identified to
	%s will be able to access services operator commands.
	
	The OPER ADD command adds the given nickname to the
	services operator list.
	
	The OPER DEL command removes the given nick from the
	services operator list.  If a list of entry numbers is given, 
	those entries are deleted.  (See the example for LIST below.)
	
	The OPER LIST command displays the services operator list.  
	If a wildcard mask is given, only those entries matching the
	mask are displayed.  If a list of entry numbers is given,
	only those entries are shown; for example:

	   OPER LIST 2-5,7-9
	      Lists services operator entries numbered 2 through 
	      5 and 7 through 9.
	      
	The OPER CLEAR command clears all entries of the
	services operator list.
	
	Any IRC operator may use the OPER LIST form of the command.
	All other use limited to services admins.

OPER_HELP_ADMIN
	Syntax: ADMIN ADD nick
	        ADMIN DEL {nick | entry-num | list}
	        ADMIN LIST [mask | list]
	        ADMIN CLEAR
	
	Allows the services roots to add or remove nicknames
	to or from the services admin list.  A user whose nickname
	is on the services admin list and who has identified to
	%s will be able to access services admin commands
	(including those reserved to services operators).
	
	The ADMIN ADD command adds the given nickname to the
	services admin list.
	
	The ADMIN DEL command removes the given nick from the
	services admin list.  If a list of entry numbers is given, 
	those entries are deleted.  (See the example for LIST below.)
	
	The ADMIN LIST command displays the services admin list.  
	If a wildcard mask is given, only those entries matching the
	mask are displayed.  If a list of entry numbers is given,
	only those entries are shown; for example:

	   ADMIN LIST 2-5,7-9
	      Lists services admin entries numbered 2 through 
	      5 and 7 through 9.
	      
	The ADMIN CLEAR command clears all entries of the
	services admin list.
	
	Any IRC operator may use the ADMIN LIST form of the command.
	All other use limited to services root.

OPER_HELP_MODE
	Syntax: MODE channel modes
	
	Allows services operators to set channel modes for any
	channel.  Parameters are the same as for the standard /MODE
	command.
	
	Limited to services operators.

OPER_HELP_CLEARMODES
	Syntax: CLEARMODES channel [ALL]
	
	Clears all binary modes (such as i,k,l,m,n,p,s,t) and bans
	from a channel.  If ALL is given, also clears all channel
	user modes (such as +o and +v modes) from the channel.
	
	Limited to services operators.

OPER_HELP_KICK
	Syntax: KICK channel user reason
	
	Allows IRCops to kick a user from any channel.
	Parameters are the same as for the standard /KICK command.
	The kick message will have the nickname of the IRCop
	sending the KICK command prepended; for example:
	
	*** SpamMan has been kicked off channel #my_channel by %S (Alcan (Flood))
	
	Limited to services operators.

OPER_HELP_AKILL
	Syntax: AKILL ADD [+expiry] {mask | nick[:type]} reason
	        AKILL DEL {mask | entry-num | list}
	        AKILL LIST [mask | list]
	        AKILL VIEW [mask | list]
	        AKILL CLEAR
	
	Allows services operators to manipulate the AKILL list.  If
	a user matching an AKILL mask attempts to connect, services
	will issue a KILL for that user and, on supported server
	types, will instruct all servers to add a ban (K-line) for
	the mask which the user matched.
	
	AKILL ADD adds the given user@host mask to the AKILL
	list for the given reason (which must be given). If
	a nick is given instead of a mask, then it is
	automatically translated to a relevant mask for this 
	connected user (a type may optionally be specified
	to change the default mask type that is computed).
	expiry is specified as an integer followed by one of w
	(weeks), d (days), h (hours), or m (minutes).
	Combinations (such as 1h30m) are allowed. To add an AKILL
	which does not expire, use +0.  If the usermask to be
	added starts with a +, an expiry time must be given, even
	if it is the same as the default.  The current AKILL
	default expiry time can be found with the STATS AKILL
	command.
	
	The AKILL DEL command removes the given mask from the
	AKILL list if it is present.  If a list of entry numbers is 
	given, those entries are deleted.  (See the example for LIST 
	below.)
	
	The AKILL LIST command displays the AKILL list.  
	If a wildcard mask is given, only those entries matching the
	mask are displayed. If the mask starts with #, then the
	entry with the same tracking ID is listed (those are
	automatically appended to the AKILL reason when
	disconnections are issued). If a list of entry numbers is
	given, only those entries are shown; for example:

	   AKILL LIST 2-5,7-9
	      Lists AKILL entries numbered 2 through 5 and 7 
	      through 9.
	      
	AKILL VIEW is a more verbose version of AKILL LIST, and 
	will show who added an AKILL, the date it was added, and when 
	it expires, as well as the user@host mask and reason.
	
	AKILL CLEAR clears all entries of the AKILL list.
	
	Limited to services operators.


OPER_HELP_SGLINE
	Syntax: SGLINE ADD [+expiry] mask:reason
	    	SGLINE DEL {mask | entry-num | list}
	        SGLINE LIST [mask | list]
	        SGLINE VIEW [mask | list]
	        SGLINE CLEAR
	
	Allows services operators to manipulate the SGLINE list. If
	a user with a realname matching a SGLINE mask attempts to 
	connect, services won't allow him to continue his IRC
	session.
	
	SGLINE ADD adds the given realname mask to the SGLINE
	list for the given reason (which must be given).
	expiry is specified as an integer followed by one of w
	(weeks), d (days), h (hours), or m (minutes).
	Combinations (such as 1h30m) are allowed. To add an
	SGLINE which does not expire, use +0. If the mask to be
	added starts with a +, an expiry time must be given, even
	if it is the same as the default. The current SGLINE
	default expiry time can be found with the STATS AKILL
	command.
	Note: because the realname mask may contain spaces, the
	separator between it and the reason is a colon, not a
	space.
	
	The SGLINE DEL command removes the given mask from the
	SGLINE list if it is present.  If a list of entry numbers
	is given, those entries are deleted.  (See the example for
	LIST below.)
	
	The SGLINE LIST command displays the SGLINE list.  
	If a wildcard mask is given, only those entries matching
	the mask are displayed. If the mask starts with #, then the
	entry with the same tracking ID is listed (those are
	automatically appended to the SGLINE reason issued to the
	IRC servers). If a list of entry numbers is given, only
	those entries are shown; for example:

	   SGLINE LIST 2-5,7-9
	      Lists SGLINE entries numbered 2 through 5 and 7 
	      through 9.
	      
	SGLINE VIEW is a more verbose version of SGLINE LIST,
	and will show who added an SGLINE, the date it was added,
	and when it expires, as well as the mask and reason.
	
	SGLINE CLEAR clears all entries of the SGLINE list.
	
	This command requires special support from the IRC server,
	and is therefore disabled on some networks.
	
	Limited to services operators.

OPER_HELP_SQLINE
	Syntax: SQLINE ADD [+expiry] mask reason
	    	SQLINE DEL {mask | entry-num | list}
	        SQLINE LIST [mask | list]
	        SQLINE VIEW [mask | list]
	        SQLINE CLEAR
	
	Allows services operators to manipulate the SQLINE list.  If
	a user with a nick matching a SQLINE mask attempts to 
	connect, services will do anything they need to prevent
	him from using it.
	
	If the first character of the mask is #, services will 
	prevent the use of matching channels.
	
	SQLINE ADD adds the given mask to the SQLINE
	list for the given reason (which must be given).
	expiry is specified as an integer followed by one of w
	(weeks), d (days), h (hours), or m (minutes).
	Combinations (such as 1h30m) are allowed. To add an
	SQLINE which does not expire, use +0. If the mask to be
	added starts with a +, an expiry time must be given, even
	if it is the same as the default. The current SQLINE
	default expiry time can be found with the STATS AKILL
	command.
	
	The SQLINE DEL command removes the given mask from the
	SQLINE list if it is present.  If a list of entry numbers
	is given, those entries are deleted.  (See the example for
	LIST below.)
	
	The SQLINE LIST command displays the SQLINE list.  
	If a wildcard mask is given, only those entries matching
	the mask are displayed. If the mask starts with #, then the
	entry with the same tracking ID is listed (those are
	automatically appended to the SQLINE reason issued to the
	IRC servers). If a list of entry numbers is given, only
	those entries are shown; for example:

	   SQLINE LIST 2-5,7-9
	      Lists SQLINE entries numbered 2 through 5 and 7 
	      through 9.
	      
	SQLINE VIEW is a more verbose version of SQLINE LIST, and 
	will show who added an SQLINE, the date it was added, and when 
	it expires, as well as the mask and reason.
	
	SQLINE CLEAR clears all entries of the SQLINE list.
	
	Limited to services operators.

OPER_HELP_SZLINE
	Syntax: SZLINE ADD [+expiry] mask reason
	    	SZLINE DEL {mask | entry-num | list}
	        SZLINE LIST [mask | list]
	        SZLINE VIEW [mask | list]
	        SZLINE CLEAR
	
	Allows services operators to manipulate the SZLINE list. If
	a user with an IP matching a SZLINE mask attempts to 
	connect, services won't allow him to continue his IRC
	session (whether the IP has a PTR RR or not).
	
	SZLINE ADD adds the given IP mask to the SZLINE
	list for the given reason (which must be given).
	CIDR notation is supported on some IRC servers.
	expiry is specified as an integer followed by one of w
	(weeks), d (days), h (hours), or m (minutes).
	Combinations (such as 1h30m) are allowed. To add an
	SZLINE which does not expire, use +0. If the mask to be
	added starts with a +, an expiry time must be given, even
	if it is the same as the default. The current SZLINE
	default expiry time can be found with the STATS AKILL
	command.
	
	The SZLINE DEL command removes the given mask from the
	SZLINE list if it is present.  If a list of entry numbers
	is given, those entries are deleted.  (See the example for
	LIST below.)
	
	The SZLINE LIST command displays the SZLINE list.  
	If a wildcard mask is given, only those entries matching
	the mask are displayed. If the mask starts with #, then the
	entry with the same tracking ID is listed (those are
	automatically appended to the SZLINE reason issued to the
	IRC servers). If a list of entry numbers is given, only
	those entries are shown; for example:

	   SZLINE LIST 2-5,7-9
	      Lists SZLINE entries numbered 2 through 5 and 7 
	      through 9.
	      
	SZLINE VIEW is a more verbose version of SZLINE LIST,
	and will show who added an SZLINE, the date it was added,
	and when it expires, as well as the mask and reason.
	
	SZLINE CLEAR clears all entries of the SZLINE list.
	
	This command requires special support from the IRC server,
	and is therefore disabled on some networks.
	
	Limited to services operators.

OPER_HELP_SET
	Syntax: SET option setting
	
	Sets various global Epona options.  Option names
	currently defined are:
	    READONLY   Set read-only or read-write mode
	    DEBUG      Activate or deactivate debug mode
	    NOEXPIRE   Activate or deactivate no expire mode
	    IGNORE     Activate or deactivate ignore code
	
	Limited to services admins.

OPER_HELP_SET_READONLY
	Syntax: SET READONLY {ON | OFF}
	
	Sets read-only mode on or off.  In read-only mode, users
	(including IRC operators) will not be allowed to modify any
	services data.
	
	This option is equivalent to the command-line option
	--readonly.

OPER_HELP_SET_DEBUG
	Syntax: SET DEBUG {ON | OFF | num}
	
	Sets debug mode on or off.  In debug mode, all data sent to
	and from services as well as a number of other debugging
	messages are logged. If num is given, debug mode is
	activated, with the debugging level set to num.
	
	This option is equivalent to the command-line option
	--debug.
	
OPER_HELP_SET_NOEXPIRE
	Syntax: SET NOEXPIRE {ON | OFF}
	
	Sets no expire mode on or off.  In no expire mode, no
	expiration will happen until the option is unset.
	
	This option is equivalent to the command-line option
	--noexpire.

OPER_HELP_SET_IGNORE
	Syntax: SET IGNORE {ON | OFF}
	
	Enables or disables the use of ignore code, whose intent
	is to prevent malicious users from exploiting services 
	through the consecutive use of commands that take more
	than one second to execute, which would prevent other
	users from actually accessing the services.
	
	It is enabled by default, which is recommended. IRC
	operators are not affected by it.

OPER_HELP_NOOP
	Syntax: NOOP SET server
	        NOOP REVOKE server
	
	NOOP SET removes all O:lines of the given server and
	kills all IRCops logged on it to prevent them from
	rehashing the server (because this would just cancel the
	effect).
	
	NOOP REVOKE makes all removed O:lines available again
	on the given server.
	
	Note: The server parameter is not checked for
	validity by services.
	
	Limited to services admins.

OPER_HELP_JUPE
	Syntax: JUPE server [reason]
	
	Tells Services to jupiter a server -- that is, to create
	a fake "server" connected to Services which prevents
	the real server of that name from connecting.  The jupe
	may be removed using a standard SQUIT.  If a reason is
	given, it is placed in the server information field;
	otherwise, the server information field will contain the
	text "Juped by <nick>", showing the nickname of the
	person who jupitered the server.
	
	Limited to services admins.

OPER_HELP_RAW
	Syntax: RAW text
	
	Sends a string of text directly to the server to which
	Services is connected.  This command has a very limited
	range of uses, and can wreak havoc on a network if used
	improperly.  DO NOT USE THIS COMMAND unless you are
	absolutely certain you know what you are doing!
	
	Limited to services admins.

OPER_HELP_SYNC
	Syntax: SYNC
	
	Causes %S to send a synchronization request to modules.
	Modules may interpret this message in whatever way they
	see fit (including ignoring it), but database modules
	generally save or import data when they receive it,
	if it's relevant to them.
	
	Limited to services admins.
	
OPER_HELP_RELOAD
	Syntax: RELOAD
	
	Causes Epona to reload its configuration. Note that
	modifying some directives still requires a restart to
	take effect.
	
	Limited to services admins.

OPER_HELP_RESTART
	Syntax: RESTART
	
	Causes Epona to stop all modules and then restart (i.e.
	exit and immediately re-run the executable).
	
	Limited to services admins.

OPER_HELP_STOP
	Syntax: STOP
	
	Causes Epona to stop all modules and itself.
	
	Limited to services admins.

OPER_HELP_FATAL
	Syntax: FATAL
	
	Causes Epona to stop abruptly; modules are not properly
	stopped. This command should not be used unless
	irreversible damage is feared if one or more modules were
	to be able to stop. To stop Epona normally, use the
	STOP command.
	
	Limited to services admins.

OPER_HELP_CHANLIST
	Syntax: CHANLIST [{[pattern] [range] [NOLIMIT] | nick} [SECRET]]
	
	Lists all channels currently in use on the IRC network,
	whether they are registered or not.
	
	If pattern is given, only channels with a matching
	name are displayed. The number of results shown at once
	is limited; you may specify a range to see results past
	this limit. For example:
	
	   CHANLIST 51-100
	       Shows results 51 to 100.
	   CHANLIST *epona* 15-20
	       Shows results 15 to 20 whose name contains "epona".
	
	(Note: unlike numbered lists, it is not possible to use
	extended syntax such as 51-60,75-80).
	
	If a pattern or range is given, specifying the NOLIMIT
	option shows an unlimited number of results.
	
	If a nick is given instead of a pattern, all channels
	this user is on are displayed.
	
	In both cases, you may specify the SECRET option to 
	display only channels having the +s or +p mode set.
	
	Limited to services admins.

OPER_HELP_USERLIST
	Syntax: USERLIST [{[pattern] [range] [NOLIMIT] | channel} [INVISIBLE]]
	
	Lists all users currently connected to the IRC network,
	whether they are registered or not.
	
	If pattern is given, only users with a matching
	usermask (in the format nick!user@host) are displayed. The
	number of results shown at once is limited; you may specify
	a range to see results past this limit. For example:
	
	   USERLIST 51-100
	       Shows results 51 to 100.
	   USERLIST *epona*!* 15-20
	       Shows results 15 to 20 whose nickname contains "epona".
	
	(Note: unlike numbered lists, it is not possible to use
	extended syntax such as 51-60,75-80).
	
	If a pattern or range is given, specifying the NOLIMIT
	option shows an unlimited number of results.
	
	If a channel is given instead of a pattern, only users
	that are in this channel are displayed..
	
	In both cases, you may specify the INVISIBLE option to 
	display only users having the +i mode set.
	
	Limited to services admins.

OPER_HELP_BLACKLIST
	Syntax: BLACKLIST DEL hostname
	        BLACKLIST LIST [pattern]
	
	The BLACKLIST DEL command deletes the specified hostname from
	the black list. Note that this change is only temporary; it
	won't update the black list file.

	The BLACKLIST LIST command lists all blacklisted hosts or 
	those that match the given pattern.
	
	Limited to services admins.

OPER_HELP_MODULES
	Syntax: MODULES LOAD name
	        MODULES UNLOAD name
	        MODULES LIST
	        MODULES INFO name
	        MODULES QUERY name query
	
	The MODULES LOAD command dynamically loads the module
	with the given name.
	
	Beware that loading a module that has been compiled for
	a different version of Epona and/or a different version
	of other loaded modules it uses may cause the whole
	program to crash in some cases. Therefore I don't
	recommend you use this command to do this unless you're
	sure that you know what you're doing.
	
	The MODULES UNLOAD command dynamically unloads the module
	with the given name. You can't unload a module that has a
	non-zero reference count.
	
	The MODULES LIST command lists the currently loaded
	modules along with their descriptions and reference counts.
	
	The MODULES INFO command shows general information about
	the module with the given name.
	
	The MODULES QUERY command sends the given query to the
	module with the given name, then shows the answer it
	eventually got.
	
	Limited to services roots.

###########################################################################
#
# BotServ help messages
#
###########################################################################
	
BOT_HELP
	%S allows you to have a bot on your own channel.
	It has been created for users that can't host or
	configure a bot, or for use on networks that don't
	allow users' bot. Available commands are listed 
	below; to use them, type /msg %S command.  For 
	more information on a specific command, type /msg 
	%S HELP command.
	
	BOTLIST            Lists available bots
	ASSIGN             Assigns a bot to a channel
	SET                Configures bot options
	KICK               Configures kickers
	BADWORDS           Maintains bad words list
	
	Other commands: ACT INFO SAY UNASSIGN
	
	Bot will join a channel whenever there is at least
	%d user(s) on it.
	
BOT_HELP_BOTLIST
	Syntax: BOTLIST
	
	Lists all available bots on this network.
	
BOT_HELP_ASSIGN
	Syntax: ASSIGN channel bot
	
	Assigns bot to the channel. You can then configure the
	bot as it fits for the channel.
	
BOT_HELP_UNASSIGN
	Syntax: UNASSIGN channel
	
	Unassigns a bot from a channel. When you use this command,
	the bot won't join the channel anymore. However, bot
	configuration for the channel is kept, so you will always
	be able to reassign a bot later without having to
	reconfigure it.
	
BOT_HELP_INFO
	Syntax: INFO {channel | bot}
	
	Allows you to see %S information about a channel or a bot.
	If the parameter is a channel, then you'll get information
	such as the bot options and enabled kickers. If the
	parameter is a nick, you'll get information about a bot,
	such as its creation time or number of channels it is on.
	
BOT_HELP_SET
	Syntax: SET channel option parameters
	
	Configures bot options.  option can be one of:
	
	    DONTKICKOPS      To protect ops against bot kicks
	    DONTKICKVOICES   To protect voices against bot kicks
	    GREET            Enable greet messages
	    FANTASY          Enable fantaisist commands
	    SYMBIOSIS        Allow the bot to act as a real bot
	
	Type /msg %S HELP SET option for more information
	on a specific option.
	
	Note: access to this command is controlled by the
	level SET.
	
BOT_HELP_SET_DONTKICKOPS
	Syntax: SET channel DONTKICKOPS {ON|OFF}
	
	Enables or disables ops protection mode on a channel.
	When it is enabled, ops won't be kicked by the bot
	even if they don't match the NOKICK level.
	
BOT_HELP_SET_DONTKICKVOICES
	Syntax: SET channel DONTKICKVOICES {ON|OFF}
	
	Enables or disables voices protection mode on a channel.
	When it is enabled, voices won't be kicked by the bot
	even if they don't match the NOKICK level.
	
BOT_HELP_SET_FANTASY
	Syntax: SET channel FANTASY {ON|OFF}
	
	Enables or disables fantasy mode on a channel.
	When it is enabled, users will be able to use
	commands !op, !deop, !voice, !devoice,
	!kick, !kb, !unban, !seen on a channel (find how 
	to use them; try with or without nick for each, 
	and with a reason for some?).
	
	Note that users wanting to use fantaisist
	commands MUST have enough level for both
	the FANTASIA and another level depending
	of the command if required (for example, to use 
	!op, user must have enough access for the OPDEOP
	level).

BOT_HELP_SET_GREET
	Syntax: SET channel GREET {ON|OFF}
	
	Enables or disables greet mode on a channel.
	When it is enabled, the bot will display greet
	messages of users joining the channel, provided
	they have enough access to the channel.

BOT_HELP_SET_SYMBIOSIS
	Syntax: SET channel SYMBIOSIS {ON|OFF}
	
	Enables or disables symbiosis mode on a channel.
	When it is enabled, the bot will do everything
	normally done by %s on channels, such as MODEs,
	KICKs, and even the entry message.

BOT_HELP_KICK
	Syntax: KICK channel option parameters
	
	Configures bot kickers.  option can be one of:
	
	    BOLDS         Sets if the bot kicks bolds
	    BADWORDS      Sets if the bot kicks bad words
	    CAPS          Sets if the bot kicks caps
	    COLORS        Sets if the bot kicks colors
	    FLOOD         Sets if the bot kicks flooding users
	    REPEAT        Sets if the bot kicks users who repeat
	                       themselves
	    REVERSES      Sets if the bot kicks reverses
	    UNDERLINES    Sets if the bot kicks underlines
	
	Type /msg %S HELP KICK option for more information
	on a specific option.
	
	Note: access to this command is controlled by the
	level SET.

BOT_HELP_KICK_BOLDS
	Syntax: KICK channel BOLDS {ON | OFF} [kbb]

	Sets the bolds kicker on or off. When enabled, this
	option tells the bot to kick users who use bolds.

	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_KICK_COLORS
	Syntax: KICK channel COLORS {ON | OFF} [kbb]

	Sets the colors kicker on or off. When enabled, this
	option tells the bot to kick users who use colors.

	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_KICK_REVERSES
	Syntax: KICK channel REVERSES {ON | OFF} [kbb]

	Sets the reverses kicker on or off. When enabled, this
	option tells the bot to kick users who use reverses.

	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_KICK_UNDERLINES
	Syntax: KICK channel UNDERLINES {ON | OFF} [kbb]

	Sets the underlines kicker on or off. When enabled, this
	option tells the bot to kick users who use underlines.

	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_KICK_CAPS
	Syntax: KICK channel CAPS {ON | OFF} [kbb [min [percent]]]

	Sets the caps kicker on or off. When enabled, this
	option tells the bot to kick users who are talking in
	CAPS.

	The bot kicks only if there are at least min caps
	and they constitute at least percent%% of the total 
	text line (by default, 10 characters and 25%%).

	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_KICK_FLOOD
	Syntax: KICK channel FLOOD {ON | OFF} [kbb [ln [secs]]]

	Sets the flood kicker on or off. When enabled, this
	option tells the bot to kick users who are flooding
	the channel using at least ln lines in secs seconds
	(by default, 6 lines in 10 seconds).
	
	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_KICK_REPEAT
	Syntax: KICK #channel REPEAT {ON | OFF} [kbb [num]]

	Sets the repeat kicker on or off. When enabled, this
	option tells the bot to kick users who are repeating
	themselves num times (by default, 3).

	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_KICK_BADWORDS
	Syntax: KICK #channel BADWORDS {ON | OFF} [kbb]

	Sets the bad words kicker on or off. When enabled, this
	option tells the bot to kick users who say certain words
	on the channels.

	You can define bad words for your channel using the
	BADWORDS command. Type /msg %S HELP BADWORDS for
	more information.

	kbb is the number of times a user can be kicked
	before he gets banned. Leave that parameter out to
	disable banning.

BOT_HELP_BADWORDS
	Syntax: BADWORDS channel ADD expr [SINGLE | START | END]
	        BADWORDS channel DEL {expr | entry-num | list}
	        BADWORDS channel LIST [mask | list]
	        BADWORDS channel CLEAR
	
	Modifies or displays the bad words list for a channel,
	that determines which words trigger the bad words kicker
	when it is enabled. For more information, type
	 /msg %S HELP KICK BADWORDS.
	
	BADWORDS ADD adds the given expression to the bad words
	list. The SINGLE option, when specified, ensures the
	expression is a word before the kicker is triggered,
	the START option ensures the expression is at the
	beginning of a word, and the END option ensures the
	expression is the end of a word. When no option is
	specified, the kicker is triggered everytime the
	expression is part of a word. For example:
	
	  BADWORDS ADD tick
	        Adds expression tick to the bad words list. The
	        kicker would be triggered by the words "tick",
	        "stick", "ticks" or "sticks".
	  BADWORDS ADD tick SINGLE
	        Adds expression tick to the bad words list with
	        the SINGLE option. The kicker would be triggered
	        by the word "tick" only.
	  BADWORDS ADD tick START
	        Adds expression tick to the bad words list with
	        the START option. The kicker would be triggered
	        by the words "tick" or "ticks".
	  BADWORDS ADD tick END
	        Adds expression tick to the bad words list with
	        the END option. The kicker would be triggered
	        by the words "tick", or "stick".
	
	Please note that the expression does NOT support wildcards.
	
	The BADWORDS DEL command removes the given expression
	from the bad words list if it is present.  If a list of
	entry numbers is given, those entries are deleted.  (See
	the example for LIST below.)
	
	The BADWORDS LIST command displays the bad words list.  
	If a wildcard mask is given, only those entries matching
	the mask are displayed. If a list of entry numbers is
	given, only those entries are shown; for example:

	   BADWORDS LIST 2,7-9
	      Lists bad words list entries numbered 2 and 7 
	      through 9.
	
	BADWORDS CLEAR clears all entries of the bad words list.

BOT_HELP_SAY
	Syntax: SAY channel text
	
	Asks the bot to say the given text on the given channel.

BOT_HELP_ACT
	Syntax: ACT channel text
	
	Asks the bot to do the equivalent of a "/me" command
	on the given channel using the given text.

BOT_SERVADMIN_HELP
	
	The following command is available to services admins:
	
	    BOT              Maintains network bot list

BOT_SERVADMIN_HELP_BOT
	Syntax: BOT ADD nick username hostname realname
	        BOT CHANGE oldnick nick [username [hostname [realname]]]
	        BOT DEL nick
	        BOT LIST
	
	Allows services admins to create, modify, and delete
	bots that users are then able to use on their own
	channels.
	
	BOT ADD adds a bot with the given nickname, username,
	hostname and realname. Since no integrity checks are done 
	for these settings, be really careful.
	BOT CHANGE allows to change nickname, username, hostname
	or realname of a bot without actually deleting it (thus
	keeping all channels assigned to it).
	BOT DEL removes the given bot from the bot list.  
	BOT LIST is an alias of BOTLIST and just lists all
	bots available on the network.
	
	Note: if you create a bot that has a nick that is 
	registered, it will be dropped. Also, if a user is
	currently using the nick, it will be killed.

BOT_SERVADMIN_HELP_SET
	
	These options are reserved to services admins:
	
	    NOBOT            Prevent a bot from being assigned to 
	                        a channel
	    PRIVATE          Prevent a bot from being assigned by
	                        non IRC operators

BOT_SERVADMIN_HELP_SET_NOBOT
	Syntax: SET channel NOBOT {ON | OFF}
	
	This option prevents a bot from being assigned to a
	channel. If a bot is already assigned to the channel, it
	is unassigned automatically once the option is enabled.
	
	Limited to services admins.

BOT_SERVADMIN_HELP_SET_PRIVATE
	Syntax: SET bot-nick PRIVATE {ON | OFF}
	
	This option prevents a bot from being assigned to a
	channel by users that aren't IRC operators.
	
	Limited to services admins.
