/* Diff for language definition files.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include <fstream>
#include <iostream>
#include <map>
#include <string>

using namespace std;

/*************************************************************************/

typedef map<string, string> langmap;
static langmap olds, news;

/*************************************************************************/

static bool generate_set(char const *filename, langmap &lmap)
{
	string line, name, contents;
	ifstream in(filename, ios::in);
	
	if (!in)
	{
		cout << filename << ": Open error: " << strerror(errno) << endl;
		return false;
	}
	
	while ((!line.empty() && line[0] != '#' && line[0] != '\t') ||
			getline(in, line))
	{
		if (line.empty() || line[0] == '#' || line[0] == '\t')
			continue;
		
		name = line;
		contents = "";
		
		while (getline(in, line))
		{	
			if (line.empty())
				continue;
			
			if (line[0] != '\t')
				break;
			
			contents += line;
			contents += '\n';
		}
		
		lmap[name] = contents;
		
		if (in.fail())
			break;
	}
	
	return true;
}

/*************************************************************************/

static void print_del()
{
	cout << "Deleted strings: " << endl;
	
	langmap::const_iterator oit, end, nit;
	
	for (oit = olds.begin(), end = olds.end(); oit != end; ++oit)
	{
		nit = news.find(oit->first);
		
		if (nit == news.end())
			cout << "  " << oit->first << endl;
	}
}

/*************************************************************************/

static void print_new()
{
	cout << "New strings: " << endl;
	
	langmap::const_iterator nit, end, oit;
	
	for (nit = news.begin(), end = news.end(); nit != end; ++nit)
	{
		oit = olds.find(nit->first);
		
		if (oit == olds.end())
			cout << "  " << nit->first << endl;
	}
	
	cout << endl;
}

/*************************************************************************/

static void print_mod()
{
	cout << "Modified strings: " << endl;
	
	langmap::const_iterator oit, end, nit;
	
	for (oit = olds.begin(), end = olds.end(); oit != end; ++oit)
	{
		nit = news.find(oit->first);
		
		if (nit == news.end())
			continue;
		
		if (oit->second != nit->second)
			cout << "  " << oit->first << endl;
	}
	
	cout << endl;
}

/*************************************************************************/

int main(int ac, char **av)
{	
	if (ac < 3)
	{
		cout << "Syntax: %s old-file new-file" << endl;
		exit(EXIT_FAILURE);
	}
	
	if (!generate_set(av[1], olds) || !generate_set(av[2], news))
		exit(EXIT_FAILURE);
	
	cout << "Old strings: " << olds.size() << " New strings: " <<
			news.size() << " " << endl;
	
	print_new();
	print_mod();
	print_del();
	
	exit(EXIT_SUCCESS);
}

/*************************************************************************/
