/* Legacy databases module.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * Based on the original code of Services by Andy Church.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/epona.h>
#include <core/version.h>
#include <databases/xml.h>
#include <log/levels.h>
#include <password/password.h>

#include "decl.h"
#include "module.h"
#include "xmlexport.h"

using namespace std;
using namespace modules;
using namespace core;
using namespace legacydb;

/*************************************************************************/

extern "C" Module *legacydb_get(Epona &core)
{
	/* Ensure the core version is the same as the version we were
	 * compiled for so that there are no compatibility problems later. */
	if (core.version() != core::core_version)
	{
		core.log(xlog::LogCritical(), "Core module version mismatch");
		return 0;
	}
	
	return (me = new LegacyDB(core));
}

/*************************************************************************/

extern "C" void legacydb_release(Epona &core, Module *module)
{
	delete module;
	me = 0;
}

/*************************************************************************/

namespace legacydb
{

/*************************************************************************/

LegacyDB *me;

/*************************************************************************/

LegacyDB::LegacyDB(Epona &core) : Module("legacydb", "Legacy databases exporter"),
		m_core(&core), m_enc()
{
	
}

/*************************************************************************/

Interface *LegacyDB::factory(modules::PSIID iid)
{
	if (databases::XMLExport::IID == iid)
		return new XMLExport;
	else
		return Module::factory(iid);
}

/*************************************************************************/

Interface *LegacyDB::interface(modules::PSIID iid)
{
	if (password::Encrypter::IID == iid)
		return &m_enc;
	else
		return Module::factory(iid);
}

/*************************************************************************/

} /* namespace legacydb */

/*************************************************************************/
