/* SMTP mail transport module.
 *
 * SMTP mail transport module (c) 2007 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/version.h>
#include <log/levels.h>
#include <modules/core.h>

#include "conf.h"
#include "decl.h"
#include "module.h"
#include "smtp.h"

using namespace std;
using namespace modules;
using namespace smtp_cli;

/*************************************************************************/

extern "C" Module *smtp_cli_get(Core &core)
{
	/* Ensure the core version is the same as the version we were
	 * compiled for so that there are no compatibility problems later. */
	if (core.version() != core::core_version)
	{
		core.log(xlog::LogCritical(), "Core module version mismatch");
		return 0;
	}
	
	return (me = new SMTPCli(core));
}

/*************************************************************************/

extern "C" void smtp_cli_release(Core &core, Module *module)
{
	delete module;
	me = 0;
}

/*************************************************************************/

namespace smtp_cli
{

/*************************************************************************/

SMTPCli *me;

/*************************************************************************/

SMTPCli::SMTPCli(Core &core) : 
		Module("smtp-cli", "SMTP mail transport module"),
		m_core(&core), m_cd(new Directives(core.conf_logger())), m_socks(),
		m_transport()
{
	
}

/*************************************************************************/

SMTPCli::~SMTPCli()
{
	delete m_cd;
}

/*************************************************************************/

conf::ConfQ *SMTPCli::config(modules::Core &core, conf::ConfGlobalQ &global)
{
	return &m_cd->parser();
}

/*************************************************************************/

modules::Interface *SMTPCli::interface(PSIID iid)
{
	if (iid == mail::Transport::IID)
		return &m_transport;
	else
		return Module::interface(iid);
}

/*************************************************************************/

void SMTPCli::notify(modules::Module &sender, int msg, void *arg)
{
	if (msg == mnReconfig)
		m_transport.update_template();
	
	Module::notify(sender, msg, arg);
}

/*************************************************************************/

bool SMTPCli::stop(modules::Core &core)
{
	socks_list::iterator it, end;
	
	for (it = m_socks.begin(), end = m_socks.end(); it != end; ++it)
		delete (*it);
	
	m_socks.clear();
	
	return Module::stop(core);
}

/*************************************************************************/

} /* namespace smtp_cli */

/*************************************************************************/
