/* Database interface.
 *
 * Twilight database module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef DATABASE_H
#define DATABASE_H

/*************************************************************************/

#include <string>

#include <databases/database.h>

/*************************************************************************/

namespace twilight 
{

/*************************************************************************/

class Database : public databases::Database
{
	public:
		/// Free all databases.
		static void free_all();
		/// Saves all databases.
		static bool save_all();
	
		/// Constructor.
		explicit Database(modules::Core &core) : databases::Database(core),
				m_dbhname(), m_dbdname(), m_fieldids() { }
		/// Destructor.
		virtual ~Database();
		
		/// Overrides databases::Database::supports().
		virtual modules::Interface *supports(modules::PSIID sid);
	protected:
		/// Overrides databases::Database::reg().
		virtual bool reg();
		/// Overrides databases::Database::start_real().
		virtual bool start_real();
	private:
		/// Database list type.
		typedef std::list<Database *> twcontainer_type;
		/// Database list.
		static twcontainer_type g_twlist;
		
		/* The name of header and data files */
		std::string m_dbhname;
		std::string m_dbdname;
	
		/* Initial import data */
		fields_type m_fieldids;
	
		/* Initial import functions */
		bool initial_import();
		bool read_header();
		bool read_data();
		
		/* Save functions */
		bool save();
};

/*************************************************************************/

class DBRemove : public databases::DBRemove
{
	public:
		/// Implements DBRemove::remove().
		virtual bool remove(std::string const &database);
		/// Implements DBRemove::clear().
		virtual bool clear();
};

/*************************************************************************/

} /* namespace twilight */

/*************************************************************************/

#endif /* DATABASE_H */
