/* OperServ functions.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include <core/directives.h>
#include <core/epona.h>
#include <databases/database.h>
#include <log/levels.h>
#include <misc/misc.h>
#include <modules/manager.h>

#include "blacklist.h"
#include "botserv.h"
#include "channels.h"
#include "chanserv.h"
#include "epona.h"
#include "nickserv.h"
#include "operserv.h"

#include "services.h"
#include "pseudo.h"

using namespace std;
using namespace misc;
using namespace core;

using modules::ModManager;
using modules::Module;

/*************************************************************************/

StatsList stats;
Stats *stats_rec = 0;

AkillList akills;
AkillCmd *akill_cmd;

SGLineList sglines;
SQLineList sqlines;
SZLineList szlines;

/* List of Services administrators */
VectorNG servadmins;
/* List of Services operators */
VectorNG servopers;

/*************************************************************************/

LOGLEVELH(InfoOperServUse, InfoOperServ, "Info::OperServ::Use",
		"operserv use: ");

LOGLEVELH(NoticeOperServ, xlog::LogNotice, "Notice::OperServ", "operserv: ");
LOGLEVELH(NoticeOperServSettings, NoticeOperServ, "Notice::OperServ::Settings",
		"operserv settings: ");

/*************************************************************************/

static void do_help(User *u);
static void do_global(User *u);
static void do_stats(User *u);
static void do_admin(User *u);
static void do_oper(User *u);
static void do_os_mode(User *u);
static void do_clearmodes(User *u);
static void do_os_kick(User *u);
static void do_akill(User *u);
static void do_sgline(User *u);
static void do_sqline(User *u);
static void do_szline(User *u);
static void do_set(User *u);
static void do_noop(User *u);
static void do_jupe(User *u);
static void do_raw(User *u);
static void do_sync(User *u);
static void do_reload(User *u);
static void do_restart(User *u);
static void do_stop(User *u);
static void do_fatal(User *u);
static void do_listignore(User *u);
static void do_chanlist(User *u);
static void do_userlist(User *u);
static void do_modules(User *u);

#ifdef DEBUG_COMMANDS
static void do_matchwild(User *u);
#endif

/*************************************************************************/

static int akill_msgs[ListCmd::cmListCmdTotal];
static int sgline_msgs[ListCmd::cmListCmdTotal];
static int sqline_msgs[ListCmd::cmListCmdTotal];
static int szline_msgs[ListCmd::cmListCmdTotal];

static int admin_msgs[ListCmd::cmListCmdTotal];
static int oper_msgs[ListCmd::cmListCmdTotal];

/*************************************************************************/

class SGLineCmd : public ListCmd
{
	public:
		/// Constructor.
		SGLineCmd();
	protected:
		/// Overrides ListCmd::container().
		virtual container_type *container(User *u);
		
		/// Overrides ListCmd::dispatch().
		virtual int dispatch(User *u, container_type &ctn, char const *sub);
		/// Overrides ListCmd::changeable().
		virtual bool changeable(User *u, container_type &ctn,
				char const *sub) const;
		
		/// Overrides ListCmd::add().
		virtual int add(User *u, container_type &ctn, char const *sub);
		
		/// Adds a record to the SQLINE list.
		int add_record(User *u, char const *mask, char const *creator,
				time_t expires, char const *reason);
		
		/// Overrides ListCmd::del_record().
		virtual bool del_record(User *u, container_type &ctn,
				char const *sub, databases::Record *rec);
		
		/// Overrides ListCmd::list_record().
		virtual bool list_record(User *u, container_type &ctn,
				char const *sub, uint16_t num, databases::Record const *rec,
				uint16_t count);
		
		/// Overrides ListCmd::clear_records().
		virtual bool clear_records(User *u, container_type &ctn,
				char const *sub);
};

/*************************************************************************/

class SQLineCmd : public ListCmd
{
	public:
		/// Constructor.
		SQLineCmd();
	protected:
		/// Overrides ListCmd::container().
		virtual container_type *container(User *u);
		
		/// Overrides ListCmd::dispatch().
		virtual int dispatch(User *u, container_type &ctn, char const *sub);
		/// Overrides ListCmd::changeable().
		virtual bool changeable(User *u, container_type &ctn,
				char const *sub) const;
		
		/// Overrides ListCmd::add().
		virtual int add(User *u, container_type &ctn, char const *sub);
		
		/// Adds a record to the SQLINE list.
		int add_record(User *u, char const *mask, char const *creator,
				time_t expires, char const *reason);
		
		/// Overrides ListCmd::del_record().
		virtual bool del_record(User *u, container_type &ctn,
				char const *sub, databases::Record *rec);
		
		/// Overrides ListCmd::list_record().
		virtual bool list_record(User *u, container_type &ctn,
				char const *sub, uint16_t num, databases::Record const *rec,
				uint16_t count);
		
		/// Overrides ListCmd::clear_records().
		virtual bool clear_records(User *u, container_type &ctn,
				char const *sub);
};

/*************************************************************************/

class SZLineCmd : public ListCmd
{
	public:
		/// Constructor.
		SZLineCmd();
	protected:
		/// Overrides ListCmd::container().
		virtual container_type *container(User *u);
		
		/// Overrides ListCmd::dispatch().
		virtual int dispatch(User *u, container_type &ctn, char const *sub);
		/// Overrides ListCmd::changeable().
		virtual bool changeable(User *u, container_type &ctn,
				char const *sub) const;
		
		/// Overrides ListCmd::add().
		virtual int add(User *u, container_type &ctn, char const *sub);
		
		/// Adds a record to the SQLINE list.
		int add_record(User *u, char const *mask, char const *creator,
				time_t expires, char const *reason);
		
		/// Overrides ListCmd::del_record().
		virtual bool del_record(User *u, container_type &ctn,
				char const *sub, databases::Record *rec);
		
		/// Overrides ListCmd::list_record().
		virtual bool list_record(User *u, container_type &ctn,
				char const *sub, uint16_t num, databases::Record const *rec,
				uint16_t count);
		
		/// Overrides ListCmd::clear_records().
		virtual bool clear_records(User *u, container_type &ctn,
				char const *sub);
};

/*************************************************************************/

class AdminCmd : public ListCmd
{
	public:
		/// Constructor.
		AdminCmd();
	protected:
		/// Overrides ListCmd::container().
		virtual container_type *container(User *u);
		
		/// Overrides ListCmd::allow_dispatch().
		virtual bool allow_dispatch(User *u, container_type &ctn,
				char const *sub);
		
		/// Overrides ListCmd::changeable().
		virtual bool changeable(User *u, container_type &ctn,
				char const *sub) const;
		
		/// Overrides ListCmd::add().
		virtual int add(User *u, container_type &ctn, char const *sub);
		
		/// Overrides ListCmd::del_record().
		virtual bool del_record(User *u, container_type &ctn,
				char const *sub, databases::Record *rec);
		
		/// Overrides ListCmd::list_record().
		virtual bool list_record(User *u, container_type &ctn,
				char const *sub, uint16_t num, databases::Record const *rec,
				uint16_t count);
};

/*************************************************************************/

class OperCmd : public ListCmd
{
	public:
		/// Constructor.
		OperCmd();
	protected:
		/// Overrides ListCmd::container().
		virtual container_type *container(User *u);
		
		/// Overrides ListCmd::allow_dispatch().
		virtual bool allow_dispatch(User *u, container_type &ctn,
				char const *sub);
		
		/// Overrides ListCmd::changeable().
		virtual bool changeable(User *u, container_type &ctn,
				char const *sub) const;
		
		/// Overrides ListCmd::add().
		virtual int add(User *u, container_type &ctn, char const *sub);
		
		/// Overrides ListCmd::del_record().
		virtual bool del_record(User *u, container_type &ctn,
				char const *sub, databases::Record *rec);
		
		/// Overrides ListCmd::list_record().
		virtual bool list_record(User *u, container_type &ctn,
				char const *sub, uint16_t num, databases::Record const *rec,
				uint16_t count);
};

/*************************************************************************/

static Command cmds[] = {
    { "HELP",       do_help,       NULL,  -1,                   -1,-1,-1,-1 },
    { "GLOBAL",     do_global,     NULL,  OPER_HELP_GLOBAL,     -1,-1,-1,-1 },
    { "STATS",      do_stats,      NULL,  OPER_HELP_STATS,      -1,-1,-1,-1 },
    { "UPTIME",     do_stats,      NULL,  OPER_HELP_STATS,      -1,-1,-1,-1 },

    /* Anyone can use the LIST option to the ADMIN and OPER commands; those
     * routines check privileges to ensure that only authorized users
     * modify the list. */
    { "ADMIN",      do_admin,      NULL,  OPER_HELP_ADMIN,      -1,-1,-1,-1 },
    { "OPER",       do_oper,       NULL,  OPER_HELP_OPER,       -1,-1,-1,-1 },
    /* Similarly, anyone can use NEWS LIST, but NEWS {ADD,DEL} are
     * reserved for Services admins. */
    { "NEWS",       do_news,       NULL,  OPER_HELP_NEWS,       -1,-1,-1,-1 },
    { "LOGONNEWS",  do_oldnews,    NULL,  -1,                   -1,-1,-1,-1 },
    { "OPERNEWS",   do_oldnews,    NULL,  -1,                   -1,-1,-1,-1 },
    { "RANDOMNEWS", do_oldnews,    NULL,  -1,                   -1,-1,-1,-1 },

    /* Commands for Services opers: */
    { "MODE",       do_os_mode,    is_services_oper,
	OPER_HELP_MODE, -1,-1,-1,-1 },
    { "CLEARMODES", do_clearmodes, is_services_oper,
	OPER_HELP_CLEARMODES, -1,-1,-1,-1 },
    { "KICK",       do_os_kick,    is_services_oper,
	OPER_HELP_KICK, -1,-1,-1,-1 },
    { "AKILL",      do_akill,      is_services_oper,
	OPER_HELP_AKILL, -1,-1,-1,-1 },
	{ "SGLINE",     do_sgline,     is_services_oper,
	OPER_HELP_SGLINE, -1,-1,-1,-1 },
	{ "SQLINE",     do_sqline,     is_services_oper,
	OPER_HELP_SQLINE, -1,-1,-1,-1 },
	{ "SZLINE",     do_szline,     is_services_oper,
	OPER_HELP_SZLINE, -1,-1,-1,-1 },

    /* Commands for Services admins: */
	{ "SET",        do_set,        is_services_admin,
	OPER_HELP_SET, -1,-1,-1,-1 },
    { "SET READONLY", NULL,         NULL,  
    OPER_HELP_SET_READONLY, -1,-1,-1,-1 },
    { "SET DEBUG",  NULL,          NULL,     
    OPER_HELP_SET_DEBUG, -1,-1,-1,-1 },
    { "SET NOEXPIRE", NULL,         NULL, 
    OPER_HELP_SET_NOEXPIRE, -1,-1,-1,-1 },
    { "SET IGNORE", NULL,         NULL, 
    OPER_HELP_SET_IGNORE, -1,-1,-1,-1 },
    { "NOOP",       do_noop,       is_services_admin,
	OPER_HELP_NOOP, -1,-1,-1,-1 },
    { "JUPE",       do_jupe,       is_services_admin,
	OPER_HELP_JUPE, -1,-1,-1,-1 },
	{ "RAW",        do_raw,        is_services_admin,
	OPER_HELP_RAW, -1,-1,-1,-1 },
	{ "SYNC",       do_sync,     is_services_admin,
	OPER_HELP_SYNC, -1,-1,-1,-1 },
    { "RELOAD",     do_reload,     is_services_admin,
	OPER_HELP_RELOAD, -1,-1,-1,-1 },
    { "RESTART",    do_restart,    is_services_admin,
	OPER_HELP_RESTART, -1,-1,-1,-1 },
    { "STOP",       do_stop,   is_services_admin,
	OPER_HELP_STOP, -1,-1,-1,-1 },
    { "FATAL",      do_fatal,    is_services_admin,
	OPER_HELP_FATAL, -1,-1,-1,-1 },
    { "LISTIGNORE", do_listignore, is_services_admin,
	-1,-1,-1,-1, -1 },
    { "SESSION",    do_session,    is_services_admin,
        OPER_HELP_SESSION, -1,-1,-1, -1 },
    { "EXCEPTION",  do_exception,  is_services_admin,
        OPER_HELP_EXCEPTION, -1,-1,-1, -1 },
    { "CHANLIST",   do_chanlist,   is_services_admin,
	OPER_HELP_CHANLIST,   -1,-1,-1,-1 },
	{ "USERLIST",   do_userlist,   is_services_admin,
	OPER_HELP_USERLIST,   -1,-1,-1,-1 },
	{ "BLACKLIST",  do_blacklist,  is_services_admin,
	OPER_HELP_BLACKLIST,      -1,-1,-1,-1 },

	/* Deprecated names for some commands */
	{ "UPDATE",     do_sync,     is_services_admin,
	OPER_HELP_SYNC, -1,-1,-1,-1 },
	{ "SHUTDOWN",   do_stop,   is_services_admin,
	OPER_HELP_STOP, -1,-1,-1,-1 },
    { "QUIT",       do_fatal,    is_services_admin,
	OPER_HELP_FATAL, -1,-1,-1,-1 },

    /* Commands for Services root: */

	{ "MODULES",    do_modules,    is_services_root,
	OPER_HELP_MODULES,      -1,-1,-1,-1 },

#ifdef DEBUG_COMMANDS
    { "LISTTIMERS", send_timeout_list,  is_services_root, -1,-1,-1,-1,-1 },
    { "MATCHWILD",  do_matchwild,       is_services_root, -1,-1,-1,-1,-1 },
#endif

    /* Fencepost: */
    { NULL }
};

/*************************************************************************/
/*************************************************************************/

/* OperServ initialization. */

void os_init(void)
{
    Command *cmd;

    cmd = lookup_cmd(cmds, "GLOBAL");
    if (cmd)
	cmd->help_param1 = s_GlobalNoticer;
    cmd = lookup_cmd(cmds, "ADMIN");
    if (cmd)
	cmd->help_param1 = s_NickServ;
    cmd = lookup_cmd(cmds, "OPER");
    if (cmd)
	cmd->help_param1 = s_NickServ;
}

/*************************************************************************/

bool operserv_load()
{
	if (!epona->modules().load(epona->cd().operserv.database_stats))
	{
		epona->log(ErrorOperServ(), "Can't load database module %s: %s",
				epona->cd().operserv.database_stats.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	if (akills.available() &&
			!epona->modules().load(epona->cd().operserv.database_akills))
	{
		epona->log(ErrorOperServ(), "Can't load database module %s: %s",
				epona->cd().operserv.database_akills.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	if (sglines.available() &&
			!epona->modules().load(epona->cd().operserv.database_sglines))
	{
		epona->log(ErrorOperServ(), "Can't load database module %s: %s",
				epona->cd().operserv.database_sglines.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	if (!epona->modules().load(epona->cd().operserv.database_sqlines))
	{
		epona->log(ErrorOperServ(), "Can't load database module %s: %s",
				epona->cd().operserv.database_sqlines.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	if (szlines.available() &&
			!epona->modules().load(epona->cd().operserv.database_szlines))
	{
		epona->log(ErrorOperServ(), "Can't load database module %s: %s",
				epona->cd().operserv.database_szlines.value().c_str(),
				epona->modules().error().c_str());
		return false;
	}
	
	akill_msgs[ListCmd::cmSyntax] = OPER_AKILL_SYNTAX;
	akill_msgs[ListCmd::cmDisabled] = OPER_AKILL_DISABLED;
	akill_msgs[ListCmd::cmEmpty] = OPER_AKILL_EMPTY;
	akill_msgs[ListCmd::cmSyntaxDel] = OPER_AKILL_DEL_SYNTAX;
	akill_msgs[ListCmd::cmNoSuchEntry] = OPER_AKILL_NO_SUCH_ENTRY;
	akill_msgs[ListCmd::cmNoMatch] = OPER_AKILL_NO_MATCH;
	akill_msgs[ListCmd::cmDeletedOne] = OPER_AKILL_DELETED_ONE;
	akill_msgs[ListCmd::cmDeletedSeveral] = OPER_AKILL_DELETED_SEVERAL;
	akill_msgs[ListCmd::cmNotFound] = OPER_AKILL_NOT_FOUND;
	akill_msgs[ListCmd::cmDeleted] = OPER_AKILL_DELETED;
	akill_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for AKILL */
	akill_msgs[ListCmd::cmListHeader] = OPER_AKILL_LIST_HEADER;
	akill_msgs[ListCmd::cmListedOne] = OPER_AKILL_LISTED_ONE;
	akill_msgs[ListCmd::cmListedSeveral] = OPER_AKILL_LISTED_SEVERAL;
	akill_msgs[ListCmd::cmClearPartially] = OPER_AKILL_CLEAR_PARTIALLY;
	akill_msgs[ListCmd::cmClear] = OPER_AKILL_CLEAR;
	
	akill_cmd = new AkillCmd;
	
	sgline_msgs[ListCmd::cmSyntax] = OPER_SGLINE_SYNTAX;
	sgline_msgs[ListCmd::cmDisabled] = OPER_SGLINE_DISABLED;
	sgline_msgs[ListCmd::cmEmpty] = OPER_SGLINE_EMPTY;
	sgline_msgs[ListCmd::cmSyntaxDel] = OPER_SGLINE_DEL_SYNTAX;
	sgline_msgs[ListCmd::cmNoSuchEntry] = OPER_SGLINE_NO_SUCH_ENTRY;
	sgline_msgs[ListCmd::cmNoMatch] = OPER_SGLINE_NO_MATCH;
	sgline_msgs[ListCmd::cmDeletedOne] = OPER_SGLINE_DELETED_ONE;
	sgline_msgs[ListCmd::cmDeletedSeveral] = OPER_SGLINE_DELETED_SEVERAL;
	sgline_msgs[ListCmd::cmNotFound] = OPER_SGLINE_NOT_FOUND;
	sgline_msgs[ListCmd::cmDeleted] = OPER_SGLINE_DELETED;
	sgline_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for SGLINE */
	sgline_msgs[ListCmd::cmListHeader] = OPER_SGLINE_LIST_HEADER;
	sgline_msgs[ListCmd::cmListedOne] = OPER_SGLINE_LISTED_ONE;
	sgline_msgs[ListCmd::cmListedSeveral] = OPER_SGLINE_LISTED_SEVERAL;
	sgline_msgs[ListCmd::cmClearPartially] = OPER_SGLINE_CLEAR_PARTIALLY;
	sgline_msgs[ListCmd::cmClear] = OPER_SGLINE_CLEAR;
	
	sqline_msgs[ListCmd::cmSyntax] = OPER_SQLINE_SYNTAX;
	sqline_msgs[ListCmd::cmDisabled] = OPER_SQLINE_DISABLED;
	sqline_msgs[ListCmd::cmEmpty] = OPER_SQLINE_EMPTY;
	sqline_msgs[ListCmd::cmSyntaxDel] = OPER_SQLINE_DEL_SYNTAX;
	sqline_msgs[ListCmd::cmNoSuchEntry] = OPER_SQLINE_NO_SUCH_ENTRY;
	sqline_msgs[ListCmd::cmNoMatch] = OPER_SQLINE_NO_MATCH;
	sqline_msgs[ListCmd::cmDeletedOne] = OPER_SQLINE_DELETED_ONE;
	sqline_msgs[ListCmd::cmDeletedSeveral] = OPER_SQLINE_DELETED_SEVERAL;
	sqline_msgs[ListCmd::cmNotFound] = OPER_SQLINE_NOT_FOUND;
	sqline_msgs[ListCmd::cmDeleted] = OPER_SQLINE_DELETED;
	sqline_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for SQLINE */
	sqline_msgs[ListCmd::cmListHeader] = OPER_SQLINE_LIST_HEADER;
	sqline_msgs[ListCmd::cmListedOne] = OPER_SQLINE_LISTED_ONE;
	sqline_msgs[ListCmd::cmListedSeveral] = OPER_SQLINE_LISTED_SEVERAL;
	sqline_msgs[ListCmd::cmClearPartially] = OPER_SQLINE_CLEAR_PARTIALLY;
	sqline_msgs[ListCmd::cmClear] = OPER_SQLINE_CLEAR;
	
	szline_msgs[ListCmd::cmSyntax] = OPER_SZLINE_SYNTAX;
	szline_msgs[ListCmd::cmDisabled] = OPER_SZLINE_DISABLED;
	szline_msgs[ListCmd::cmEmpty] = OPER_SZLINE_EMPTY;
	szline_msgs[ListCmd::cmSyntaxDel] = OPER_SZLINE_DEL_SYNTAX;
	szline_msgs[ListCmd::cmNoSuchEntry] = OPER_SZLINE_NO_SUCH_ENTRY;
	szline_msgs[ListCmd::cmNoMatch] = OPER_SZLINE_NO_MATCH;
	szline_msgs[ListCmd::cmDeletedOne] = OPER_SZLINE_DELETED_ONE;
	szline_msgs[ListCmd::cmDeletedSeveral] = OPER_SZLINE_DELETED_SEVERAL;
	szline_msgs[ListCmd::cmNotFound] = OPER_SZLINE_NOT_FOUND;
	szline_msgs[ListCmd::cmDeleted] = OPER_SZLINE_DELETED;
	szline_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for SZLINE */
	szline_msgs[ListCmd::cmListHeader] = OPER_SZLINE_LIST_HEADER;
	szline_msgs[ListCmd::cmListedOne] = OPER_SZLINE_LISTED_ONE;
	szline_msgs[ListCmd::cmListedSeveral] = OPER_SZLINE_LISTED_SEVERAL;
	szline_msgs[ListCmd::cmClearPartially] = OPER_SZLINE_CLEAR_PARTIALLY;
	szline_msgs[ListCmd::cmClear] = OPER_SZLINE_CLEAR;
	
	admin_msgs[ListCmd::cmSyntax] = OPER_ADMIN_SYNTAX;
	admin_msgs[ListCmd::cmDisabled] = OPER_ADMIN_DISABLED;
	admin_msgs[ListCmd::cmEmpty] = OPER_ADMIN_EMPTY;
	admin_msgs[ListCmd::cmSyntaxDel] = OPER_ADMIN_DEL_SYNTAX;
	admin_msgs[ListCmd::cmNoSuchEntry] = OPER_ADMIN_NO_SUCH_ENTRY;
	admin_msgs[ListCmd::cmNoMatch] = OPER_ADMIN_NO_MATCH;
	admin_msgs[ListCmd::cmDeletedOne] = OPER_ADMIN_DELETED_ONE;
	admin_msgs[ListCmd::cmDeletedSeveral] = OPER_ADMIN_DELETED_SEVERAL;
	admin_msgs[ListCmd::cmNotFound] = OPER_ADMIN_NOT_FOUND;
	admin_msgs[ListCmd::cmDeleted] = OPER_ADMIN_DELETED;
	admin_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for ADMIN */
	admin_msgs[ListCmd::cmListHeader] = OPER_ADMIN_LIST_HEADER;
	admin_msgs[ListCmd::cmListedOne] = OPER_ADMIN_LISTED_ONE;
	admin_msgs[ListCmd::cmListedSeveral] = OPER_ADMIN_LISTED_SEVERAL;
	admin_msgs[ListCmd::cmClearPartially] = OPER_ADMIN_CLEAR_PARTIALLY;
	admin_msgs[ListCmd::cmClear] = OPER_ADMIN_CLEAR;
	
	oper_msgs[ListCmd::cmSyntax] = OPER_OPER_SYNTAX;
	oper_msgs[ListCmd::cmDisabled] = OPER_OPER_DISABLED;
	oper_msgs[ListCmd::cmEmpty] = OPER_OPER_EMPTY;
	oper_msgs[ListCmd::cmSyntaxDel] = OPER_OPER_DEL_SYNTAX;
	oper_msgs[ListCmd::cmNoSuchEntry] = OPER_OPER_NO_SUCH_ENTRY;
	oper_msgs[ListCmd::cmNoMatch] = OPER_OPER_NO_MATCH;
	oper_msgs[ListCmd::cmDeletedOne] = OPER_OPER_DELETED_ONE;
	oper_msgs[ListCmd::cmDeletedSeveral] = OPER_OPER_DELETED_SEVERAL;
	oper_msgs[ListCmd::cmNotFound] = OPER_OPER_NOT_FOUND;
	oper_msgs[ListCmd::cmDeleted] = OPER_OPER_DELETED;
	oper_msgs[ListCmd::cmSyntaxList] = -1; /* Not used for OPER */
	oper_msgs[ListCmd::cmListHeader] = OPER_OPER_LIST_HEADER;
	oper_msgs[ListCmd::cmListedOne] = OPER_OPER_LISTED_ONE;
	oper_msgs[ListCmd::cmListedSeveral] = OPER_OPER_LISTED_SEVERAL;
	oper_msgs[ListCmd::cmClearPartially] = OPER_OPER_CLEAR_PARTIALLY;
	oper_msgs[ListCmd::cmClear] = OPER_OPER_CLEAR;
	
	return true;
}

/*************************************************************************/

void operserv_notify(modules::Module &sender, int msg, void *arg)
{
	switch (msg)
	{
		case modules::Module::mnLoaded:
			if (sender.name() == epona->cd().operserv.database_stats.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database operserv_stats: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				stats.setup_database(db);
			}
			
			if (akills.available() &&
					(sender.name() == epona->cd().operserv.database_akills.value()))
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database operserv_akills: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				akills.setup_database(db);
			}
			
			if (sglines.available() && 
					(sender.name() == epona->cd().operserv.database_sglines.value()))
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database operserv_sglines: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				sglines.setup_database(db);
			}
			
			if (sender.name() == epona->cd().operserv.database_sqlines.value())
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database operserv_sqlines: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				sqlines.setup_database(db);
			}
			
			if (szlines.available() && 
					(sender.name() == epona->cd().operserv.database_szlines.value()))
			{
				databases::Database *db = (databases::Database *)
						sender.factory(databases::Database::IID);
				if (!db)
				{
					epona->fatalf("Can't create database operserv_szlines: "
							"module %s does not expose the Database interface",
							sender.name().c_str());	
				}
				
				/* Add a reference to the module */
				sender.references_add();
				/* Setup the database */
				szlines.setup_database(db);
			}
		default:
			break;
	}
}

/*************************************************************************/

void operserv_start()
{
	if (!stats.db() || !stats.db()->start())
		epona->fatal("Can't start database operserv_stats");
	
	stats.make_record();
	
	if (akills.available() && (!akills.db() || !akills.db()->start()))
		epona->fatal("Can't start database operserv_akills");
	
	if (szlines.available() && (!sglines.db() || !sglines.db()->start()))
		epona->fatal("Can't start database operserv_sglines");
	
	if (!sqlines.db() || !sqlines.db()->start())
		epona->fatal("Can't start database operserv_sqlines");
	
	if (szlines.available() && (!szlines.db() || !szlines.db()->start()))
		epona->fatal("Can't start database operserv_szlines");
}

/*************************************************************************/

/* Main OperServ routine. */

void operserv(User *u, char *buf)
{
	char *cmd;
	char *s;
	
	epona->log(InfoOperServUse(), "%s: %s", u->nick, buf);
	
	cmd = strtok(buf, " ");
	if (!cmd) {
		return;
	} else if (stricmp(cmd, "\1PING") == 0) {
		s = strtok(NULL, "\1");
	    
		notice(s_OperServ, u->nick, "\1PING%s%s\1", (s ? " " : ""),
				(s ? s : ""));
	} else {
		run_cmd(s_OperServ, u, cmds, cmd);
	}
}

/*************************************************************************/
/**************************** Privilege checks ***************************/
/*************************************************************************/

/* Does the given user have Services root privileges?
   Now enhanced. */

int is_services_root(User *u)
{
    if ((epona->cd().nickserv.strict_privileges && !is_oper(u)) ||
    		!nick_identified(u))
    	return 0;
    if (u->na->ng()->flags() & NickGroup::gfServicesRoot)
		return 1;
    return 0;
}

/*************************************************************************/

/* Does the given user have Services admin privileges? */

int is_services_admin(User *u)
{
    if ((epona->cd().nickserv.strict_privileges && !is_oper(u)) || 
    		!nick_identified(u))
		return 0;
    if (u->na->ng()->flags() &
    		(NickGroup::gfServicesAdmin | NickGroup::gfServicesRoot))
		return 1;
	return 0;
}

/*************************************************************************/

/* Does the given user have Services oper privileges? */

int is_services_oper(User *u)
{
    if ((epona->cd().nickserv.strict_privileges && !is_oper(u)) || 
    		!nick_identified(u))
		return 0;
    if (u->na->ng()->flags() & 
    		(NickGroup::gfServicesOper | NickGroup::gfServicesAdmin |
    		 NickGroup::gfServicesRoot))
		return 1;
	return 0;
}

/*************************************************************************/

/* Is the given nick a Services admin/root nick? */

int nick_is_services_admin(NickGroup *nc)
{
    if (nc->flags() & (NickGroup::gfServicesAdmin | NickGroup::gfServicesRoot)) 
		return 1;
	
	return 0;
}

/*************************************************************************/

/* Is the given nick a Services oper/admin/root nick? */

int nick_is_services_oper(NickGroup *nc)
{
    if (nc->flags() & (NickGroup::gfServicesOper | NickGroup::gfServicesAdmin | NickGroup::gfServicesRoot)) 
		return 1;
	
	return 0;
}

/*************************************************************************/
/*********************** OperServ command functions **********************/
/*************************************************************************/

/* HELP command. */

static void do_help(User *u)
{
    const char *cmd = strtok(NULL, "");

    if (!cmd) {
	notice_help(s_OperServ, u, OPER_HELP);
    } else {
	help_cmd(s_OperServ, u, cmds, cmd);
    }
}

/*************************************************************************/

/* Global notice sending via GlobalNoticer. */

static void do_global(User *u)
{
    char *msg = strtok(NULL, "");

    if (!msg) {
		syntax_error(s_OperServ, u, "GLOBAL", OPER_GLOBAL_SYNTAX);
		return;
    }
    
    if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waGlobal)
   		wallops(s_OperServ, "\2%s\2 just used GLOBAL command.", u->nick);

	if (epona->cd().global.network_domain) {
		send_cmd(s_GlobalNoticer, "NOTICE $*.%s :%s",
				(char const *)epona->cd().global.network_domain, msg);
	} else {
		/* Go through all common top-level domains.  If you have others,
         * add them here.
         */
		notice(s_GlobalNoticer, "$*.com", "%s", msg);
		notice(s_GlobalNoticer, "$*.net", "%s", msg);
    	notice(s_GlobalNoticer, "$*.org", "%s", msg);
    	notice(s_GlobalNoticer, "$*.edu", "%s", msg);
	}
}

/*************************************************************************/

/* STATS command. */

static void do_stats(User *u)
{
    time_t uptime = time(NULL) - epona->start_time();
    char *extra = strtok(NULL, "");
    int days = uptime/86400, hours = (uptime/3600)%24,
        mins = (uptime/60)%60, secs = uptime%60;
    struct tm *tm;
    char timebuf[64];

    if (extra && stricmp(extra, "ALL") != 0) {
	if (stricmp(extra, "AKILL") == 0) {
	    int timeout; 
	    /* AKILLs */
	    notice_lang(s_OperServ, u, OPER_STATS_AKILL_COUNT, akills.size());
	    timeout = epona->cd().operserv.expire_akills+59;
	    if (timeout >= 172800)
		notice_lang(s_OperServ, u, OPER_STATS_AKILL_EXPIRE_DAYS, timeout/86400);
	    else if (timeout >= 86400)
		notice_lang(s_OperServ, u, OPER_STATS_AKILL_EXPIRE_DAY);
	    else if (timeout >= 7200)
		notice_lang(s_OperServ, u, OPER_STATS_AKILL_EXPIRE_HOURS, timeout/3600);
	    else if (timeout >= 3600)
		notice_lang(s_OperServ, u, OPER_STATS_AKILL_EXPIRE_HOUR);
	    else if (timeout >= 120)
		notice_lang(s_OperServ, u, OPER_STATS_AKILL_EXPIRE_MINS, timeout/60);
	    else if (timeout >= 60)
		notice_lang(s_OperServ, u, OPER_STATS_AKILL_EXPIRE_MIN);
	    else
		notice_lang(s_OperServ, u, OPER_STATS_AKILL_EXPIRE_NONE);
		/* SGLINEs */
		if (sglines.available())
		{
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_COUNT, sglines.size());
		    timeout = epona->cd().operserv.expire_sglines+59;
		    if (timeout >= 172800)
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_EXPIRE_DAYS, timeout/86400);
		    else if (timeout >= 86400)
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_EXPIRE_DAY);
		    else if (timeout >= 7200)
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_EXPIRE_HOURS, timeout/3600);
		    else if (timeout >= 3600)
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_EXPIRE_HOUR);
		    else if (timeout >= 120)
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_EXPIRE_MINS, timeout/60);
		    else if (timeout >= 60)
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_EXPIRE_MIN);
		    else
			notice_lang(s_OperServ, u, OPER_STATS_SGLINE_EXPIRE_NONE);
		}
		/* SQLINEs */
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_COUNT, sqlines.size());
	    timeout = epona->cd().operserv.expire_sqlines+59;
	    if (timeout >= 172800)
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_EXPIRE_DAYS, timeout/86400);
	    else if (timeout >= 86400)
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_EXPIRE_DAY);
	    else if (timeout >= 7200)
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_EXPIRE_HOURS, timeout/3600);
	    else if (timeout >= 3600)
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_EXPIRE_HOUR);
	    else if (timeout >= 120)
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_EXPIRE_MINS, timeout/60);
	    else if (timeout >= 60)
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_EXPIRE_MIN);
	    else
		notice_lang(s_OperServ, u, OPER_STATS_SQLINE_EXPIRE_NONE);
		if (szlines.available())
		{
			/* SZLINEs */
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_COUNT, szlines.size());
		    timeout = epona->cd().operserv.expire_szlines+59;
		    if (timeout >= 172800)
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_EXPIRE_DAYS, timeout/86400);
		    else if (timeout >= 86400)
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_EXPIRE_DAY);
		    else if (timeout >= 7200)
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_EXPIRE_HOURS, timeout/3600);
		    else if (timeout >= 3600)
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_EXPIRE_HOUR);
		    else if (timeout >= 120)
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_EXPIRE_MINS, timeout/60);
		    else if (timeout >= 60)
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_EXPIRE_MIN);
		    else
			notice_lang(s_OperServ, u, OPER_STATS_SZLINE_EXPIRE_NONE);
		}
	    return;
	} else if (!stricmp(extra, "RESET")) 
	{
		if (!is_services_admin(u)) {
			notice_lang(s_OperServ, u, PERMISSION_DENIED);
		} else if (!stats_rec || !stats.writable()) {
			notice_lang(s_OperServ, u, OPER_STATS_RESET_DISABLED);
		} else
		{
			stats_rec->new_max(usercnt, time(0));
			notice_lang(s_OperServ, u, OPER_STATS_RESET);
		}
		
		return;
	} else {
	    notice_lang(s_OperServ, u, OPER_STATS_UNKNOWN_OPTION,
			extra);
	}
    }

    notice_lang(s_OperServ, u, OPER_STATS_CURRENT_USERS, usercnt, opcnt);
    time_t t = (stats_rec ? stats_rec->maxusers_time() : 0);
    tm = localtime(&t);
    strftime_lang(timebuf, sizeof(timebuf), u, STRFTIME_DATE_TIME_FORMAT, tm);
    notice_lang(s_OperServ, u, OPER_STATS_MAX_USERS,
    		(stats_rec ? stats_rec->maxusers() : 0), timebuf);
    if (days > 1) {
	notice_lang(s_OperServ, u, OPER_STATS_UPTIME_DHMS,
		days, hours, mins, secs);
    } else if (days == 1) {
	notice_lang(s_OperServ, u, OPER_STATS_UPTIME_1DHMS,
		days, hours, mins, secs);
    } else {
	if (hours > 1) {
	    if (mins != 1) {
		if (secs != 1) {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_HMS,
				hours, mins, secs);
		} else {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_HM1S,
				hours, mins, secs);
		}
	    } else {
		if (secs != 1) {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_H1MS,
				hours, mins, secs);
		} else {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_H1M1S,
				hours, mins, secs);
		}
	    }
	} else if (hours == 1) {
	    if (mins != 1) {
		if (secs != 1) {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_1HMS,
				hours, mins, secs);
		} else {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_1HM1S,
				hours, mins, secs);
		}
	    } else {
		if (secs != 1) {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_1H1MS,
				hours, mins, secs);
		} else {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_1H1M1S,
				hours, mins, secs);
		}
	    }
	} else {
	    if (mins != 1) {
		if (secs != 1) {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_MS,
				mins, secs);
		} else {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_M1S,
				mins, secs);
		}
	    } else {
		if (secs != 1) {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_1MS,
				mins, secs);
		} else {
		    notice_lang(s_OperServ, u, OPER_STATS_UPTIME_1M1S,
				mins, secs);
		}
	    }
	}
    }

    if (extra && stricmp(extra, "ALL") == 0 && is_services_admin(u)) {
		long count, mem;

		notice_lang(s_OperServ, u, OPER_STATS_BYTES_READ, total_read / 1024);
		notice_lang(s_OperServ, u, OPER_STATS_BYTES_WRITTEN, total_written / 1024);

		get_user_stats(&count, &mem);
		notice_lang(s_OperServ, u, OPER_STATS_USER_MEM, count, (mem+512) / 1024);
		get_channel_stats(&count, &mem);
		notice_lang(s_OperServ, u, OPER_STATS_CHANNEL_MEM, count, (mem+512) / 1024);
		get_session_stats(&count, &mem);
		notice_lang(s_OperServ, u, OPER_STATS_SESSIONS_MEM, count, (mem+512) / 1024);
		if (epona->cd().blacklist.enable) 
		{
			get_blacklist_stats(&count, &mem);
			notice_lang(s_OperServ, u, OPER_STATS_BLACKLIST_MEM, count, (mem+512) / 1024);	
		}
		
		databases::Database::const_iterator it, end;
		
		for (it = databases::Database::begin(),
				end = databases::Database::end(); it != end; ++it)
		{
			databases::Database *db = (*it);
			
			count = db->coll()->size();
			mem = db->coll()->memory_usage();
			
			notice_lang(s_OperServ, u,
					((count > 1) ? OPER_STATS_MEM : OPER_STATS_MEM_ONE),
					db->name().c_str(), count, (mem + 512) / 1024);
		}
	}
}

/*************************************************************************/

/* Channel mode changing (MODE command). */

static void do_os_mode(User *u)
{
    int ac;
    char **av;
    char *chan = strtok(NULL, " "), *modes = strtok(NULL, "");
    Channel *c;

    if (!chan || !modes) {
		syntax_error(s_OperServ, u, "MODE", OPER_MODE_SYNTAX);
		return;
    }
    
    if (!(c = findchan(chan))) {
		notice_lang(s_OperServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (c->bouncy_modes) {
		notice_lang(s_OperServ, u, OPER_BOUNCY_MODES_U_LINE);
		return;
    } else {
#ifdef IRC_BAHAMUT
    	send_cmd(s_OperServ, "MODE %s 0 %s", chan, modes);
#else
    	send_cmd(s_OperServ, "MODE %s %s", chan, modes);
#endif
		
		ac = split_buf(modes, &av, 1);
		chan_set_modes(s_OperServ, c, ac, av, 0);
		
		if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waMode)
	    	wallops(s_OperServ, "%s used MODE %s on %s", u->nick, modes, chan);
    }
}

/*************************************************************************/

/* Clear all modes from a channel. */

static void do_clearmodes(User *u)
{
    char *s;
    int i;
    char const *argv[2];
    char *chan = strtok(NULL, " ");
    Channel *c;
    int all = 0, status = 0;
    int count;		/* For saving ban/exempt info */
    char **list;	/* For saving ban/exempt info */
    Channel::c_userlist *cu, *next;
    CUModeInfo *cmi;

    if (!chan) {
		syntax_error(s_OperServ, u, "CLEARMODES", OPER_CLEARMODES_SYNTAX);
    } else if (!(c = findchan(chan))) {
		notice_lang(s_OperServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (c->bouncy_modes) {
		notice_lang(s_OperServ, u, OPER_BOUNCY_MODES_U_LINE);
		return;
    } else {
		s = strtok(NULL, " ");
		if (s) {
	    	if (stricmp(s, "ALL") == 0) {
				all = 1;
	    	} else {
				syntax_error(s_OperServ, u, "CLEARMODES", OPER_CLEARMODES_SYNTAX);
				return;
	    	}
		}
		
		if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waClearModes)
	    	wallops(s_OperServ, "%s used CLEARMODES%s on %s", u->nick,
	    			all ? " ALL" : "", chan);
	
		if (all) {
	    	/* Clear user modes */
	    	
	    	for (cu = c->users; cu; cu = next) {
				next = cu->next;
				
				status = chan_get_user_status(c, cu->user);
				
				for (cmi = cumodeinfos; cmi->mode; cmi++) 
				{
					if (status & cmi->status) 
					{
						char modebuf[4];
						snprintf(modebuf, sizeof(modebuf), "-%c", cmi->mode);
						
						argv[0] = modebuf;
						argv[1] = cu->user->nick;
				
						s_mode(s_OperServ, c->name, argv[0], cu->user->nick);
						chan_set_modes(s_OperServ, c, 2, argv, 0);
					}
				}
			}
		}

		/* Clear modes */
#ifdef IRC_BAHAMUT
		send_cmd(s_OperServ, "MODE %s 0 %s %s", c->name, MODESTOREMOVE, c->key ? c->key : "");
#else
		send_cmd(s_OperServ, "MODE %s %s %s", c->name, MODESTOREMOVE, c->key ? c->key : "");
#endif
		argv[0] = MODESTOREMOVE;
		argv[1] = c->key ? c->key : NULL;
		chan_set_modes(s_OperServ, c, c->key ? 2 : 1, argv, 0);

		/* Clear bans */
		if (c->bancount) {
			count = c->bancount;
			list = static_cast<char **>(smalloc(sizeof(char *) * count));
		
			for (i = 0; i < count; i++)
	    		list[i] = misc::nstrdup(c->bans[i]);
		
			for (i = 0; i < count; i++) {
	    		argv[0] = "-b";
	    		argv[1] = list[i];
	    		s_mode(s_OperServ, c->name, "-b", argv[1]);
				chan_set_modes(s_OperServ, c, 2, argv, 0);
	    		delete [] list[i];
			}
		
			free(list);
		}

#ifdef HAS_EXEMPTS
		if (c->exemptcount) {
			count = c->exemptcount;
			list = static_cast<char **>(smalloc(sizeof(char *) * count));
		
			for (i = 0; i < count; i++)
	    		list[i] = misc::nstrdup(c->exempts[i]);
		
			for (i = 0; i < count; i++) {
	    		argv[0] = "-e";
	    		argv[1] = list[i];
	    		s_mode(s_OperServ, c->name, "-e", argv[1]);
				chan_set_modes(s_OperServ, c, 2, argv, 0);
	    		delete [] list[i];
			}
		
			free(list);
		}
#endif

		notice_lang(s_OperServ, u, OPER_CLEARMODES_ALL_DONE, chan);
	}
}

/*************************************************************************/

/* Kick a user from a channel (KICK command). */

static void do_os_kick(User *u)
{
    char *argv[3];
    char *chan, *nick, *s;
    Channel *c;

    chan = strtok(NULL, " ");
    nick = strtok(NULL, " ");
    s = strtok(NULL, "");
    if (!chan || !nick || !s) {
	syntax_error(s_OperServ, u, "KICK", OPER_KICK_SYNTAX);
	return;
    }
    if (!(c = findchan(chan))) {
	notice_lang(s_OperServ, u, CHAN_X_NOT_IN_USE, chan);
    } else if (c->bouncy_modes) {
	notice_lang(s_OperServ, u, OPER_BOUNCY_MODES_U_LINE);
	return;
    }
    send_cmd(s_OperServ, "KICK %s %s :%s (%s)", chan, nick, u->nick, s);
    if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waKick)
	wallops(s_OperServ, "%s used KICK on %s/%s", u->nick, nick, chan);
    argv[0] = chan;
    argv[1] = nick;
    argv[2] = s;
    do_kick(s_OperServ, 3, argv);
}

/*************************************************************************/

AkillCmd::AkillCmd() : ListCmd("AKILL", s_OperServ, akill_msgs, true)
{ 

}

/*************************************************************************/

int AkillCmd::add(User *u, container_type &ctn, char const *sub)
{
	char *expiry = NULL, *mask = strtok(NULL, " ");
	
	if (mask && *mask == '+')
	{
		expiry = mask;
		mask = strtok(NULL, " ");
	}
	
	char *reason = strtok(NULL, "");
	
	if (!mask || !reason)
	{
		syntax_error(m_service, u, m_name, OPER_AKILL_ADD_SYNTAX);
		return crStop;
	}
	
	/* Computes the expiration time */
	time_t expires = epona->cd().operserv.expire_akills;
	
	if (expiry && !duration_scan(expiry, &expires))
	{
		notice_lang(m_service, u, BAD_EXPIRY_TIME);
		return crStop;
	}
	
	if (expires)
		expires += time(0);
	
	/* Check for masks in the format nick[:type] */
	misc::cstring nmask;
	
	if (!strchr(mask, '@'))
	{
		uint16_t type = epona->cd().operserv.default_akilltype;
		
		char *dp = strchr(mask, ':');
		if (dp)
		{
			*dp++ = 0;
			
			char *end;
			type = strtoul(dp, &end, 10);
			
			if (!*dp || *end || type > 3)
			{
				*--dp = ':';	
				notice_lang(s_OperServ, u, OPER_AKILL_MASK_INVALID, mask);
				return crStop;
			}
		}
		
		User *target;
		
		if (!(target = finduser(mask)))
		{
			notice_lang(s_OperServ, u, NICK_X_NOT_IN_USE, mask);
			return crStop;
		}
		
		nmask.reserve(Akill::mask_len + 1);
		
		switch (type) 
		{
			case 0:
				snprintf(nmask, Akill::mask_len + 1, "%s@%s",
						target->username, target->host);
				break;
			case 1:
				snprintf(nmask, Akill::mask_len + 1, "%s%s@%s",
						(strlen(target->username) < (*(target->username) ==
							'~' ? USERMAX+1 : USERMAX) ? "*" : ""),
						(*(target->username) == '~' ? target->username+1 :
							target->username),
						target->host);
				break;
			case 2:
				snprintf(nmask, Akill::mask_len + 1, "*@%s", target->host);
				break;
			case 3:
				mask = create_mask(target, false); 
				snprintf(nmask, Akill::mask_len + 1, "%s", mask);
				free(mask);
				break;
			default:
				break;
		}
		
		mask = nmask;
	}
	
	/* Validates the mask */
	
	if (strlen(mask) > Akill::mask_len)
	{
		notice_lang(m_service, u, OPER_AKILL_MASK_TOO_LONG,
				Akill::mask_len);
		return crStop;
	}
	
	if (strchr(mask, '!'))
	{
		notice_lang(m_service, u, OPER_AKILL_MASK_NO_NICK);
		return crStop;
	}
	
	/* Validates the reason */
	
	if (strlen(reason) > Akill::reason_len)
	{
		notice_lang(m_service, u, OPER_AKILL_REASON_TOO_LONG,
				Akill::reason_len);
		return crStop;
	}
	
	if (add_record(u, mask, u->nick, expires, reason) < 0)
		return crStop;
	
	if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waAkill)
	{
		char buf[128];
		
		if(!expires) {
			strcpy(buf, "does not expire");
		} else {
			int wall_expiry = expires-time(NULL);
			char const *s = NULL;
			
			if (wall_expiry >= 86400) {
				wall_expiry /= 86400;
				s = "day";
			} else if (wall_expiry >= 3600) { 
				wall_expiry /= 3600;
				s = "hour";
			} else if (wall_expiry >= 60) {
				wall_expiry /= 60;
				s = "minute";
			} else {
				s = "second";
			}
			
			snprintf(buf, sizeof(buf), "expires in %d %s%s", wall_expiry, s,
					(wall_expiry == 1) ? "" : "s");
		}
		
		wallops(s_OperServ, "%s added an AKILL for %s (%s)",
				u->nick, mask, buf);
	}
	
	return crStop;
}

/*************************************************************************/

int AkillCmd::add_record(User *u, char const *mask, char const *creator,
		time_t expires, char const *reason)
{
	int deleted = 0;
	Akill *ak;
	
	ak = (Akill *)akills.create();
	
	ak->mask(mask);
	
	if (!(ak->netban()->flags() & NetBan::bfComplete))
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_AKILL_MASK_INVALID, mask);
		delete ak;
		
		return -1;
	}
	
	ak->creator(creator);
	ak->reason(reason);
	ak->expires(expires);
	
	/* Checks whether there is an AKILL that already covers
	 * the one we want to add, and whether there are AKILLs
	 * that would be covered by this one. The masks AND the
	 * expiry times are used to determine this, because some
	 * AKILLs may become useful when another one expires.
	 * If so, warn the user in the first case and cleanup
	 * the useless AKILLs in the second.
	 */
	if (akills.size()) 
	{
		Akill *ak2;
		
		for (int i = 0; i < akills.array().size(); ++i)
		{
			ak2 = (Akill *)akills.array()[i];
			
			if (!ak2)
				continue;
			
			if (!stricmp(ak2->mask(), ak->mask())) 
			{
				/* We change the AKILL expiry time if its current one is less
				 * than the new. This is preferable to be sure we don't change
				 * an important AKILL by mistake. */
				if (ak2->expires() >= ak->expires() || ak2->expires() == 0) 
				{
					if (u) 
						notice_lang(s_OperServ, u,
								OPER_AKILL_ALREADY_PRESENT, ak->mask());
					delete ak;
					return -1;
				} 
				else 
				{
					ak2->expires(expires);
					if (u)
						notice_lang(s_OperServ, u,
								OPER_AKILL_CHANGED, ak2->mask());
					delete ak;
					return -2;
				}
			}
			
			if (ak2->netban()->wider(ak->netban()) &&
					(ak2->expires() >= ak->expires() || ak2->expires() == 0))
			{
				if (u)
					notice_lang(s_OperServ, u, OPER_AKILL_ALREADY_COVERED,
							ak->mask(), ak2->mask());
				delete ak;
				return -1;
			}
			
			if (ak->netban()->wider(ak2->netban()) &&
					(ak2->expires() <= ak->expires() || ak->expires() == 0))
			{
				if (!akills.remove(ak2))
				{
					notice_lang(s_OperServ, u, m_msgs[cmDisabled]);
					return -1;
				}
				
				deleted++;
			}
		}
	}
	
	if (u && deleted)
	{
		if (deleted == 1)
			notice_lang(s_OperServ, u, OPER_AKILL_DELETED_ONE);
		else
			notice_lang(s_OperServ, u, OPER_AKILL_DELETED_SEVERAL, deleted);
	}
	
	/* We can now check whether the list is full or not. */
	if (akills.size() == akills.array().max_size()) 
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_AKILL_TOO_MANY);
		delete ak;
		
		return -1;	
	}
	
	/* Try to add the AKILL to the list */
	
	int error;
	
	if (!akills.add(ak, &error))
	{
		if (u)
		{
			if (error == databases::ArrayCollection::ceName ||
					error == databases::ArrayCollection::ceList)
				notice_lang(s_OperServ, u, OPER_AKILL_ALREADY_PRESENT,
				 		ak->mask());
			else
				notice_lang(m_service, u, m_msgs[cmDisabled]);
		}
				
		delete ak;
		return -1;
	} else {
		if (u)
			notice_lang(m_service, u, OPER_AKILL_ADDED, ak->mask());
	}
	
	return deleted;
}

/*************************************************************************/

bool AkillCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return akills.writable();
}

/*************************************************************************/

bool AkillCmd::clear_records(User *u, container_type &ctn,
		char const *sub)
{
	if (!akills.clear())
		return true; /* Partial clear (on paper :P) */
	
	ctn.clear();
	return false;
}

/*************************************************************************/

AkillCmd::container_type *AkillCmd::container(User *u)
{
	if (!akills.available()) 
	{
		notice_lang(s_OperServ, u, OPER_AKILL_UNSUPPORTED);
		return 0;
	}
	
	return &akills.array();
}

/*************************************************************************/

bool AkillCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	return akills.remove(rec);
}

/*************************************************************************/

int AkillCmd::dispatch(User *u, container_type &ctn, char const *sub)
{
	if (!stricmp(sub, "VIEW"))
		return list(u, ctn, sub);
	else
		return ListCmd::dispatch(u, ctn, sub);
}

/*************************************************************************/

bool AkillCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	Akill *ak = (Akill *)rec;
	
	if (!stricmp(sub, "LIST"))
	{
		if (count == 0)
			notice_lang(m_service, u, OPER_AKILL_LIST_COLHEAD);
	
		notice_lang(m_service, u, OPER_AKILL_LIST_FORMAT, num,
				ak->mask(), ak->reason());
	} else if (!stricmp(sub, "VIEW")) 
	{
		time_t t = ak->since();
		struct tm tm = *localtime(&t);
		
		char timebuf[32];
		strftime_lang(timebuf, sizeof(timebuf), u,
				STRFTIME_SHORT_DATE_FORMAT, &tm);
		
		char expirebuf[256];
		expire_left(u->na, expirebuf, sizeof(expirebuf), ak->expires());
		
		notice_lang(s_OperServ, u, OPER_AKILL_VIEW_FORMAT, num, ak->mask(),
				ak->creator(), timebuf, expirebuf, ak->reason());
	}
	
	return true;
}

/*************************************************************************/

/* Manage the AKILL list. */

static void do_akill(User *u)
{
	akill_cmd->execute(u);
}

/*************************************************************************/

SGLineCmd::SGLineCmd() : ListCmd("SGLINE", s_OperServ, sgline_msgs, true)
{ 

}

/*************************************************************************/

int SGLineCmd::add(User *u, container_type &ctn, char const *sub)
{
	char *expiry = NULL, *mask = strtok(NULL, ":");
	
	if (mask && *mask == '+')
	{
		expiry = mask;
		mask = strchr(expiry, ' ');
		
		if (mask)
		{
			*mask = 0;
			++mask;
		}
	}
	
	char *reason = strtok(NULL, "");
	
	if (!mask || !reason)
	{
		syntax_error(m_service, u, m_name, OPER_SGLINE_ADD_SYNTAX);
		return crStop;
	}
	
	/* Computes the expiration time */
	time_t expires = epona->cd().operserv.expire_sglines;
	
	if (expiry && !duration_scan(expiry, &expires))
	{
		notice_lang(m_service, u, BAD_EXPIRY_TIME);
		return crStop;
	}
	
	if (expires)
		expires += time(0);
	
	/* Validates the mask */
	
	if (strlen(mask) > SGLine::mask_len)
	{
		notice_lang(m_service, u, OPER_SGLINE_MASK_TOO_LONG,
				SGLine::mask_len);
		return crStop;
	}
	
	/* Validates the reason */
	
	if (strlen(reason) > SGLine::reason_len)
	{
		notice_lang(m_service, u, OPER_SGLINE_REASON_TOO_LONG,
				SGLine::reason_len);
		return crStop;
	}
	
	if (add_record(u, mask, u->nick, expires, reason) < 0)
		return crStop;
	
	if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waSGLine)
	{
		char buf[128];
		
		if(!expires) {
			strcpy(buf, "does not expire");
		} else {
			int wall_expiry = expires-time(NULL);
			char const *s = NULL;
			
			if (wall_expiry >= 86400) {
				wall_expiry /= 86400;
				s = "day";
			} else if (wall_expiry >= 3600) { 
				wall_expiry /= 3600;
				s = "hour";
			} else if (wall_expiry >= 60) {
				wall_expiry /= 60;
				s = "minute";
			} else {
				s = "second";
			}
			
			snprintf(buf, sizeof(buf), "expires in %d %s%s", wall_expiry, s,
					(wall_expiry == 1) ? "" : "s");
		}
		
		wallops(s_OperServ, "%s added a SGLINE for %s (%s)",
				u->nick, mask, buf);
	}
	
	return crStop;
}

/*************************************************************************/

int SGLineCmd::add_record(User *u, char const *mask, char const *creator,
		time_t expires, char const *reason)
{
	int deleted = 0;
	
	SGLine *sg;
	
	sg = (SGLine *)sglines.create();
	
	sg->mask(mask);
	
	if (!(sg->netban()->flags() & NetBan::bfComplete))
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_SGLINE_MASK_INVALID, mask);
		delete sg;
		
		return -1;
	}
	
	sg->creator(creator);
	sg->reason(reason);
	sg->expires(expires);
	
	/* Checks whether there is an SGLINE that already covers
	 * the one we want to add, and whether there are SGLINEs
	 * that would be covered by this one. The masks AND the
	 * expiry times are used to determine this, because some
	 * SGLINEs may become useful when another one expires.
	 * If so, warn the user in the first case and cleanup
	 * the useless SGLINEs in the second.
	 */
	if (sglines.size()) 
	{
		SGLine *sg2;
		
		for (int i = 0; i < sglines.array().size(); ++i)
		{
			sg2 = (SGLine *)sglines.array()[i];
			
			if (!sg2)
				continue;
			
			if (!stricmp(sg2->mask(), sg->mask())) 
			{
				/* We change the SGLINE expiry time if its current one is less
				 * than the new. This is preferable to be sure we don't change
				 * an important SGLINE by mistsge. */
				if (sg2->expires() >= sg->expires() || sg2->expires() == 0) 
				{
					if (u) 
						notice_lang(s_OperServ, u,
								OPER_SGLINE_ALREADY_PRESENT, sg->mask());
					delete sg;
					return -1;
				} 
				else 
				{
					sg2->expires(expires);
					if (u)
						notice_lang(s_OperServ, u,
								OPER_SGLINE_CHANGED, sg2->mask());
					delete sg;
					return -2;
				}
			}
			
			if (sg2->netban()->wider(sg->netban()) &&
					(sg2->expires() >= sg->expires() || sg2->expires() == 0))
			{
				if (u)
					notice_lang(s_OperServ, u, OPER_SGLINE_ALREADY_COVERED,
							sg->mask(), sg2->mask());
				delete sg;
				return -1;
			}
			
			if (sg->netban()->wider(sg2->netban()) &&
					(sg2->expires() <= sg->expires() || sg->expires() == 0))
			{
				if (!sglines.remove(sg2))
				{
					notice_lang(s_OperServ, u, m_msgs[cmDisabled]);
					return -1;
				}
				
				deleted++;
			}
		}
	}
	
	if (u && deleted)
	{
		if (deleted == 1)
			notice_lang(s_OperServ, u, OPER_SGLINE_DELETED_ONE);
		else
			notice_lang(s_OperServ, u, OPER_SGLINE_DELETED_SEVERAL, deleted);
	}
	
	/* We can now check whether the list is full or not. */
	if (sglines.size() == sglines.array().max_size()) 
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_SGLINE_TOO_MANY);
		delete sg;
		
		return -1;	
	}
	
	/* Try to add the SGLINE to the list */
	
	int error = 0;
	
	if (!sglines.add(sg, &error))
	{
		if (u)
		{
			if (error == databases::ArrayCollection::ceName ||
					error == databases::ArrayCollection::ceList)
				notice_lang(s_OperServ, u, OPER_SGLINE_ALREADY_PRESENT,
				 		sg->mask());
			else
				notice_lang(m_service, u, m_msgs[cmDisabled]);
		}
				
		delete sg;
		return -1;
	} else {
		if (u)
			notice_lang(m_service, u, OPER_SGLINE_ADDED, sg->mask());
	}
	
	return deleted;
}

/*************************************************************************/

bool SGLineCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return sglines.writable();
}

/*************************************************************************/

bool SGLineCmd::clear_records(User *u, container_type &ctn,
		char const *sub)
{
	if (!sglines.clear())
		return true; /* Partial clear (on paper :P) */
	
	ctn.clear();
	return false;
}

/*************************************************************************/

SGLineCmd::container_type *SGLineCmd::container(User *u)
{
	if (!sglines.available()) 
	{
		notice_lang(s_OperServ, u, OPER_SGLINE_UNSUPPORTED);
		return 0;
	}
	
	return &sglines.array();
}

/*************************************************************************/

bool SGLineCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	return sglines.remove(rec);
}

/*************************************************************************/

int SGLineCmd::dispatch(User *u, container_type &ctn, char const *sub)
{
	if (!stricmp(sub, "VIEW"))
		return list(u, ctn, sub);
	else
		return ListCmd::dispatch(u, ctn, sub);
}

/*************************************************************************/

bool SGLineCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	SGLine *sg = (SGLine *)rec;
	
	if (!stricmp(sub, "LIST"))
	{
		if (count == 0)
			notice_lang(m_service, u, OPER_SGLINE_LIST_COLHEAD);
	
		notice_lang(m_service, u, OPER_SGLINE_LIST_FORMAT, num,
				sg->mask(), sg->reason());
	} else if (!stricmp(sub, "VIEW")) 
	{
		time_t t = sg->since();
		struct tm tm = *localtime(&t);
		
		char timebuf[32];
		strftime_lang(timebuf, sizeof(timebuf), u,
				STRFTIME_SHORT_DATE_FORMAT, &tm);
		
		char expirebuf[256];
		expire_left(u->na, expirebuf, sizeof(expirebuf), sg->expires());
		
		notice_lang(s_OperServ, u, OPER_AKILL_VIEW_FORMAT, num, sg->mask(),
				sg->creator(), timebuf, expirebuf, sg->reason());
	}
	
	return true;
}

/*************************************************************************/

/* Manage the SGLINE list. */

static void do_sgline(User *u)
{
	static SGLineCmd sgline_cmd;
	sgline_cmd.execute(u);
}

/*************************************************************************/

SQLineCmd::SQLineCmd() : ListCmd("SQLINE", s_OperServ, sqline_msgs, true)
{ 

}

/*************************************************************************/

int SQLineCmd::add(User *u, container_type &ctn, char const *sub)
{
	char *expiry = NULL, *mask = strtok(NULL, " ");
	
	if (mask && *mask == '+')
	{
		expiry = mask;
		mask = strtok(NULL, " ");
	}
	
	char *reason = strtok(NULL, "");
	
	if (!mask || !reason)
	{
		syntax_error(m_service, u, m_name, OPER_SQLINE_ADD_SYNTAX);
		return crStop;
	}
	
	/* Computes the expiration time */
	time_t expires = epona->cd().operserv.expire_sqlines;
	
	if (expiry && !duration_scan(expiry, &expires))
	{
		notice_lang(m_service, u, BAD_EXPIRY_TIME);
		return crStop;
	}
	
	if (expires)
		expires += time(0);
	
	/* Validates the mask */
	
	if (strlen(mask) > SQLine::mask_len)
	{
		notice_lang(m_service, u, OPER_SQLINE_MASK_TOO_LONG,
				SQLine::mask_len);
		return crStop;
	}
	
	/* Validates the reason */
	
	if (strlen(reason) > SQLine::reason_len)
	{
		notice_lang(m_service, u, OPER_SQLINE_REASON_TOO_LONG,
				SQLine::reason_len);
		return crStop;
	}
	
	if (add_record(u, mask, u->nick, expires, reason) < 0)
		return crStop;
	
	if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waSQLine)
	{
		char buf[128];
		
		if(!expires) {
			strcpy(buf, "does not expire");
		} else {
			int wall_expiry = expires-time(NULL);
			char const *s = NULL;
			
			if (wall_expiry >= 86400) {
				wall_expiry /= 86400;
				s = "day";
			} else if (wall_expiry >= 3600) { 
				wall_expiry /= 3600;
				s = "hour";
			} else if (wall_expiry >= 60) {
				wall_expiry /= 60;
				s = "minute";
			} else {
				s = "second";
			}
			
			snprintf(buf, sizeof(buf), "expires in %d %s%s", wall_expiry, s,
					(wall_expiry == 1) ? "" : "s");
		}
		
		wallops(s_OperServ, "%s added a SQLINE for %s (%s)",
				u->nick, mask, buf);
	}
	
	return crStop;
}

/*************************************************************************/

int SQLineCmd::add_record(User *u, char const *mask, char const *creator,
		time_t expires, char const *reason)
{
	int deleted = 0;

	SQLine *sq;
	
	sq = (SQLine *)sqlines.create();
	
	sq->mask(mask);
	
	if (!(sq->netban()->flags() & NetBan::bfComplete))
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_SQLINE_MASK_INVALID, mask);
		delete sq;
		
		return -1;
	}
	
	sq->creator(creator);
	sq->reason(reason);
	sq->expires(expires);
	
	/* Checks whether there is an SQLINE that already covers
	 * the one we want to add, and whether there are SQLINEs
	 * that would be covered by this one. The masks AND the
	 * expiry times are used to determine this, because some
	 * SQLINEs may become useful when another one expires.
	 * If so, warn the user in the first case and cleanup
	 * the useless SQLINEs in the second.
	 */
	if (sqlines.size()) 
	{
		SQLine *sq2;
		
		for (int i = 0; i < sqlines.array().size(); ++i)
		{
			sq2 = (SQLine *)sqlines.array()[i];
			
			if (!sq2)
				continue;
			
			if (!stricmp(sq2->mask(), sq->mask())) 
			{
				/* We change the SQLINE expiry time if its current one is less
				 * than the new. This is preferable to be sure we don't change
				 * an important SQLINE by mistsqe. */
				if (sq2->expires() >= sq->expires() || sq2->expires() == 0) 
				{
					if (u) 
						notice_lang(s_OperServ, u,
								OPER_SQLINE_ALREADY_PRESENT, sq->mask());
					delete sq;
					return -1;
				} 
				else 
				{
					sq2->expires(expires);
					if (u)
						notice_lang(s_OperServ, u,
								OPER_SQLINE_CHANGED, sq2->mask());
					delete sq;
					return -2;
				}
			}
			
			if (sq2->netban()->wider(sq->netban()) &&
					(sq2->expires() >= sq->expires() || sq2->expires() == 0))
			{
				if (u)
					notice_lang(s_OperServ, u, OPER_SQLINE_ALREADY_COVERED,
							sq->mask(), sq2->mask());
				delete sq;
				return -1;
			}
			
			if (sq->netban()->wider(sq2->netban()) &&
					(sq2->expires() <= sq->expires() || sq->expires() == 0))
			{
				if (!sqlines.remove(sq2))
				{
					notice_lang(s_OperServ, u, m_msgs[cmDisabled]);
					return -1;
				}
				
				deleted++;
			}
		}
	}
	
	if (u && deleted)
	{
		if (deleted == 1)
			notice_lang(s_OperServ, u, OPER_SQLINE_DELETED_ONE);
		else
			notice_lang(s_OperServ, u, OPER_SQLINE_DELETED_SEVERAL, deleted);
	}
	
	/* We can now check whether the list is full or not. */
	if (sqlines.size() == sqlines.array().max_size()) 
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_SQLINE_TOO_MANY);
		delete sq;
		
		return -1;	
	}
	
	/* Try to add the SQLINE to the list */
	
	int error;
	
	if (!sqlines.add(sq, &error))
	{
		if (u)
		{
			if (error == databases::ArrayCollection::ceName ||
					error == databases::ArrayCollection::ceList)
				notice_lang(s_OperServ, u, OPER_SQLINE_ALREADY_PRESENT,
				 		sq->mask());
			else
				notice_lang(m_service, u, m_msgs[cmDisabled]);
		}
				
		delete sq;
		return -1;
	} else {
		if (u)
			notice_lang(m_service, u, OPER_SQLINE_ADDED, sq->mask());
	}
	
	return deleted;
}

/*************************************************************************/

bool SQLineCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return sqlines.writable();
}

/*************************************************************************/

bool SQLineCmd::clear_records(User *u, container_type &ctn,
		char const *sub)
{
	if (!sqlines.clear())
		return true; /* Partial clear (on paper :P) */
	
	ctn.clear();
	return false;
}

/*************************************************************************/

SQLineCmd::container_type *SQLineCmd::container(User *u)
{
	return &sqlines.array();
}

/*************************************************************************/

bool SQLineCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	return sqlines.remove(rec);
}

/*************************************************************************/

int SQLineCmd::dispatch(User *u, container_type &ctn, char const *sub)
{
	if (!stricmp(sub, "VIEW"))
		return list(u, ctn, sub);
	else
		return ListCmd::dispatch(u, ctn, sub);
}

/*************************************************************************/

bool SQLineCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	SQLine *sq = (SQLine *)rec;
	
	if (!stricmp(sub, "LIST"))
	{
		if (count == 0)
			notice_lang(m_service, u, OPER_SQLINE_LIST_COLHEAD);
	
		notice_lang(m_service, u, OPER_SQLINE_LIST_FORMAT, num,
				sq->mask(), sq->reason());
	} else if (!stricmp(sub, "VIEW")) 
	{
		time_t t = sq->since();
		struct tm tm = *localtime(&t);
		
		char timebuf[32];
		strftime_lang(timebuf, sizeof(timebuf), u,
				STRFTIME_SHORT_DATE_FORMAT, &tm);
		
		char expirebuf[256];
		expire_left(u->na, expirebuf, sizeof(expirebuf), sq->expires());
		
		notice_lang(s_OperServ, u, OPER_SQLINE_VIEW_FORMAT, num, sq->mask(),
				sq->creator(), timebuf, expirebuf, sq->reason());
	}
	
	return true;
}

/*************************************************************************/

/* Manage the SQLINE list. */

static void do_sqline(User *u)
{
	static SQLineCmd sqline_cmd;
	sqline_cmd.execute(u);
}

/*************************************************************************/

SZLineCmd::SZLineCmd() : ListCmd("SZLINE", s_OperServ, szline_msgs, true)
{ 

}

/*************************************************************************/

int SZLineCmd::add(User *u, container_type &ctn, char const *sub)
{
	char *expiry = NULL, *mask = strtok(NULL, " ");
	
	if (mask && *mask == '+')
	{
		expiry = mask;
		mask = strtok(NULL, " ");
	}
	
	char *reason = strtok(NULL, "");
	
	if (!mask || !reason)
	{
		syntax_error(m_service, u, m_name, OPER_SZLINE_ADD_SYNTAX);
		return crStop;
	}
	
	/* Computes the expiration time */
	time_t expires = epona->cd().operserv.expire_szlines;
	
	if (expiry && !duration_scan(expiry, &expires))
	{
		notice_lang(m_service, u, BAD_EXPIRY_TIME);
		return crStop;
	}
	
	if (expires)
		expires += time(0);
	
	/* Validates the mask */
	
	if (strlen(mask) > SZLine::mask_len)
	{
		notice_lang(m_service, u, OPER_SZLINE_MASK_TOO_LONG,
				SZLine::mask_len);
		return crStop;
	}
	
	if (strchr(mask, '!') || strchr(mask, '@'))
	{
		notice_lang(m_service, u, OPER_SZLINE_MASK_ONLY_IPS);
		return crStop;
	}
	
	/* Validates the reason */
	
	if (strlen(reason) > SZLine::reason_len)
	{
		notice_lang(m_service, u, OPER_SZLINE_REASON_TOO_LONG,
				SZLine::reason_len);
		return crStop;
	}
	
	if (add_record(u, mask, u->nick, expires, reason) < 0)
		return crStop;
	
	if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waSZLine)
	{
		char buf[128];
		
		if(!expires) {
			strcpy(buf, "does not expire");
		} else {
			int wall_expiry = expires-time(NULL);
			char const *s = NULL;
			
			if (wall_expiry >= 86400) {
				wall_expiry /= 86400;
				s = "day";
			} else if (wall_expiry >= 3600) { 
				wall_expiry /= 3600;
				s = "hour";
			} else if (wall_expiry >= 60) {
				wall_expiry /= 60;
				s = "minute";
			} else {
				s = "second";
			}
			
			snprintf(buf, sizeof(buf), "expires in %d %s%s", wall_expiry, s,
					(wall_expiry == 1) ? "" : "s");
		}
		
		wallops(s_OperServ, "%s added a SZLINE for %s (%s)",
				u->nick, mask, buf);
	}
	
	return crStop;
}

/*************************************************************************/

int SZLineCmd::add_record(User *u, char const *mask, char const *creator,
		time_t expires, char const *reason)
{
	int deleted = 0;
	
	SZLine *sz;
	
	sz = (SZLine *)szlines.create();
	
	sz->mask(mask);
	
	if (!(sz->netban()->flags() & NetBan::bfComplete))
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_SZLINE_MASK_INVALID, mask);
		delete sz;
		
		return -1;
	}
	
	sz->creator(creator);
	sz->reason(reason);
	sz->expires(expires);
	
	/* Checks whether there is an SZLINE that already covers
	 * the one we want to add, and whether there are SZLINEs
	 * that would be covered by this one. The masks AND the
	 * expiry times are used to determine this, because some
	 * SZLINEs may become useful when another one expires.
	 * If so, warn the user in the first case and cleanup
	 * the useless SZLINEs in the second.
	 */
	if (szlines.size()) 
	{
		SZLine *sz2;
		
		for (int i = 0; i < szlines.array().size(); ++i)
		{
			sz2 = (SZLine *)szlines.array()[i];
			
			if (!sz2)
				continue;
			
			if (!stricmp(sz2->mask(), sz->mask())) 
			{
				/* We change the SZLINE expiry time if its current one is less
				 * than the new. This is preferable to be sure we don't change
				 * an important SZLINE by mistsze. */
				if (sz2->expires() >= sz->expires() || sz2->expires() == 0) 
				{
					if (u) 
						notice_lang(s_OperServ, u,
								OPER_SZLINE_ALREADY_PRESENT, sz->mask());
					delete sz;
					return -1;
				} 
				else 
				{
					sz2->expires(expires);
					if (u)
						notice_lang(s_OperServ, u,
								OPER_SZLINE_CHANGED, sz2->mask());
					delete sz;
					return -2;
				}
			}
			
			if (sz2->netban()->wider(sz->netban()) &&
					(sz2->expires() >= sz->expires() || sz2->expires() == 0))
			{
				if (u)
					notice_lang(s_OperServ, u, OPER_SZLINE_ALREADY_COVERED,
							sz->mask(), sz2->mask());
				delete sz;
				return -1;
			}
			
			if (sz->netban()->wider(sz2->netban()) &&
					(sz2->expires() <= sz->expires() || sz->expires() == 0))
			{
				if (!szlines.remove(sz2))
				{
					notice_lang(s_OperServ, u, m_msgs[cmDisabled]);
					return -1;
				}
				
				deleted++;
			}
		}
	}
	
	if (u && deleted)
	{
		if (deleted == 1)
			notice_lang(s_OperServ, u, OPER_SZLINE_DELETED_ONE);
		else
			notice_lang(s_OperServ, u, OPER_SZLINE_DELETED_SEVERAL, deleted);
	}
	
	/* We can now check whether the list is full or not. */
	if (szlines.size() == szlines.array().max_size()) 
	{
		if (u)
			notice_lang(s_OperServ, u, OPER_SZLINE_TOO_MANY);
		delete sz;
		
		return -1;	
	}
	
	/* Try to add the SZLINE to the list */
	
	int error = 0;
	
	if (!szlines.add(sz, &error))
	{
		if (u)
		{
			if (error == databases::ArrayCollection::ceName ||
					error == databases::ArrayCollection::ceList)
				notice_lang(s_OperServ, u, OPER_SZLINE_ALREADY_PRESENT,
				 		sz->mask());
			else
				notice_lang(m_service, u, m_msgs[cmDisabled]);
		}
				
		delete sz;
		return -1;
	} else {
		if (u)
			notice_lang(m_service, u, OPER_SZLINE_ADDED, sz->mask());
	}
	
	return deleted;
}

/*************************************************************************/

bool SZLineCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return szlines.writable();
}

/*************************************************************************/

bool SZLineCmd::clear_records(User *u, container_type &ctn,
		char const *sub)
{
	if (!szlines.clear())
		return true; /* Partial clear (on paper :P) */
	
	ctn.clear();
	return false;
}

/*************************************************************************/

SZLineCmd::container_type *SZLineCmd::container(User *u)
{
	if (!szlines.available()) 
	{
		notice_lang(s_OperServ, u, OPER_SZLINE_UNSUPPORTED);
		return 0;
	}
	
	return &szlines.array();
}

/*************************************************************************/

bool SZLineCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	return szlines.remove(rec);
}

/*************************************************************************/

int SZLineCmd::dispatch(User *u, container_type &ctn, char const *sub)
{
	if (!stricmp(sub, "VIEW"))
		return list(u, ctn, sub);
	else
		return ListCmd::dispatch(u, ctn, sub);
}

/*************************************************************************/

bool SZLineCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	SZLine *sz = (SZLine *)rec;
	
	if (!stricmp(sub, "LIST"))
	{
		if (count == 0)
			notice_lang(m_service, u, OPER_SZLINE_LIST_COLHEAD);
	
		notice_lang(m_service, u, OPER_SZLINE_LIST_FORMAT, num,
				sz->mask(), sz->reason());
	} else if (!stricmp(sub, "VIEW")) 
	{
		time_t t = sz->since();
		struct tm tm = *localtime(&t);
		
		char timebuf[32];
		strftime_lang(timebuf, sizeof(timebuf), u,
				STRFTIME_SHORT_DATE_FORMAT, &tm);
		
		char expirebuf[256];
		expire_left(u->na, expirebuf, sizeof(expirebuf), sz->expires());
		
		notice_lang(s_OperServ, u, OPER_AKILL_VIEW_FORMAT, num, sz->mask(),
				sz->creator(), timebuf, expirebuf, sz->reason());
	}
	
	return true;
}

/*************************************************************************/

/* Manage the SZLINE list. */

static void do_szline(User *u)
{
	static SZLineCmd szline_cmd;
	szline_cmd.execute(u);
}

/*************************************************************************/

static void do_chanlist(User *u)
{
	uint32_t modes = 0;
	bool nolimit = false;
	char *pattern = strtok(NULL, " "), *range = 0, *opt;
	
	if (pattern && isdigit(*pattern))
	{
		range = pattern;
		pattern = 0;
	}
	
	while ((pattern || range) && (opt = strtok(NULL, " ")))
	{
		if (isdigit(*opt) && !range && !modes)
		{
			range = opt;
			continue;
		}
		
		if (!stricmp(opt, "NOLIMIT") /*&& (pattern || range) implied */)
		{
			nolimit = true;
			continue;
		}
		
		if (!stricmp(opt, "SECRET"))
		{
			modes |= (CMODE_s | CMODE_p);
			continue;
		} 
		
		syntax_error(s_OperServ, u, "CHANLIST", OPER_CHANLIST_SYNTAX);
		return;
	}
	
	User *u2 = 0;
	unsigned long first = 0, last = 0, index = 0;
	int count = 0;
	
	if (pattern && (u2 = finduser(pattern)))
	{
		if (range)
		{
			syntax_error(s_OperServ, u, "CHANLIST", OPER_CHANLIST_SYNTAX);
			return;
		}
		
		if (!u2->chans)
		{
			notice_lang(s_OperServ, u, OPER_CHANLIST_NO_CHANNELS_USER,
					u2->nick);
			return;
		}
		
		struct User::u_chanlist *uc;
		
		notice_lang(s_OperServ, u, OPER_CHANLIST_HEADER_USER, u2->nick);
		
		for (uc = u2->chans; uc; uc = uc->next) 
		{
			++index;
			
			if (modes && !(uc->chan->mode & modes)) 
				continue;

			++count;

			notice_lang(s_OperServ, u, OPER_CHANLIST_ENTRY, uc->chan->name,
					uc->chan->usercount, chan_get_modes(uc->chan, 1, 1),
					(uc->chan->topic ? uc->chan->topic : ""));
		}
	}
	else 
	{
		if (!misc::simple_range(range, &first, &last)) 
		{
			notice_lang(s_OperServ, u, OPER_CHANLIST_INVALID_RANGE, range);
			return;
		}
		
		bool header = false;
		
		for (int i = 0; i < 1024; ++i)
		{
			for (Channel *c = chanlist[i]; c; c = c->next)
			{
				if (modes && !(c->mode & modes))
					continue;
				if (pattern && !misc::irc_match(pattern, c->name))
					continue;
				
				if (++index < first || index > last)
					continue;
				
				if (!header)
				{
					notice_lang(s_OperServ, u,
							(pattern ? OPER_CHANLIST_HEADER_MATCH : 
							OPER_CHANLIST_HEADER), pattern);
					header = true;
				}
				
				if ((++count < epona->cd().operserv.list_max) || nolimit) 
					notice_lang(s_OperServ, u, OPER_CHANLIST_ENTRY, c->name,
							c->usercount, chan_get_modes(c, 1, 1),
							(c->topic ? c->topic : ""));
			}
		}
		
		if (!header)
		{
			notice_lang(s_OperServ, u,
					(pattern ? OPER_CHANLIST_NO_CHANNELS_MATCH :
					OPER_CHANLIST_NO_CHANNELS), pattern);
			return;
		}
	}
	
	if (!u2 && !nolimit && count > epona->cd().operserv.list_max)
		count = epona->cd().operserv.list_max;
	
	if (range && count)
	{
		if (first == last)
			notice_lang(s_OperServ, u,
					OPER_CHANLIST_RESULTS_RANGE_ONE, first);
		else
			notice_lang(s_OperServ, u, OPER_CHANLIST_RESULTS_RANGE,
					first, (first + count - 1));
	} else {
		notice_lang(s_OperServ, u, OPER_CHANLIST_RESULTS, count, index);
	}
}

/*************************************************************************/

static void do_userlist(User *u)
{
	uint32_t modes = 0;
	bool nolimit = false;
	char *pattern = strtok(NULL, " "), *range = 0, *opt;
	
	if (pattern && isdigit(*pattern))
	{
		range = pattern;
		pattern = 0;
	}
	
	while ((pattern || range) && (opt = strtok(NULL, " ")))
	{
		if (isdigit(*opt) && !range && !modes)
		{
			range = opt;
			continue;
		}
		
		if (!stricmp(opt, "NOLIMIT") /*&& (pattern || range) implied */)
		{
			nolimit = true;
			continue;
		}
		
		if (!stricmp(opt, "INVISIBLE"))
		{
			modes |= UMODE_i;
			continue;
		} 
		
		syntax_error(s_OperServ, u, "USERLIST", OPER_USERLIST_SYNTAX);
		return;
	}
	
	Channel *c = 0;
	unsigned long first = 0, last = 0, index = 0;
	int count = 0;
	
	if (pattern && *pattern == '#')
	{	
		if (range)
		{
			syntax_error(s_OperServ, u, "USERLIST", OPER_USERLIST_SYNTAX);
			return;
		}
		
		Channel *c = findchan(pattern);
		if (!c)
		{
			notice_lang(s_OperServ, u, CHAN_X_NOT_IN_USE, pattern);
			return;
		}
		
		Channel::c_userlist *cu;
		
		notice_lang(s_OperServ, u, OPER_USERLIST_HEADER_CHAN, c->name);
		
		for (cu = c->users; cu; cu = cu->next) 
		{
			++index;
			
			if (modes && !(cu->user->mode & modes)) 
				continue;

			++count;

			notice_lang(s_OperServ, u, OPER_USERLIST_ENTRY, cu->user->nick,
					cu->user->username, GetHost(cu->user));
		}
	}
	else 
	{
		if (!misc::simple_range(range, &first, &last)) 
		{
			notice_lang(s_OperServ, u, OPER_USERLIST_INVALID_RANGE, range);
			return;
		}
		
		bool header = false;
		
		for (int i = 0; i < 1024; ++i)
		{
			for (User *u2 = userlist[i]; u2; u2 = u2->next)
			{
				if (modes && !(u2->mode & modes))
					continue;
				
				if (pattern && strcmp(pattern, "*"))
				{
					char mask[BUFSIZE];
					
					snprintf(mask, sizeof(mask), "%s!%s@%s", u2->nick,
							u2->username, u2->host);
					if (!misc::irc_match(pattern, mask)) 
					{
#ifdef HAS_VHOST
						if (!u2->vhost)
#endif
							continue;
#ifdef HAS_VHOST
						snprintf(mask, sizeof(mask), "%s!%s@%s", u2->nick,
							u2->username, u2->vhost);
						
						if (!misc::irc_match(pattern, mask))
							continue;
#endif
					}
				}
				
				if (++index < first || index > last)
					continue;
				
				if (!header)
				{
					notice_lang(s_OperServ, u,
							(pattern ? OPER_USERLIST_HEADER_MATCH : 
							OPER_USERLIST_HEADER), pattern);
					header = true;
				}
				
				if ((++count < epona->cd().operserv.list_max) || nolimit) 
					notice_lang(s_OperServ, u, OPER_USERLIST_ENTRY, u2->nick,
							u2->username, GetHost(u2));
			}
		}
		
		if (!header && pattern)
		{
			notice_lang(s_OperServ, u, OPER_USERLIST_NO_USERS_MATCH, pattern);
			return;
		}
	}
	
	if (!c && !nolimit && count > epona->cd().operserv.list_max)
		count = epona->cd().operserv.list_max;
	
	if (range && count)
	{
		if (first == last)
			notice_lang(s_OperServ, u,
					OPER_USERLIST_RESULTS_RANGE_ONE, first);
		else
			notice_lang(s_OperServ, u, OPER_USERLIST_RESULTS_RANGE,
					first, (first + count - 1));
	} else {
		notice_lang(s_OperServ, u, OPER_USERLIST_RESULTS, count, index);
	}
}

/*************************************************************************/

AdminCmd::AdminCmd() : ListCmd("ADMIN", s_OperServ, admin_msgs, true)
{ 

}

/*************************************************************************/

int AdminCmd::add(User *u, container_type &ctn, char const *sub)
{
	if (ctn.size() == ctn.max_size()) 
	{
		notice_lang(s_OperServ, u, OPER_ADMIN_TOO_MANY);
		return crStop;
	}
	
	char *nick = strtok(NULL, " ");
	
	if (!nick)
	{
		syntax_error(m_service, u, m_name, OPER_ADMIN_ADD_SYNTAX);
		return crStop;
	}
	
	NickAlias *na = aliases.find(nick);
	
	if (!na)
	{
		notice_lang(s_OperServ, u, NICK_X_NOT_REGISTERED, nick);
		return crStop;
	}
	
	if (na->ng()->flags() & NickGroup::gfServicesAdmin)
	{
		notice_lang(s_OperServ, u, OPER_ADMIN_ALREADY_PRESENT, na->nick());
		return crStop;
	}
	
	if (!(na->ng()->flags_add(NickGroup::gfServicesAdmin) & NickGroup::gfServicesAdmin))
	{
		notice_lang(s_OperServ, u, OPER_ADMIN_DISABLED);
		return crStop;
	}
	
	notice_lang(s_OperServ, u, OPER_ADMIN_ADDED, na->nick());
	
	return crStop;
}

/*************************************************************************/

bool AdminCmd::allow_dispatch(User *u, container_type &ctn, char const *sub)
{
	if (!stricmp(sub, "LIST"))
		return is_oper(u);
	else
		return is_services_root(u);
}

/*************************************************************************/

bool AdminCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return groups.writable();
}

/*************************************************************************/

AdminCmd::container_type *AdminCmd::container(User *u)
{
	return &servadmins;
}

/*************************************************************************/

bool AdminCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	NickGroup *ng = (NickGroup *)rec;
	
	if (ng->flags_remove(NickGroup::gfServicesAdmin) & NickGroup::gfServicesAdmin)
		return false;

	return true;
}

/*************************************************************************/

bool AdminCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	NickGroup *ng = (NickGroup *)rec;
	
	if (count == 0)
		notice_lang(m_service, u, OPER_ADMIN_LIST_COLHEAD);
	
	notice_lang(m_service, u, OPER_ADMIN_LIST_FORMAT, num, ng->name());	
	return true;
}

/*************************************************************************/

/* Services admin list viewing/modification. */

static void do_admin(User *u)
{
	static AdminCmd admin_cmd;
	admin_cmd.execute(u);
}

/*************************************************************************/

OperCmd::OperCmd() : ListCmd("OPER", s_OperServ, oper_msgs, true)
{ 

}

/*************************************************************************/

int OperCmd::add(User *u, container_type &ctn, char const *sub)
{
	if (ctn.size() == ctn.max_size()) 
	{
		notice_lang(s_OperServ, u, OPER_OPER_TOO_MANY);
		return crStop;
	}
	
	char *nick = strtok(NULL, " ");
	
	if (!nick)
	{
		syntax_error(m_service, u, m_name, OPER_OPER_ADD_SYNTAX);
		return crStop;
	}
	
	NickAlias *na = aliases.find(nick);
	
	if (!na)
	{
		notice_lang(s_OperServ, u, NICK_X_NOT_REGISTERED, nick);
		return crStop;
	}
	
	if (na->ng()->flags() & NickGroup::gfServicesOper)
	{
		notice_lang(s_OperServ, u, OPER_OPER_ALREADY_PRESENT, na->nick());
		return crStop;
	}
	
	if (!(na->ng()->flags_add(NickGroup::gfServicesOper) & NickGroup::gfServicesOper))
	{
		notice_lang(s_OperServ, u, OPER_OPER_DISABLED);
		return crStop;
	}
	
	notice_lang(s_OperServ, u, OPER_OPER_ADDED, na->nick());
	
	return crStop;
}

/*************************************************************************/

bool OperCmd::allow_dispatch(User *u, container_type &ctn, char const *sub)
{
	if (!stricmp(sub, "LIST"))
		return is_oper(u);
	else
		return is_services_admin(u);
}

/*************************************************************************/

bool OperCmd::changeable(User *u, container_type &ctn,
		char const *sub) const
{
	return groups.writable();
}

/*************************************************************************/

OperCmd::container_type *OperCmd::container(User *u)
{
	return &servopers;
}

/*************************************************************************/

bool OperCmd::del_record(User *u, container_type &ctn, char const *sub,
		databases::Record *rec)
{
	NickGroup *ng = (NickGroup *)rec;
	
	if (ng->flags_remove(NickGroup::gfServicesOper) & NickGroup::gfServicesOper)
		return false;

	return true;
}

/*************************************************************************/

bool OperCmd::list_record(User *u, container_type &ctn, char const *sub,
		uint16_t num, databases::Record const *rec, uint16_t count)
{
	NickGroup *ng = (NickGroup *)rec;
	
	if (count == 0)
		notice_lang(m_service, u, OPER_OPER_LIST_COLHEAD);
	
	notice_lang(m_service, u, OPER_OPER_LIST_FORMAT, num, ng->name());	
	return true;
}

/*************************************************************************/

/* Services operator list viewing/modification. */

static void do_oper(User *u)
{
	static OperCmd oper_cmd;
	oper_cmd.execute(u);
}

/*************************************************************************/

/* Set various Services runtime options. */

static void do_set(User *u)
{
    char *option = strtok(NULL, " ");
    char *setting = strtok(NULL, " ");

    if (!option || !setting) {
	syntax_error(s_OperServ, u, "SET", OPER_SET_SYNTAX);

    } else if (stricmp(option, "IGNORE") == 0) {
	if (stricmp(setting, "on") == 0) {
	    allow_ignore = 1;
	    notice_lang(s_OperServ, u, OPER_SET_IGNORE_ON);
	} else if (stricmp(setting, "off") == 0) {
	    allow_ignore = 0;
	    notice_lang(s_OperServ, u, OPER_SET_IGNORE_OFF);
	} else {
	    notice_lang(s_OperServ, u, OPER_SET_IGNORE_ERROR);
	}

    } else if (stricmp(option, "READONLY") == 0) {
	if (stricmp(setting, "on") == 0) {
	    epona->flags_add(Epona::pfReadOnly);
	    epona->log(NoticeOperServSettings(), "Read-only mode activated");
	    notice_lang(s_OperServ, u, OPER_SET_READONLY_ON);
	} else if (stricmp(setting, "off") == 0) {
	    epona->flags_remove(Epona::pfReadOnly);
	    epona->log(NoticeOperServSettings(), "Read-only mode deactivated");
	    notice_lang(s_OperServ, u, OPER_SET_READONLY_OFF);
	} else {
	    notice_lang(s_OperServ, u, OPER_SET_READONLY_ERROR);
	}

    } else if (stricmp(option, "DEBUG") == 0) {
	if (stricmp(setting, "on") == 0) {
	    epona->debuglevel(1);
	    epona->log(NoticeOperServSettings(), "Debug mode activated");
	    notice_lang(s_OperServ, u, OPER_SET_DEBUG_ON);
	} else if (stricmp(setting, "off") == 0 ||
				(*setting == '0' && atoi(setting) == 0)) {
	    epona->log(NoticeOperServSettings(), "Debug mode deactivated");
	    epona->debuglevel(0);
	    notice_lang(s_OperServ, u, OPER_SET_DEBUG_OFF);
	} else if (isdigit(*setting) && atoi(setting) > 0) {
	    epona->debuglevel(atoi(setting));
	    epona->log(NoticeOperServSettings(), "Debug mode activated (level %d)",
	    		epona->debuglevel());
	    notice_lang(s_OperServ, u, OPER_SET_DEBUG_LEVEL, epona->debuglevel());
	} else {
	    notice_lang(s_OperServ, u, OPER_SET_DEBUG_ERROR);
	}
	
	} else if (stricmp(option, "NOEXPIRE") == 0) {
	if (stricmp(setting, "ON") == 0) {
	    epona->flags_add(Epona::efNoExpire);
	    epona->log(NoticeOperServSettings(), "No expire mode activated");
	    notice_lang(s_OperServ, u, OPER_SET_NOEXPIRE_ON);
	} else if (stricmp(setting, "OFF") == 0) {
	    epona->flags_remove(Epona::efNoExpire);
	    epona->log(NoticeOperServSettings(), "No expire mode deactivated");
	    notice_lang(s_OperServ, u, OPER_SET_NOEXPIRE_OFF);
	} else {
	    notice_lang(s_OperServ, u, OPER_SET_NOEXPIRE_ERROR);
	}

    } else {
	notice_lang(s_OperServ, u, OPER_SET_UNKNOWN_OPTION, option);
    }
}

/*************************************************************************/

static void do_noop(User *u)
{
	char *cmd = strtok(NULL, " ");
    char *server = strtok(NULL, " ");

	if (!cmd || !server) {
		syntax_error(s_OperServ, u, "NOOP", OPER_NOOP_SYNTAX);
	} else if (!stricmp(cmd, "SET")) {
		User *u2;
		char reason[NICKMAX+32];
		
		/* Remove the O:lines */
		s_svsnoop(server, 1);
		
		snprintf(reason, sizeof(reason), "NOOP command used by %s", u->nick);
		if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waNoOp)
			wallops(s_OperServ, "\2%s\2 used NOOP on \2%s\2", u->nick, server);
		notice_lang(s_OperServ, u, OPER_NOOP_SET, server);
		
		/* Kill all the IRCops of the server */
		for (u2 = firstuser(); u2; u2 = nextuser())
			if (is_oper(u2) && u2->server && !stricmp(u2->server->name, server))
#ifdef IRC_BAHAMUT
				send_cmd(epona->cd().services.sname, "SVSKILL %s :%s",
						u2->nick, reason);
#else
				kill_user(s_OperServ, u2->nick, reason);
#endif
	} else if (!stricmp(cmd, "REVOKE")) {
		s_svsnoop(server, 0);
		notice_lang(s_OperServ, u, OPER_NOOP_REVOKE, server);
	} else {
		syntax_error(s_OperServ, u, "NOOP", OPER_NOOP_SYNTAX);
	}
}

/*************************************************************************/

static void do_jupe(User *u)
{
/*#ifdef HAS_SERVER_IP_AKILL
	notice_lang(s_OperServ, u, OPER_JUPE_USE_SZLINE);
#else*/
    char *jserver = strtok(NULL, " ");
    char *reason = strtok(NULL, "");
    char rbuf[256];

    if (!jserver) {
		syntax_error(s_OperServ, u, "JUPE", OPER_JUPE_SYNTAX);
    } else {
    	snprintf(rbuf, sizeof(rbuf), "Juped by %s%s%s", u->nick, reason ? ": " : "", reason ? reason : "");

		if (server_find(jserver)) {
#ifndef IRC_BAHAMUT
			char *av[2];
#endif
			send_cmd(s_OperServ, "SQUIT %s :%s", jserver, rbuf);
#ifndef IRC_BAHAMUT
			av[0] = jserver;
			av[1] = rbuf;
			do_squit(s_OperServ, 2, av);
#endif
		}
		send_cmd(epona->cd().services.sname, "SERVER %s 2 :%s", jserver, rbuf);
		
		if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waJupe)
			wallops(s_OperServ, "\2%s\2 used JUPE on \2%s\2", u->nick, jserver);
    }
/*#endif*/
}

/*************************************************************************/

static void do_raw(User *u)
{

	if (epona->cd().operserv.allow_raw) 
	{
		char *text = strtok(NULL, "");
	
	    if (!text)
		syntax_error(s_OperServ, u, "RAW", OPER_RAW_SYNTAX);
	    else {
			send_cmd(NULL, "%s", text);
			if (epona->cd().operserv.wall & Epona::Directives::OperServ::Wall::waRaw) {
				char *kw = strtok(text, " ");
	    		while (kw && *kw == ':')
	    			kw = strtok(NULL, " ");
				wallops(s_OperServ, "\2%s\2 used RAW command for \2%s\2",
						u->nick, (kw ? kw : "\2non RFC compliant message\2"));
			}
		}
	} else {
		notice_lang(s_OperServ, u, ACCESS_DENIED);
	}

}

/*************************************************************************/

static void do_sync(User *u)
{
	epona->schedule(Epona::sfSync);
	notice_lang(s_OperServ, u, OPER_SYNC_IN_PROGRESS);
}

/*************************************************************************/

static void do_reload(User *u)
{
	epona->schedule(Epona::sfSync || Epona::sfReload);
	if (epona->cd().operserv.wall &
			Epona::Directives::OperServ::Wall::waReload)
		wallops(s_OperServ, "\2%s\2 used RELOAD command", u->nick);
	notice_lang(s_OperServ, u, OPER_RELOAD);
}

/*************************************************************************/

static void do_restart(User *u)
{
	snprintf(epona->exitmsg, epona->exitmsg_size,
			"RESTART command received from %s", u->nick);
	epona->schedule(Epona::sfRestart);
}

/*************************************************************************/

static void do_stop(User *u)
{
	snprintf(epona->exitmsg, epona->exitmsg_size,
			"STOP command received from %s", u->nick);
	epona->schedule(Epona::sfStop);
}

/*************************************************************************/

static void do_fatal(User *u)
{
	epona->fatalf("FATAL command received from %s", u->nick);
}

/*************************************************************************/

/* XXX - this function is broken!! */

static void do_listignore(User *u)
{
    int sent_header = 0;
    IgnoreData *id;
    int i;

    notice(s_OperServ, u->nick, "Command disabled - it's broken.");
    return;
    
    for (i = 0; i < 256; i++) {
	for (id = ignore[i]; id; id = id->next) {
	    if (!sent_header) {
		notice_lang(s_OperServ, u, OPER_IGNORE_LIST);
		sent_header = 1;
	    }
	    notice(s_OperServ, u->nick, "%ld %s", (long) id->time, id->who);
	}
    }
    if (!sent_header)
	notice_lang(s_OperServ, u, OPER_IGNORE_LIST_EMPTY);
}

/*************************************************************************/

#ifdef DEBUG_COMMANDS

static void do_matchwild(User *u)
{
    char *pat = strtok(NULL, " ");
    char *str = strtok(NULL, " ");
    if (pat && str)
	notice(s_OperServ, u->nick, "%d", match_wild(pat, str));
    else
	notice(s_OperServ, u->nick, "Syntax error.");
}

#endif	/* DEBUG_COMMANDS */

/*************************************************************************/

static void do_modules(User *u)
{
	char *cmd = strtok(NULL, " ");
	char *name = strtok(NULL, " ");
	
	if (!cmd) 
	{
		syntax_error(s_OperServ, u, "MODULES", OPER_MODULES_SYNTAX);
	} else if (!epona->cd().operserv.allow_modules &&
			(!stricmp(cmd, "LOAD") || !stricmp(cmd, "UNLOAD"))) {
		notice_lang(s_OperServ, u, OPER_MODULES_DISABLED, cmd);
	} else if (!stricmp(cmd, "LOAD") && name) {
		Module *module = epona->modules().find(name, 0);
		
		if (module) {
			notice_lang(s_OperServ, u, OPER_MODULES_LOAD_ALREADY, name);
		} else if (!epona->modules().load(name)) {
			epona->log(ErrorOperServ(), "Failed to load module %s: %s",
					name, epona->modules().error().c_str());
			notice_lang(s_OperServ, u, OPER_MODULES_LOAD_ERROR, name);
		} else {
			if (epona->cd().operserv.wall &
					Epona::Directives::OperServ::Wall::waModules)
				wallops(s_OperServ, "\2%s\2 used MODULES LOAD on %s",
						u->nick, name);
			notice_lang(s_OperServ, u, OPER_MODULES_LOADED, name);
		}
	} else if (!stricmp(cmd, "UNLOAD") && name) {
		Module *module = epona->modules().find(name);
		
		if (!module) {
			notice_lang(s_OperServ, u, OPER_MODULES_NOT_LOADED, name);
		} else if (!epona->modules().unload(*module)) {
			epona->log(ErrorOperServ(), "Failed to unload module %s: %s",
					name, epona->modules().error().c_str());
			notice_lang(s_OperServ, u, OPER_MODULES_UNLOAD_ERROR, name);
		} else {
			if (epona->cd().operserv.wall &
					Epona::Directives::OperServ::Wall::waModules)
				wallops(s_OperServ, "\2%s\2 used MODULES UNLOAD on %s",
						u->nick, name);
			notice_lang(s_OperServ, u, OPER_MODULES_UNLOADED, name);
		}
	} else if (!stricmp(cmd, "LIST")) {
		notice_lang(s_OperServ, u, OPER_MODULES_HEADER);
		
		int count = 0;
		ModManager::list_type::const_iterator it, end;
		
		for (it = epona->modules().modules().begin(),
				end = epona->modules().modules().end(); it != end; ++it)
		{
			if ((*it)->state() & Module::msLoaded)
			{
				notice_lang(s_OperServ, u, OPER_MODULES_LIST,
						(*it)->name().c_str(), (*it)->references(),
						(*it)->description().c_str());
				++count;
			}
		}
		
		notice_lang(s_OperServ, u, OPER_MODULES_FOOTER, count);
	} else if (!stricmp(cmd, "INFO") && name) {
		Module *module = epona->modules().find(name);
		
		if (!module) 
		{
			notice_lang(s_OperServ, u, OPER_MODULES_NOT_LOADED, name);
			return;
		}
		
		notice_lang(s_OperServ, u, OPER_MODULES_INFO_HEADER,
				module->name().c_str());
		int const ver = module->version();
		notice_lang(s_OperServ, u, OPER_MODULES_INFO_VERSION,
				((ver >> 24) & 0xFF), ((ver >> 16) & 0xFF), (ver & 0xFFFF));
		notice_lang(s_OperServ, u, OPER_MODULES_INFO_DESC,
				module->description().c_str());
		if (!module->author().empty())
			notice_lang(s_OperServ, u, OPER_MODULES_INFO_AUTHOR,
					module->author().c_str());
		if (!module->url().empty())
			notice_lang(s_OperServ, u, OPER_MODULES_INFO_URL,
					module->url().c_str());
		ssize_t const mem = module->memory_usage();
		if (mem != -1)
		{
			if (mem < 1024)
				notice_lang(s_OperServ, u, OPER_MODULES_INFO_MEMORY_USAGE,
						mem);
			else
				notice_lang(s_OperServ, u, OPER_MODULES_INFO_MEMORY_USAGE_K,
						(mem / 1024));
		}
	} else if (!stricmp(cmd, "QUERY") && name) {
		char *query = strtok(NULL, "");
		
		if (!query)
		{
			syntax_error(s_OperServ, u, "MODULES",
					OPER_MODULES_QUERY_SYNTAX);
			return;
		}
		
		Module *module = epona->modules().find(name);
		
		if (!module) 
		{
			notice_lang(s_OperServ, u, OPER_MODULES_NOT_LOADED, name);
			return;
		}
		
		std::string feedback;
		
		if (!module->query(query, feedback))
			notice_lang(s_OperServ, u, OPER_MODULES_QUERY_UNSUPPORTED, name);
		else if (feedback.empty())
			notice_lang(s_OperServ, u, OPER_MODULES_QUERY_UNANSWERED, name);
		else
		{
			notice_lang(s_OperServ, u, OPER_MODULES_QUERY_ANSWERED, name);
			notice_user(s_OperServ, u, "%s", feedback.c_str());
		}
	} else {
		syntax_error(s_OperServ, u, "MODULES", OPER_MODULES_SYNTAX);
	}
}

/*************************************************************************/
