/* OperServ records management.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include <core/directives.h>
#include <core/epona.h>
#include <databases/database.h>
#include <databases/field.h>
#include <databases/record.h>
#include <log/levels.h>
#include <misc/misc.h>
#include <misc/string.h>

#include "epona.h"
#include "netbans.h"
#include "operserv.h"
#include "services.h"

using namespace std;
using namespace core;
using namespace databases;

/*************************************************************************/

Field *Akill::f_mask = 0;
Field *Akill::f_creator = 0;
Field *Akill::f_reason = 0;
Field *Akill::f_since = 0;
Field *Akill::f_expires = 0;

Field *Exception::f_mask = 0;
Field *Exception::f_limit = 0;
Field *Exception::f_creator = 0;
Field *Exception::f_reason = 0;
Field *Exception::f_since = 0;
Field *Exception::f_expires = 0;

Field *News::f_type = 0;
Field *News::f_text = 0;
Field *News::f_creator = 0;
Field *News::f_since = 0;

Field *SGLine::f_mask = 0;
Field *SGLine::f_creator = 0;
Field *SGLine::f_reason = 0;
Field *SGLine::f_since = 0;
Field *SGLine::f_expires = 0;

Field *SQLine::f_mask = 0;
Field *SQLine::f_creator = 0;
Field *SQLine::f_reason = 0;
Field *SQLine::f_since = 0;
Field *SQLine::f_expires = 0;

Field *Stats::f_maxusers = 0;
Field *Stats::f_maxusers_time = 0;

Field *SZLine::f_mask = 0;
Field *SZLine::f_creator = 0;
Field *SZLine::f_reason = 0;
Field *SZLine::f_since = 0;
Field *SZLine::f_expires = 0;

/*************************************************************************/

Akill::Akill(UmaskNetBan *nb) : NBInfo()
{
	m_nb = nb;
}

/*************************************************************************/

Akill::Akill(UmaskNetBan *nb, identifier_type id) : NBInfo(id)
{
	m_nb = nb;
}

/*************************************************************************/

bool Akill::change_creator(char const *value)
{
	return akills.change_s(this, f_creator, value);
}

/*************************************************************************/

bool Akill::change_expires(time_t value)
{
	return akills.change_ut(this, f_expires, value);
}

/*************************************************************************/

bool Akill::change_reason(char const *value)
{
	return akills.change_s(this, f_reason, value);
}

/*************************************************************************/

bool Akill::change_since(time_t value)
{
	return akills.change_ut(this, f_since, value);
}

/*************************************************************************/

char const *Akill::get_s(Field const *field) const
{
	if (field == f_mask)
		return mask();
	else if (field == f_creator)
		return creator();
	else if (field == f_reason)
		return reason();
	else
		return Record::get_s(field);
}

/*************************************************************************/

time_t Akill::get_ut(Field const *field) const
{
	if (field == f_since)
		return since();
	else if (field == f_expires)
		return expires();
	else
		return Record::get_ut(field);
}

/*************************************************************************/

char const *Akill::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_s(field, value);
	
	if (field == f_mask)
		return mask(value);
	else if (field == f_creator)
		return creator(value);
	else if (field == f_reason)
		return reason(value);
	else
		return Record::set_s(field, value);
}

/*************************************************************************/

time_t Akill::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ut(field, value);
	
	if (field == f_since)
		return since(value);
	else if (field == f_expires)
		return expires(value);
	else
		return Record::set_ut(field, value);
}

/*************************************************************************/

AkillList::AkillList() : databases::ArrayCollection(true), m_banner(0)
{ 
	m_banner = (UmaskNetBanner *)netforce.ban_supports(UmaskNetBanner::IID);
}

/*************************************************************************/

bool AkillList::add_list(databases::Record *record)
{
	Akill *ak = (Akill *)record;
	
	if (!m_banner->add(ak->netban()))
		return false;
	if (!(status() & csInitial))
		ak->netban()->apply();
	
	return ArrayCollection::add_list(record);
}

/*************************************************************************/

Record *AkillList::create(Record::identifier_type id) const
{
	if (!m_banner)
		return 0;
	
	UmaskNetBan *nb = (UmaskNetBan *)m_banner->create();
	
	if (!nb)
		return 0;
	
	if (id)
		return new Akill(nb, id);
	else
		return new Akill(nb);
}

/*************************************************************************/

void AkillList::clear_list()
{
	iterator it, end;
	
	for (it = this->begin(), end = this->end(); it != end; ++it)
	{
		Akill *ak = (Akill *)(*it);
		m_banner->remove(ak->netban());
	}
	
	ArrayCollection::clear_list();
}

/*************************************************************************/

void AkillList::expire()
{
	if (!writable())
		return;
	
	time_t now = time(0);
	
	Akill *ak;
	iterator it, end;
	
	for (it = begin(), end = this->end(); it != end; )
	{
		ak = (Akill *)*it++;
		
		if (!ak->expires() || ak->expires() > now)
			continue;
		
		if (epona->cd().operserv.wall &
				Epona::Directives::OperServ::Wall::waAkillExpire)
		    wallops(s_OperServ, "AKILL on %s has expired", ak->mask());
		
		/* If this returns false, ensure we've not switched to
		 * read-only before continuing. */
		if (!akills.remove(ak) && !akills.writable())
			return;
	}
}

/*************************************************************************/

void AkillList::remove_list(databases::Record *record)
{
	Akill *ak = (Akill *)record;
	
	m_banner->remove(ak->netban());
	ArrayCollection::remove_list(record);
}

/*************************************************************************/

void AkillList::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("operserv_akills");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	Akill::f_mask = db->field_create("mask", 1, Field::ftString,
			Akill::mask_len, 0, NULL, 1);
	Akill::f_creator = db->field_create("creator", 1, Field::ftString,
			Akill::creator_len, 0, NULL, 2);
	Akill::f_reason = db->field_create("reason", 1, Field::ftString,
			Akill::reason_len, 0, NULL, 3);
	Akill::f_since = db->field_create("since", 1, Field::ftUnixTime,
			0, 0, "0", 4);
	Akill::f_expires = db->field_create("expires", 1, Field::ftUnixTime,
			0, 0, "0", 5);
	
	if (!db->field_add(Akill::f_mask) || 
			!db->field_add(Akill::f_creator) ||
			!db->field_add(Akill::f_reason) ||
			!db->field_add(Akill::f_since) ||
			!db->field_add(Akill::f_expires))
		epona->fatal("Can't add fields to database operserv_akills");
	
	this->db(db);
}

/*************************************************************************/

Exception::Exception() : NameRecord(), m_mask(0), m_limit(0),
		m_creator(0), m_reason(0), m_since(time(0)), m_expires(0)
{
}

/*************************************************************************/

Exception::Exception(identifier_type id) : NameRecord(id), m_mask(0),
		m_limit(0), m_creator(0), m_reason(0), m_since(time(0)), m_expires(0)
{	
}

/*************************************************************************/

Exception::Exception(Exception const &ex) : NameRecord(ex), m_mask(ex.m_mask),
		m_limit(ex.m_limit), m_creator(ex.m_creator), m_reason(ex.m_reason),
		m_since(ex.m_since), m_expires(ex.m_expires)
{
}

/*************************************************************************/

Exception &Exception::operator=(Exception const &right)
{
	if (this != &right)
	{
		NameRecord::operator=(right);
		
		m_mask = right.m_mask;
		m_limit = right.m_limit;
		m_creator = right.m_creator;
		m_reason = right.m_reason;
		m_since = right.m_since;
		m_expires = right.m_expires;
	}
	
	return *this;
}

/*************************************************************************/

char const *Exception::creator(char const *creator)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL creator. */
		if (!creator)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_creator;
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL creator... =) */
			return m_creator;
		}
	}
	
	if (m_creator.compare(creator, creator_len) != 0)
	{
		misc::cstring tcreator(creator, creator_len);
		
		if (!exceptions.change_s(this, f_creator, tcreator))
			return m_creator;
		m_creator.swap(tcreator);
	}
	
	return m_creator;
}

/*************************************************************************/

time_t Exception::expires(time_t expires)
{
	if (db_status() & dbsUpdating)
	{
		/* Reject exceptions with expiration time in the past. They have no
		 * sense. */
		if (expires != 0 && (time(0) >= expires))
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_expires;
			/* Else accept the value anyway, but make end_update() fail
			 * so that the record is deleted. */
			db_status_add(dbsUpdateError);
			epona->log(InfoOperServList(),
					"Ignoring exception %s (#" RECORD_FORMAT ") with"
					" expiration time in the past",
					(name() ? name() : "<unknown>"), id());
		}
	}
	
	if (expires != m_expires) 
	{
		if (!exceptions.change_ut(this, f_expires, expires))
			return m_expires;
		m_expires = expires;
	}
	
	return m_expires;
}

/*************************************************************************/

char const *Exception::get_s(Field const *field) const
{
	if (field == f_mask)
		return mask();
	else if (field == f_creator)
		return creator();
	else if (field == f_reason)
		return reason();
	else
		return NameRecord::get_s(field);
}

/*************************************************************************/

uint32_t Exception::get_ui(Field const *field) const
{
	if (field == f_limit)
		return limit();
	else
		return NameRecord::get_ui(field);
}

/*************************************************************************/

time_t Exception::get_ut(Field const *field) const
{
	if (field == f_since)
		return since();
	else if (field == f_expires)
		return expires();
	else
		return NameRecord::get_ut(field);
}

/*************************************************************************/

uint16_t Exception::limit(uint16_t limit)
{
	if (db_status() & dbsUpdating)
	{
		/* There's a limit on the maximum limit ;)
		 * If the limit is greater than what's allowed, then set it
		 * to the maximum allowed. Don't consider this as an error
		 * since the maximum is a configuration option. */
		if (limit > epona->cd().session.max_limit)
			limit = epona->cd().session.max_limit;
	}
	
	if (limit != m_limit) 
	{
		if (!exceptions.change_ui(this, f_limit, limit))
			return m_limit;
		m_limit = limit;
	}
	
	return m_limit;
}

/*************************************************************************/

char const *Exception::mask(char const *mask)
{
	/* Don't allow mask changes. */
	if (m_mask)
		return m_mask;
	
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL masks. */
		if (!mask)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_mask;
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL mask... =) */
			return m_mask;
		}
	}
	
	/* Limit the length of masks */
	m_mask.assign(mask, mask_len);
	
	return m_mask;
}

/*************************************************************************/

size_t Exception::memory_usage() const
{
	size_t result = NameRecord::memory_usage();

	result += sizeof(Exception);
	result += strlen(m_mask) + 1;
	result += strlen(m_creator) + 1;
	result += strlen(m_reason) + 1;

	return result;
}

/*************************************************************************/

char const *Exception::reason(char const *reason)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL reasons. */
		if (!reason)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_reason;
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL reason... =) */
			return m_reason;
		}
	}
	
	if (m_reason.compare(reason, reason_len) != 0)
	{
		misc::cstring treason(reason, reason_len);
		
		if (!exceptions.change_s(this, f_reason, treason))
			return m_reason;
		m_reason.swap(treason);
	}
	
	return m_reason;
}

/*************************************************************************/

char const *Exception::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_s(field, value);
	
	if (field == f_mask)
		return mask(value);
	else if (field == f_creator)
		return creator(value);
	else if (field == f_reason)
		return reason(value);
	else
		return NameRecord::set_s(field, value);
}

/*************************************************************************/

uint32_t Exception::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ui(field, value);
	
	if (field == f_limit)
		return limit(value);
	else
		return NameRecord::set_ui(field, value);
}

/*************************************************************************/

time_t Exception::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return NameRecord::set_ut(field, value);
	
	if (field == f_since)
		return since(value);
	else if (field == f_expires)
		return expires(value);
	else
		return NameRecord::set_ut(field, value);
}

/*************************************************************************/

time_t Exception::since(time_t since)
{	
	if (db_status() & dbsUpdating)
	{
		/* We don't allow time in the future. */
		if (since > time(0))
		{
			/* If it's already a member, just return the old time. */
			if (db_status() & dbsMember)
				return m_since;
			/* Else, assign a correct time. */
			since = time(0);
		}
	}
	
	if (since != m_since)
	{
		if (!exceptions.change_ut(this, f_since, since))
			return m_since;
		m_since = since;
	}
	
	return m_since;
}

/*************************************************************************/

Record *ExceptionList::create(Record::identifier_type id) const
{
	if (id)
		return new Exception(id);
	else
		return new Exception;
}

/*************************************************************************/

bool ExceptionList::expire(Exception *ex)
{
	if (epona->cd().operserv.wall &
			Epona::Directives::OperServ::Wall::waExceptionExpire)
		wallops(s_OperServ,
				"Session limit exception for %s has expired.", ex->mask());
		
	return exceptions.remove(ex);
}

/*************************************************************************/

Exception *ExceptionList::find_match(char const *host)
{	
	Exception *ex;
	const_iterator it, last;

	time_t now = time(0);
	
	for (it = begin(), last = end(); it != last;)
	{
		ex = (Exception *)*it++;
		
		if (db()->writable() && ex->expires() != 0 &&
				ex->expires() <= now && expire(ex))
			continue;
		
		if (misc::match(ex->mask(), host))
			return ex;
	}
	
	return 0;
}

/*************************************************************************/

void ExceptionList::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("operserv_exceptions");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	Exception::f_mask = db->field_create("mask", 1, Field::ftString,
			Exception::mask_len, 0, NULL, 1);
	Exception::f_limit = db->field_create("limit", 1, Field::ftUnsigned,
			2, 0, "0", 2);
	Exception::f_creator = db->field_create("creator", 1, Field::ftString,
			Exception::creator_len, 0, NULL, 3);
	Exception::f_reason = db->field_create("reason", 1, Field::ftString,
			Exception::reason_len, 0, NULL, 4);
	Exception::f_since = db->field_create("since", 1, Field::ftUnixTime,
			0, 0, "0", 5);
	Exception::f_expires = db->field_create("expires", 1, Field::ftUnixTime,
			0, 0, "0", 6);
	
	if (!db->field_add(Exception::f_mask) || 
			!db->field_add(Exception::f_limit) ||
			!db->field_add(Exception::f_creator) ||
			!db->field_add(Exception::f_reason) ||
			!db->field_add(Exception::f_since) ||
			!db->field_add(Exception::f_expires))
		epona->fatal("Can't add fields to database operserv_exceptions");
	
	this->db(db);
}

/*************************************************************************/

NBInfo::NBInfo() : NameRecord(), m_nb(0), m_expires(0)
{
}

/*************************************************************************/

NBInfo::NBInfo(identifier_type id) : NameRecord(id), m_nb(0), m_expires(0)
{
}

/*************************************************************************/

NBInfo::~NBInfo()
{
	if (m_nb)
		delete m_nb;
}

/*************************************************************************/

char const *NBInfo::creator(char const *creator)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL creator. */
		if (!creator)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_nb->creator();
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL creator... =) */
			return 0;
		}
	}
	
	if (strncmp(creator, m_nb->creator(), creator_len))
	{
		misc::cstring old_creator(m_nb->creator()),
				tcreator(creator, creator_len);
		
		if (!m_nb->creator(tcreator))
			return m_nb->creator();
		
		if (!change_creator(tcreator))
		{
			m_nb->creator(old_creator);
			return m_nb->creator();
		}
	}
	
	return m_nb->creator();
}

/*************************************************************************/

time_t NBInfo::expires(time_t expires)
{
	if (db_status() & dbsUpdating)
	{
		/* Reject exceptions with expiration time in the past. They have no
		 * sense. */
		if (expires != 0 && (time(0) >= expires))
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_expires;
			/* Else accept the value anyway, but make end_update() fail
			 * so that the record is deleted. */
			db_status_add(dbsUpdateError);
			epona->log(InfoOperServList(),
					"Ignoring network ban %s (#" RECORD_FORMAT ") with"
					" expiration time in the past",
					(name() ? name() : "<unknown>"), id());
		}
	}
	
	if (expires != m_expires) 
	{
		if (!change_expires(expires))
			return m_expires;
		m_expires = expires;
	}

	return m_expires;
}

/*************************************************************************/

char const *NBInfo::mask(char const *mask)
{
	/* Don't allow mask changes. */
	if (m_nb->flags() & NetBan::bfComplete)
		return m_nb->mask();
	
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL masks. */
		if (!mask)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_nb->mask();
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL mask... =) */
			return 0;
		}
	}
	
	misc::cstring tmask(mask, mask_len);
	
	if (!m_nb->mask(tmask))
	{
		if (db_status() & dbsUpdating)
			db_status_add(dbsUpdateError);
		return mask; /* Return mask, since the record is going
		              * to be deleted anyway. */
	}
	
	return m_nb->mask();
}

/*************************************************************************/

bool NBInfo::matches(char const *pattern) const
{
	if (!name())
		return false;
	
	if (*pattern == '#')
	{
		uint16_t pcrc = strtoul(++pattern, 0, 16);
		return (pcrc == misc::crc16_str(name()));
	}
	
	return NameRecord::matches(pattern);
}

/*************************************************************************/

size_t NBInfo::memory_usage() const
{
	size_t result = NameRecord::memory_usage();

	result += sizeof(NBInfo);
	result += m_nb->memory_usage();
	
	return result;
}

/*************************************************************************/

char const *NBInfo::reason(char const *reason)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL reasons. */
		if (!reason)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_nb->reason();
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL reason... =) */
			return 0;
		}
	}
	
	if (strncmp(reason, m_nb->reason(), reason_len))
	{
		misc::cstring old_reason(m_nb->reason()),
				treason(reason, reason_len);
		
		if (!m_nb->reason(treason))
			return m_nb->reason();
		
		if (!change_reason(treason))
		{
			m_nb->reason(old_reason);
			return m_nb->reason();
		}
	}
	
	return m_nb->reason();
}

/*************************************************************************/

time_t NBInfo::since(time_t since)
{
	if (db_status() & dbsUpdating)
	{
		/* We don't allow time in the future. */
		if (since > time(0))
		{
			/* If it's already a member, just return the old time. */
			if (db_status() & dbsMember)
				return m_nb->creation_time();
			/* Else, assign a correct time. */
			since = time(0);
		}
	}
	
	if (since != m_nb->creation_time())
	{
		time_t old_since = m_nb->creation_time();
		
		if (!m_nb->creation_time(since))
			return m_nb->creation_time();
		
		if (!change_since(m_nb->creation_time()))
		{
			m_nb->creation_time(old_since);
			return m_nb->creation_time();
		}
	}
	
	return m_nb->creation_time();
}

/*************************************************************************/

News::News() : Record(), m_type(0), m_text(0), m_creator(0), m_since(time(0))
{
}

/*************************************************************************/

News::News(identifier_type id) : Record(id), m_type(0), m_text(0),
		m_creator(0), m_since(time(0))
{
}

/*************************************************************************/

News::News(News const &n) : Record(n), m_type(n.m_type), m_text(n.m_text),
		m_creator(n.m_creator), m_since(n.m_since)
{
}

/*************************************************************************/

News &News::operator=(News const &right)
{
	if (this != &right)
	{
		Record::operator=(right);
		
		m_type = right.m_type;
		m_text = right.m_text;
		m_creator = right.m_creator;
		m_since = right.m_since;
	}
	
	return *this;
}

/*************************************************************************/

char const *News::creator(char const *creator)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL creator. */
		if (!creator)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_creator;
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Returns the NULL creator... =) */
			return m_creator;
		}
	}
	
	if (m_creator.compare(creator, creator_len) != 0)
	{
		misc::cstring tcreator(creator, creator_len);
		
		if (!news.change_s(this, f_creator, tcreator))
			return m_creator;
		m_creator.swap(tcreator);
	}
	
	return m_creator;
}

/*************************************************************************/

char const *News::get_s(Field const *field) const
{
	if (field == f_text)
		return text();
	else if (field == f_creator)
		return creator();
	else
		return Record::get_s(field);
}

/*************************************************************************/

uint32_t News::get_ui(Field const *field) const
{
	if (field == f_type)
		return type();
	else
		return Record::get_ui(field);
}

/*************************************************************************/

time_t News::get_ut(Field const *field) const
{
	if (field == f_since)
		return since();
	else
		return Record::get_ut(field);
}

/*************************************************************************/

size_t News::memory_usage() const
{
	size_t result = Record::memory_usage();

	result += sizeof(News);
	result += strlen(m_text) + 1;
	result += strlen(m_creator) + 1;

	return result;
}

/*************************************************************************/

char const *News::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_s(field, value);
	
	if (field == f_text)
		return text(value);
	else if (field == f_creator)
		return creator(value);
	else
		return Record::set_s(field, value);
}

/*************************************************************************/

uint32_t News::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ui(field, value);
	
	if (field == f_type)
		return type(value);
	else
		return Record::set_ui(field, value);
}

/*************************************************************************/

time_t News::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ut(field, value);
	
	if (field == f_since)
		return since(value);
	else
		return Record::set_ut(field, value);
}

/*************************************************************************/

time_t News::since(time_t since)
{
	if (db_status() & dbsUpdating)
	{
		/* We don't allow time in the future. */
		if (since > time(0))
		{
			/* If it's already a member, just return the old time. */
			if (db_status() & dbsMember)
				return m_since;
			/* Else, assign a correct time. */
			since = time(0);
		}
	}
				
	
	if (since != m_since)
	{
		if (!news.change_ut(this, f_since, since))
			return m_since;
		m_since = since;
	}
	
	return m_since;
}

/*************************************************************************/

char const *News::text(char const *text)
{
	if (db_status() & dbsUpdating)
	{
		/* Don't allow NULL texts. */
		if (!text)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_text;
			/* Else make end_update() fail. */
			db_status_add(dbsUpdateError);
			/* Return the NULL text... =) */
			return m_text;
		}
	}
	
	if (m_text.compare(text, text_len) != 0)
	{
		misc::cstring ttext(text, text_len);
		
		if (!news.change_s(this, f_text, ttext))
			return m_text;
		m_text.swap(ttext);
	}
	
	return m_text;
}

/*************************************************************************/

uint8_t News::type(uint8_t type)
{
	if (db_status() & dbsUpdating)
	{
		/* Reject news with invalid type. */
		if (type < ntFirst || type > ntLast)
		{
			/* If the record is already a member, just keep the old value. */
			if (db_status() & dbsMember)
				return m_type;
			/* Else accept the value anyway, but make end_update() fail
			 * so that the record is deleted. */
			db_status_add(dbsUpdateError);
		}
	}
	
	if (type != m_type) 
	{
		if (!news.change_ui(this, f_type, type))
			return m_type;
		m_type = type;
	}
	
	return m_type;
}

/*************************************************************************/

Record *NewsList::create(Record::identifier_type id) const
{
	if (id)
		return new News(id);
	else
		return new News;
}

/*************************************************************************/

void NewsList::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("operserv_news");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	News::f_type = db->field_create("type", 1, Field::ftUnsigned,
			1, 0, "1", 1);
	News::f_text = db->field_create("text", 1, Field::ftString,
			News::text_len, 0, NULL, 3);
	News::f_creator = db->field_create("creator", 1, Field::ftString,
			News::creator_len, 0, NULL, 2);
	News::f_since = db->field_create("since", 1, Field::ftUnixTime,
			0, 0, "0", 4);
	
	if (!db->field_add(News::f_type) || 
			!db->field_add(News::f_text) ||
			!db->field_add(News::f_creator) ||
			!db->field_add(News::f_since))
		epona->fatal("Can't add fields to database operserv_news");
	
	this->db(db);
}

/*************************************************************************/

SGLine::SGLine(RealNetBan *nb) : NBInfo()
{
	m_nb = nb;
}

/*************************************************************************/

SGLine::SGLine(RealNetBan *nb, identifier_type id) : NBInfo(id)
{
	m_nb = nb;
}

/*************************************************************************/

bool SGLine::change_creator(char const *value)
{
	return szlines.change_s(this, f_creator, value);
}

/*************************************************************************/

bool SGLine::change_expires(time_t value)
{
	return szlines.change_ut(this, f_expires, value);
}

/*************************************************************************/

bool SGLine::change_reason(char const *value)
{
	return szlines.change_s(this, f_reason, value);
}

/*************************************************************************/

bool SGLine::change_since(time_t value)
{
	return szlines.change_ut(this, f_since, value);
}

/*************************************************************************/

char const *SGLine::get_s(Field const *field) const
{
	if (field == f_mask)
		return mask();
	else if (field == f_creator)
		return creator();
	else if (field == f_reason)
		return reason();
	else
		return Record::get_s(field);
}

/*************************************************************************/

time_t SGLine::get_ut(Field const *field) const
{
	if (field == f_since)
		return since();
	else if (field == f_expires)
		return expires();
	else
		return Record::get_ut(field);
}

/*************************************************************************/

char const *SGLine::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_s(field, value);
	
	if (field == f_mask)
		return mask(value);
	else if (field == f_creator)
		return creator(value);
	else if (field == f_reason)
		return reason(value);
	else
		return Record::set_s(field, value);
}

/*************************************************************************/

time_t SGLine::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ut(field, value);
	
	if (field == f_since)
		return since(value);
	else if (field == f_expires)
		return expires(value);
	else
		return Record::set_ut(field, value);
}

/*************************************************************************/

SGLineList::SGLineList() : databases::ArrayCollection(true), m_banner(0)
{ 
	m_banner = (RealNetBanner *)netforce.ban_supports(RealNetBanner::IID);
}

/*************************************************************************/

bool SGLineList::add_list(databases::Record *record)
{
	SGLine *sg = (SGLine *)record;
	
	if (!m_banner->add(sg->netban()))
		return false;
	
	if (!(status() & csInitial))
		sg->netban()->apply();
	
	return ArrayCollection::add_list(record);
}

/*************************************************************************/

Record *SGLineList::create(Record::identifier_type id) const
{
	if (!m_banner)
		return 0;
	
	RealNetBan *nb = (RealNetBan *)m_banner->create();
	
	if (!nb)
		return 0;
	
	if (id)
		return new SGLine(nb, id);
	else
		return new SGLine(nb);
}

/*************************************************************************/

void SGLineList::clear_list()
{
	iterator it, end;
	
	for (it = this->begin(), end = this->end(); it != end; ++it)
	{
		SGLine *sg = (SGLine *)(*it);
		m_banner->remove(sg->netban());
	}
	
	ArrayCollection::clear_list();
}

/*************************************************************************/

void SGLineList::expire()
{
	if (!writable())
		return;
	
	time_t now = time(0);
	
	SGLine *sg;
	iterator it, end;
	
	for (it = begin(), end = this->end(); it != end; )
	{
		sg = (SGLine *)*it++;
		
		if (!sg->expires() || sg->expires() > now)
			continue;
		
		if (epona->cd().operserv.wall &
				Epona::Directives::OperServ::Wall::waSGLineExpire)
		    wallops(s_OperServ, "SGLINE on %s has expired", sg->mask());
		
		/* If this returns false, ensure we've not switched to
		 * read-only before continuing. */
		if (!sglines.remove(sg) && !sglines.writable())
			return;
	}
}

/*************************************************************************/

void SGLineList::remove_list(databases::Record *record)
{
	SGLine *sg = (SGLine *)record;
	
	m_banner->remove(sg->netban());
	ArrayCollection::remove_list(record);
}

/*************************************************************************/

void SGLineList::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("operserv_sglines");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	SGLine::f_mask = db->field_create("mask", 1, Field::ftString,
			SGLine::mask_len, 0, NULL, 1);
	SGLine::f_creator = db->field_create("creator", 1, Field::ftString,
			SGLine::creator_len, 0, NULL, 2);
	SGLine::f_reason = db->field_create("reason", 1, Field::ftString,
			SGLine::reason_len, 0, NULL, 3);
	SGLine::f_since = db->field_create("since", 1, Field::ftUnixTime,
			0, 0, "0", 4);
	SGLine::f_expires = db->field_create("expires", 1, Field::ftUnixTime,
			0, 0, "0", 5);
	
	if (!db->field_add(SGLine::f_mask) || 
			!db->field_add(SGLine::f_creator) ||
			!db->field_add(SGLine::f_reason) ||
			!db->field_add(SGLine::f_since) ||
			!db->field_add(SGLine::f_expires))
		epona->fatal("Can't add fields to database operserv_sglines");
	
	this->db(db);
}

/*************************************************************************/

SQLine::SQLine(NickChanNetBan *nb) : NBInfo()
{
	m_nb = nb;
}

/*************************************************************************/

SQLine::SQLine(NickChanNetBan *nb, identifier_type id) : NBInfo(id)
{
	m_nb = nb;
}

/*************************************************************************/

bool SQLine::change_creator(char const *value)
{
	return sqlines.change_s(this, f_creator, value);
}

/*************************************************************************/

bool SQLine::change_expires(time_t value)
{
	return sqlines.change_ut(this, f_expires, value);
}

/*************************************************************************/

bool SQLine::change_reason(char const *value)
{
	return sqlines.change_s(this, f_reason, value);
}

/*************************************************************************/

bool SQLine::change_since(time_t value)
{
	return sqlines.change_ut(this, f_since, value);
}

/*************************************************************************/

char const *SQLine::get_s(Field const *field) const
{
	if (field == f_mask)
		return mask();
	else if (field == f_creator)
		return creator();
	else if (field == f_reason)
		return reason();
	else
		return Record::get_s(field);
}

/*************************************************************************/

time_t SQLine::get_ut(Field const *field) const
{
	if (field == f_since)
		return since();
	else if (field == f_expires)
		return expires();
	else
		return Record::get_ut(field);
}

/*************************************************************************/

char const *SQLine::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_s(field, value);
	
	if (field == f_mask)
		return mask(value);
	else if (field == f_creator)
		return creator(value);
	else if (field == f_reason)
		return reason(value);
	else
		return Record::set_s(field, value);
}

/*************************************************************************/

time_t SQLine::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ut(field, value);
	
	if (field == f_since)
		return since(value);
	else if (field == f_expires)
		return expires(value);
	else
		return Record::set_ut(field, value);
}

/*************************************************************************/

SQLineList::SQLineList() : databases::ArrayCollection(true), m_nbanner(0),
		m_cbanner(0)
{ 
	m_nbanner = (NickNetBanner *)netforce.ban_supports(NickNetBanner::IID);
	m_cbanner = (ChanNameNetBanner *)
			netforce.ban_supports(ChanNameNetBanner::IID);
}

/*************************************************************************/

bool SQLineList::add_list(databases::Record *record)
{
	SQLine *sq = (SQLine *)record;
	
	if ((*sq->mask() != '#') ? !m_nbanner->add(sq->netban()) : 
				!m_cbanner->add(sq->netban()))
		return false;
	if (!(status() & csInitial))
		sq->netban()->apply();
	
	return ArrayCollection::add_list(record);
}

/*************************************************************************/

Record *SQLineList::create(Record::identifier_type id) const
{
	NickChanNetBan *nb = (NickChanNetBan *)m_nbanner->create();
	
	if (!nb)
		return 0;
	
	if (id)
		return new SQLine(nb, id);
	else
		return new SQLine(nb);
}

/*************************************************************************/

void SQLineList::clear_list()
{
	iterator it, end;
	
	for (it = this->begin(), end = this->end(); it != end; ++it)
	{
		SQLine *sq = (SQLine *)(*it);
		if (*sq->mask() != '#')
			m_nbanner->remove(sq->netban());
		else
			m_cbanner->remove(sq->netban());
	}
	
	ArrayCollection::clear_list();
}

/*************************************************************************/

void SQLineList::expire()
{
	if (!writable())
		return;
	
	time_t now = time(0);
	
	SQLine *sq;
	iterator it, end;
	
	for (it = begin(), end = this->end(); it != end; )
	{
		sq = (SQLine *)*it++;
		
		if (!sq->expires() || sq->expires() > now)
			continue;
		
		if (epona->cd().operserv.wall &
				Epona::Directives::OperServ::Wall::waSQLineExpire)
		    wallops(s_OperServ, "SQLINE on %s has expired", sq->mask());
		
		/* If this returns false, ensure we've not switched to
		 * read-only before continuing. */
		if (!sqlines.remove(sq) && !sqlines.writable())
			return;
	}
}

/*************************************************************************/

void SQLineList::remove_list(databases::Record *record)
{
	SQLine *sq = (SQLine *)record;
	
	if (*sq->mask() != '#')
		m_nbanner->remove(sq->netban());
	else
		m_cbanner->remove(sq->netban());
	ArrayCollection::remove_list(record);
}

/*************************************************************************/

void SQLineList::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("operserv_sqlines");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	SQLine::f_mask = db->field_create("mask", 1, Field::ftString,
			SQLine::mask_len, 0, NULL, 1);
	SQLine::f_creator = db->field_create("creator", 1, Field::ftString,
			SQLine::creator_len, 0, NULL, 2);
	SQLine::f_reason = db->field_create("reason", 1, Field::ftString,
			SQLine::reason_len, 0, NULL, 3);
	SQLine::f_since = db->field_create("since", 1, Field::ftUnixTime,
			0, 0, "0", 4);
	SQLine::f_expires = db->field_create("expires", 1, Field::ftUnixTime,
			0, 0, "0", 5);
	
	if (!db->field_add(SQLine::f_mask) || 
			!db->field_add(SQLine::f_creator) ||
			!db->field_add(SQLine::f_reason) ||
			!db->field_add(SQLine::f_since) ||
			!db->field_add(SQLine::f_expires))
		epona->fatal("Can't add fields to database operserv_sqlines");
	
	this->db(db);
}

/*************************************************************************/

Stats::Stats() : Record(), m_maxusers(0), m_maxusers_time(0)
{
}

/*************************************************************************/

Stats::Stats(identifier_type id) : Record(id), m_maxusers(0),
		m_maxusers_time(0)
{	
}

/*************************************************************************/

Stats::Stats(Stats const &s) : Record(s), m_maxusers(s.m_maxusers),
		m_maxusers_time(s.m_maxusers_time)
{
}

/*************************************************************************/

Stats &Stats::operator=(Stats const &right)
{
	if (this != &right)
	{
		Record::operator=(right);
		
		m_maxusers = right.m_maxusers;
		m_maxusers_time = right.m_maxusers_time;
	}
	
	return *this;
}

/*************************************************************************/

uint32_t Stats::get_ui(Field const *field) const
{
	if (field == f_maxusers)
		return maxusers();
	else
		return Record::get_ui(field);
}

/*************************************************************************/

time_t Stats::get_ut(Field const *field) const
{
	if (field == f_maxusers_time)
		return maxusers_time();
	else
		return Record::get_ut(field);
}

/*************************************************************************/

uint32_t Stats::maxusers(uint32_t maxusers)
{
	if (maxusers != m_maxusers) 
	{
		if (!stats.change_ui(this, f_maxusers, maxusers))
			return m_maxusers;
		m_maxusers = maxusers;
	}
	
	return m_maxusers;
}

/*************************************************************************/

time_t Stats::maxusers_time(time_t maxusers_time)
{
	if (db_status() & dbsUpdating)
	{
		/* We don't allow time in the future. */
		if (maxusers_time > time(0))
		{
			/* If it's already a member, just return the old time. */
			if (db_status() & dbsMember)
				return m_maxusers_time;
			/* Else, assign a correct time. */
			maxusers_time = time(0);
		}
	}
				
	
	if (maxusers_time != m_maxusers_time)
	{
		if (!stats.change_ut(this, f_maxusers_time, maxusers_time))
			return m_maxusers_time;
		m_maxusers_time = maxusers_time;
	}
	
	return m_maxusers_time;
}

/*************************************************************************/

void Stats::new_max(uint32_t max, time_t time)
{
	if (max == m_maxusers && time == m_maxusers_time)
		;
	else if (max == m_maxusers)
		maxusers_time(time);
	else if (time == m_maxusers_time)
		maxusers(max);
	else
	{
		if (!stats.change_multi(this, 2, f_maxusers, max,
				f_maxusers_time, time))
			return;
		m_maxusers = max;
		m_maxusers_time = time;
	}
}

/*************************************************************************/

size_t Stats::memory_usage() const
{
	size_t result = Record::memory_usage();
	result += sizeof(Stats);
	return result;
}

/*************************************************************************/

uint32_t Stats::set_ui(Field const *field, uint32_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ui(field, value);
	
	if (field == f_maxusers)
		return maxusers(value);
	else
		return Record::set_ui(field, value);
}

/*************************************************************************/

time_t Stats::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ut(field, value);
	
	if (field == f_maxusers_time)
		return maxusers_time(value);
	else
		return Record::set_ut(field, value);
}

/*************************************************************************/

bool StatsList::add(databases::Record *record, int *err)
{
	if (!Collection::add(record, err))
		return false;
	
	stats_rec = (Stats *)record;
	return true;
}

/*************************************************************************/

Record *StatsList::create(Record::identifier_type id) const
{
	if (size() != 0)
		return 0;
	
	if (id)
		return new Stats(id);
	else
		return new Stats;
}

/*************************************************************************/

bool StatsList::make_record()
{
	if (stats.size() == 0 && stats.db()->writable())
	{
		Stats *s = (Stats *)stats.create();
		s->maxusers(usercnt);
		s->maxusers_time(time(0));
		
		if (!stats.add(s, 0))
			epona->fatal("Can't add stats record to database operserv_stats");
	}
	
	return true;
}

/*************************************************************************/

void StatsList::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("operserv_stats");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpUpdate | Database::dpExport);
	
	Stats::f_maxusers = db->field_create("maxusers", 1, Field::ftUnsigned,
			4, 0, "0", 1);
	Stats::f_maxusers_time = db->field_create("maxusers_time", 1,
			Field::ftUnixTime, 0, 0, "0", 4);
	
	if (!db->field_add(Stats::f_maxusers) || 
			!db->field_add(Stats::f_maxusers_time))
		epona->fatal("Can't add fields to database operserv_stats");
	
	this->db(db);
}

/*************************************************************************/

SZLine::SZLine(IPNetBan *nb) : NBInfo()
{
	m_nb = nb;
}

/*************************************************************************/

SZLine::SZLine(IPNetBan *nb, identifier_type id) : NBInfo(id)
{
	m_nb = nb;
}

/*************************************************************************/

bool SZLine::change_creator(char const *value)
{
	return szlines.change_s(this, f_creator, value);
}

/*************************************************************************/

bool SZLine::change_expires(time_t value)
{
	return szlines.change_ut(this, f_expires, value);
}

/*************************************************************************/

bool SZLine::change_reason(char const *value)
{
	return szlines.change_s(this, f_reason, value);
}

/*************************************************************************/

bool SZLine::change_since(time_t value)
{
	return szlines.change_ut(this, f_since, value);
}

/*************************************************************************/

char const *SZLine::get_s(Field const *field) const
{
	if (field == f_mask)
		return mask();
	else if (field == f_creator)
		return creator();
	else if (field == f_reason)
		return reason();
	else
		return Record::get_s(field);
}

/*************************************************************************/

time_t SZLine::get_ut(Field const *field) const
{
	if (field == f_since)
		return since();
	else if (field == f_expires)
		return expires();
	else
		return Record::get_ut(field);
}

/*************************************************************************/

char const *SZLine::set_s(Field const *field, char const *value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_s(field, value);
	
	if (field == f_mask)
		return mask(value);
	else if (field == f_creator)
		return creator(value);
	else if (field == f_reason)
		return reason(value);
	else
		return Record::set_s(field, value);
}

/*************************************************************************/

time_t SZLine::set_ut(Field const *field, time_t value)
{
	if (!(db_status() & dbsUpdating))
		return Record::set_ut(field, value);
	
	if (field == f_since)
		return since(value);
	else if (field == f_expires)
		return expires(value);
	else
		return Record::set_ut(field, value);
}

/*************************************************************************/

SZLineList::SZLineList() : databases::ArrayCollection(true), m_banner(0)
{ 
	m_banner = (IPNetBanner *)netforce.ban_supports(IPNetBanner::IID);
}

/*************************************************************************/

bool SZLineList::add_list(databases::Record *record)
{
	SZLine *sz = (SZLine *)record;
	
	if (!m_banner->add(sz->netban()))
		return false;
	
	if (!(status() & csInitial))
		sz->netban()->apply();
	
	return ArrayCollection::add_list(record);
}

/*************************************************************************/

Record *SZLineList::create(Record::identifier_type id) const
{
	if (!m_banner)
		return 0;
	
	IPNetBan *nb = (IPNetBan *)m_banner->create();
	
	if (!nb)
		return 0;
	
	if (id)
		return new SZLine(nb, id);
	else
		return new SZLine(nb);
}

/*************************************************************************/

void SZLineList::clear_list()
{
	iterator it, end;
	
	for (it = this->begin(), end = this->end(); it != end; ++it)
	{
		SZLine *sz = (SZLine *)(*it);
		m_banner->remove(sz->netban());
	}
	
	ArrayCollection::clear_list();
}

/*************************************************************************/

void SZLineList::expire()
{
	if (!writable())
		return;
	
	time_t now = time(0);
	
	SZLine *sz;
	iterator it, end;
	
	for (it = begin(), end = this->end(); it != end; )
	{
		sz = (SZLine *)*it++;
		
		if (!sz->expires() || sz->expires() > now)
			continue;
		
		if (epona->cd().operserv.wall &
				Epona::Directives::OperServ::Wall::waSZLineExpire)
		    wallops(s_OperServ, "SZLINE on %s has expired", sz->mask());
		
		/* If this returns false, ensure we've not switched to
		 * read-only before continuing. */
		if (!szlines.remove(sz) && !szlines.writable())
			return;
	}
}

/*************************************************************************/

void SZLineList::remove_list(databases::Record *record)
{
	SZLine *sz = (SZLine *)record;
	
	m_banner->remove(sz->netban());
	ArrayCollection::remove_list(record);
}

/*************************************************************************/

void SZLineList::setup_database(databases::Database *db)
{
	db->coll(this);
	db->name("operserv_szlines");
	db->version(Database::vtExport, 1);
	db->version(Database::vtMinimum, 1);
	db->perms_add(Database::dpInitial |
			Database::dpImport | Database::dpExport);
	
	SZLine::f_mask = db->field_create("mask", 1, Field::ftString,
			SZLine::mask_len, 0, NULL, 1);
	SZLine::f_creator = db->field_create("creator", 1, Field::ftString,
			SZLine::creator_len, 0, NULL, 2);
	SZLine::f_reason = db->field_create("reason", 1, Field::ftString,
			SZLine::reason_len, 0, NULL, 3);
	SZLine::f_since = db->field_create("since", 1, Field::ftUnixTime,
			0, 0, "0", 4);
	SZLine::f_expires = db->field_create("expires", 1, Field::ftUnixTime,
			0, 0, "0", 5);
	
	if (!db->field_add(SZLine::f_mask) || 
			!db->field_add(SZLine::f_creator) ||
			!db->field_add(SZLine::f_reason) ||
			!db->field_add(SZLine::f_since) ||
			!db->field_add(SZLine::f_expires))
		epona->fatal("Can't add fields to database operserv_szlines");
	
	this->db(db);
}

/*************************************************************************/
